package ru.loylabs.portal.aop;

import lombok.RequiredArgsConstructor;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Component;
import ru.loylabs.portal.entity.Achievement;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.CredentialPermission;
import ru.loylabs.portal.presentation.SecurityUtil;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.TaskRepository;

import java.util.UUID;

import static ru.loylabs.portal.enums.TaskStatus.DONE;

@Aspect
@Component
@RequiredArgsConstructor
public class AchievementAspect {

	private final TaskRepository taskRepository;


	@Pointcut("execution(ru.loylabs.portal.entity.Achievement " +
			"ru.loylabs.portal.service.interfaces.AchievementService.*(..))")
	private void allWhichReturnAchievement() {
	}

	@Pointcut("execution(org.springframework.data.domain.Page<ru.loylabs.portal.entity.Achievement> " +
			"ru.loylabs.portal.service.interfaces.AchievementService.*(..))")
	private void allWhichReturnAchievementsPage() {
	}


	@AfterReturning(returning = "achievement", pointcut = "allWhichReturnAchievement()")
	private void initCountOfDoneTasks(Achievement achievement) {
		achievement.setCountOfDoneTasks(taskRepository.countByAchievementAndStatus(achievement, DONE));
	}

	@AfterReturning(returning = "achievements", pointcut = "allWhichReturnAchievementsPage()")
	private void initCountOfDoneTasksInPage(Page<Achievement> achievements) {
		achievements.forEach(this::initCountOfDoneTasks);
	}

	@AfterReturning(returning = "achievement", pointcut = "allWhichReturnAchievement()")
	private void initTasksSize(Achievement achievement) {
		achievement.setTasksSize(taskRepository.countByAchievement(achievement));
	}

	@AfterReturning(returning = "achievements",
			pointcut = "allWhichReturnAchievementsPage()")
	private void initTasksSizesInPage(Page<Achievement> achievements) {
		achievements.forEach(this::initTasksSize);
	}

	@Before(value = "execution(* ru.loylabs.portal.service.interfaces.AchievementService.getAllPersonalAchievements(..))" +
			" && args(page, size, sort, criteria, ownerId)", argNames = "page,size,sort,criteria,ownerId")
	public void beforeGetAllPersonalAchievements(int page,
												 int size,
												 String sort,
												 SearchReversePolishNotation criteria,
												 UUID ownerId) {
		var person = SecurityUtil.getCredential().getPerson();
		if (!(person.getId().equals(ownerId) || personCanAllAchievementRead(person))) {
			criteria.add(SearchCriteria.builder().key("tasks.observers.id").operation(":").value(person.getId()).build());
		}
	}

	private boolean personCanAllAchievementRead(Person person) {
		return person.getRole().getPermissions()
				.stream()
				.anyMatch(perm -> perm.getPermission().equals(CredentialPermission.ACHIEVEMENT_READ));
	}
}
