package ru.loylabs.portal.config.jwt;

import io.jsonwebtoken.Claims;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.security.web.authentication.WebAuthenticationDetailsSource;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;
import org.springframework.web.filter.OncePerRequestFilter;
import ru.loylabs.portal.exception.ForbiddenException;
import ru.loylabs.portal.service.security.Credential;

import javax.servlet.FilterChain;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_BANNED;

@Slf4j
@Component
@RequiredArgsConstructor
public class JwtFilter extends OncePerRequestFilter {

	private static final String AUTHORIZATION = "Authorization";
	private final JwtProvider jwtProvider;
	private final JwtUtils jwtUtils;

	@Override
	public void doFilterInternal(@NonNull HttpServletRequest request,
								 @NonNull HttpServletResponse response,
								 @NonNull FilterChain fc) throws IOException, ServletException {
		final String token = getTokenFromRequest(request);

		if (token != null && jwtProvider.validateAccessToken(token)) {
			final Claims claims = jwtProvider.getAccessClaims(token);
			final Credential credential = jwtUtils.generate(claims);

			if (!credential.isEnabled()) throw new ForbiddenException(PERSON_BANNED, credential.getPerson().getId());

			UsernamePasswordAuthenticationToken authentication =
					new UsernamePasswordAuthenticationToken(
							credential,
							null,
							credential.getAuthorities());
			authentication.setDetails(new WebAuthenticationDetailsSource().buildDetails(request));
			SecurityContextHolder.getContext().setAuthentication(authentication);
		}
		fc.doFilter(request, response);
	}

	private String getTokenFromRequest(HttpServletRequest request) {
		final String bearer = request.getHeader(AUTHORIZATION);

		if (StringUtils.hasText(bearer) && bearer.startsWith("Bearer ")) {
			return bearer.substring(7);
		} else {
			return bearer;
		}
	}
}