package ru.loylabs.portal.config.ws;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.messaging.simp.stomp.StompCommand;
import org.springframework.messaging.simp.stomp.StompEncoder;
import org.springframework.messaging.simp.stomp.StompHeaderAccessor;
import org.springframework.web.socket.*;
import org.springframework.web.socket.handler.WebSocketHandlerDecorator;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.service.interfaces.WebSocketMessageParser;
import ru.loylabs.portal.service.interfaces.WebSocketSecurity;

import java.io.IOException;

import static org.springframework.messaging.simp.stomp.DefaultStompSession.EMPTY_PAYLOAD;
import static ru.loylabs.portal.exception.BaseException.Exception.WS_AUTH_ERROR;

@Slf4j
public class CustomWebSocketHandler extends WebSocketHandlerDecorator {

	private WebSocketMessageParser messageParser;
	private WebSocketSecurity security;
	private final StompEncoder stompEncoder = new StompEncoder();

	public CustomWebSocketHandler(WebSocketHandler delegate) {
		super(delegate);
	}

	@Autowired
	public void setMessageParser(WebSocketMessageParser messageParser) {
		this.messageParser = messageParser;
	}

	@Autowired
	public void setSecurity(WebSocketSecurity security) {
		this.security = security;
	}

	@Override
	public void handleMessage(WebSocketSession session, WebSocketMessage<?> message) throws Exception {
		try {
			validateToken(session, message);
		} catch (ApplicationException ex) {
			sendErrorMessageAndTerminateConnection(session, ex);
		}

		getDelegate().handleMessage(session, message);
	}

	private void validateToken(WebSocketSession session, WebSocketMessage<?> message) throws ApplicationException {
		if (!security.validateToken(messageParser.parseHeaders(message)))
			throw new ApplicationException(WS_AUTH_ERROR, session.getId());
	}

	//from org.springframework.web.socket.messaging.StompSubProtocolHandler
	private void sendErrorMessageAndTerminateConnection(WebSocketSession session, Throwable error) {
		log.error("WS error {}", error.getMessage());

		StompHeaderAccessor headerAccessor = StompHeaderAccessor.create(StompCommand.ERROR);
		headerAccessor.setMessage(error.getMessage());

		byte[] bytes = this.stompEncoder.encode(headerAccessor.getMessageHeaders(), EMPTY_PAYLOAD);

		try {
			session.sendMessage(new TextMessage(bytes));
		} catch (Throwable ex) {
			// Could be part of normal workflow (e.g. browser tab closed)
		} finally {
			try {
				session.close(CloseStatus.PROTOCOL_ERROR);
			} catch (IOException ex) {
				// Ignore
			}
		}
	}
}
