package ru.loylabs.portal.exception;

import lombok.extern.slf4j.Slf4j;
import org.flywaydb.core.internal.util.ExceptionUtils;
import org.springframework.data.mapping.PropertyReferenceException;
import org.springframework.data.redis.RedisConnectionFailureException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseStatus;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.multipart.MultipartException;
import ru.loylabs.portal.presentation.view.common.FailResponse;

import java.io.FileNotFoundException;

@Slf4j
@RestControllerAdvice
public class WebExceptionHandler {

	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ExceptionHandler(ApplicationException.class)
	public ResponseEntity<Object> handleApplicationException(ApplicationException e) {
		log.error("Service error {}", e.getMessage());
		return ResponseEntity.badRequest().body(new FailResponse(e.getErrorCode(), e.getMessage()));
	}

	@ResponseStatus(HttpStatus.FORBIDDEN)
	@ExceptionHandler(ForbiddenException.class)
	public ResponseEntity<Object> handleForbiddenException(ForbiddenException e) {
		log.error("Service error {}", e.getMessage());
		return new ResponseEntity<>(new FailResponse(e.getErrorCode(), e.getMessage()), HttpStatus.FORBIDDEN);
	}

	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ExceptionHandler(IllegalArgumentException.class)
	public ResponseEntity<Object> handleIllegalArgumentException(IllegalArgumentException e) {
		log.error("Service error {}", ExceptionUtils.getRootCause(e).getMessage());
		return ResponseEntity.badRequest().body(new FailResponse(0, ExceptionUtils.getRootCause(e).getMessage()));
	}

	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ExceptionHandler(MultipartException.class)
	public ResponseEntity<Object> handleMultipartException(MultipartException e) {
		log.error("Service error {}", ExceptionUtils.getRootCause(e).getMessage());
		return ResponseEntity.badRequest().body(new FailResponse(0, ExceptionUtils.getRootCause(e).getMessage()));
	}

	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ExceptionHandler(PropertyReferenceException.class)
	public ResponseEntity<Object> handlePropertyReferenceException(PropertyReferenceException e) {
		log.error("Service error {}", ExceptionUtils.getRootCause(e).getMessage());
		return ResponseEntity.badRequest().body(new FailResponse(0, ExceptionUtils.getRootCause(e).getMessage()));
	}

	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ExceptionHandler(MethodArgumentNotValidException.class)
	public ResponseEntity<Object> handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
		String error = e.getBindingResult().getFieldErrors()
				.stream()
				.map(x -> x.getField() + " " + x.getDefaultMessage())
				.findFirst().orElse("INTERNAL SERVER ERROR");

		log.error("Service error {}", error);
		return ResponseEntity.badRequest().body(new FailResponse(0, error));
	}

	@ResponseStatus(HttpStatus.NOT_FOUND)
	@ExceptionHandler(FileNotFoundException.class)
	public ResponseEntity<Object> handleFileNotFoundException(FileNotFoundException e) {
		log.error("Service error {}", ExceptionUtils.getRootCause(e).getMessage());
		return ResponseEntity.notFound().build();
	}

	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	@ExceptionHandler(RedisConnectionFailureException.class)
	public ResponseEntity<Object> handleRedisConnectionFailureException(RedisConnectionFailureException e) {
		log.error("Service error {}", e.getMessage());
		return ResponseEntity.internalServerError().body(new FailResponse(-1000, "Redis not setup"));
	}
}
