package ru.loylabs.portal.listener.task;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.BellNotification;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.entity.Task;
import ru.loylabs.portal.listener.BaseEventListener;
import ru.loylabs.portal.listener.events.TaskEvents;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.PersonalBellNotificationRepository;
import ru.loylabs.portal.repository.TaskRepository;
import ru.loylabs.portal.service.interfaces.BellNotificationFactory;
import ru.loylabs.portal.service.interfaces.BellNotificationPersister;
import ru.loylabs.portal.service.interfaces.NotificationSender;
import ru.loylabs.portal.service.notification.bell.data.TaskAppointObserverNotificationData;

import java.util.Collection;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

@Slf4j
@Component
@RequiredArgsConstructor
public class TaskObserverAppointListener implements BaseEventListener<TaskEvents.TaskUpdateObserversEvent> {

	private final BellNotificationPersister persister;
	private final BellNotificationFactory notificationFactory;
	private final PersonalBellNotificationRepository notificationRepository;
	private final TaskRepository taskRepository;
	private final PersonRepository personRepository;
	@Qualifier("bellNotificationSenderImpl")
	private final NotificationSender<BellNotification> wsSender;

	@Override
	@EventListener
	@Transactional
	public void handle(TaskEvents.TaskUpdateObserversEvent event) {
		taskRepository.findByIdAndAchievementId(event.getTaskId(), event.getAchievementId())
				.ifPresent(task -> {
					personRepository.findById(event.getAppointerId())
							.ifPresent(appointer -> sentNotification(event.getOldObserverIds(), task, appointer));
					deleteNotificationRemoteObservers(task, event.getOldObserverIds());
				});
	}

	private void sentNotification(Collection<UUID> oldObserverIds, Task task, Person appointer) {
		var notification = notificationFactory.create(
				new TaskAppointObserverNotificationData(
						task.getAchievement(),
						task,
						appointer)
		);
		persister.persist(notification, filterNewObservers(task, oldObserverIds));
		wsSender.send(notification);
	}


	private void deleteNotificationRemoteObservers(Task task, Collection<UUID> oldObserversId) {
		var remoteObserversIds = filterRemoteObserversIds(task, oldObserversId);
		notificationRepository
				.deleteAllByPersonIdInAndBellNotificationSubjectId(
						remoteObserversIds,
						task.getId());

		task.getComments().stream()
				.map(Record::getId)
				.forEach(commentId -> notificationRepository
						.deleteAllByPersonIdInAndBellNotificationSubjectId(
								remoteObserversIds,
								commentId));
	}

	private Set<Person> filterNewObservers(Task task, Collection<UUID> oldObserversId) {
		return task.getObservers()
				.stream()
				.filter(person -> !oldObserversId.contains(person.getId()))
				.collect(Collectors.toSet());
	}

	private Set<UUID> filterRemoteObserversIds(Task task, Collection<UUID> oldObserversId) {
		var observersIds = task.getObservers()
				.stream()
				.map(Person::getId)
				.collect(Collectors.toSet());

		return oldObserversId.stream()
				.filter(id -> !observersIds.contains(id))
				.collect(Collectors.toSet());
	}
}
