package ru.loylabs.portal.listener.task;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.event.EventListener;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.BellNotification;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Task;
import ru.loylabs.portal.enums.TaskStatus;
import ru.loylabs.portal.listener.BaseEventListener;
import ru.loylabs.portal.service.notification.bell.data.TaskObservedStatusUpdateNotificationData;
import ru.loylabs.portal.service.notification.bell.data.TaskStatusUpdateNotificationData;
import ru.loylabs.portal.listener.events.TaskEvents;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.TaskRepository;
import ru.loylabs.portal.repository.redis.TaskDoneRepository;
import ru.loylabs.portal.service.interfaces.BellNotificationFactory;
import ru.loylabs.portal.service.interfaces.BellNotificationPersister;
import ru.loylabs.portal.service.interfaces.NotificationSender;

@Slf4j
@Component
@RequiredArgsConstructor
public class TaskStatusUpdateListener implements BaseEventListener<TaskEvents.TaskStatusUpdateEvent> {

	private final BellNotificationPersister persister;
	private final BellNotificationFactory notificationFactory;
	private final TaskRepository taskRepository;
	private final PersonRepository personRepository;
	private final TaskDoneRepository delayedNotificationRepository;
	@Qualifier("bellNotificationSenderImpl")
	private final NotificationSender<BellNotification> wsSender;

	@Override
	@EventListener
	@Transactional
	public void handle(TaskEvents.TaskStatusUpdateEvent event) {
		taskRepository.findByIdAndAchievementId(event.getTaskId(), event.getAchievementId())
				.ifPresent(task -> {
					personRepository.findById(event.getUpdatedById())
							.ifPresent(updateBy -> {
										if (task.getStatus() == TaskStatus.DONE) {
											delayedNotificationRepository.save(task);
										}
										notifyAchievementOwner(task, updateBy);
										notifyTaskObservers(task, updateBy);
									}
							);
				});
	}

	private void notifyTaskObservers(Task task, Person updateBy) {
		var notificationForObserver = notificationFactory.create(
				new TaskObservedStatusUpdateNotificationData(task.getAchievement(), task, updateBy)
		);
		persister.persist(notificationForObserver, task.getObservers());
		wsSender.send(notificationForObserver);
	}

	private void notifyAchievementOwner(Task task, Person updateBy) {
		var notification = notificationFactory.create(
				new TaskStatusUpdateNotificationData(task.getAchievement(), task, updateBy)
		);
		persister.persist(notification, task.getAchievement().getOwner());
		wsSender.send(notification);
	}


}
