package ru.loylabs.portal.mapper;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.MappingConstants;
import org.mapstruct.Named;
import org.mapstruct.NullValuePropertyMappingStrategy;
import org.mapstruct.ReportingPolicy;
import ru.loylabs.portal.entity.Contact;
import ru.loylabs.portal.entity.Job;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.SecurityRole;
import ru.loylabs.portal.enums.Gender;
import ru.loylabs.portal.service.yandex.dto.Yandex360CreateUserRequest;
import ru.loylabs.portal.service.yandex.dto.Yandex360UpdateUserRequest;
import ru.loylabs.portal.service.yandex.dto.Yandex360UserContactRequest;
import ru.loylabs.portal.service.yandex.dto.Yandex360UserNameRequest;
import ru.loylabs.portal.service.yandex.dto.Yandex360UserResponse;
import ru.loylabs.portal.utility.StringUtils;

@Mapper(componentModel = MappingConstants.ComponentModel.SPRING,
		uses = { ContactMapper.class, JobMapper.class },
		unmappedTargetPolicy = ReportingPolicy.IGNORE,
		nullValuePropertyMappingStrategy = NullValuePropertyMappingStrategy.IGNORE,
		imports = { Gender.class, StringUtils.class })
public interface Yandex360UserMapper {

	@Mapping(target = "externalId", source = "id")
	@Mapping(target = "isAdmin", constant = "false")
	@Mapping(target = "name", source = "person", qualifiedByName = "mapNameFromPerson")
	@Mapping(target = "nickname", source = "username")
	@Mapping(target = "password", ignore = true)
	@Mapping(target = "gender", expression = "java( person.getGender().name().toLowerCase() )")
	@Mapping(target = "birthday", source = "dob")
	@Mapping(target = "about", source = "description")
	@Mapping(target = "contacts", source = "person", qualifiedByName = "mapContactsFromPerson")
	@Mapping(target = "departmentId", expression = "java( person.getJob().getDepartment().getYandexDepartmentId() )")
	@Mapping(target = "position", expression = "java( person.getJob().getTitle().getName() )")
	@Mapping(target = "language", constant = "ru") // Always Russkiy yazik.
	@Mapping(target = "timezone", constant = "Europe/Moscow")
		// TODO: Perhaps determine timezone from Person's city
	Yandex360CreateUserRequest toCreateUserRequest(Person person);

	@Mapping(target = "externalId", source = "id")
	@Mapping(target = "isAdmin", ignore = true)
	@Mapping(target = "isEnabled", ignore = true)
//	@Mapping(target = "isEnabled", expression = "java( !person.isBlocked() )")
	@Mapping(target = "name", source = "person", qualifiedByName = "mapNameFromPerson")
	@Mapping(target = "password", ignore = true)
	@Mapping(target = "passwordChangeRequired", ignore = true)
	@Mapping(target = "gender", expression = "java( person.getGender().name().toLowerCase() )")
	@Mapping(target = "birthday", source = "dob")
	@Mapping(target = "about", source = "description")
	@Mapping(target = "contacts", source = "person", qualifiedByName = "mapContactsFromPerson")
	@Mapping(target = "departmentId", expression = "java( person.getJob().getDepartment().getYandexDepartmentId() )")
	@Mapping(target = "position", expression = "java( person.getJob().getTitle().getName() )")
	@Mapping(target = "language", constant = "ru") // Always Russkiy yazik.
	@Mapping(target = "timezone", constant = "Europe/Moscow")
		// TODO: Perhaps determine timezone from Person's city
	Yandex360UpdateUserRequest toUpdateUserRequest(Person person);

	@Mapping(target = "id", ignore = true)
	@Mapping(target = "username", source = "response.email")
	@Mapping(target = "password", source = "password")
	@Mapping(target = "name", source = "response.name.first")
	@Mapping(target = "surname", source = "response.name.last")
	@Mapping(target = "patronymic", source = "response.name.middle")
	@Mapping(target = "dob", source = "response.birthday")
	@Mapping(target = "gender", expression = "java( Gender.fromString(response.getGender()) )")
	@Mapping(target = "description", source = "response.about")
	@Mapping(target = "city", ignore = true)
	@Mapping(target = "job", source = "job")
	@Mapping(target = "contact", source = "contact")
	@Mapping(target = "hired", source = "hired")
	@Mapping(target = "fired", ignore = true)
	@Mapping(target = "externalUserId", source = "response.id")
	@Mapping(target = "blocked", constant = "false")
	@Mapping(target = "role", source = "role")
	Person toEntity(Yandex360UserResponse response, String password, Job job,
					Contact contact, LocalDate hired, SecurityRole role);

	@Named("mapNameFromPerson")
	default Yandex360UserNameRequest mapNameFromPerson(Person person) {
		return new Yandex360UserNameRequest(person.getName(), person.getSurname(), person.getPatronymic());
	}

	@Named("mapContactsFromPerson")
	default List<Yandex360UserContactRequest> mapContactsFromPerson(Person person) {
		List<Yandex360UserContactRequest> contacts = new ArrayList<>(8);
		Contact contact = person.getContact();

		Optional.ofNullable(contact.getEmail())
				.filter(email -> !email.isBlank())
				.ifPresent(email -> contacts.add(new Yandex360UserContactRequest("E-mail", "email", email)));

		Optional.ofNullable(contact.getPhone())
				.filter(phone -> !phone.isBlank())
				.ifPresent(phone -> contacts.add(new Yandex360UserContactRequest("Телефон", "phone", phone)));

		Optional.ofNullable(contact.getSkype())
				.filter(skype -> !skype.isBlank())
				.ifPresent(skype -> contacts.add(new Yandex360UserContactRequest("Skype", "skype", skype)));

		return contacts;
	}

}
