package ru.loylabs.portal.presentation;

import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.exception.BaseException;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchElement;
import ru.loylabs.portal.presentation.search.SearchOperation;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.UUID;
import java.util.regex.Pattern;

import static ru.loylabs.portal.presentation.search.SearchOperation.AND;
import static ru.loylabs.portal.presentation.search.SearchOperation.OR;

public class WebUtils {

	private static final String STRING_PATTERN = "([\\w\\-.]?|([а-яА-ЯёЁ0-9_\\-.]?))+?";
	private static final String LOCAL_DATE_TIME_PATTERN = "(\\d{4})-(\\d{2})-(\\d{2})T(\\d{2}):(\\d{2}):(\\d{2})";
	private static final String LOCAL_DATE_PATTERN = "(\\d{4})-(\\d{2})-(\\d{2})";
	private static final String EMAIL_PATTERN = "(?:[\\w!#$%&'*+/=?^_`{|}~-]+(?:\\.[\\w!#$%&'*+/=?^_`{|}~-]+)*|\"(?:[\\x01-\\x08\\x0b\\x0c\\x0e-\\x1f\\x21\\x23-\\x5b\\x5d-\\x7f]|\\\\[\\x01-\\x09\\x0b\\x0c\\x0e-\\x7f])*\")@(?:(?:[\\w](?:[\\w-]*[\\w])?\\.)+[\\w](?:[\\w-]*[\\w])?|\\[(?:(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9]))\\.){3}(?:(2(5[0-5]|[0-4][0-9])|1[0-9][0-9]|[1-9]?[0-9])|[a-z0-9-]*[\\w]:(?:[\\x01-\\x08\\x0b\\x0c\\x0e-\\x1f\\x21-\\x5a\\x53-\\x7f]|\\\\[\\x01-\\x09\\x0b\\x0c\\x0e-\\x7f])+)\\])";
	private static final String UUID_PATTERN = "([0-9a-fA-F]{8})-([0-9a-fA-F]{4})-([0-9a-fA-F]{4})-([0-9a-fA-F]{4})-([0-9a-fA-F]{12})";
	private static final String NUMBER_PATTERN = "^\\d+$";
	private static final String BOOLEAN_PATTERN = "false|true";

	private static final char LOGIC_START = '(';
	private static final char LOGIC_END = ')';
	private static final char LOGIC_AND = ',';
	private static final char LOGIC_OR = '+';

	private static final int START_GROUP = 1;
	private static final int KEY_GROUP = 2;
	private static final int OPERATION_GROUP = 4;
	private static final int VALUE_GROUP = 5;
	private static final int END_GROUP = 26;
	private static final int LOGIC_GROUP = 27;

	private static final int STACK_MAX = 100;

	public static SearchReversePolishNotation getCriteria(String search) {
		var pattern = Pattern.compile("(\\" + LOGIC_START + "*)(\\w+?(\\.?\\w+?)*)(:|>|>:|<|<:|!:)("
				+ STRING_PATTERN + "|"
				+ EMAIL_PATTERN + "|"
				+ LOCAL_DATE_PATTERN + "|"
				+ LOCAL_DATE_TIME_PATTERN + "|"
				+ UUID_PATTERN + ")(\\" + LOGIC_END + "*)(" + LOGIC_AND + "|\\" + LOGIC_OR + ")");
		var matcher = pattern.matcher(search + LOGIC_AND);

		var executionOrder = new ArrayList<SearchElement>();
		var operatorStack = new LinkedList<Character>();

		while (matcher.find()) {
			if (executionOrder.size() > STACK_MAX)
				throw new ApplicationException(BaseException.Exception.SEARCH_STACK_OVERFLOW);

			var start = matcher.group(START_GROUP).length();
			var end = matcher.group(END_GROUP).length();
			var logic = getLogic(matcher.group(LOGIC_GROUP).charAt(0));

			var criteria = SearchCriteria.builder()
					.key(matcher.group(KEY_GROUP))
					.operation(matcher.group(OPERATION_GROUP))
					.value(getValue(matcher.group(VALUE_GROUP)))
					.build();

			for (; start > 0; start--) {
				operatorStack.push(LOGIC_START);
			}

			executionOrder.add(criteria);

			for (; end > 0; end--) {
				while (!operatorStack.isEmpty() && operatorStack.peek() != LOGIC_START) {
					executionOrder.add(getLogic(operatorStack.pop()));
				}
				if (!operatorStack.isEmpty() && operatorStack.peek() == LOGIC_START) {
					operatorStack.pop();
				}
			}

			while (!operatorStack.isEmpty() && isOperator(operatorStack.peek()) &&
					getPriority(logic) <= getPriority(getLogic(operatorStack.peek()))) {
				executionOrder.add(getLogic(operatorStack.pop()));
			}
			if (logic == OR) operatorStack.push(LOGIC_OR);
			else operatorStack.push(LOGIC_AND);
		}

		if (!operatorStack.isEmpty()) operatorStack.pop();

		while (!operatorStack.isEmpty()) {
			executionOrder.add(getLogic(operatorStack.pop()));
		}

		return new SearchReversePolishNotation(executionOrder);
	}

	private static Object getValue(String group) {
		var localDateTimePattern = Pattern.compile(LOCAL_DATE_TIME_PATTERN);
		var localDatePattern = Pattern.compile(LOCAL_DATE_PATTERN);
		var booleanPattern = Pattern.compile(BOOLEAN_PATTERN);
		var numberPatter = Pattern.compile(NUMBER_PATTERN);
		var UUIDPattern = Pattern.compile(UUID_PATTERN);
		if (localDateTimePattern.matcher(group).matches())
			return LocalDateTime.parse(group);
		if (localDatePattern.matcher(group).matches())
			return LocalDate.parse(group);
		if (UUIDPattern.matcher(group).matches())
			return UUID.fromString(group);
		if (numberPatter.matcher(group).matches())
			return Long.parseLong(group);
		if (booleanPattern.matcher(group).matches())
			return Boolean.parseBoolean(group);
		return group;
	}

	private static SearchOperation getLogic(char logicOperator) {
		switch (logicOperator) {
			case LOGIC_AND -> {
				return AND;
			}
			case LOGIC_OR -> {
				return OR;
			}
			default -> {
				return null;
			}
		}
	}

	private static boolean isOperator(char logicOperator) {
		return logicOperator == LOGIC_AND || logicOperator == LOGIC_OR;
	}

	private static int getPriority(SearchOperation operation) {
		if (operation == null) return 0;
		if (operation == OR) return 1;
		return 2;
	}
}
