package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.facade.interfaces.TaskFacade;
import ru.loylabs.portal.mapper.AchievementMapper;
import ru.loylabs.portal.mapper.CommentMapper;
import ru.loylabs.portal.mapper.TaskMapper;
import ru.loylabs.portal.mapper.TaskTagMapper;
import ru.loylabs.portal.presentation.request.*;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.view.AchievementView;
import ru.loylabs.portal.presentation.view.CommentView;
import ru.loylabs.portal.presentation.view.TaskTagView;
import ru.loylabs.portal.presentation.view.TaskView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.Collections;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.AchievementStatus.ARCHIVED;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.*;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + ACHIEVEMENTS_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class AchievementController {

	private final TaskService taskService;
	private final TaskFacade taskFacade;
	private final TaskMapper taskMapper;

	private final AchievementService achievementService;
	private final AchievementMapper achievementMapper;

	private final TaskTagService tagService;
	private final TaskTagMapper tagMapper;

	private final CommentService commentService;
	private final CommentMapper commentMapper;

	private final TaskRailService railService;

//--ACHIEVEMENTS------------------------------------------------------------------------------------------------------//

	@ApiIgnore
	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_READ + "')")
	@ApiOperation("Получения всех целей с сортировкой и фильтрацией")
	@GetMapping
	public ListView<AchievementView> findAllAchievements(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,name") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for Achievement list");
		var criteria = getCriteria(search);
		var achievements = achievementService.getAllAchievements(page, size, sort, criteria);
		return new ListView<>(achievements.getTotalElements(),
							  achievements.getContent()
										  .stream()
										  .map(achievementMapper::toView)
										  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_READ + "') || hasAuthority('" + ACHIEVEMENT_READ_OWN + "')")
	@ApiOperation("Получения всех целей юзера по ID")
	@GetMapping(PERSONAL_URI + PERSON_ID_URI_PARAM)
	public ListView<AchievementView> findAllPersonalAchievements(
			@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "creationDate,asc") String sort,
			@RequestParam(required = false) String search,
			@RequestParam boolean archived
	) {
		log.info("Request for personal Achievement list for Person with id '{}'", personId);
		var criteria = getCriteria(search);
		criteria.add(SearchCriteria.builder().key("status").operation(archived ? ":" : "!:").value(ARCHIVED.toString()).build());
		var achievements = achievementService.getAllPersonalAchievements(page, size, sort, criteria, personId);
		return new ListView<>(achievements.getTotalElements(),
							  achievements.getContent()
										  .stream()
										  .map(achievementMapper::toView)
										  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_READ + "') || (hasAuthority('" + ACHIEVEMENT_READ_OWN + "')" +
				  " && @sec.isPrincipalAchievementParticipant(#achievementId))")
	@ApiOperation("Получение цели по ID")
	@GetMapping(ACHIEVEMENT_ID_URI_PARAM)
	public AchievementView findOneAchievement(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId) {
		log.info("Request for Achievement with id '{}'", achievementId);
		return achievementMapper.toView(achievementService.getAchievement(achievementId));
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_WRITE + "') && " +
				  "!@sec.isPrincipalAccountOwner(#request.ownerId)")
	@ApiOperation("Создание новой цели")
	@PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
	public AchievementView createAchievement(@Valid @RequestBody CreateAchievementRequest request) {
		log.info("Request to create new Achievement with request: {}", request);
		return achievementMapper.toView(
				achievementService.createAchievement(request, getCredential().getPerson())
		);
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_WRITE + "')")
	@ApiOperation("Редактирование цели по ID")
	@PutMapping(path = ACHIEVEMENT_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public AchievementView updateAchievement(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
											 @Valid @RequestBody UpdateAchievementRequest request) {
		log.info("Request to update Achievement with id '{}' with request: {}", achievementId, request);
		return achievementMapper.toView(achievementService.updateAchievement(achievementId, request));
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_APPROVAL + "') && " +
			"!@sec.isPrincipalAchievementOwner(#achievementId)")
	@ApiOperation("Подтверждение цели")
	@PostMapping(ACHIEVEMENT_ID_URI_PARAM + CONFIRM_URI)
	public AchievementView approveAchievement(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId) {
		log.info("Request to approve Achievement with id '{}'", achievementId);
		return achievementMapper.toView(achievementService.confirmAchievement(achievementId,
																			 getCredential().getPerson()));
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_ARCHIVE + "') && @sec.isPrincipalAchievementOwner(#achievementId)")
	@ApiOperation("Архивация цели")
	@PostMapping(ACHIEVEMENT_ID_URI_PARAM + ARCHIVE_URI)
	public AchievementView archiveAchievement(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId) {
		log.info("Request to archive Achievement with id '{}'", achievementId);
		return achievementMapper.toView(achievementService.archiveAchievements(achievementId));
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_UNARCHIVE + "')")
	@ApiOperation("Вернуть цель в статус 'NEW'")
	@PostMapping(ACHIEVEMENT_ID_URI_PARAM + REASSIGN_URI)
	public AchievementView unarchiveAchievement(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId) {
		log.info("Request to un-archive Achievement with id '{}'", achievementId);
		return achievementMapper.toView(achievementService.setAchievementStatusNew(achievementId));
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_WRITE + "') && " +
			"!@sec.isPrincipalAchievementOwner(#achievementId)")
	@ApiOperation("Удаление цели по id")
	@DeleteMapping(ACHIEVEMENT_ID_URI_PARAM)
	public SuccessResponse deleteAchievement(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId) {
		log.info("Request to delete Achievement with id '{}'", achievementId);
		achievementService.deleteAchievement(achievementId);
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + ACHIEVEMENT_WRITE + "')")
	@ApiOperation("Очистить все архивированные достижения у пользователя по id")
	@DeleteMapping(PERSONAL_URI + PERSON_ID_URI_PARAM + ARCHIVED_URI)
	public SuccessResponse deleteArchivedAchievements(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request to delete archived Achievements for Person with id '{}'", personId);
		achievementService.deleteArchivedAchievementForPerson(personId);
		return new SuccessResponse();
	}

//--ACHIEVEMENT-TASKS-------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + TASK_READ + "') || (hasAuthority('" + TASK_READ_OWN + "')" +
				  " && @sec.isPrincipalAchievementParticipant(#achievementId))")
	@ApiOperation("Получения всех тасков с сортировкой и фильтрацией")
	@GetMapping(ACHIEVEMENT_ID_URI_PARAM + TASKS_URI)
	public ListView<TaskView> findAllTasks(
			@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "sequence,asc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for Task list for Achievement with id '{}'", achievementId);
		var criteria = getCriteria(search);
		var tasks = taskService.getAllTasks(achievementId, page, size, sort, criteria);
		return new ListView<>(tasks.getTotalElements(),
							  tasks.getContent()
								   .stream()
								   .map(taskMapper::toView)
								   .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + TASK_READ + "') || (hasAuthority('" + TASK_READ_OWN + "')" +
				  " && @sec.isPrincipalTaskParticipant(#achievementId, #taskId))")
	@ApiOperation("Получения таска по ID")
	@GetMapping(ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM)
	public TaskView findOneTask(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
								@PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId) {
		log.info("Request for Task with id '{}' for Achievement with id '{}'", taskId, achievementId);
		return taskMapper.toView(taskService.getTask(achievementId, taskId));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE + "') && " +
			"!@sec.isPrincipalAchievementOwner(#achievementId)")
	@ApiOperation("Создание таска по ID цели")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
			dataType = "ru.loylabs.portal.presentation.request.CreateTaskRequest", paramType = "body")
	@PostMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI,
				 consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public TaskView createTask(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
							   @Valid @RequestPart CreateTaskRequest request,
							   @RequestPart(required = false) MultipartFile[] files) {
		log.info("Request to create new Task for Achievement with id '{}' with request: {}", achievementId, request);
		return taskMapper.toView(taskService.createTask(request,
				achievementId,
				Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()),
				getCredential().getPerson()));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE + "')")
	@ApiOperation("Изменение таска по ID")
	@PutMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public TaskView updateTask(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
							   @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
							   @Valid @RequestBody UpdateTaskRequest request) {
		log.info("Request to update Task with id '{}' for Achievement with id '{}' with request: {}",
				 taskId, achievementId, request);
		return taskMapper.toView(taskService.updateTask(request, achievementId, taskId,getCredential().getPerson()));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE + "') || (hasAuthority('" + TASK_WRITE_OWN + "')" +
				  " && @sec.isPrincipalTaskParticipant(#achievementId, #taskId))")
	@ApiOperation("Изменение статуса и последовательности задачи")
	@PutMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + MOVE_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public TaskView updateTaskStatusAndOrSequence(
			@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
			@PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
			@Valid @RequestBody UpdateTaskSequenceStatusRequest request
	) {
		log.info("Request to update sequence and status for Task with id '{}' for Achievement with id '{}' " +
				 "with request: {}", achievementId, taskId, request);
		return taskMapper.toView(taskFacade.updateTaskStatusAndSequence(request, achievementId, taskId,
																		getCredential().getPerson()));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE + "')")
	@ApiOperation("Добавление вложений к задаче")
	@PutMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + ATTACHMENTS_URI,
			consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public TaskView createTaskAttachment(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
										 @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
										 @RequestPart("file") MultipartFile[] files) {
		log.info("Request to create attachment for Task with id '{}'", taskId);
		var task = taskService.addAttachments(achievementId, taskId, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()));
		return taskMapper.toView(task);
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE + "')")
	@ApiOperation("Удаление вложений к задаче по ID")
	@DeleteMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + ATTACHMENTS_URI,
			consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public TaskView deleteTaskAttachment(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
										 @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
										 @RequestBody @Valid DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for Task with id '{}'", taskId);
		var task = taskService.deleteAttachments(achievementId, taskId, request);
		return taskMapper.toView(task);
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE + "')")
	@ApiOperation("Удаление таска по ID")
	@DeleteMapping(ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM)
	public SuccessResponse deleteTask(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
									  @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId) {
		log.info("Request to delete Task with id '{}' for Achievement with id '{}'", taskId, achievementId);
		taskService.deleteTask(achievementId, taskId);
		return new SuccessResponse();
	}

//--ACHIEVEMENT-TASK-TAGS---------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + TASK_READ_TAG + "')")
	@ApiOperation("Получение существующих тэгов в достижении")
	@GetMapping(ACHIEVEMENT_ID_URI_PARAM + TAGS_URI)
	public ListView<TaskTagView> findAllTags(
			@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "name,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for all Tags list for Achievement with id '{}'", achievementId);
		var criteria = getCriteria(search);
		var tags = tagService.getTagsInAchievement(page, size, sort, criteria, achievementId);
		return new ListView<>(tags.getTotalElements(),
							  tags.getContent()
								  .stream()
								  .map(tagMapper::toView)
								  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE_TAG + "')")
	@ApiOperation("Создание тэга для тасок в достижении")
	@PostMapping(path = ACHIEVEMENT_ID_URI_PARAM + TAGS_URI,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public TaskTagView createTag(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
								 @Valid @RequestBody CreateTaskTagRequest request) {
		log.info("Request to create new Tag for Achievement with id '{}' with request: {}", achievementId, request);
		return tagMapper.toView(tagService.createTaskTag(achievementId, request));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE_TAG + "')")
	@ApiOperation("Обновление тэга для тасок в достижении")
	@PutMapping(path = ACHIEVEMENT_ID_URI_PARAM + TAGS_URI + TAG_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public TaskTagView updateTag(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
								 @PathVariable(TAG_ID_URI_PARAM_NAME) UUID tagId,
								 @Valid @RequestBody UpdateTaskTagRequest request) {
		log.info("Request to update Tag with id '{}' for Achievement with id '{}' with request: {}",
				 tagId, achievementId, request);
		return tagMapper.toView(tagService.updateTaskTag(achievementId, tagId, request));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE_TAG + "')")
	@ApiOperation("Удаление тэга для тасок в достижении")
	@DeleteMapping(ACHIEVEMENT_ID_URI_PARAM + TAGS_URI + TAG_ID_URI_PARAM)
	public SuccessResponse deleteTag(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
									 @PathVariable(TAG_ID_URI_PARAM_NAME) UUID tagId) {
		log.info("Request to delete Tag with id '{}' for Achievement with id '{}'", tagId, achievementId);
		tagService.deleteTaskTag(achievementId, tagId);
		return new SuccessResponse();
	}

//--ACHIEVEMENT-TASK-TAGS-FOR-TASKS-----------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + TASK_WRITE_TAG + "')")
	@ApiOperation("Добавление тэга на таску")
	@PostMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + TAGS_URI,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public SuccessResponse addTagsToTask(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
										 @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
										 @Valid @RequestBody AddTaskTagsRequest request) {
		log.info("Request to add Tags to Task with id '{}' for Achievement with id '{}' with request: {}",
				 taskId, achievementId, request);
		tagService.addTagsOnTask(achievementId, taskId, request.getTagIds());
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE_TAG + "')")
	@ApiOperation("Удаление тэга с таски")
	@DeleteMapping(ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + TAGS_URI + TAG_ID_URI_PARAM)
	public SuccessResponse deleteTagFromTask(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
											 @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
											 @PathVariable(TAG_ID_URI_PARAM_NAME) UUID tagId) {
		log.info("Request for delete tag {} on task {} in achievement {}", tagId, taskId, achievementId);
		tagService.deleteTagFromTask(achievementId, taskId, tagId);
		return new SuccessResponse();
	}

//--ACHIEVEMENT-TASK-COMMENTS-----------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + TASK_READ + "') || (hasAuthority('" + TASK_READ_OWN + "')" +
				  " && hasAuthority('" + TASK_READ_COMMENT + "')" +
				  " && @sec.isPrincipalAchievementParticipant(#achievementId))")
	@ApiOperation("Получение событий и комментарий по ID таска")
	@GetMapping(ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + COMMENTS_URI)
	public ListView<CommentView> findAllTaskComments(
			@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
			@PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for Comment list for Task with id '{}' for Achievement with id '{}'", taskId, achievementId);
		var criteria = getCriteria(search);
		var comments = railService.getAllRecordInTaskRail(page, size, sort, criteria, taskId, RecordType.TASK);
		return new ListView<>(comments.getTotalElements(),
							  comments.getContent().stream().map(commentMapper::toView).collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE_COMMENT + "') || (hasAuthority('" + TASK_WRITE_OWN_COMMENT + "')" +
				  " && @sec.isPrincipalTaskParticipant(#achievementId, #taskId))")
	@ApiOperation("Создание комментария к таску по ID")
	@PostMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + COMMENTS_URI,
				 consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public CommentView createTaskComment(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
										 @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
										 @Valid @RequestPart(required = false) CreateRecordRequest request,
										 @RequestPart(required = false) MultipartFile[] files) {
		log.info("Request to create new comment for Task with id '{}' for Achievement with id '{}' with request: {}",
				 taskId, achievementId, request);
		return commentMapper.toView(
				commentService.saveComment(
						request,
						getCredential().getPerson(),
						taskId,
						RecordType.TASK,
						Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())
				)
		);
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE_COMMENT + "') || (hasAuthority('" + TASK_WRITE_OWN_COMMENT + "')" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Редактирование комментария по ID")
	@PutMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommentView updateTaskComment(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
										 @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
										 @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
										 @Valid @RequestBody UpdateRecordRequest request) {
		log.info("Request to update comment with id '{}' for Task with id '{}' for Achievement with id '{}' " +
				 "with request: {}", commentId, taskId, achievementId, request);
		return commentMapper.toView(
				commentService.updateComment(request, commentId, taskService.getTask(achievementId, taskId).getId())
		);
	}

	@PreAuthorize("hasAuthority('" + TASK_WRITE_COMMENT + "') || (hasAuthority('" + TASK_WRITE_OWN_COMMENT + "')" +
				  " && @sec.isPrincipalTaskParticipant(#achievementId, #taskId)" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Удаление комментария по ID")
	@DeleteMapping(ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM)
	public SuccessResponse deleteTaskComment(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
											 @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
											 @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId) {
		log.info("Request to delete comment with id '{}' for Task with id '{}' for Achievement with id '{}'",
				 commentId, taskId, achievementId);
		commentService.deleteComment(commentId, taskService.getTask(achievementId, taskId).getId());
		return new SuccessResponse();
	}

//--ACHIEVEMENT-TASK-COMMENT-ATTACHMENTS------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + COMMENT_WRITE_ATTACHMENT + "')" +
				  " || (hasAuthority('" + COMMENT_WRITE_OWN_ATTACHMENT + "')" +
				  " && @sec.isPrincipalTaskParticipant(#achievementId, #taskId)" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Добавление вложений к комментарию по ID")
	@PutMapping(path = ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + COMMENTS_URI +
					   COMMENT_ID_URI_PARAM + ATTACHMENTS_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public CommentView createTaskCommentAttachment(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
												   @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
												   @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
												   @RequestPart MultipartFile[] files) {
		log.info("Request to add attachment for comment with id '{}' for Task with id '{}' for Achievement with " +
				 "id '{}'", commentId, taskId, achievementId);
		return commentMapper.toView(commentService.addAttachments(
				commentId,
				taskService.getTask(achievementId, taskId).getId(),
				RecordType.TASK,
				Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())
		));
	}

	@PreAuthorize("hasAuthority('" + COMMENT_WRITE_ATTACHMENT + "')" +
				  " || (hasAuthority('" + COMMENT_WRITE_OWN_ATTACHMENT + "')" +
				  " && @sec.isPrincipalTaskParticipant(#achievementId, #taskId)" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Удаление вложений комментария по ID")
	@DeleteMapping(ACHIEVEMENT_ID_URI_PARAM + TASKS_URI + TASK_ID_URI_PARAM + COMMENTS_URI +
				   COMMENT_ID_URI_PARAM + ATTACHMENTS_URI)
	public CommentView deleteTaskCommentAttachment(@PathVariable(ACHIEVEMENT_ID_URI_PARAM_NAME) UUID achievementId,
												   @PathVariable(TASK_ID_URI_PARAM_NAME) UUID taskId,
												   @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
												   @Valid @RequestBody DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for comment with id '{}' for Task with id '{}' for Achievement with " +
				 "id '{}'", commentId, taskId, achievementId);
		return commentMapper.toView(commentService.deleteAttachments(
				commentId,
				taskService.getTask(achievementId, taskId).getId(),
				RecordType.TASK,
				request
		));
	}

}
