package ru.loylabs.portal.presentation.controller;

import static java.util.stream.Collectors.toList;
import static ru.loylabs.portal.config.URIConstants.ALL_URI;
import static ru.loylabs.portal.config.URIConstants.BELL_NOTIFICATIONS_URI;
import static ru.loylabs.portal.config.URIConstants.MARK_URI;
import static ru.loylabs.portal.config.URIConstants.NOTIFICATION_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.NOTIFICATION_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.NOTIFICATION_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.NOTIFICATION_WRITE;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

import java.util.Arrays;
import java.util.List;
import java.util.UUID;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.ApiOperation;
import ru.loylabs.portal.mapper.BellNotificationMapper;
import ru.loylabs.portal.presentation.view.BellNotificationView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.PersonalBellNotificationService;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + BELL_NOTIFICATIONS_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class BellNotificationController {

	private final PersonalBellNotificationService personalNotificationService;
	private final BellNotificationMapper bellNotificationMapper;

//--BELL-NOTIFICATIONS------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + NOTIFICATION_READ + "')")
	@ApiOperation("Получение всех оповещений для пользователя")
	@GetMapping
	public ListView<BellNotificationView> findAllNotificationsForPerson(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "bellNotification.date,desc") String sort,
			@RequestParam(required = false) String search,
			@RequestParam boolean seen
	) {
		var principal = getCredential().getPerson();
		log.info("Request for all{} BellNotification list for Person with id '{}'",
				 seen ? " unseen" : "", principal.getId());
		var criteria = getCriteria(search);
		var notifications = personalNotificationService.getAllForPerson(
				page, size, sort, criteria, principal.getId(), seen
		);
		return new ListView<>(notifications.getTotalElements(),
							  notifications.getContent()
										   .stream()
										   .map(notification -> bellNotificationMapper.toView(
												   notification.getBellNotification(),
												   notification.isSeen(),
												   personalNotificationService.countUnseenForPerson(principal.getId())))
										   .collect(toList()));
	}

	@PreAuthorize("hasAuthority('" + NOTIFICATION_WRITE + "')")
	@ApiOperation("Пометка оповещения пользователя как 'просмотренного'")
	@PutMapping(NOTIFICATION_ID_URI_PARAM + MARK_URI)
	public SuccessResponse markNotificationAsSeen(@PathVariable(NOTIFICATION_ID_URI_PARAM_NAME) UUID notificationId) {
		var principal = getCredential().getPerson();
		log.info("Request to mark BellNotification with id '{}' of Person with id '{}' as 'seen'",
				 notificationId, principal.getId());
		personalNotificationService.markAsSeen(principal.getId(), notificationId);
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + NOTIFICATION_WRITE + "')")
	@ApiOperation("Пометка оповещений пользователя как 'просмотренных'")
	@PutMapping(path = MARK_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public SuccessResponse markNotificationsAsSeen(@RequestBody UUID[] notificationIds) {
		var principal = getCredential().getPerson();
		log.info("Request to mark BellNotifications with ids '{}' of Person with id '{}' as 'seen'",
				 Arrays.toString(notificationIds), principal.getId());
		personalNotificationService.markAsSeen(principal.getId(), List.of(notificationIds));
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + NOTIFICATION_WRITE + "')")
	@ApiOperation("Пометка всех оповещений пользователя как 'просмотренных'")
	@PutMapping(MARK_URI + ALL_URI)
	public SuccessResponse markAllNotificationsAsSeen() {
		var principal = getCredential().getPerson();
		log.info("Request to mark all BellNotifications of Person with id '{}' as 'seen'", principal.getId());
		personalNotificationService.markAllAsSeen(principal.getId());
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + NOTIFICATION_WRITE + "')")
	@ApiOperation("Удаление личного уведомления")
	@DeleteMapping(NOTIFICATION_ID_URI_PARAM)
	public SuccessResponse deleteNotification(@PathVariable(NOTIFICATION_ID_URI_PARAM_NAME) UUID notificationId) {
		var principal = getCredential().getPerson();
		log.info("Request to delete BellNotification with id '{}' of Person with id '{}'",
				 notificationId, principal.getId());
		personalNotificationService.deleteNotificationForPerson(notificationId, principal.getId());
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + NOTIFICATION_WRITE + "')")
	@ApiOperation("Удаление всех архивированных уведомлений")
	@DeleteMapping
	public SuccessResponse deleteAllNotifications() {
		var principal = getCredential().getPerson();
		log.info("Request to delete all archived BellNotification of Person with id '{}'", principal.getId());
		personalNotificationService.deleteAllArchivedNotification(principal.getId());
		return new SuccessResponse();
	}

}
