package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import ru.loylabs.portal.entity.CorporateCalendarEvent;
import ru.loylabs.portal.entity.IntegratedCalendarLink;
import ru.loylabs.portal.entity.MonthHolidayData;
import ru.loylabs.portal.entity.YandexCalendarEvent;
import ru.loylabs.portal.mapper.CalendarLinkMapper;
import ru.loylabs.portal.presentation.request.AddEventYandexRequest;
import ru.loylabs.portal.presentation.request.CorporateCalendarEventAddRequest;
import ru.loylabs.portal.presentation.request.CreateIntegrateCalendarLink;
import ru.loylabs.portal.presentation.request.DeleteEventYandexRequest;
import ru.loylabs.portal.presentation.request.IntegratedCalendarEvent;
import ru.loylabs.portal.presentation.request.SaveYandexCredentialsRequest;
import ru.loylabs.portal.presentation.request.UpdateIntegrateCalendarLink;
import ru.loylabs.portal.presentation.view.IntegratedCalendarLinkView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.CorporateCalendarEventService;
import ru.loylabs.portal.service.interfaces.IntegratedCalendarService;
import ru.loylabs.portal.service.interfaces.WorkingCalendarService;
import ru.loylabs.portal.service.interfaces.YandexCalendarService;

import javax.validation.Valid;
import javax.validation.constraints.Max;
import javax.validation.constraints.Min;
import javax.validation.constraints.Pattern;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.UUID;

import static java.util.stream.Collectors.toList;
import static ru.loylabs.portal.config.URIConstants.CALENDAR_EVENT_URI_DATE_NAME;
import static ru.loylabs.portal.config.URIConstants.CALENDAR_EVENT_URI_DATE_PARAM;
import static ru.loylabs.portal.config.URIConstants.CALENDAR_LINK_URI;
import static ru.loylabs.portal.config.URIConstants.CALENDAR_LINK_URI_NAME;
import static ru.loylabs.portal.config.URIConstants.CALENDAR_LINK_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.CALENDAR_URI;
import static ru.loylabs.portal.config.URIConstants.CORPORATE_URI;
import static ru.loylabs.portal.config.URIConstants.CREDENTIALS_URI;
import static ru.loylabs.portal.config.URIConstants.EVENTS_URI;
import static ru.loylabs.portal.config.URIConstants.ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.INTEGRATED_CALENDAR_URI;
import static ru.loylabs.portal.config.URIConstants.PERSONS_URI;
import static ru.loylabs.portal.config.URIConstants.PERSON_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.PERSON_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.config.URIConstants.YANDEX_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.CALENDAR_READ_CORPORATE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.CALENDAR_READ_WORKING;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.CALENDAR_READ_YANDEX;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.CALENDAR_WRITE_CORPORATE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.CALENDAR_WRITE_YANDEX;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class CalendarController {


	private final YandexCalendarService yandexCalendarService;

	private final WorkingCalendarService workingCalendarService;

	private final CorporateCalendarEventService corporateCalendarEventService;

	private final IntegratedCalendarService integratedCalendarService;

	private final CalendarLinkMapper calendarLinkMapper;

//--CALENDAR----------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + CALENDAR_READ_CORPORATE + "')")
	@ApiOperation("Получить все события из корпоративного календаря")
	@GetMapping(CALENDAR_URI + CORPORATE_URI + EVENTS_URI)
	public List<CorporateCalendarEvent> findAllCorporateEvents() {
		log.info("Request for Corporate Event list");
		return corporateCalendarEventService.getAllEvents();
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_READ_CORPORATE + "')")
	@ApiOperation("Получить все повторяющиеся события по repeatId из корпоративного календаря")
	@GetMapping(CALENDAR_URI + CORPORATE_URI + EVENTS_URI + "/repeating" + ID_URI_PARAM)
	public ListView<CorporateCalendarEvent> findAllRepeatingCorporateEvents(@PathVariable(ID_URI_PARAM_NAME) UUID repeatId) {
		log.info("Request for repeating Corporate Event list with repeat id {}", repeatId);
		var events = corporateCalendarEventService.getAllRepeatingEvents(repeatId);
		return new ListView<>(events.size(),events);
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_READ_CORPORATE + "')")
	@ApiOperation("Получить все события и дни рождения из корпоративного календаря на ближайшие i месяцев")
	@GetMapping(CALENDAR_URI + CORPORATE_URI + EVENTS_URI + "/next-{i}-months")
	public List<CorporateCalendarEvent> findAllCorporateEventsForNextMonths(
			@PathVariable("i") @Min(0) @Max(12) int i
	) {
		log.info("Request for Corporate Event list for next {} months", i);
		return corporateCalendarEventService.getEventsAndBirthdaysForNextMonths(i);
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_READ_CORPORATE + "')")
	@ApiOperation("Получить событие из корпоративного календаря")
	@GetMapping(CALENDAR_URI + CORPORATE_URI + EVENTS_URI + ID_URI_PARAM)
	public CorporateCalendarEvent findOneCorporateEvent(@PathVariable(ID_URI_PARAM_NAME) UUID eventId) {
		log.info("Request for Corporate Event with id '{}'", eventId);
		return corporateCalendarEventService.getEvent(eventId);
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_READ_WORKING + "')")
	@ApiOperation("Получение рабочего календаря на год")
	@GetMapping(CALENDAR_URI + "/{year}")
	public ListView<MonthHolidayData> findWorkingCalendarForYear(
			@PathVariable("year") @Pattern(regexp = "[1-2][0-9]{3}") String year
	) {
		log.info("Request for Working calendar for {} year", year);
		final List<MonthHolidayData> calendar = workingCalendarService.getWorkingCalendar(year);
		return new ListView<>(calendar.size(), calendar);
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_WRITE_CORPORATE + "')")
	@ApiOperation("Добавить событие в корпоративный календарь")
	@PostMapping(path = CALENDAR_URI + CORPORATE_URI + EVENTS_URI,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public void createCorporateEvent(@Valid @RequestBody CorporateCalendarEventAddRequest request) {
		log.info("Request to create new Corporate Event with request: {}", request);
		corporateCalendarEventService.addEvent(
				request.getEventName(),
				request.getDescription(),
				request.getEventType(),
				request.getDateStart(),
				request.getDateEnd(),
				request.getRecurrence(),
				getCredential().getPerson());
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_WRITE_CORPORATE + "')")
	@ApiOperation("Удалить событие из корпоративного календаря")
	@DeleteMapping(CALENDAR_URI + CORPORATE_URI + EVENTS_URI + ID_URI_PARAM)
	public void deleteCorporateEvent(@PathVariable(ID_URI_PARAM_NAME) UUID eventId,
									 @RequestParam(name = "repeatable", defaultValue = "false") boolean repeatable) {
		log.info("Request to delete{} Corporate Event with id '{}'", repeatable ? " repeatable" : "", eventId);
		corporateCalendarEventService.deleteEvent(eventId,repeatable);
	}

//--YANDEX-CALENDAR---------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + CALENDAR_READ_YANDEX + "')")
	@ApiOperation("Получение списка встреч из Яндекс-календаря")
	@GetMapping(PERSONS_URI + PERSON_ID_URI_PARAM + YANDEX_URI + CALENDAR_URI + EVENTS_URI)
	public ListView<YandexCalendarEvent> findAllYandexCalendarEvents(
			@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId
	) {
		log.info("Request for events from Person with id '{}' calendar", personId);
		final List<YandexCalendarEvent> events = yandexCalendarService.getEvents(personId);
		return new ListView<>(events.size(), events);
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_WRITE_YANDEX + "')")
	@ApiOperation("Отправить встречу в Яндекс-календарь")
	@PostMapping(path = PERSONS_URI + PERSON_ID_URI_PARAM + YANDEX_URI + CALENDAR_URI + EVENTS_URI,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public void createYandexCalendarEvent(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
										  @Valid @RequestBody AddEventYandexRequest request) {
		log.info("Request to create new Yandex Calendar Event for Person with id '{}' with request: {}",
				 personId, request);
		final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd'T'HHmmss");
		final String dateStartString = formatter.format(request.getDateStart());
		final String dateEndString = formatter.format(request.getDateEnd());
		yandexCalendarService.addEvent(personId,
									   request.getEventDirectoryName(),
									   request.getEventName(),
									   dateStartString,
									   dateEndString);
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_WRITE_YANDEX + "')")
	@ApiOperation("Удалить встречу из Яндекс-календаря")
	@DeleteMapping(PERSONS_URI + PERSON_ID_URI_PARAM + YANDEX_URI + CALENDAR_URI + EVENTS_URI)
	public void deleteYandexCalendarEvent(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
										  @Valid @RequestBody DeleteEventYandexRequest request) {
		log.info("Request to delete Yandex Calendar Event for Person with id '{}'", personId);
		yandexCalendarService.deleteEvent(personId, request.getEventDirectoryName(), request.getEventUid());
	}

//--YANDEX-CALENDAR-CREDENTIALS---------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + CALENDAR_WRITE_YANDEX + "')")
	@ApiOperation("Сохранение данных для доступа к Яндекс-календарю")
	@PostMapping(path = PERSONS_URI + PERSON_ID_URI_PARAM + YANDEX_URI + CALENDAR_URI + CREDENTIALS_URI,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public SuccessResponse createYandexCalendarCredentials(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
														   @Valid @RequestBody SaveYandexCredentialsRequest request) {
		log.info("Request to create Yandex Calendar credentials for Person with id '{}' with request: {}",
				 personId, request);
		yandexCalendarService.saveYandexCredentials(personId, request.getEmail(), request.getPassword());
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + CALENDAR_WRITE_YANDEX + "')")
	@ApiOperation("Удаление данных для доступа к Яндекс-календарю")
	@DeleteMapping(PERSONS_URI + PERSON_ID_URI_PARAM + YANDEX_URI + CALENDAR_URI + CREDENTIALS_URI)
	public SuccessResponse deleteYandexCalendarCredentials(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request to delete Yandex Calendar credentials for Person with id '{}'", personId);
		yandexCalendarService.deleteYandexCredentials(personId);
		return new SuccessResponse();
	}

//--INTEGRATION-CALENDARS-ICAL----------------------------------------------------------------------------------------//
	@GetMapping(PERSONS_URI + INTEGRATED_CALENDAR_URI + CALENDAR_URI + EVENTS_URI)
	@ApiOperation("Получить все события пользователя")
	public ListView<IntegratedCalendarEvent> findCalendarEventsInICal() {
		log.info("Request to receive all events of the person integrated calendars");
		List<IntegratedCalendarEvent> events = integratedCalendarService.getAllCalendarEvents();
		return new ListView<>(events.size(), events);
	}

	@GetMapping(PERSONS_URI + INTEGRATED_CALENDAR_URI + CALENDAR_URI + EVENTS_URI + CALENDAR_EVENT_URI_DATE_PARAM)
	@ApiOperation("Получить события из интегрированных календарей по дате")
	public List<IntegratedCalendarEvent> findCalendarEventsByDate(@PathVariable(CALENDAR_EVENT_URI_DATE_NAME) String date) {
		log.info("Request to receive integrated calendar events by date '{}' of the person", date);
		return integratedCalendarService.getAllCalendarEventsById(date);
	}

	@GetMapping(PERSONS_URI + INTEGRATED_CALENDAR_URI + CALENDAR_URI + CALENDAR_LINK_URI)
	@ApiOperation("Получить все ссылки на интегрированные календари пользователя")
	public ListView<IntegratedCalendarLinkView> findCalendarEventsByDate() {
		log.info("Request to receive all links of the integrated person calendars");
		List<IntegratedCalendarLink> links = integratedCalendarService.getAllLinkCalendar();
		return new ListView<>(links.size(),
				links.stream()
						.map(calendarLinkMapper::toView)
						.collect(toList()));
	}

	@GetMapping(PERSONS_URI + INTEGRATED_CALENDAR_URI + CALENDAR_URI + CALENDAR_LINK_URI + CALENDAR_LINK_URI_PARAM)
	@ApiOperation("Получить ссылку на интегрированный календарь пользователя")
	public IntegratedCalendarLinkView findCalendarEventsByDate(@PathVariable(CALENDAR_LINK_URI_NAME) UUID linkId) {
		log.info("Request to get a link from the ID '{}' of the integrated person calendars", linkId);
		return calendarLinkMapper.toView(integratedCalendarService.getLinkById(linkId));
	}

	@PostMapping(path = PERSONS_URI + INTEGRATED_CALENDAR_URI + CALENDAR_URI + CALENDAR_LINK_URI,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	@ApiOperation("Добавление ссылки интегрированного календаря к аккаунту")
	public IntegratedCalendarLinkView addCalendarLink(@Valid @RequestBody CreateIntegrateCalendarLink request) {
		log.info("Request to add integrated calendar link to person");
		return calendarLinkMapper.toView(integratedCalendarService.addCalendarLink(request));
	}

	@PutMapping(path = PERSONS_URI + INTEGRATED_CALENDAR_URI + CALENDAR_URI + CALENDAR_LINK_URI + CALENDAR_LINK_URI_PARAM,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	@ApiOperation("Изменить ссылку интегрированного календаря")
	public IntegratedCalendarLinkView updateCalendarLink(@PathVariable(CALENDAR_LINK_URI_NAME) UUID linkId,
														 @Valid @RequestBody UpdateIntegrateCalendarLink updateLink) {
		log.info("Request to change the link from ID '{}' of the integrated person calendars", linkId);
		return calendarLinkMapper.toView(integratedCalendarService.updateLink(linkId, updateLink));
	}

	@DeleteMapping(PERSONS_URI  + INTEGRATED_CALENDAR_URI + CALENDAR_URI + CALENDAR_LINK_URI + CALENDAR_LINK_URI_PARAM)
	@ApiOperation("Удаление ссылки интегрированного календаря из аккаунта пользователя")
	public SuccessResponse deleteCalendarLink(@PathVariable(CALENDAR_LINK_URI_NAME) UUID linkId) {
		log.info("Request to remove a link from the ID '{}' of the integrated person calendars", linkId);
		integratedCalendarService.deleteLink(linkId);
		return new SuccessResponse();
	}
}
