package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.mapper.CommentMapper;
import ru.loylabs.portal.mapper.CommunityMapper;
import ru.loylabs.portal.mapper.CommunityRecordMapper;
import ru.loylabs.portal.mapper.PersonMapper;
import ru.loylabs.portal.presentation.request.*;
import ru.loylabs.portal.presentation.view.CommentView;
import ru.loylabs.portal.presentation.view.CommunityListView;
import ru.loylabs.portal.presentation.view.CommunityRecordView;
import ru.loylabs.portal.presentation.view.CommunityView;
import ru.loylabs.portal.presentation.view.PersonCommunityView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.CommentService;
import ru.loylabs.portal.service.interfaces.CommunityPersonRoleService;
import ru.loylabs.portal.service.interfaces.CommunityRecordService;
import ru.loylabs.portal.service.interfaces.CommunityService;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.Collections;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static java.util.stream.Collectors.toList;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.*;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredentialPersonId;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + COMMUNITY_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class CommunityController {

	private final CommunityService communityService;
	private final CommunityMapper communityMapper;

	private final CommunityRecordService communityRecordService;
	private final CommunityRecordMapper communityRecordMapper;

	private final CommunityPersonRoleService communityPersonService;

	private final CommentMapper commentMapper;
	private final CommentService commentService;

	private final PersonMapper personMapper;

//--COMMUNITIES-------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + COMMUNITY_READ + "')")
	@ApiOperation("Получение списка групп с сортировкой и фильтрацией")
	@GetMapping
	public ListView<CommunityListView> findAllCommunities(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for Community list");
		var criteria = getCriteria(search);
		var communities = communityService.getAllCommunities(page, size, sort, criteria);
		return new ListView<>(communities.getTotalElements(),
							  communities.getContent()
										 .stream()
										 .map(c -> communityMapper.toCommunityListView(c, c.getMembers().size()))
										 .collect(toList()));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_READ_OWN + "')")
	@ApiOperation("Получение списка групп в которых состоит участник с сортировкой и фильтрацией")
	@GetMapping(PERSONAL_URI)
	public ListView<CommunityListView> findAllPersonalCommunities(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		var principal = getCredential().getPerson();
		log.info("Request for personal Community list for Person with id '{}'", principal.getId());
		var criteria = getCriteria(search);
		var communities = communityPersonService.getAllPersonalCommunities(
				page, size, sort, criteria, principal.getId()
		);
		return new ListView<>(communities.getTotalElements(),
							  communities.getContent()
										 .stream()
										 .map(c -> communityMapper.toCommunityListView(
												 c.getCommunity(), c.getCommunity().getMembers().size()))
										 .collect(toList()));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_READ_PUBLIC + "')")
	@ApiOperation("Получение списка открытых групп")
	@GetMapping(PUBLIC_URI)
	public ListView<CommunityListView> findAllPublicCommunities(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search) {
		log.info("Request for public Community list");
		var criteria = getCriteria(search);
		var communities = communityService.getAllPublicCommunities(page, size, sort, criteria);
		return new ListView<>(communities.getTotalElements(),
							  communities.stream()
										 .map(c -> communityMapper.toCommunityListView(c, c.getMembers().size()))
										 .collect(toList()));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_READ + "')" +
				  " || (hasAuthority('" + COMMUNITY_READ_PUBLIC + "') && @sec.isCommunityPublic(#communityId))" +
				  " || (hasAuthority('" + COMMUNITY_READ_OWN + "') && @sec.isPrincipalCommunityMember(#communityId))")
	@ApiOperation("Получить группу по ID")
	@GetMapping(COMMUNITY_ID_URI_PARAM)
	public CommunityView findOneCommunity(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId) {
		log.info("Request for Community with id '{}'", communityId);
		return communityMapper.toView(communityService.getCommunity(communityId),
									  communityPersonService.getPersonalRoleInCommunity(
											  communityId, getCredentialPersonId()));
	}

	@PreAuthorize("hasAnyAuthority('" + COMMUNITY_WRITE + "', '" + COMMUNITY_WRITE_OWN + "')")
	@ApiOperation("Создать группу")
	@PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommunityView createCommunity(@Valid @RequestBody CreateCommunityRequest request) {
		log.info("Request to create new Community with request: {}", request);
		var community = communityService.saveCommunity(request, getCredential().getPerson());
		return communityMapper.toView(
				community,
				communityPersonService.getPersonalRoleInCommunity(
						community.getId(), getCredentialPersonId()
				)
		);
	}

	@PreAuthorize("hasAnyAuthority('" + COMMUNITY_WRITE + "', '" + COMMUNITY_WRITE_OWN + "')")
	@ApiOperation("Создать группу с аватаром")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
					  dataType = "ru.loylabs.portal.presentation.request.CreateCommunityRequest", paramType = "body")
	@PostMapping(path = NEW_AVATAR_URI,
				 consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public CommunityView createCommunity(@Valid @RequestPart CreateCommunityRequest request,
										 @RequestPart("file") MultipartFile file) {
		log.info("Request to create new Community with avatar with request: {}", request);
		var community = communityService.saveCommunity(request, file, getCredential().getPerson());
		return communityMapper.toView(
				community,
				communityPersonService.getPersonalRoleInCommunity(
						community.getId(), getCredentialPersonId()
				)
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN + "') && @sec.isPrincipalCommunityOwner(#communityId))")
	@ApiOperation("Обновить группу")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommunityView updateCommunity(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
										 @Valid @RequestBody UpdateCommunityRequest request) {
		log.info("Request to update Community with id '{}' with request: {}", communityId, request);
		return communityMapper.toView(
				communityService.updateCommunity(communityId, request),
				communityPersonService.getPersonalRoleInCommunity(
						communityId, getCredentialPersonId()
				)
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "') || (hasAuthority('" + COMMUNITY_WRITE_OWN + "')" +
				  " && @sec.isPrincipalCommunityOwnerOrAdmin(#communityId))")
	@ApiOperation("Установить аватар группы по ID")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM + AVATAR_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public CommunityView updateCommunityAvatar(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											   @RequestPart("file") MultipartFile file) {
		log.info("Request to update Community avatar with id '{}'", communityId);
		return communityMapper.toView(
				communityService.saveAvatar(communityId, file),
				communityPersonService.getPersonalRoleInCommunity(
						communityId, getCredentialPersonId()
				)
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN + "') && @sec.isPrincipalCommunityOwner(#communityId))")
	@ApiOperation("Удалить группу")
	@DeleteMapping(COMMUNITY_ID_URI_PARAM)
	public SuccessResponse deleteCommunity(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId) {
		log.info("Request to delete Community with id '{}'", communityId);
		communityService.deleteCommunity(communityId);
		return new SuccessResponse();
	}

//--COMMUNITY-MEMBERS-------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + COMMUNITY_READ + "')" +
				  " || (hasAuthority('" + COMMUNITY_READ_PUBLIC + "') && @sec.isCommunityPublic(#communityId))" +
				  " || (hasAuthority('" + COMMUNITY_READ_OWN + "') && @sec.isPrincipalCommunityMember(#communityId))")
	@ApiOperation("Получить всех пользователей в группе")
	@GetMapping(COMMUNITY_ID_URI_PARAM + MEMBERS_URI)
	public ListView<PersonCommunityView> findAllCommunityMembers(
			@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for all members of Community with id '{}'", communityId);
		var criteria = getCriteria(search);
		var members = communityPersonService.getAllPersonRoleInCommunity(communityId, page, size, sort, criteria);
		return new ListView<>(members.getTotalElements(),
							  members.getContent()
									 .stream()
									 .map(e -> personMapper.toPersonCommunityView(e.getPerson(), e.getCommunityRole()))
									 .collect(toList()));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_INVITE_KICK + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN + "') && @sec.isPrincipalCommunityOwner(#communityId))" +
				  " || (hasAuthority('" + COMMUNITY_INVITE_KICK_OWN + "')" +
				  " && @sec.isPrincipalCommunityOwnerOrAdmin(#communityId))")
	@ApiOperation("Добавить пользователей в группу")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM + MEMBERS_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommunityView addCommunityMember(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											@Valid @RequestBody AddMembersRequest[] request) {
		log.info("Request to add members to Community with id '{}' with request: {}", communityId, request);
		return communityMapper.toView(
				communityService.addMembers(communityId, request,getCredential().getPerson()),
				communityPersonService.getPersonalRoleInCommunity(
						communityId, getCredentialPersonId()
				)
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_JOIN_PRIVATE + "')" +
			" || (hasAuthority('" + COMMUNITY_JOIN_LEAVE + "') && @sec.isCommunityPublic(#communityId))")
	@ApiOperation("Присоединиться к группе")
	@PutMapping(JOIN_URI)
	public CommunityView joinCommunity(@RequestParam("communityId") UUID communityId) {
		var principal = getCredential().getPerson();
		log.info("Request to join Community with id '{}' from Person with id '{}'", communityId, principal.getId());
		return communityMapper.toView(
				communityService.joinCommunity(communityId, principal.getId()),
				communityPersonService.getPersonalRoleInCommunity(communityId, principal.getId())
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_JOIN_LEAVE + "') && @sec.isPrincipalCommunityMember(#communityId)")
	@ApiOperation("Выйти из группы")
	@PutMapping(LEAVE_URI)
	public SuccessResponse leaveCommunity(@RequestParam("communityId") UUID communityId) {
		var principal = getCredential().getPerson();
		log.info("Request to leave Community with id '{}' from Person with id '{}'", communityId, principal.getId());
		communityService.leaveCommunity(communityId, principal.getId());
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_INVITE_KICK + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN + "') && @sec.isPrincipalCommunityOwner(#communityId))" +
				  " || (hasAuthority('" + COMMUNITY_INVITE_KICK_OWN + "')" +
				  " && @sec.isPrincipalCommunityOwnerOrAdmin(#communityId))")
	@ApiOperation("Изменение роли участника группы")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommunityView updateCommunityMemberRole(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
												   @Valid @RequestBody UpdateRoleInCommunityRequest request) {
		log.info("Request to update Person role in Community with id '{}' with request: {}", communityId, request);
		return communityMapper.toView(
				communityService.updatePersonRoleInCommunity(communityId, request),
				communityPersonService.getPersonalRoleInCommunity(
						communityId, getCredentialPersonId()
				)
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN + "') && @sec.isPrincipalCommunityOwner(#communityId))")
	@ApiOperation("Назначить нового владельца группы")
	@PutMapping(COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI + OWNER_URI)
	public CommunityView updateCommunityOwner(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											  @RequestParam("personId") UUID personId) {
		log.info("Request to change owner of Community with id '{}' to Person with id '{}'", communityId, personId);
		return communityMapper.toView(
				communityService.changeOwner(communityId, personId),
				communityPersonService.getPersonalRoleInCommunity(
						communityId, getCredentialPersonId()
				)
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_INVITE_KICK + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN + "') && @sec.isPrincipalCommunityOwner(#communityId))" +
				  " || (hasAuthority('" + COMMUNITY_INVITE_KICK_OWN + "')" +
				  " && @sec.isPrincipalCommunityOwnerOrAdmin(#communityId))")
	@ApiOperation("Удалить пользователей из группы")
	@DeleteMapping(COMMUNITY_ID_URI_PARAM + MEMBERS_URI)
	public SuccessResponse removeCommunityMember(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
												 @Valid @RequestBody DeleteMemberRequest request) {
		log.info("Request to remove members with ids '{}' to Community with id '{}'",
				 request.getMembersIds(), communityId);
		communityService.deleteMembers(communityId, request.getMembersIds().toArray(UUID[]::new));
		return new SuccessResponse();
	}

//--COMMUNITY-MESSAGES------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + COMMUNITY_READ + "')" +
				  " || (hasAuthority('" + COMMUNITY_READ_PUBLIC + "') && @sec.isCommunityPublic(#communityId))" +
				  " || (hasAuthority('" + COMMUNITY_READ_OWN + "') && @sec.isPrincipalCommunityMember(#communityId))")
	@ApiOperation("Получение списка сообщений группы сортировкой и фильтрацией")
	@GetMapping(COMMUNITY_ID_URI_PARAM + MESSAGES_URI)
	public ListView<CommunityRecordView> findAllMessages(
			@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for Community message list from Community with id '{}'", communityId);
		var criteria = getCriteria(search);
		var messages = communityRecordService.getAllCommunityRecords(communityId, page, size, sort, criteria);
		return new ListView<>(messages.getTotalElements(),
							  messages.getContent()
									  .stream()
									  .map(communityRecordMapper::toView)
									  .collect(toList()));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_READ + "')" +
				  " || (hasAuthority('" + COMMUNITY_READ_PUBLIC + "') && @sec.isCommunityPublic(#communityId))" +
				  " || (hasAuthority('" + COMMUNITY_READ_OWN + "') && @sec.isPrincipalCommunityMember(#communityId))")
	@ApiOperation("Получение сообщения группы по ID")
	@GetMapping(COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM)
	public CommunityRecordView findOneMessage(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											  @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId) {
		log.info("Request for Community message with id '{}' from Community with id '{}'", messageId, communityId);
		return communityRecordMapper.toView(communityRecordService.getCommunityRecord(messageId, communityId));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE_MESSAGE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN_MESSAGE + "') && @sec.isPrincipalCommunityMember(#communityId))")
	@ApiOperation("Написать сообщение в группу")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
					  dataType = "ru.loylabs.portal.presentation.request.CreateRecordRequest", paramType = "body")
	@PostMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI,
				 consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public CommunityRecordView createMessage(
			@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
			@Valid @RequestPart CreateRecordRequest request,
			@RequestPart(required = false, name = "files") MultipartFile[] files
	) {
		log.info("Request to create new message in Community with id '{}' with request: {}", communityId, request);
		return communityRecordMapper.toView(
				communityRecordService.sendMessage(
						communityId,
						request,
						getCredential().getPerson(),
						Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()))
		);
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "')" +
				  " || (@sec.isPrincipalCommunityMember(#communityId)" +
				  " && (hasAuthority('" + COMMUNITY_WRITE_MESSAGE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN_MESSAGE + "') && @sec.isPrincipalRecordAuthor(#messageId))))")
	@ApiOperation("Редактировать сообщение в группе по ID")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommunityRecordView updateMessage(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											 @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
											 @Valid @RequestBody UpdateRecordRequest request) {
		log.info("Request to update message with id '{}' in Community with id '{}' with request: {}",
				 messageId, communityId, request);
		return communityRecordMapper.toView(communityRecordService.updateMessage(messageId, communityId, request));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "')" +
				  " || (@sec.isPrincipalCommunityMember(#communityId)" +
				  " && (hasAuthority('" + COMMUNITY_WRITE_MESSAGE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN_MESSAGE + "') && @sec.isPrincipalRecordAuthor(#messageId))))")
	@ApiOperation("Удалить сообщение из группы по ID")
	@DeleteMapping(COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM)
	public SuccessResponse deleteMessage(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
										 @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId) {
		log.info("Request to delete message with id '{}' in Community with id '{}'", messageId, communityId);
		communityRecordService.deleteMessage(messageId, communityId);
		return new SuccessResponse();
	}

//--COMMUNITY-MESSAGE-COMMENTS----------------------------------------------------------------------------------------//
	@PreAuthorize("hasAuthority('" + COMMUNITY_READ_COMMENT + "')" +
				  "|| ((hasAuthority('" + COMMUNITY_READ_OWN_COMMENT + "')) && @sec.isPrincipalCommunityMember(#communityId))"
	)
	@ApiOperation("Получить все комментарии к сообщению в группе")
	@GetMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI)
	public ListView<CommentView> findAllGroupComments(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
													  @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
													  @RequestParam(required = false, defaultValue = "0") int page,
													  @RequestParam(required = false, defaultValue = "10") int size,
													  @RequestParam(required = false, defaultValue = "id,desc") String sort,
													  @RequestParam(required = false) String search) {

		log.info("Request to get comments for a post with the ID '{}' in the community with the ID '{}' ",
				messageId, communityId);

		var criteria = getCriteria(search);
		var comments = commentService.getAllComments(page, size, sort, criteria, messageId, RecordType.COMMUNITY);
		return new ListView<>(comments.getTotalElements(),
				comments.getContent()
						.stream()
						.map(commentMapper::toView)
						.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAnyAuthority('" + COMMUNITY_WRITE_COMMENT + "')" +
	"|| (hasAnyAuthority('" + COMMUNITY_WRITE_OWN_COMMENT + "') && @sec.isPrincipalCommunityMember(#communityId))")
	@ApiOperation("Добавить комментарий к сообщению в группе")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
			dataType = "ru.loylabs.portal.presentation.request.CreateRecordRequest", paramType = "body")
	@PostMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI,
			consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public CommentView createCommunityComment(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											  @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
											  @Valid @RequestPart CreateRecordRequest request,
											  @RequestPart(required = false) MultipartFile[] files) {
		log.info("Request to create a new comment for a post with the ID '{}' in the community with the ID '{}' with the request: {}",
				messageId, communityId, request);
		return commentMapper.toView(commentService.saveComment(
				request,
				getCredential().getPerson(),
				messageId,
				RecordType.COMMUNITY,
				Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())
		));
	}

	@PreAuthorize("hasAnyAuthority('" + COMMUNITY_WRITE_COMMENT + "')" +
			"|| (hasAnyAuthority('" + COMMUNITY_WRITE_OWN_COMMENT + "') && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Редактировать комментарий к сообщению в группе")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommentView updateCommunityComment(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											  @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
											  @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
											  @Valid @RequestBody UpdateRecordRequest request) {

		log.info("Request to update a comment with the ID '{}' to an entry with the ID '{}' in the community with the ID '{}' with the request: {}",
				commentId, messageId, communityId, request);

		return commentMapper.toView(commentService.updateComment(request, commentId, messageId));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE_COMMENT + "') || (hasAuthority('" + COMMUNITY_WRITE_OWN_COMMENT + "')" +
			" && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Удалить комментарий к сообщению в группе")
	@DeleteMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM)
	public SuccessResponse deleteGroupComment(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
											  @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
											  @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId) {
		log.info("Request to delete comment with the ID '{}' for a post with the ID '{}' in the community with the ID '{}' ",
				commentId, messageId, communityId);

		commentService.deleteComment(commentId, messageId);
		return new SuccessResponse();
	}
//--COMMUNITY-MESSAGE-ATTACHMENTS-------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "')" +
				  " || (@sec.isPrincipalCommunityMember(#communityId)" +
				  " && (hasAuthority('" + COMMUNITY_WRITE_MESSAGE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN_MESSAGE + "') && @sec.isPrincipalRecordAuthor(#messageId))))")
	@ApiOperation("Добавление вложений к сообщению группы по ID")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public CommunityRecordView createMessageAttachment(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
													   @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
													   @RequestPart("file") MultipartFile[] files) {
		log.info("Request to create attachment for message with id '{}' in Community with id '{}'",
				 messageId, communityId);
		return communityRecordMapper.toView(communityRecordService.addAttachments(messageId, communityId, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE + "')" +
				  " || (@sec.isPrincipalCommunityMember(#communityId)" +
				  " && (hasAuthority('" + COMMUNITY_WRITE_MESSAGE + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN_MESSAGE + "') && @sec.isPrincipalRecordAuthor(#messageId))))")
	@ApiOperation("Удаление вложений сообщения группы по ID")
	@DeleteMapping(COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI)
	public CommunityRecordView deleteMessageAttachment(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
													   @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
													   @Valid @RequestBody DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for message with id '{}' in Community with id '{}'",
				 messageId, communityId);
		return communityRecordMapper.toView(communityRecordService.deleteAttachments(messageId, communityId, request));
	}

//--COMMUNITY-MESSAGE-COMMENT-ATTACHMENTS-----------------------------------------------------------------------------//
	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE_COMMENT + "')" +
				  " || (hasAuthority('" + COMMUNITY_WRITE_OWN_COMMENT + "') && @sec.isPrincipalCommunityMember(#communityId)" +
				  " && @sec.isPrincipalRecordAuthor(#messageId))")
	@ApiOperation("Добавление вложений к комментарию в сообщении группы по ID")
	@PutMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI,
			consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public CommentView createCommunityCommentAttachment(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
														@PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
														@PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
														@RequestPart("file") MultipartFile[] files) {
		log.info("Request to create attachment for comment with id '{}' in message with id '{}' in community with id '{}'", commentId, messageId, communityId);
		return commentMapper.toView(commentService.addAttachments(commentId, messageId, RecordType.COMMUNITY, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())));
	}

	@PreAuthorize("hasAuthority('" + COMMUNITY_WRITE_COMMENT + "')" +
			" || (hasAuthority('" + COMMUNITY_WRITE_OWN_COMMENT + "') && @sec.isPrincipalCommunityMember(#communityId)" +
			" && @sec.isPrincipalRecordAuthor(#messageId))")
	@ApiOperation("Удаление вложений к комментарию в сообщении группы по ID")
	@DeleteMapping(path = COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI)
	public CommentView deleteAttachmentCommunityCommentAttachment(@PathVariable(COMMUNITY_ID_URI_PARAM_NAME) UUID communityId,
																  @PathVariable(MESSAGE_ID_URI_PARAM_NAME) UUID messageId,
																  @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
																  @Valid @RequestBody DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for comment with id '{}' in message with id '{}' in community with id '{}'", commentId, messageId, communityId);
		return commentMapper.toView(commentService.deleteAttachments(commentId, messageId, RecordType.COMMUNITY, request));
	}
}
