package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import ru.loylabs.portal.mapper.DepartmentMapper;
import ru.loylabs.portal.presentation.request.CreateDepartmentRequest;
import ru.loylabs.portal.presentation.request.UpdateDepartmentRequest;
import ru.loylabs.portal.presentation.view.DepartmentView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.DepartmentService;

import javax.validation.Valid;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.DEPARTMENTS_URI;
import static ru.loylabs.portal.config.URIConstants.DEPARTMENT_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.DEPARTMENT_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.DEPARTMENT_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.DEPARTMENT_WRITE;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + DEPARTMENTS_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class DepartmentController {

	private final DepartmentService departmentService;
	private final DepartmentMapper departmentMapper;

//--DEPARTMENTS-------------------------------------------------------------------------------------------------------//

	@SneakyThrows
	@PreAuthorize("hasAuthority('" + DEPARTMENT_READ + "')")
	@ApiOperation("Получение всех департаментов с сортировкой и фильтрацией")
	@GetMapping
	public ListView<DepartmentView> findAllDepartments(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "name,asc") String sort,
			@RequestParam(required = false) String search) {
		log.info("Request for Department list");
		var criteria = getCriteria(search);
		var departments = departmentService.getAllDepartment(page, size, sort, criteria);
		return new ListView<>(departments.getTotalElements(),
							  departments.getContent()
										 .stream()
										 .map(departmentMapper::toView)
										 .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + DEPARTMENT_READ + "')")
	@ApiOperation("Получение департамента по ID")
	@GetMapping(DEPARTMENT_ID_URI_PARAM)
	public DepartmentView findOneDepartment(@PathVariable(DEPARTMENT_ID_URI_PARAM_NAME) UUID departmentId) {
		log.info("Request for Department with id '{}'", departmentId);
		return departmentMapper.toView(departmentService.getDepartmentById(departmentId));
	}

	@PreAuthorize("hasAuthority('" + DEPARTMENT_WRITE + "')")
	@ApiOperation("Создание департамента")
	@PostMapping
	public DepartmentView createDepartment(@Valid @RequestBody CreateDepartmentRequest request) {
		log.info("Request to create new Department with request: {}", request);
		return departmentMapper.toView(departmentService.createDepartment(request));
	}

	@PreAuthorize("hasAuthority('" + DEPARTMENT_WRITE + "')")
	@ApiOperation("Редактирование департамента")
	@PutMapping(DEPARTMENT_ID_URI_PARAM)
	public DepartmentView updateDepartment(@PathVariable(DEPARTMENT_ID_URI_PARAM_NAME) UUID departmentId,
										   @Valid @RequestBody UpdateDepartmentRequest request) {
		log.info("Request to update Department with id '{}' with request: {}", departmentId, request);
		return departmentMapper.toView(departmentService.updateDepartment(departmentId, request));
	}

	@PreAuthorize("hasAuthority('" + DEPARTMENT_WRITE + "')")
	@ApiOperation("Удаление департамента")
	@DeleteMapping(DEPARTMENT_ID_URI_PARAM)
	public SuccessResponse deleteDepartment(@PathVariable(DEPARTMENT_ID_URI_PARAM_NAME) UUID departmentId) {
		log.info("Request to delete Department with id: {}", departmentId);
		departmentService.deleteDepartment(departmentId);
		return new SuccessResponse();
	}

}
