package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.mapper.CommentMapper;
import ru.loylabs.portal.mapper.FeedMapper;
import ru.loylabs.portal.presentation.request.CreateFeedRequest;
import ru.loylabs.portal.presentation.request.CreateRecordRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.request.UpdateRecordRequest;
import ru.loylabs.portal.presentation.view.CommentView;
import ru.loylabs.portal.presentation.view.FeedRecordView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.CommentService;
import ru.loylabs.portal.service.interfaces.FeedService;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.Collections;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.ATTACHMENTS_URI;
import static ru.loylabs.portal.config.URIConstants.COMMENTS_URI;
import static ru.loylabs.portal.config.URIConstants.COMMENT_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.COMMENT_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.FEED_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.FEED_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.FEED_URI;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMENT_WRITE_ATTACHMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMENT_WRITE_OWN_ATTACHMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_CREATE_GENERAL;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_CREATE_IMPORTANT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_MODIFY;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_MODIFY_OWN;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_READ_COMMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_WRITE_ATTACHMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_WRITE_COMMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_WRITE_OWN_ATTACHMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.FEED_WRITE_OWN_COMMENT;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + FEED_URI,
		produces = MediaType.APPLICATION_JSON_VALUE)
public class FeedController {

	private final FeedService feedService;
	private final FeedMapper feedMapper;

	private final CommentService commentService;
	private final CommentMapper commentMapper;

//--FEED--------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + FEED_READ + "')")
	@ApiOperation("Получение списка записей с сортировкой и фильтрацией")
	@GetMapping
	public ListView<FeedRecordView> findAllFeed(@RequestParam(required = false, defaultValue = "0") int page,
												@RequestParam(required = false, defaultValue = "10") int size,
												@RequestParam(required = false, defaultValue = "id,desc") String sort,
												@RequestParam(required = false) String search,
												@RequestParam(required = false, defaultValue = "false") boolean important) {
		log.info("Request for{} Feed list", important ? " important" : "");
		var criteria = getCriteria(search);
		var feed = feedService.getAllFeedRecord(page, size, sort, criteria, important);
		return new ListView<>(feed.getTotalElements(),
				feed.getContent()
						.stream()
						.map(record -> feedMapper.toView(record, feedService.countComments(record.getId())))
						.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + FEED_READ + "')")
	@ApiOperation("Получение записи ленты по ID")
	@GetMapping(FEED_ID_URI_PARAM)
	public FeedRecordView findOneFeed(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId) {
		log.info("Request for Feed with id '{}'", feedId);
		return feedMapper.toView(feedService.getFeed(feedId), feedService.countComments(feedId));
	}

	@PreAuthorize("(hasAuthority('" + FEED_CREATE_GENERAL + "') && !#request.isImportant())" +
			" || (hasAuthority('" + FEED_CREATE_IMPORTANT + "') && #request.isImportant())")
	@ApiOperation("Создание записи ленты")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
			dataType = "ru.loylabs.portal.presentation.request.CreateFeedRequest", paramType = "body")
	@PostMapping(consumes = {MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE})
	public FeedRecordView createFeed(@Valid @RequestPart CreateFeedRequest request,
									 @RequestPart(required = false) MultipartFile[] files) {
		log.info("Request to create new Feed with request: {}", request);
		var feed = feedService.saveFeedRecord(
				request,
				getCredential().getPerson(),
				Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()));
		return feedMapper.toView(feed, feedService.countComments(feed.getId()));
	}

	@PreAuthorize("hasAuthority('" + FEED_MODIFY + "') || (hasAuthority('" + FEED_MODIFY_OWN + "')" +
			" && @sec.isPrincipalRecordAuthor(#feedId))")
	@ApiOperation("Обновление записи ленты по Id")
	@PutMapping(path = FEED_ID_URI_PARAM,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	public FeedRecordView updateFeed(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
									 @Valid @RequestBody UpdateRecordRequest request) {
		log.info("Request to update Feed with id '{}' with request: {}", feedId, request);
		var feed = feedService.updateFeed(feedId, request);
		return feedMapper.toView(feed, feedService.countComments(feed.getId()));
	}

	@PreAuthorize("hasAuthority('" + FEED_MODIFY + "') || (hasAuthority('" + FEED_MODIFY_OWN + "')" +
			" && @sec.isPrincipalRecordAuthor(#feedId))")
	@ApiOperation("Удаление записи ленты по ID")
	@DeleteMapping(FEED_ID_URI_PARAM)
	public SuccessResponse deleteFeed(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId) {
		log.info("Request to delete Feed with id '{}'", feedId);
		feedService.deleteFeedRecord(feedId);
		return new SuccessResponse();
	}

//--FEED-ATTACHMENTS--------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + FEED_WRITE_ATTACHMENT + "')" +
			" || (hasAuthority('" + FEED_WRITE_OWN_ATTACHMENT + "') && @sec.isPrincipalRecordAuthor(#feedId))")
	@ApiOperation("Добавление вложений к записи ленты по ID")
	@PutMapping(path = FEED_ID_URI_PARAM + ATTACHMENTS_URI,
			consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public FeedRecordView createFeedAttachment(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
											   @RequestPart("file") MultipartFile[] files) {
		log.info("Request to create attachment for Feed with id '{}'", feedId);
		var feed = feedService.addAttachments(feedId, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()));
		return feedMapper.toView(feed, feedService.countComments(feedId));
	}

	@PreAuthorize("hasAuthority('" + FEED_WRITE_ATTACHMENT + "')" +
			" || (hasAuthority('" + FEED_WRITE_OWN_ATTACHMENT + "') && @sec.isPrincipalRecordAuthor(#feedId))")
	@ApiOperation("Удаление вложений записи ленты по ID")
	@DeleteMapping(FEED_ID_URI_PARAM + ATTACHMENTS_URI)
	public FeedRecordView deleteFeedAttachment(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
											   @RequestBody @Valid DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for Feed with id '{}'", feedId);
		var feed = feedService.deleteAttachments(feedId, request);
		return feedMapper.toView(feed, feedService.countComments(feedId));
	}

//--FEED-COMMENTS-----------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + FEED_READ_COMMENT + "')")
	@ApiOperation("Получение комментариев к записи ленты с сортировкой и фильтрацией по ID идеи")
	@GetMapping(FEED_ID_URI_PARAM + COMMENTS_URI)
	public ListView<CommentView> findAllFeedComments(
			@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for Feed comments list for Feed with id '{}'", feedId);
		var criteria = getCriteria(search);
		var comments = commentService.getAllComments(page, size, sort, criteria, feedId, RecordType.FEED);
		return new ListView<>(comments.getTotalElements(),
				comments.getContent()
						.stream()
						.map(commentMapper::toView)
						.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAnyAuthority('" + FEED_WRITE_COMMENT + "', '" + FEED_WRITE_OWN_COMMENT + "')")
	@ApiOperation("Оставить комментарий к записи ленты по ID")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
			dataType = "ru.loylabs.portal.presentation.request.CreateRecordRequest", paramType = "body")
	@PostMapping(path = FEED_ID_URI_PARAM + COMMENTS_URI,
			consumes = {MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE})
	public CommentView createFeedComment(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
										 @Valid @RequestPart CreateRecordRequest request,
										 @RequestPart(required = false) MultipartFile[] files) {
		log.info("Request to create new comment for Feed with id '{}' with request: {}", feedId, request);
		return commentMapper.toView(
				commentService.saveComment(
						request,
						getCredential().getPerson(),
						feedId,
						RecordType.FEED,
						Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()))
		);
	}

	@PreAuthorize("hasAuthority('" + FEED_WRITE_COMMENT + "') || (hasAuthority('" + FEED_WRITE_OWN_COMMENT + "')" +
			" && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Редактирование комментария к записи ленты по ID")
	@PutMapping(path = FEED_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommentView updateFeedComment(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
										 @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
										 @Valid @RequestBody UpdateRecordRequest request) {
		log.info("Request to update comment with id '{}' for Feed with id '{}' with request: {}",
				commentId, feedId, request);
		return commentMapper.toView(commentService.updateComment(request, commentId, feedId));
	}

	@PreAuthorize("hasAuthority('" + FEED_WRITE_COMMENT + "') || (hasAuthority('" + FEED_WRITE_OWN_COMMENT + "')" +
			" && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation(value = "Удаление комментария к записи ленты по ID")
	@DeleteMapping(FEED_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM)
	public SuccessResponse deleteFeedComment(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
											 @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId) {
		log.info("Request to delete comment for Idea with id '{}'", feedId);
		commentService.deleteComment(commentId, feedId);
		return new SuccessResponse();
	}

//--FEED-COMMENT-ATTACHMENTS------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + COMMENT_WRITE_ATTACHMENT + "')" +
			" || (hasAuthority('" + COMMENT_WRITE_OWN_ATTACHMENT + "')" +
			" && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Добавление вложений к комментарию по ID")
	@PutMapping(path = FEED_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI,
			consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public CommentView createFeedCommentAttachment(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
												   @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
												   @RequestPart MultipartFile[] files) {
		log.info("Request to create attachment for comment with id '{}' for Feed with id '{}'", commentId, feedId);
		return commentMapper.toView(commentService.addAttachments(commentId, feedId, RecordType.FEED, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())));
	}

	@PreAuthorize("hasAuthority('" + COMMENT_WRITE_ATTACHMENT + "')" +
			" || (hasAuthority('" + COMMENT_WRITE_OWN_ATTACHMENT + "')" +
			" && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Удаление вложений комментария по ID")
	@DeleteMapping(FEED_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI)
	public CommentView deleteFeedCommentAttachment(@PathVariable(FEED_ID_URI_PARAM_NAME) UUID feedId,
												   @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
												   @Valid @RequestBody DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for comment with id '{}' for Feed with id '{}'", commentId, feedId);
		return commentMapper.toView(commentService.deleteAttachments(commentId, feedId, RecordType.FEED, request));
	}

}
