package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.loylabs.portal.mapper.GradeMapper;
import ru.loylabs.portal.presentation.request.CreateGradeRequest;
import ru.loylabs.portal.presentation.request.UpdateGradeRequest;
import ru.loylabs.portal.presentation.view.GradeView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.GradeService;

import javax.validation.Valid;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.GRADE_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.GRADE_WRITE;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(VERSION_1_URI + GRADE_URI)
public class GradeController {

    private final GradeService gradeService;
    private final GradeMapper gradeMapper;

    @PreAuthorize("hasAuthority('" + GRADE_READ + "')")
    @ApiOperation("Получить список грэйдов с сортировкой и фильтрацией")
    @GetMapping
    public ListView<GradeView> getAllGrade(@RequestParam(required = false, defaultValue = "0") int page,
                                           @RequestParam(required = false, defaultValue = "10") int size,
                                           @RequestParam(required = false, defaultValue = "name,asc") String sort,
                                           @RequestParam(required = false) String search) {
        log.info("Request for Grade list");
        var criteria = getCriteria(search);
        var titles = gradeService.getAllGrade(page, size, sort, criteria);
        return new ListView<>(titles.getTotalElements(),
                titles.getContent()
                        .stream()
                        .map(gradeMapper::toView)
                        .collect(Collectors.toList()));
    }

    @PreAuthorize("hasAuthority('" + GRADE_READ + "')")
    @ApiOperation("Получить грэйд по ID")
    @GetMapping(GRADE_ID_URI_PARAM)
    public GradeView getGrade(@PathVariable(GRADE_ID_URI_PARAM_NAME) UUID gradeId) {
        log.info("Request for Grade with id '{}'", gradeId);

        return gradeMapper.toView(gradeService.getGrade(gradeId));
    }

    @PreAuthorize("hasAuthority('" + GRADE_WRITE + "')")
    @ApiOperation("Создание грэйда")
    @ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
            dataType = "ru.loylabs.portal.presentation.request.CreateGradeRequest", paramType = "body")
    @PostMapping(
            consumes = MediaType.APPLICATION_JSON_VALUE)
    public GradeView createFeed(@Valid @RequestBody CreateGradeRequest request) {
        log.info("Request to create new Grade with request: {}", request);
        return gradeMapper.toView(gradeService.createGrade(request));
    }

    @PreAuthorize("hasAuthority('" + GRADE_WRITE + "')")
    @ApiOperation("Обновление грэйда по ID")
    @PutMapping(path = GRADE_ID_URI_PARAM, consumes = MediaType.APPLICATION_JSON_VALUE)
    public GradeView updateGrade(@PathVariable(GRADE_ID_URI_PARAM_NAME) UUID gradeId,
                                 @Valid @RequestBody UpdateGradeRequest request) {
        log.info("Request to update Grade with id '{}' with request {}", gradeId, request);
        return gradeMapper.toView(gradeService.updateGrade(gradeId, request));
    }

    @PreAuthorize("hasAuthority('" + GRADE_WRITE + "')")
    @ApiOperation("Удаление грэйда по ID")
    @DeleteMapping(GRADE_ID_URI_PARAM)
    public SuccessResponse deleteIdea(@PathVariable(GRADE_ID_URI_PARAM_NAME) UUID gradeId) {
        log.info("Request to delete Grade with id '{}'", gradeId);
        gradeService.deleteGrade(gradeId);
        return new SuccessResponse();
    }
}
