package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.mapper.CommentMapper;
import ru.loylabs.portal.mapper.IdeaMapper;
import ru.loylabs.portal.presentation.request.*;
import ru.loylabs.portal.presentation.view.CommentView;
import ru.loylabs.portal.presentation.view.IdeaView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.CommentService;
import ru.loylabs.portal.service.interfaces.IdeaService;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.Collections;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.*;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + IDEAS_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class IdeaController {

	private final IdeaService ideaService;
	private final IdeaMapper ideaMapper;

	private final CommentService commentService;
	private final CommentMapper commentMapper;

//--IDEAS-------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + IDEA_READ + "')")
	@ApiOperation("Получение списка идей с сортировкой и фильтрацией")
	@GetMapping
	public ListView<IdeaView> findAllIdeas(@RequestParam(required = false, defaultValue = "0") int page,
										   @RequestParam(required = false, defaultValue = "10") int size,
										   @RequestParam(required = false, defaultValue = "id,desc") String sort,
										   @RequestParam(required = false) String search) {
		log.info("Request for Idea list");
		var principal = getCredential().getPerson();
		var criteria = getCriteria(search);
		var ideas = ideaService.getAllIdeas(page, size, sort, criteria);
		return new ListView<>(ideas.getTotalElements(),
							  ideas.getContent()
								   .stream()
								   .map(i -> ideaMapper.toView(i, i.getLiked().contains(principal)))
								   .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + IDEA_READ + "')")
	@ApiOperation("Получение избранных идей")
	@GetMapping(FAVORITE_URI)
	public ListView<IdeaView> findAllFavoriteIdeas(@RequestParam(required = false, defaultValue = "0") int page,
												   @RequestParam(required = false, defaultValue = "10") int size,
												   @RequestParam(required = false, defaultValue = "id,desc") String sort) {
		var principal = getCredential().getPerson();
		log.info("Request for favorite Idea list for Person with id '{}'", principal.getId());
		var criteria = getCriteria("search");
		var ideas = ideaService.getFavoriteIdeas(page, size, sort, criteria, principal);
		return new ListView<>(ideas.getTotalElements(),
							  ideas.getContent()
								   .stream()
								   .map(idea -> ideaMapper.toView(idea, idea.getLiked().contains(principal)))
								   .collect(Collectors.toList())
		);
	}

	@PreAuthorize("hasAuthority('" + IDEA_READ + "')")
	@ApiOperation("Получение идеи по ID")
	@GetMapping(IDEA_ID_URI_PARAM)
	public IdeaView findOneIdea(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId) {
		log.info("Request for Idea with id '{}'", ideaId);
		return ideaMapper.toView(ideaService.getIdea(ideaId),
								 ideaService.isFavoriteForPerson(ideaId, getCredential().getPerson()));
	}

	@PreAuthorize("hasAnyAuthority('" + IDEA_WRITE + "', '" + IDEA_WRITE_OWN + "')")
	@ApiOperation("Создание идеи")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
					  dataType = "ru.loylabs.portal.presentation.request.CreateIdeaRequest", paramType = "body")
	@PostMapping(consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public IdeaView createIdea(@Valid @RequestPart CreateIdeaRequest request,
							   @RequestPart(required = false) MultipartFile[] files) {
		log.info("Request to create new Idea with request: {}", request);
		return ideaMapper.toView(ideaService.saveIdea(request,
				getCredential().getPerson(),
				Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())),
				false);
	}

	@PreAuthorize("hasAuthority('" + IDEA_WRITE + "') || (hasAuthority('" + IDEA_WRITE_OWN + "')" +
				  " && @sec.isPrincipalRecordAuthor(#ideaId))")
	@ApiOperation("Обновление идеи по ID")
	@PutMapping(path = IDEA_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public IdeaView updateIdea(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
							   @Valid @RequestBody UpdateIdeaRequest request) {
		log.info("Request to update Idea with id '{}' with request {}", ideaId, request);
		return ideaMapper.toView(ideaService.updateIdea(ideaId, request),
								 ideaService.isFavoriteForPerson(ideaId, getCredential().getPerson()));
	}

	@PreAuthorize("hasAuthority('" + IDEA_FAVORITE + "')")
	@ApiOperation("Добавление идеи в избранное")
	@PutMapping(IDEA_ID_URI_PARAM + FAVORITE_URI + MARK_URI)
	public SuccessResponse markFavorite(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId) {
		var principal = getCredential().getPerson();
		log.info("Request to mark as favorite Idea with id '{}' for Person with id '{}'", ideaId, principal.getId());
		ideaService.markAsFavorites(ideaId, principal);
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + IDEA_FAVORITE + "')")
	@ApiOperation("Убрать идею из избранного")
	@PutMapping(IDEA_ID_URI_PARAM + FAVORITE_URI + UNMARK_URI)
	public SuccessResponse unmarkFavorite(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId) {
		var principal = getCredential().getPerson();
		log.info("Request to unmark as favorite Idea with id '{}' for Person with id '{}'", ideaId, principal.getId());
		ideaService.unmarkAsFavorites(ideaId, principal);
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + IDEA_APPROVAL + "')")
	@ApiOperation("Принятие идеи по ID")
	@PutMapping(path = IDEA_ID_URI_PARAM + ACCEPT_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public SuccessResponse approveIdea(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
									   @Valid @RequestBody ProvideFeedbackRequest request) {
		log.info("Request to approve Idea with id '{}'", ideaId);
		ideaService.acceptIdea(request, ideaId, getCredential().getPerson());
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + IDEA_APPROVAL + "')")
	@ApiOperation("Отклонение идеи по ID")
	@PutMapping(path = IDEA_ID_URI_PARAM + REJECT_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public SuccessResponse rejectIdea(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
									  @Valid @RequestBody ProvideFeedbackRequest request) {
		log.info("Request to reject Idea with id '{}'", ideaId);
		ideaService.rejectIdea(request, ideaId, getCredential().getPerson());
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + IDEA_WRITE + "') || (hasAuthority('" + IDEA_WRITE_OWN + "')" +
				  " && @sec.isPrincipalRecordAuthor(#ideaId))")
	@ApiOperation("Удаление идеи по ID")
	@DeleteMapping(IDEA_ID_URI_PARAM)
	public SuccessResponse deleteIdea(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId) {
		log.info("Request to delete Idea with id '{}'", ideaId);
		ideaService.deleteIdea(ideaId);
		return new SuccessResponse();
	}

//--IDEA-ATTACHMENTS--------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + IDEA_WRITE_ATTACHMENT + "')" +
				  " || (hasAuthority('" + IDEA_WRITE_OWN_ATTACHMENT + "') && @sec.isPrincipalRecordAuthor(#ideaId))")
	@ApiOperation("Добавление вложений к идее по ID")
	@PutMapping(path = IDEA_ID_URI_PARAM + ATTACHMENTS_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public IdeaView createIdeaAttachment(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
										 @RequestPart("file") MultipartFile[] files) {
		log.info("Request to create attachment for Idea with id '{}'", ideaId);
		return ideaMapper.toView(ideaService.addAttachments(ideaId, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())),
								 ideaService.isFavoriteForPerson(ideaId, getCredential().getPerson()));
	}

	@PreAuthorize("hasAuthority('" + IDEA_WRITE_ATTACHMENT + "')" +
				  " || (hasAuthority('" + IDEA_WRITE_OWN_ATTACHMENT + "') && @sec.isPrincipalRecordAuthor(#ideaId))")
	@ApiOperation("Удаление вложений идее по ID")
	@DeleteMapping(IDEA_ID_URI_PARAM + ATTACHMENTS_URI)
	public IdeaView deleteIdeaAttachment(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
										 @Valid @RequestBody DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for Idea with id '{}'", ideaId);
		return ideaMapper.toView(ideaService.deleteAttachments(ideaId, request),
								 ideaService.isFavoriteForPerson(ideaId, getCredential().getPerson()));
	}

//--IDEA-COMMENTS-----------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + IDEA_READ_COMMENT + "')")
	@ApiOperation("Получение комментариев к идее с сортировкой и фильтрацией по ID идеи")
	@GetMapping(IDEA_ID_URI_PARAM + COMMENTS_URI)
	public ListView<CommentView> findAllIdeaComments(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
													 @RequestParam(required = false, defaultValue = "0") int page,
													 @RequestParam(required = false, defaultValue = "10") int size,
													 @RequestParam(required = false, defaultValue = "id,desc") String sort,
													 @RequestParam(required = false) String search) {
		log.info("Request for Idea comments list for Idea with id '{}'", ideaId);
		var criteria = getCriteria(search);
		var comments = commentService.getAllComments(page, size, sort, criteria, ideaId, RecordType.IDEA);
		return new ListView<>(comments.getTotalElements(),
							  comments.getContent()
									  .stream()
									  .map(commentMapper::toView)
									  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAnyAuthority('" + IDEA_WRITE_COMMENT + "', '" + IDEA_WRITE_OWN_COMMENT + "')")
	@ApiOperation("Оставить комментарий к идее по ID")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
					  dataType = "ru.loylabs.portal.presentation.request.CreateRecordRequest", paramType = "body")
	@PostMapping(path = IDEA_ID_URI_PARAM + COMMENTS_URI,
				 consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public CommentView createIdeaComment(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
										 @Valid @RequestPart CreateRecordRequest request,
										 @RequestPart(required = false) MultipartFile[] files) {
		log.info("Request to create new comment for Idea with id '{}' with request: {}", ideaId, request);
		return commentMapper.toView(
				commentService.saveComment(request,
						getCredential().getPerson(),
						ideaId,
						RecordType.IDEA,
						Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()))
		);
	}

	@PreAuthorize("hasAuthority('" + IDEA_WRITE_COMMENT + "') || (hasAuthority('" + IDEA_WRITE_OWN_COMMENT + "')" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Редактирование комментария к идее по ID")
	@PutMapping(path = IDEA_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public CommentView updateIdeaComment(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
										 @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
										 @Valid @RequestBody UpdateRecordRequest request) {
		log.info("Request to update comment with id '{}' for Idea with id '{}' with request: {}",
				 commentId, ideaId, request);
		return commentMapper.toView(commentService.updateComment(request, commentId, ideaId));
	}

	@PreAuthorize("hasAuthority('" + IDEA_WRITE_COMMENT + "') || (hasAuthority('" + IDEA_WRITE_OWN_COMMENT + "')" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Удаление комментария к идее по ID")
	@DeleteMapping(IDEA_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM)
	public SuccessResponse deleteIdeaComment(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
											 @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId) {
		log.info("Request to delete comment for Idea with id '{}'", ideaId);
		commentService.deleteComment(commentId, ideaId);
		return new SuccessResponse();
	}

//--IDEA-COMMENT-ATTACHMENTS------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + COMMENT_WRITE_ATTACHMENT + "')" +
				  " || (hasAuthority('" + COMMENT_WRITE_OWN_ATTACHMENT + "')" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Добавление вложений к комментарию по ID")
	@PutMapping(path = IDEA_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public CommentView createIdeaCommentAttachment(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
												   @PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
												   @RequestPart MultipartFile[] files) {
		log.info("Request to create attachment for comment with id '{}' for Idea with id '{}'", commentId, ideaId);
		return commentMapper.toView(commentService.addAttachments(commentId, ideaId, RecordType.IDEA, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())));
	}

	@PreAuthorize("hasAuthority('" + COMMENT_WRITE_ATTACHMENT + "')" +
				  " || (hasAuthority('" + COMMENT_WRITE_OWN_ATTACHMENT + "')" +
				  " && @sec.isPrincipalRecordAuthor(#commentId))")
	@ApiOperation("Удаление вложений комментария по ID")
	@DeleteMapping(IDEA_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI)
	public CommentView deleteIdeaCommentAttachments(@PathVariable(IDEA_ID_URI_PARAM_NAME) UUID ideaId,
													@PathVariable(COMMENT_ID_URI_PARAM_NAME) UUID commentId,
													@Valid @RequestBody DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for comment with id '{}' for Idea with id '{}'", commentId, ideaId);
		return commentMapper.toView(commentService.deleteAttachments(commentId, ideaId, RecordType.IDEA, request));
	}

}
