package ru.loylabs.portal.presentation.controller;

import static ru.loylabs.portal.config.URIConstants.ARTICLES_URI;
import static ru.loylabs.portal.config.URIConstants.ATTACHMENTS_URI;
import static ru.loylabs.portal.config.URIConstants.KB_ARTICLE_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.KB_ARTICLE_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.KB_SECTION_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.KB_SECTION_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.KB_URI;
import static ru.loylabs.portal.config.URIConstants.MOVE_URI;
import static ru.loylabs.portal.config.URIConstants.SEARCH_URI;
import static ru.loylabs.portal.config.URIConstants.SECTIONS_URI;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.KB_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.KB_WRITE_ARTICLE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.KB_WRITE_ATTACHMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.KB_WRITE_SECTION;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;
import javax.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.mapstruct.util.Experimental;
import ru.loylabs.portal.mapper.KbArticleMapper;
import ru.loylabs.portal.mapper.KbSectionMapper;
import ru.loylabs.portal.presentation.request.ChangeKbArticleSectionRequest;
import ru.loylabs.portal.presentation.request.CreateKbArticleRequest;
import ru.loylabs.portal.presentation.request.CreateKbSectionRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.request.UpdateKbArticleRequest;
import ru.loylabs.portal.presentation.request.UpdateKbSectionRequest;
import ru.loylabs.portal.presentation.view.KbArticleListView;
import ru.loylabs.portal.presentation.view.KbArticleView;
import ru.loylabs.portal.presentation.view.KbSectionSearchView;
import ru.loylabs.portal.presentation.view.KbSectionView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.KbArticleService;
import ru.loylabs.portal.service.interfaces.KbSectionService;

@Slf4j
@RestController
@RequiredArgsConstructor
@RequestMapping(path = VERSION_1_URI + KB_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class KbController {

	private final KbSectionService sectionService;
	private final KbArticleMapper articleMapper;

	private final KbArticleService articleService;
	private final KbSectionMapper sectionMapper;

//--KB-SECTION--------------------------------------------------------------------------------------------------------//

	@Experimental
	@PreAuthorize("hasAuthority('" + KB_READ + "')")
	@ApiOperation("Получение секций базы знаний по содержимому статей")
	@GetMapping(SECTIONS_URI + SEARCH_URI)
	public ListView<KbSectionSearchView> searchArticles(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam String searchPhrase
	) {
		log.info("Request for KB search");
		var criteria = getCriteria("search");
		var sections = sectionService.searchSectionWithRelevantContent(page, size, sort, criteria, searchPhrase);
		return new ListView<>(sections.getTotalElements(),
							  sections.getContent()
									  .stream()
									  .map(section -> sectionMapper.toSearchView(
											  section, section.getArticles().size()))
									  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + KB_READ + "')")
	@ApiOperation("Получение списка разделов с сортировкой и фильтрацией")
	@GetMapping
	public ListView<KbSectionView> findAllSections(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false, value = "search") String search
	) {
		log.info("Request for KbSection list");
		var criteria = getCriteria(search);
		var sections = sectionService.getAllSections(page, size, sort, criteria);
		return new ListView<>(sections.getTotalElements(),
							  sections.getContent()
									  .stream()
									  .map(section -> sectionMapper.toView(
											  section, sectionService.countArticles(section.getId())))
									  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + KB_READ + "')")
	@ApiOperation("Получение раздела по ID")
	@GetMapping(KB_SECTION_ID_URI_PARAM)
	public KbSectionView findOneSection(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId) {
		log.info("Request for KbSection with id '{}'", sectionId);
		var section = sectionService.getSection(sectionId);
		return sectionMapper.toView(section, sectionService.countArticles(section.getId()));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_SECTION + "')")
	@ApiOperation("Добавить раздел")
	@PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
	public KbSectionView createSection(@Valid @RequestBody CreateKbSectionRequest request) {
		log.info("Request to create new KbSection with request: {}", request);
		var section = sectionService.createSection(request);
		return sectionMapper.toView(section, sectionService.countArticles(section.getId()));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_SECTION + "')")
	@ApiOperation("Изменение раздела по ID")
	@PutMapping(path = KB_SECTION_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public KbSectionView updateSection(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
									   @Valid @RequestBody UpdateKbSectionRequest request) {
		log.info("Request to update KbSection with id '{}' with request: {}", sectionId, request);
		var section = sectionService.updateSection(sectionId, request);
		return sectionMapper.toView(section, sectionService.countArticles(section.getId()));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_SECTION + "')")
	@ApiOperation("Удаление раздела по ID")
	@DeleteMapping(KB_SECTION_ID_URI_PARAM)
	public SuccessResponse deleteSection(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId) {
		log.info("Request for deleting section with id {}", sectionId);
		sectionService.deleteSection(sectionId);
		return new SuccessResponse();
	}

//--KB-SECTION-ARTICLES-----------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + KB_READ + "')")
	@ApiOperation("Получение списка всех статей с сортировкой и фильтрацией")
	@GetMapping(ARTICLES_URI)
	public ListView<KbArticleListView> findAllArticles(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for KbArticles list");
		var searchCriteria = getCriteria(search);
		var allSections = articleService.getAllArticles(page, size, sort, searchCriteria);
		return new ListView<>(allSections.getTotalElements(),
							  allSections.getContent()
										 .stream()
										 .map(articleMapper::toArticleListView)
										 .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + KB_READ + "')")
	@ApiOperation("Получение списка статей из раздела по ID с сортировкой и фильтрацией")
	@GetMapping(SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI)
	public ListView<KbArticleListView> findAllSectionArticles(
			@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "id,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for KbArticles list from KbSection with id '{}'", sectionId);
		var criteria = getCriteria(search);
		var sections = articleService.getAllArticles(page, size, sort, criteria, sectionId);
		return new ListView<>(sections.getTotalElements(),
							  sections.getContent()
									  .stream()
									  .map(articleMapper::toArticleListView)
									  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + KB_READ + "')")
	@ApiOperation("Получение статьи по ID")
	@GetMapping(SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI + KB_ARTICLE_ID_URI_PARAM)
	public KbArticleView findOneArticle(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
										@PathVariable(KB_ARTICLE_ID_URI_PARAM_NAME) UUID articleId) {
		log.info("Request for KbArticle with id '{}' from KbSection with id '{}'", articleId, sectionId);
		return articleMapper.toView(articleService.getArticle(sectionId, articleId));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_ARTICLE + "')")
	@ApiOperation("Добавить статью")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
					  dataType = "ru.loylabs.portal.presentation.request.CreateKbArticleRequest", paramType = "body")
	@PostMapping(path = SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI,
				 consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public KbArticleView createArticle(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
									   @Valid @RequestPart CreateKbArticleRequest request,
									   @RequestPart(name = "file", required = false) MultipartFile[] files) {
		log.info("Request to create new KbArticle for KbSection with id '{}' with request: {}", sectionId, request);
		return articleMapper.toView(articleService.saveArticle(sectionId, request, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_ARTICLE + "')")
	@ApiOperation("Изменение статьи по ID")
	@PutMapping(path = SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI + KB_ARTICLE_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public KbArticleView updateArticle(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
									   @PathVariable(KB_ARTICLE_ID_URI_PARAM_NAME) UUID articleId,
									   @Valid @RequestBody UpdateKbArticleRequest request) {
		log.info("Request to update KbArticle with id '{}' from KbSection with id '{}' with request: {}",
				 articleId, sectionId, request);
		return articleMapper.toView(articleService.updateArticle(sectionId, articleId, request));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_ARTICLE + "')")
	@ApiOperation("Изменение раздела статьи по ID")
	@PutMapping(path = SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI + KB_ARTICLE_ID_URI_PARAM + MOVE_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public KbArticleView moveArticle(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
									 @PathVariable(KB_ARTICLE_ID_URI_PARAM_NAME) UUID articleId,
									 @Valid @RequestBody ChangeKbArticleSectionRequest request) {
		log.info("Request to update KbArticle's section with id '{}' from KbSection with id '{}' with request: {}",
				 articleId, sectionId, request);
		return articleMapper.toView(articleService.moveArticle(sectionId, articleId, request));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_ARTICLE + "')")
	@ApiOperation("Удаление статьи по ID")
	@DeleteMapping(SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI + KB_ARTICLE_ID_URI_PARAM)
	public SuccessResponse deleteArticle(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
										 @PathVariable(KB_ARTICLE_ID_URI_PARAM_NAME) UUID articleId) {
		log.info("Request to delete KbArticle with id '{}' from KbSection with id '{}'", articleId, sectionId);
		articleService.deleteArticle(sectionId, articleId);
		return new SuccessResponse();
	}

//--KB-ARTICLES-ATTACHMENTS-------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + KB_WRITE_ATTACHMENT + "')")
	@ApiOperation("Добавление вложений к статье по ID")
	@PutMapping(path = SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI + KB_ARTICLE_ID_URI_PARAM +
					   ATTACHMENTS_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public KbArticleView createArticleAttachment(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
												 @PathVariable(KB_ARTICLE_ID_URI_PARAM_NAME) UUID articleId,
												 @RequestPart("file") MultipartFile[] files) {
		log.info("Request to create attachment for KbArticle with id '{}' from KbSection with id '{}'",
				 articleId, sectionId);
		return articleMapper.toView(articleService.addAttachments(sectionId, articleId, Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList())));
	}

	@PreAuthorize("hasAuthority('" + KB_WRITE_ATTACHMENT + "')")
	@ApiOperation("Удаление вложений статьи по ID")
	@DeleteMapping(SECTIONS_URI + KB_SECTION_ID_URI_PARAM + ARTICLES_URI + KB_ARTICLE_ID_URI_PARAM + ATTACHMENTS_URI)
	public KbArticleView deleteArticleAttachment(@PathVariable(KB_SECTION_ID_URI_PARAM_NAME) UUID sectionId,
												 @PathVariable(KB_ARTICLE_ID_URI_PARAM_NAME) UUID articleId,
												 @Valid @RequestBody DeleteAttachmentsRequest request) {
		log.info("Request to delete attachment for KbArticle with id '{}' from KbSection with id '{}'",
				 articleId, sectionId);
		return articleMapper.toView(articleService.deleteAttachments(sectionId, articleId, request));
	}

}
