package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import ru.loylabs.portal.mapper.NoteMapper;
import ru.loylabs.portal.presentation.request.CreateNoteRequest;
import ru.loylabs.portal.presentation.request.UpdateNoteRequest;
import ru.loylabs.portal.presentation.view.NoteView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.NoteService;

import javax.validation.Valid;
import java.util.UUID;

import static ru.loylabs.portal.config.URIConstants.NOTE_URI;
import static ru.loylabs.portal.config.URIConstants.PERSON_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.PERSON_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.PERSON_URI;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.NOTE_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.NOTE_WRITE;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredentialPersonId;

@Slf4j
@RestController
@RequestMapping(path = VERSION_1_URI + PERSON_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
@RequiredArgsConstructor
public class NoteController {

	private final NoteService noteService;
	private final NoteMapper noteMapper;

//--NOTES-------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + NOTE_READ + "')")
	@ApiOperation("Получение заметки по ID пользователя")
	@GetMapping(PERSON_ID_URI_PARAM + NOTE_URI)
	public NoteView findOneNote(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request for note for person by id: {}", personId);
		return noteMapper.toView(noteService.getNote(getCredentialPersonId(), personId));
	}

	@PreAuthorize("hasAuthority('" + NOTE_WRITE + "')")
	@ApiOperation("Добавить заметку по ID пользователя")
	@PostMapping(path = PERSON_ID_URI_PARAM + NOTE_URI,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public NoteView createNote(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
							   @Valid @RequestBody CreateNoteRequest request) {
		log.info("Request for creating note for person with id {}: {}", personId, request);
		return noteMapper.toView(noteService.createNote(request, getCredentialPersonId(), personId));
	}

	@PreAuthorize("hasAuthority('" + NOTE_WRITE + "')")
	@ApiOperation("Изменить заметку по ID пользователя")
	@PutMapping(path = PERSON_ID_URI_PARAM + NOTE_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public NoteView updateNote(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
							   @Valid @RequestBody UpdateNoteRequest request) {
		log.info("Request for updating note for person with id {}: {}", personId, request);
		return noteMapper.toView(noteService.updateNote(request, getCredentialPersonId(), personId));
	}

	@PreAuthorize("hasAuthority('" + NOTE_WRITE + "')")
	@ApiOperation("Удалить заметку по ID пользователя")
	@DeleteMapping(PERSON_ID_URI_PARAM + NOTE_URI)
	public SuccessResponse deleteNote(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request for deleting note for person with id: {}", personId);
		noteService.deleteNote(getCredentialPersonId(), personId);
		return new SuccessResponse();
	}

}
