package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.City;
import ru.loylabs.portal.mapper.PersonMapper;
import ru.loylabs.portal.presentation.request.*;
import ru.loylabs.portal.presentation.view.PersonLoginView;
import ru.loylabs.portal.presentation.view.PersonView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.CityService;
import ru.loylabs.portal.service.interfaces.PersonService;

import javax.validation.Valid;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.*;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + PERSONS_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class PersonController {

	private final PersonService personService;
	private final PersonMapper personMapper;

	private final CityService cityService;

//--PERSONS-----------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + PERSON_READ + "')")
	@ApiOperation("Получение списка пользователей с сортировкой и фильтрацией")
	@GetMapping
	public ListView<PersonView> findAllPersons(@RequestParam(required = false, defaultValue = "0") int page,
											   @RequestParam(required = false, defaultValue = "10") int size,
											   @RequestParam(required = false, defaultValue = "id,desc") String sort,
											   @RequestParam(required = false) String search) {
		log.info("Request for Person list");
		var criteria = getCriteria(search);
		var persons = personService.getAllPersons(page, size, sort, criteria);
		return new ListView<>(persons.getTotalElements(),
							  persons.getContent()
									 .stream()
									 .map(personMapper::toView)
									 .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + PERSON_READ + "') || (hasAuthority('" + PERSON_READ_SELF + "')" +
				  " && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Получение пользователя по ID")
	@GetMapping(PERSON_ID_URI_PARAM)
	public PersonView findOnePerson(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request for Person with id '{}'", personId);
		return personMapper.toView(personService.getPerson(personId));
	}

	@ApiOperation("Получение себя")
	@GetMapping(SELF_URI)
	public PersonView findSelf() {
		var principal = getCredential().getPerson();
		log.info("Request for self from Person with id '{}'", principal.getId());
		return personMapper.toView(principal);
	}

	@PreAuthorize("hasAuthority('" + PERSON_READ_LOGIN + "') || (hasAuthority('" + PERSON_READ_LOGIN_SELF + "')" +
				  " && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Получение логина пользователя по ID")
	@GetMapping(PERSON_ID_URI_PARAM + USERNAME_URI)
	public PersonLoginView findOnePersonLogin(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request for login of Person with id '{}'", personId);
		return personMapper.toLoginView(personService.getPerson(personId));
	}

	@PreAuthorize("hasAuthority('" + PERSON_READ + "')")
	@ApiOperation("Получение предстоящих дней рождений")
	@GetMapping(UPCOMING_BIRTHDATS_URI)
	public ListView<PersonView> findAllUpcomingBirthdays(@RequestParam(required = false, defaultValue = "20") int size,
														 @RequestParam(required = false, defaultValue = "0") int page) {
		log.info("Request for upcoming birthdays list");
		var upcomingBirthdays = personService.getAllWithUpcomingBirthdays(size, page);
		return new ListView<>(upcomingBirthdays.getTotalElements(),
							  upcomingBirthdays.getContent()
											   .stream()
											   .map(personMapper::toView)
											   .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + PERSON_CREATE + "')")
	@ApiOperation("Добавить пользователя")
	@PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
	public PersonView createPerson(@Valid @RequestBody CreatePersonRequest request) {
		log.info("Request to create new Person with request: {}", request);
		return personMapper.toView(personService.createPerson(request));
	}

	@ApiOperation("Добавить пользователя через корпоративную Яндекс.Почту")
	@PostMapping(path = YANDEX_URI,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public PersonView createPersonFromYandex(@RequestBody @Valid CreatePersonFromYandexRequest request) {
		log.info("Request to create new Person from existing Yandex account with email '{}'", request.getEmail());
		return personMapper.toView(personService.createPersonFromYandex(request.getEmail()));
	}

	@PreAuthorize("(hasAuthority('" + PERSON_MODIFY + "'))" +
				  " || (hasAuthority('" + PERSON_MODIFY_SELF + "') && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Изменение пользователя по ID")
	@PutMapping(path = PERSON_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public PersonView updatePerson(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
								   @Valid @RequestBody UpdatePersonRequest request) {
		log.info("Request to update Person with id '{}' with request: {}", personId, request);
		return personMapper.toView(personService.updatePerson(request, personId));
	}

	@PreAuthorize("hasAuthority('" + PERSON_MODIFY_PERSONAL + "')" +
				  " || (hasAuthority('" + PERSON_MODIFY_PERSONAL_SELF + "')" +
				  " && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Изменение информации пользователя по ID")
	@PutMapping(path = PERSON_ID_URI_PARAM + PERSONAL_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public PersonView updatePersonPersonalInfo(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
											   @Valid @RequestBody UpdatePersonInfoRequest request) {
		log.info("Request to update personal info of Person with id '{}' with request: {}", personId, request);
		return personMapper.toView(personService.updatePersonInfo(request, personId));
	}

	@PreAuthorize("(hasAuthority('" + PERSON_MODIFY + "'))" +
				  " || (hasAuthority('" + PERSON_MODIFY_SELF + "') && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Изменение работы пользователя по ID пользователя")
	@PutMapping(path = PERSON_ID_URI_PARAM + JOB_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public PersonView updatePersonJob(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
									  @Valid @RequestBody UpdateJobRequest request) {
		log.info("Request to update Job of Person with id '{}' with request: {}", personId, request);
		return personMapper.toView(personService.updateJob(request, personId));
	}

	@PreAuthorize("(hasAuthority('" + PERSON_MODIFY + "'))" +
			" || (hasAuthority('" + PERSON_MODIFY_SELF + "') && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Изменение набора навыков пользователя по ID пользователя")
	@PutMapping(path = PERSON_ID_URI_PARAM + SKILL_URI,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	public PersonView updatePersonJob(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
									  @RequestBody UpdatePersonSkillsRequest request) {
		log.info("Request to update set skills of Person with id '{}' with request: {}", personId, request);
		return personMapper.toView(personService.updateSkills(request, personId));
	}

	@PreAuthorize("hasAuthority('" + PERSON_MODIFY_ROLE + "') && !@sec.isPrincipalAccountOwner(#personId)")
	@ApiOperation("Изменение роли пользователя по ID")
	@PutMapping(path = PERSON_ID_URI_PARAM + ROLE_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public PersonView updatePersonRole(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
									   @Valid @RequestBody UpdatePersonRoleRequest request) {
		log.info("Request to update Role of Person with id '{}' with request: {}", personId, request);
		return personMapper.toView(personService.updateRole(request, personId));
	}

	@PreAuthorize("(hasAuthority('" + PERSON_MODIFY_PASSWORD + "'))" +
				  " || (hasAuthority('" + PERSON_MODIFY_PASSWORD_SELF + "')" +
				  " && @sec.isPrincipalAccountOwner(#personId)" +
				  " && @sec.isPasswordCorrect(#personId, #request.getOldPassword()))")
	@ApiOperation("Изменение пароля пользователя по ID")
	@PutMapping(path = PERSON_ID_URI_PARAM + PASSWORD_URI,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public SuccessResponse updatePersonPassword(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
												@Valid @RequestBody UpdatePersonPasswordRequest request) {
		log.info("Request to update password of Person with id '{}' with request: {}", personId, request);
		personService.updatePassword(request.getNewPassword(), personId);
		return new SuccessResponse();
	}

	@PreAuthorize("(hasAuthority('" + PERSON_MODIFY_IMAGERY + "'))" +
				  " || (hasAuthority('" + PERSON_MODIFY_IMAGERY_SELF + "')" +
				  " && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Установить аватар пользователя по ID")
	@PutMapping(path = PERSON_ID_URI_PARAM + AVATAR_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public PersonView updatePersonAvatar(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
										 @RequestPart("file") MultipartFile file) {
		log.info("Request to update avatar of Person with id '{}'", personId);
		return personMapper.toView(personService.setAvatar(personId, file));
	}

	@PreAuthorize("(hasAuthority('" + PERSON_MODIFY_IMAGERY + "'))" +
				  " || (hasAuthority('" + PERSON_MODIFY_IMAGERY_SELF + "')" +
				  " && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Установить фон пользователя по ID")
	@PutMapping(path = PERSON_ID_URI_PARAM + BACKGROUND_URI,
			consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public PersonView updatePersonBackground(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId,
											 @RequestPart("file") MultipartFile file) {
		log.info("Request to update background of Person with id '{}'", personId);
		return personMapper.toView(personService.setBackground(personId, file));
	}

	@PreAuthorize("(hasAuthority('" + PERSON_MODIFY_IMAGERY + "'))" +
			" || (hasAuthority('" + PERSON_MODIFY_IMAGERY_SELF + "')" +
			" && @sec.isPrincipalAccountOwner(#personId))")
	@ApiOperation("Удалить фон пользователя по ID")
	@DeleteMapping( PERSON_ID_URI_PARAM + BACKGROUND_URI)
	public PersonView deletePersonBackground(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request to remove background of Person with id '{}'", personId);
		return personMapper.toView(personService.setBackground(personId, null));
	}

	@PreAuthorize("hasAuthority('" + PERSON_BLOCK + "') && !@sec.isPrincipalAccountOwner(#personId)")
	@ApiOperation("Блокировка пользователя по ID")
	@PutMapping(PERSON_ID_URI_PARAM + BLOCK_URI)
	public SuccessResponse blockPerson(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request to block Person with id '{}'", personId);
		personService.blockPerson(personId);
		return new SuccessResponse();
	}

	@PreAuthorize("hasAuthority('" + PERSON_BLOCK + "') && !@sec.isPrincipalAccountOwner(#personId)")
	@ApiOperation("Разблокировка пользователя по ID")
	@PutMapping(PERSON_ID_URI_PARAM + UNBLOCK_URI)
	public SuccessResponse unblockPerson(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId) {
		log.info("Request to unblock Person with id '{}'", personId);
		personService.unblockPerson(personId);
		return new SuccessResponse();
	}

    @PreAuthorize("(hasAuthority('" + PERSON_MODIFY_STATUS + "'))" +
            " || (hasAuthority('" + PERSON_MODIFY_STATUS_SELF + "')" +
            " && @sec.isPrincipalAccountOwner(#personId))")
    @ApiOperation("Установить статус пользователя по ID")
    @PutMapping(path = PERSON_ID_URI_PARAM + STATUS_URI + STATUS_ID_URI_PARAM)
    public PersonView updateStatus(@PathVariable(PERSON_ID_URI_PARAM_NAME) UUID personId, @PathVariable(STATUS_ID_URI_PARAM_NAME) UUID statusId) {
        log.info("Request to updating status of Person with id '{}'", personId);
        return personMapper.toView(personService.updateStatus(personId, statusId));
    }

    @PreAuthorize("(hasAuthority('" + PERSON_MODIFY_STATUS + "'))" +
            " || (hasAuthority('" + PERSON_MODIFY_STATUS_SELF + "')" +
            " && @sec.isPrincipalAccountOwner(#personId))")
    @ApiOperation("Удалить статус пользователя по ID")
    @DeleteMapping(PERSON_ID_URI_PARAM + STATUS_URI)
    public PersonView deleteStatus(@PathVariable UUID personId) {
        log.info("Request to delete status of Person with id '{}'", personId);
        return personMapper.toView(personService.deleteStatus(personId));
    }

//--CITIES------------------------------------------------------------------------------------------------------------//

	@ApiOperation("Получение списка городов")
	@GetMapping(CITIES_URI)
	public ListView<City> findAllCities(@RequestParam(required = false, defaultValue = "0") int page,
										@RequestParam(required = false, defaultValue = "10") int size,
										@RequestParam(required = false, defaultValue = "id,desc") String sort,
										@RequestParam(required = false) String search) {
		log.info("Request for City list");
		var criteria = getCriteria(search);
		var cities = cityService.getAllCity(page, size, sort, criteria);
		return new ListView<>(cities.getTotalElements(), cities.getContent());
	}

}
