package ru.loylabs.portal.presentation.controller;

import static ru.loylabs.portal.config.URIConstants.REACTIONS_URI;
import static ru.loylabs.portal.config.URIConstants.REACTION_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.REACTION_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.SMILES_URI;
import static ru.loylabs.portal.config.URIConstants.SMILE_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.SMILE_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REACTION_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REACTION_WRITE;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

import java.util.UUID;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.ApiOperation;
import ru.loylabs.portal.mapper.EmojiMapper;
import ru.loylabs.portal.mapper.SmileMapper;
import ru.loylabs.portal.presentation.view.EmojiView;
import ru.loylabs.portal.presentation.view.SmileView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.service.interfaces.EmojiService;
import ru.loylabs.portal.service.interfaces.SmileService;

// TODO 2023-10-31: Move to individual controllers akin to comments etc.
@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + REACTIONS_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class ReactionController {

	private final EmojiService emojiService;
	private final EmojiMapper emojiMapper;

	private final SmileService smileService;
	private final SmileMapper smileMapper;

//--SMILES------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + REACTION_READ + "')")
	@ApiOperation("Получить список смайлов с сортировкой и фильтрацией")
	@GetMapping(SMILES_URI)
	public ListView<SmileView> findAllSmiles(@RequestParam(required = false, defaultValue = "0") int page,
											 @RequestParam(required = false, defaultValue = "10") int size,
											 @RequestParam(required = false, defaultValue = "id,desc") String sort,
											 @RequestParam(required = false, value = "search") String search) {
		log.info("Request for Smile list");
		var criteria = getCriteria(search);
		var smiles = smileService.getAllSmiles(page, size, sort, criteria);
		return new ListView<>(smiles.getTotalElements(),
							  smiles.getContent()
									.stream()
									.map(smileMapper::toView)
									.collect(Collectors.toList()));
	}

//--EMOJIS------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + REACTION_READ + "')")
	@ApiOperation("Получить список реакций с сортировкой и фильтрацией")
	@GetMapping(REACTION_ID_URI_PARAM)
	public ListView<EmojiView> findAllEmojis(@PathVariable(REACTION_ID_URI_PARAM_NAME) UUID reactionId,
											 @RequestParam(required = false, defaultValue = "0") int page,
											 @RequestParam(required = false, defaultValue = "10") int size,
											 @RequestParam(required = false, defaultValue = "id,desc") String sort,
											 @RequestParam(required = false) String search) {
		log.info("Request for Emoji list");
		var criteria = getCriteria(search);
		var emojis = emojiService.getAllEmoji(
				page, size, sort, criteria, reactionId, getCredential().getPerson()
		);
		return new ListView<>(emojis.getTotalElements(),
							  emojis.getContent()
									.stream()
									.map(emojiMapper::toView)
									.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + REACTION_WRITE + "')")
	@ApiOperation("Поставить реакцию на запись по ID смайла")
	@PostMapping(REACTION_ID_URI_PARAM + SMILE_ID_URI_PARAM)
	public EmojiView incrementEmoji(@PathVariable(REACTION_ID_URI_PARAM_NAME) UUID reactionId,
									@PathVariable(SMILE_ID_URI_PARAM_NAME) UUID smileId) {
		log.info("Request for incrementing smile count with id '{}' for reaction with id '{}'", smileId, reactionId);
		return emojiMapper.toView(
				emojiService.increaseEmoji(reactionId, smileId, getCredential().getPerson())
		);
	}

	@PreAuthorize("hasAuthority('" + REACTION_WRITE + "')")
	@ApiOperation("Убрать реакцию на запись по ID смайла")
	@DeleteMapping(REACTION_ID_URI_PARAM + SMILE_ID_URI_PARAM)
	public EmojiView decrementEmoji(@PathVariable(REACTION_ID_URI_PARAM_NAME) UUID reactionId,
									@PathVariable(SMILE_ID_URI_PARAM_NAME) UUID smileId) {
		log.info("Request for decrementing smile count with id '{}' for reaction with id '{}'", smileId, reactionId);
		return emojiMapper.toView(
				emojiService.decreaseEmoji(reactionId, smileId, getCredential().getPerson())
		);
	}

}
