package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import ru.loylabs.portal.entity.ReviewPersonSkill;
import ru.loylabs.portal.mapper.PersonMapper;
import ru.loylabs.portal.mapper.ReviewPersonSkillMapper;
import ru.loylabs.portal.mapper.ReviewPersonSkillRatingMapper;
import ru.loylabs.portal.mapper.ReviewerSkillMapper;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkill;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkillRating;
import ru.loylabs.portal.presentation.request.UpdateReviewPersonSkill;
import ru.loylabs.portal.presentation.view.ReviewPersonSkillRatingView;
import ru.loylabs.portal.presentation.view.ReviewPersonSkillView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.ReviewPersonService;

import javax.validation.Valid;
import java.util.Collections;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.OWNER_REVIEW_URI;
import static ru.loylabs.portal.config.URIConstants.PERSON_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.PERSON_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.PERSON_URI;
import static ru.loylabs.portal.config.URIConstants.REVIEWER_URI;
import static ru.loylabs.portal.config.URIConstants.REVIEW_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.REVIEW_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.REVIEW_RATING_URI;
import static ru.loylabs.portal.config.URIConstants.REVIEW_URI;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REVIEW_RATING_WRITE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REVIEW_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REVIEW_WRITE;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;
import static ru.loylabs.portal.utility.EncodingObject.encodingObject;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + REVIEW_URI,
		produces = MediaType.APPLICATION_JSON_VALUE)
public class ReviewController {

	private final ReviewPersonService reviewPersonService;
	private final ReviewPersonSkillMapper reviewSkillMapper;
	private final ReviewPersonSkillRatingMapper reviewPersonSkillRatingMapper;
	private final ReviewerSkillMapper reviewerSkillMapper;
	private final PersonMapper personMapper;

	@PreAuthorize("hasAuthority('" + REVIEW_READ + "')")
	@ApiOperation("Получить все ревью пользователя")
	@GetMapping(path = PERSON_URI + PERSON_ID_URI_PARAM + OWNER_REVIEW_URI + REVIEW_URI)
	public ListView<ReviewPersonSkillView> getAllReviewPersonOwner(@RequestParam(required = false, defaultValue = "0") int page,
																   @RequestParam(required = false, defaultValue = "10") int size,
																   @RequestParam(required = false, defaultValue = "id,desc") String sort,
																   @RequestParam(required = false) String search,
																   @PathVariable(PERSON_ID_URI_PARAM_NAME) UUID ownerId) {
		log.info("Request for a list of owner reviews {}", ownerId);
		var criteria = getCriteria(search);
		var review = reviewPersonService.getAllReviewPersonSkillByOwner(page, size, sort, criteria, ownerId);
		return new ListView<>(review.getTotalElements(),
				review.getContent()
						.stream()
						.map(this::encodingSingleReviewer)
						.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + REVIEW_READ + "')")
	@ApiOperation("Получить все ревью в которых участвует пользователь")
	@GetMapping(path = PERSON_URI + PERSON_ID_URI_PARAM + REVIEWER_URI + REVIEW_URI)
	public ListView<ReviewPersonSkillView> getAllReviewPersonReviewer(@RequestParam(required = false, defaultValue = "0") int page,
																	  @RequestParam(required = false, defaultValue = "10") int size,
																	  @RequestParam(required = false, defaultValue = "id,desc") String sort,
																	  @RequestParam(required = false) String search,
																	  @PathVariable(PERSON_ID_URI_PARAM_NAME) UUID reviewerId) {
		log.info("Request for a list reviews of reviewer {}", reviewerId);
		var criteria = getCriteria(search);
		var review = reviewPersonService.getAllReviewPersonSkillByIdReviewer(page, size, sort, criteria, reviewerId);
		return new ListView<>(review.getTotalElements(),
				review.getContent()
						.stream()
						.map(this::encodingSingleReviewer)
						.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + REVIEW_READ + "')")
	@ApiOperation("Получить ревью по ID")
	@GetMapping(path = PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM)
	public ReviewPersonSkillView getReviewById(@PathVariable(REVIEW_ID_URI_PARAM_NAME) UUID reviewId) {
		log.info("Request for a review {}", reviewId);
		return encodingSingleReviewer(reviewPersonService.getReviewPersonSkillById(reviewId));
	}

	@PreAuthorize("hasAuthority('" + REVIEW_READ + "')")
	@ApiOperation("Получить оценки по ID ревью")
	@GetMapping(path = PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM + REVIEW_RATING_URI)
	public ListView<ReviewPersonSkillRatingView> getRatingByReviewId(@PathVariable(REVIEW_ID_URI_PARAM_NAME) UUID reviewId) {
		log.info("Request for a rating by id review {}", reviewId);
		var ratingReview = reviewPersonService.getRatingByReviewId(reviewId);
		return new ListView<>(ratingReview.size(),
				ratingReview.stream()
						.map(reviewPersonSkillRatingMapper::toView)
						.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + REVIEW_WRITE + "')")
	@ApiOperation("Добавить ревью")
	@PostMapping(path = PERSON_URI + REVIEW_URI,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	public ReviewPersonSkillView createReview(@Valid @RequestBody CreateReviewPersonSkill request) {
		log.info("Request to create a new person review {} for the skill {}", request.getOwnerId(), request.getSkillId());
		return encodingSingleReviewer(reviewPersonService.createReview(request));
	}

	@PreAuthorize("hasAuthority('" + REVIEW_RATING_WRITE + "')")
	@ApiOperation("Добавить оценку к ревью")
	@PostMapping(path = PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM + REVIEW_RATING_URI,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	public ReviewPersonSkillRatingView createReviewRating(@Valid @RequestBody CreateReviewPersonSkillRating request,
														  @PathVariable(REVIEW_ID_URI_PARAM_NAME) UUID reviewId) {
		log.info("Request to create a new rating for the review {}", reviewId);
		return reviewPersonSkillRatingMapper.toView(reviewPersonService.createRating(reviewId, request));
	}

	@PreAuthorize("hasAuthority('" + REVIEW_WRITE + "')")
	@ApiOperation("Редактировать ревью")
	@PutMapping(path = PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM,
			consumes = MediaType.APPLICATION_JSON_VALUE)
	public ReviewPersonSkillView updateReview(@PathVariable(REVIEW_ID_URI_PARAM_NAME) UUID reviewId,
											  @Valid @RequestBody UpdateReviewPersonSkill request) {
		log.info("Request to update review {}", reviewId);
		return encodingSingleReviewer(reviewPersonService.updateReviewPersonSkill(reviewId, request));
	}

	@PreAuthorize("hasAuthority('" + REVIEW_WRITE + "')")
	@ApiOperation("Удалить ревью")
	@DeleteMapping(path = PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM)
	public SuccessResponse removeReview(@PathVariable(REVIEW_ID_URI_PARAM_NAME) UUID reviewId) {
		log.info("Request to delete review {}", reviewId);
		reviewPersonService.removeReview(reviewId);
		return new SuccessResponse();
	}

	private ReviewPersonSkillView encodingSingleReviewer(ReviewPersonSkill review) {
		if (review.getReviewers() != null && review.getReviewers().size() == 1) {
			var reviewer = personMapper.toPersonReviewView(encodingObject(review.getReviewers().get(0).getReviewer().getId().toString()));
			var reviewersView = reviewerSkillMapper.toReviewersView(reviewer, review.getReviewers().get(0).isVoted());
			return reviewSkillMapper.toView(review, Collections.singletonList(reviewersView));
		}
		return reviewSkillMapper.toView(review);
	}
}
