package ru.loylabs.portal.presentation.controller;

import static ru.loylabs.portal.config.URIConstants.ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.PERMISSIONS_URI;
import static ru.loylabs.portal.config.URIConstants.ROLES_URI;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.ROLE_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.ROLE_WRITE;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;
import javax.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.ApiOperation;
import ru.loylabs.portal.mapper.SecurityPermissionMapper;
import ru.loylabs.portal.mapper.SecurityRoleMapper;
import ru.loylabs.portal.presentation.request.CreateSecurityRoleRequest;
import ru.loylabs.portal.presentation.request.UpdateSecurityRoleRequest;
import ru.loylabs.portal.presentation.view.SecurityPermissionView;
import ru.loylabs.portal.presentation.view.SecurityRoleView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.SecurityPermissionService;
import ru.loylabs.portal.service.interfaces.SecurityRoleService;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + ROLES_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class RoleController {

	private final SecurityRoleService securityRoleService;
	private final SecurityRoleMapper securityRoleMapper;

	private final SecurityPermissionService securityPermissionService;
	private final SecurityPermissionMapper securityPermissionMapper;

//--ROLES-------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + ROLE_READ + "')")
	@ApiOperation("Получение списка ролей с сортировкой и фильтрацией")
	@GetMapping
	public ListView<SecurityRoleView> findAllRoles(@RequestParam(required = false, defaultValue = "0") int page,
												   @RequestParam(required = false, defaultValue = "10") int size,
												   @RequestParam(required = false, defaultValue = "id,desc") String sort,
												   @RequestParam(required = false) String search) {
		log.info("Request for Role list");
		var criteria = getCriteria(search);
		var roles = securityRoleService.getAllRoles(page, size, sort, criteria);
		return new ListView<>(roles.getTotalElements(),
							  roles.getContent()
								   .stream()
								   .map(securityRoleMapper::toView)
								   .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + ROLE_READ + "')")
	@ApiOperation("Получение роли по ID")
	@GetMapping(ID_URI_PARAM)
	public SecurityRoleView findOneRole(@PathVariable(ID_URI_PARAM_NAME) UUID roleId) {
		log.info("Request for Role with id '{}'", roleId);
		return securityRoleMapper.toView(securityRoleService.getRoleById(roleId));
	}

	@PreAuthorize("hasAuthority('" + ROLE_WRITE + "')")
	@ApiOperation("Добавить роль")
	@PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
	public SecurityRoleView createRole(@Valid @RequestBody CreateSecurityRoleRequest request) {
		log.info("Request to create new Role with request: {}", request);
		return securityRoleMapper.toView(securityRoleService.createRole(request));
	}

	@PreAuthorize("hasAuthority('" + ROLE_WRITE + "')")
	@ApiOperation("Изменение роли по ID")
	@PutMapping(path = ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public SecurityRoleView updateRole(@PathVariable(ID_URI_PARAM_NAME) UUID roleId,
									   @Valid @RequestBody UpdateSecurityRoleRequest request) {
		log.info("Request to update Role with id '{}' with request: {}", roleId, request);
		return securityRoleMapper.toView(securityRoleService.updateRole(request, roleId));
	}

	@PreAuthorize("hasAuthority('" + ROLE_WRITE + "')")
	@ApiOperation("Удаление роли")
	@DeleteMapping(ID_URI_PARAM)
	public SuccessResponse deleteRole(@PathVariable(ID_URI_PARAM_NAME) UUID roleId) {
		log.info("Request to delete Role with id '{}'", roleId);
		securityRoleService.deleteRole(roleId);
		return new SuccessResponse();
	}

//--PERMISSIONS-------------------------------------------------------------------------------------------------------//

	@ApiOperation("Получение всех существующих прав")
	@GetMapping(PERMISSIONS_URI)
	public ListView<SecurityPermissionView> findAllPermissions() {
		log.info("Request for Permission list");
		List<SecurityPermissionView> permissions = securityPermissionService.getAll()
																			.stream()
																			.map(securityPermissionMapper::toView)
																			.collect(Collectors.toList());
		return new ListView<>(permissions.size(), permissions);
	}

	@PreAuthorize("hasAuthority('" + ROLE_READ + "') || @sec.doesPrincipalHaveRole(#id)")
	@ApiOperation("Получение доступных прав роли")
	@GetMapping(ID_URI_PARAM + PERMISSIONS_URI)
	public ListView<SecurityPermissionView> findAllRolePermissions(@PathVariable(ID_URI_PARAM_NAME) UUID id) {
		log.info("Request for Role Permission list");
		List<SecurityPermissionView> permissions = securityPermissionService.findByRoleId(id)
																			.stream()
																			.map(securityPermissionMapper::toView)
																			.collect(Collectors.toList());
		return new ListView<>(permissions.size(), permissions);
	}

}
