package ru.loylabs.portal.presentation.controller;

import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;
import static ru.loylabs.portal.config.URIConstants.BELL_URI;
import static ru.loylabs.portal.config.URIConstants.CURRENT_URI;
import static ru.loylabs.portal.config.URIConstants.EMAIL_URI;
import static ru.loylabs.portal.config.URIConstants.ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.NOTIFICATIONS_URI;
import static ru.loylabs.portal.config.URIConstants.SETTINGS_URI;
import static ru.loylabs.portal.config.URIConstants.STORIES_URI;
import static ru.loylabs.portal.config.URIConstants.TAGS_URI;
import static ru.loylabs.portal.config.URIConstants.TYPES_URI;
import static ru.loylabs.portal.config.URIConstants.TYPE_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.TYPE_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.SETTINGS_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.SETTINGS_WRITE;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;
import javax.validation.Valid;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import io.swagger.annotations.ApiOperation;
import ru.loylabs.portal.enums.BellNotificationType;
import ru.loylabs.portal.enums.EmailNotificationType;
import ru.loylabs.portal.enums.StoryGroupType;
import ru.loylabs.portal.mapper.settings.BellNotificationSettingsMapper;
import ru.loylabs.portal.mapper.settings.EmailNotificationSettingsMapper;
import ru.loylabs.portal.mapper.settings.SettingsTagMapper;
import ru.loylabs.portal.mapper.settings.StorySettingsMapper;
import ru.loylabs.portal.presentation.request.settings.CreateBellNotificationSettingsRequest;
import ru.loylabs.portal.presentation.request.settings.CreateEmailNotificationSettingsRequest;
import ru.loylabs.portal.presentation.request.settings.CreateStorySettingsRequest;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.settings.BellNotificationSettingsView;
import ru.loylabs.portal.presentation.view.settings.BellNotificationTypeView;
import ru.loylabs.portal.presentation.view.settings.EmailNotificationSettingsView;
import ru.loylabs.portal.presentation.view.settings.EmailNotificationTypeView;
import ru.loylabs.portal.presentation.view.settings.SettingsTagView;
import ru.loylabs.portal.presentation.view.settings.StorySettingsView;
import ru.loylabs.portal.service.interfaces.BellNotificationSettingsService;
import ru.loylabs.portal.service.interfaces.EmailNotificationSettingsService;
import ru.loylabs.portal.service.interfaces.SettingsTagService;
import ru.loylabs.portal.service.interfaces.StorySettingsService;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + SETTINGS_URI,
				produces = APPLICATION_JSON_VALUE)
public class SettingsController {

	private final EmailNotificationSettingsService emailNotificationSettingsService;
	private final EmailNotificationSettingsMapper emailNotificationSettingsMapper;

	private final BellNotificationSettingsService bellNotificationSettingsService;
	private final BellNotificationSettingsMapper bellNotificationSettingsMapper;

	private final StorySettingsService storySettingsService;
	private final StorySettingsMapper storySettingsMapper;

	private final SettingsTagService settingsTagService;
	private final SettingsTagMapper settingsTagMapper;

//--SETTINGS-TAGS-----------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение списка всех тегов для настроек")
	@GetMapping(TAGS_URI)
	public ListView<SettingsTagView> findAllSettingsTags() {
		log.info("Request for SettingsTag list");
		var allTags = settingsTagService.getAllTags();
		return new ListView<>(allTags.size(),
							  allTags.stream()
									 .map(settingsTagMapper::toView)
									 .collect(Collectors.toList()));
	}

//--EMAIL-NOTIFICATION-SETTINGS---------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение списка настроек почтовых уведомлений с сортировкой и фильтрацией")
	@GetMapping(NOTIFICATIONS_URI + EMAIL_URI)
	public ListView<EmailNotificationSettingsView> findAllEmailNotificationSettings(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "creationDate,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for EmailNotificationSettings list");
		var criteria = getCriteria(search);
		var emailNotificationSettings = emailNotificationSettingsService.getAllSettings(page, size, sort, criteria);
		return new ListView<>(emailNotificationSettings.getTotalElements(),
							  emailNotificationSettings.getContent()
													   .stream()
													   .map(emailNotificationSettingsMapper::toView)
													   .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение настроек почтовых уведомлений по ID")
	@GetMapping(NOTIFICATIONS_URI + EMAIL_URI + ID_URI_PARAM)
	public EmailNotificationSettingsView findOneEmailNotificationSettings(@PathVariable(ID_URI_PARAM_NAME) UUID id) {
		log.info("Request for EmailNotificationSettings with id '{}'", id);
		return emailNotificationSettingsMapper.toView(emailNotificationSettingsService.getSettings(id));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение актуальных настроек почтовых уведомлений")
	@GetMapping(NOTIFICATIONS_URI + EMAIL_URI + TYPE_URI_PARAM + CURRENT_URI)
	public EmailNotificationSettingsView findCurrentEmailNotificationSettings(
			@PathVariable(TYPE_URI_PARAM_NAME) EmailNotificationType notificationType
	) {
		log.info("Request for current EmailNotificationSettings with '{}' EmailNotificationType",
				 notificationType.name());
		return emailNotificationSettingsMapper.toView(emailNotificationSettingsService.getCurrentSettings(notificationType));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_WRITE + "')")
	@ApiOperation("Обновить актуальные настройки почтовых уведомлений путем добавления новых")
	@PostMapping(path = NOTIFICATIONS_URI + EMAIL_URI + TYPE_URI_PARAM,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public EmailNotificationSettingsView createEmailNotificationSettings(
			@PathVariable(TYPE_URI_PARAM_NAME) EmailNotificationType emailNotificationType,
			@Valid @RequestBody CreateEmailNotificationSettingsRequest request
	) {
		log.info("Request to create new EmailNotificationSettings with request: {}", request);
		return emailNotificationSettingsMapper.toView(
				emailNotificationSettingsService.createSettings(request, emailNotificationType)
		);
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение всех типов почтовых оповещений")
	@GetMapping(NOTIFICATIONS_URI + EMAIL_URI + TYPES_URI)
	public EmailNotificationTypeView findAllEmailNotificationTypes() {
		log.info("Request for EmailNotificationType list");
		return new EmailNotificationTypeView(List.of(EmailNotificationType.values()));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение доступных тегов для типа почтовых оповещений")
	@GetMapping(NOTIFICATIONS_URI + EMAIL_URI + TYPE_URI_PARAM + TAGS_URI)
	public ListView<SettingsTagView> findEmailNotificationTypeAllowedTags(
			@PathVariable(TYPE_URI_PARAM_NAME) EmailNotificationType notificationType
	) {
		log.info("Request for allowed tags for '{}' EmailNotificationType", notificationType.name());
		var tags = settingsTagService.getAllowedTagsForType(notificationType);
		return new ListView<>(tags.size(),
							  tags.stream()
								  .map(settingsTagMapper::toView)
								  .collect(Collectors.toList()));
	}

//--BELL-NOTIFICATION-SETTINGS----------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение списка настроек уведомлений портала с сортировкой и фильтрацией")
	@GetMapping(NOTIFICATIONS_URI + BELL_URI)
	public ListView<BellNotificationSettingsView> findAllBellNotificationSettings(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "creationDate,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for BellNotificationSettings list");
		var criteria = getCriteria(search);
		var bellNotificationSettings = bellNotificationSettingsService.getAllSettings(page, size, sort, criteria);
		return new ListView<>(bellNotificationSettings.getTotalElements(),
							  bellNotificationSettings.getContent()
													  .stream()
													  .map(bellNotificationSettingsMapper::toView)
													  .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение настроек уведомлений портала по ID")
	@GetMapping(NOTIFICATIONS_URI + BELL_URI + ID_URI_PARAM)
	public BellNotificationSettingsView findOneBellNotificationSettings(@PathVariable(ID_URI_PARAM_NAME) UUID id) {
		log.info("Request for BellNotificationSettings with id '{}'", id);
		return bellNotificationSettingsMapper.toView(bellNotificationSettingsService.getSettings(id));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение актуальных настроек уведомлений портала")
	@GetMapping(NOTIFICATIONS_URI + BELL_URI + TYPE_URI_PARAM + CURRENT_URI)
	public BellNotificationSettingsView findCurrentBellNotificationSettings(
			@PathVariable(TYPE_URI_PARAM_NAME) BellNotificationType notificationType
	) {
		log.info("Request for current BellNotificationSettings with '{}' BellNotificationType",
				 notificationType.name());
		return bellNotificationSettingsMapper.toView(bellNotificationSettingsService.getCurrentSettings(notificationType));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_WRITE + "')")
	@ApiOperation("Обновить актуальные настройки уведомлений портала путем добавления новых")
	@PostMapping(path = NOTIFICATIONS_URI + BELL_URI + TYPE_URI_PARAM,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public BellNotificationSettingsView createBellNotificationSettings(
			@PathVariable(TYPE_URI_PARAM_NAME) BellNotificationType notificationType,
			@Valid @RequestBody CreateBellNotificationSettingsRequest request
	) {
		log.info("Request to create new BellNotificationSettings with request: {}", request);
		return bellNotificationSettingsMapper.toView(
				bellNotificationSettingsService.createSettings(request, notificationType)
		);
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение всех типов оповещений портала")
	@GetMapping(NOTIFICATIONS_URI + BELL_URI + TYPES_URI)
	public BellNotificationTypeView findAllBellNotificationTypes() {
		log.info("Request for BellNotificationType list");
		return new BellNotificationTypeView(List.of(BellNotificationType.values()));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение актуальных настроек уведомлений портала")
	@GetMapping(NOTIFICATIONS_URI + BELL_URI + TYPE_URI_PARAM + TAGS_URI)
	public ListView<SettingsTagView> findBellNotificationTypeAllowedTags(
			@PathVariable(TYPE_URI_PARAM_NAME) BellNotificationType notificationType
	) {
		log.info("Request for allowed tags for '{}' BellNotificationType", notificationType.name());
		var tags = settingsTagService.getAllowedTagsForType(notificationType);
		return new ListView<>(tags.size(),
							  tags.stream()
								  .map(settingsTagMapper::toView)
								  .collect(Collectors.toList()));
	}

//--STORY-SETTINGS----------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение списка настроек историй с сортировкой и фильтрацией")
	@GetMapping(STORIES_URI)
	public ListView<StorySettingsView> findAllStorySettings(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "creationDate,desc") String sort,
			@RequestParam(required = false) String search) {
		log.info("Request for StorySettings list");
		var criteria = getCriteria(search);
		var storySettings = storySettingsService.getAllSettings(page, size, sort, criteria);
		return new ListView<>(storySettings.getTotalElements(),
							  storySettings.getContent()
										   .stream()
										   .map(storySettingsMapper::toView)
										   .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение настроек историй по ID")
	@GetMapping(STORIES_URI + ID_URI_PARAM)
	public StorySettingsView findOneStorySettings(@PathVariable(ID_URI_PARAM_NAME) UUID id) {
		log.info("Request for StorySettings with id '{}'", id);
		return storySettingsMapper.toView(storySettingsService.getSettings(id));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_READ + "')")
	@ApiOperation("Получение актуальных настроек историй для конкретной группы")
	@GetMapping(STORIES_URI + TYPE_URI_PARAM + CURRENT_URI)
	public StorySettingsView findCurrentStorySettings(@PathVariable(TYPE_URI_PARAM_NAME) StoryGroupType storyGroupType) {
		log.info("Request for current StorySettings with '{}' StoryGroupType", storyGroupType);
		return storySettingsMapper.toView(storySettingsService.getCurrentSettings(storyGroupType));
	}

	@PreAuthorize("hasAuthority('" + SETTINGS_WRITE + "')")
	@ApiOperation("Обновить актуальные настройки историй путем добавления новых")
	@PostMapping(path = STORIES_URI + TYPE_URI_PARAM,
				 consumes = MediaType.APPLICATION_JSON_VALUE)
	public StorySettingsView createStorySettings(@PathVariable StoryGroupType storyGroupType,
												 @Valid @RequestBody CreateStorySettingsRequest request) {
		log.info("Request to create new StorySettings with request: {}", request);
		return storySettingsMapper.toView(storySettingsService.createSettings(request, storyGroupType));
	}

}
