package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.loylabs.portal.mapper.SkillMapper;
import ru.loylabs.portal.presentation.request.CreateSkillRequest;
import ru.loylabs.portal.presentation.request.UpdateSkillRequest;
import ru.loylabs.portal.presentation.view.SkillView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.SkillService;

import javax.validation.Valid;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.SKILL_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.SKILL_WRITE;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + SKILL_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class SkillController {

	private final SkillService skillService;
	private final SkillMapper skillMapper;

//--SKILLS------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + SKILL_READ + "')")
	@ApiOperation("Получение всех навыков с сортировкой и фильтрацией")
	@GetMapping
	public ListView<SkillView> findAllSkill(@RequestParam(required = false, defaultValue = "0") int page,
											 @RequestParam(required = false, defaultValue = "10") int size,
											 @RequestParam(required = false, defaultValue = "name,asc") String sort,
											 @RequestParam(required = false) String search) {
		log.info("Request for Skill list");
		var criteria = getCriteria(search);
		var skills = skillService.getAllSkill(page, size, sort, criteria);
		return new ListView<>(skills.getTotalElements(),
									skills.getContent()
									.stream()
									.map(skillMapper::toView)
									.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + SKILL_READ + "')")
	@ApiOperation("Получение навыка по ID")
	@GetMapping(SKILL_ID_URI_PARAM)
	public SkillView findOneSkill(@PathVariable(SKILL_ID_URI_PARAM_NAME) UUID skillId) {
		log.info("Request for Skill with id '{}'", skillId);
		return skillMapper.toView(skillService.getSkillById(skillId));
	}

	@PreAuthorize("hasAuthority('" + SKILL_WRITE + "')")
	@ApiOperation("Создание навыка")
	@PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
	public SkillView createSkill(@Valid @RequestBody CreateSkillRequest request) {
		log.info("Request to create new Skill with request: {}", request);
		return skillMapper.toView(skillService.createSkill(request));
	}

	@PreAuthorize("hasAuthority('" + SKILL_WRITE + "')")
	@ApiOperation("Редактирование навыка")
	@PutMapping(path = SKILL_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public SkillView updateSkill(@PathVariable(SKILL_ID_URI_PARAM_NAME) UUID skillId,
								 @Valid @RequestBody UpdateSkillRequest request) {
		log.info("Request to update Skill with id '{}' with request: {}", skillId, request);
		return skillMapper.toView(skillService.updateSkill(skillId, request));
	}

	@PreAuthorize("hasAuthority('" + SKILL_WRITE + "')")
	@ApiOperation("Удаление навыка")
	@DeleteMapping(SKILL_ID_URI_PARAM)
	public SuccessResponse deleteSkill(@PathVariable(SKILL_ID_URI_PARAM_NAME) UUID skillId) {
		log.info("Request to delete Skill with id: {}", skillId);
		skillService.deleteSkill(skillId);
		return new SuccessResponse();
	}

}
