package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;
import ru.loylabs.portal.mapper.StatusMapper;
import ru.loylabs.portal.presentation.request.CreateStatusRequest;
import ru.loylabs.portal.presentation.request.UpdateStatusRequest;
import ru.loylabs.portal.presentation.view.StatusView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.StatusService;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.presentation.WebUtils.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.STATUS_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.STATUS_WRITE;

@RestController
@RequestMapping(produces = MediaType.APPLICATION_JSON_VALUE,
        path = VERSION_1_URI + STATUS_URI)
@RequiredArgsConstructor
@Slf4j
public class StatusController {

    private final StatusMapper statusMapper;
    private final StatusService service;

    @ApiOperation("Получение статуса по id")
    @PreAuthorize("hasAuthority('" + STATUS_READ + "')")
    @GetMapping(STATUS_ID_URI_PARAM)
    public StatusView getStatus(@PathVariable(STATUS_ID_URI_PARAM_NAME) UUID id) {
        log.info("Request to getting Status by id {}", id);
        return statusMapper.toView(service.getStatus(id));
    }

    @ApiOperation("Получение статуса по имени")
    @PreAuthorize("hasAuthority('" + STATUS_READ + "')")
    @GetMapping(STATUS_NAME_URI_PARAM)
    public StatusView getStatus(@NotBlank @PathVariable(STATUS_NAME_URI_PARAM_NAME) String name) {
        log.info("Request to getting Status by name {}", name);
        return statusMapper.toView(service.getStatus(name));
    }

    @ApiOperation("Получение всех статусов")
    @PreAuthorize("hasAuthority('" + STATUS_READ + "')")
    @GetMapping()
    public ListView<StatusView> getAllStatuses(@RequestParam(required = false, defaultValue = "0") int page,
                                               @RequestParam(required = false, defaultValue = "10") int size,
                                               @RequestParam(required = false, defaultValue = "name,asc") String sort,
                                               @RequestParam(required = false) String search) {
        log.info("Request to getting list of Statuses with sorting and filters");
        var criteria = getCriteria(search);
        var statuses = service.getAllStatuses(page, size, sort, criteria);
        return new ListView<>(statuses.getTotalElements(),
                statuses.getContent()
                        .stream()
                        .map(statusMapper::toView)
                        .collect(Collectors.toList()));
    }

    @ApiOperation("Создание нового статуса")
    @ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
            dataType = "ru.loylabs.portal.presentation.request.CreateStatusRequest", paramType = "body")
    @PreAuthorize("hasAuthority('" + STATUS_WRITE + "')")
    @PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
    public StatusView createStatus(@Valid @RequestBody CreateStatusRequest request) {
        log.info("Request to creating new Status with name {}", request.getName());
        return statusMapper.toView(service.createStatus(statusMapper.toEntity(request)));
    }

    @ApiOperation("Обновление статуса по ID")
    @PreAuthorize("hasAuthority('" + STATUS_WRITE + "')")
    @PutMapping(path =STATUS_ID_URI_PARAM, consumes = MediaType.APPLICATION_JSON_VALUE)
    public StatusView updateStatus(@PathVariable(STATUS_ID_URI_PARAM_NAME) UUID id,
                             @Valid @RequestBody UpdateStatusRequest request) {
        log.info("Request to update Status with id {}", id);
        return statusMapper.toView(service.updateStatus(id, request));
    }

    @ApiOperation("Удаление статуса по ID")
    @PreAuthorize("hasAuthority('" + STATUS_WRITE + "')")
    @DeleteMapping(STATUS_ID_URI_PARAM)
    public SuccessResponse deleteStatus(@PathVariable(STATUS_ID_URI_PARAM_NAME) UUID id) {
        log.info("Request to delete Status by id {}", id);
        service.deleteStatus(id);
        return new SuccessResponse();
    }
}
