package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RequestPart;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.mapper.StoryGroupMapper;
import ru.loylabs.portal.mapper.StoryMapper;
import ru.loylabs.portal.presentation.request.CreateStoryGroupRequest;
import ru.loylabs.portal.presentation.request.CreateStoryRequest;
import ru.loylabs.portal.presentation.request.UpdateStoryGroupRequest;
import ru.loylabs.portal.presentation.request.UpdateStoryRequest;
import ru.loylabs.portal.presentation.view.StoryGroupView;
import ru.loylabs.portal.presentation.view.StoryView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.StoryGroupService;
import ru.loylabs.portal.service.interfaces.StoryService;

import javax.validation.Valid;
import java.util.Arrays;
import java.util.Collections;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.ATTACHMENTS_URI;
import static ru.loylabs.portal.config.URIConstants.STORIES_URI;
import static ru.loylabs.portal.config.URIConstants.STORY_GROUPS_URI;
import static ru.loylabs.portal.config.URIConstants.STORY_GROUP_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.STORY_GROUP_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.STORY_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.STORY_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.STORY_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.STORY_WRITE;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + STORY_GROUPS_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class StoryController {

	private final StoryGroupService storyGroupService;
	private final StoryGroupMapper storyGroupMapper;

	private final StoryService storyService;
	private final StoryMapper storyMapper;

//--STORY-GROUPS------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + STORY_READ + "')")
	@ApiOperation("Получение списка групп историй с сортировкой и фильтрацией")
	@GetMapping
	public ListView<StoryGroupView> findAllStoryGroups(
			@RequestParam(required = false, defaultValue = "0") int page,
			@RequestParam(required = false, defaultValue = "10") int size,
			@RequestParam(required = false, defaultValue = "pinned,updateTime,desc") String sort,
			@RequestParam(required = false) String search
	) {
		log.info("Request for StoryGroup list");
		var criteria = getCriteria(search);
		var storyGroups = storyGroupService.getAllStoryGroups(page, size, sort, criteria);
		return new ListView<>(storyGroups.getTotalElements(),
							  storyGroups.getContent()
										 .stream()
										 .map(storyGroupMapper::toView)
										 .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + STORY_READ + "')")
	@ApiOperation("Получение группы историй по ID")
	@GetMapping(STORY_GROUP_ID_URI_PARAM)
	public StoryGroupView findOneStoryGroup(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId) {
		log.info("Request for StoryGroup with id '{}'", storyGroupId);
		return storyGroupMapper.toView(storyGroupService.getStoryGroup(storyGroupId));
	}

	@PreAuthorize("hasAuthority('" + STORY_WRITE + "')")
	@ApiOperation("Создание группы историй")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
					  dataType = "ru.loylabs.portal.presentation.request.CreateStoryGroupRequest", paramType = "body")
	@PostMapping(consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public StoryGroupView createStoryGroup(@Valid @RequestPart CreateStoryGroupRequest request,
										   @RequestPart MultipartFile[] files) {
		log.info("Request to create new StoryGroup with request: {}", request);
		return storyGroupMapper.toView(
				storyGroupService.saveStoryGroup(
						request,
						getCredential().getPerson(),
						request.getRequests(),
						Optional.ofNullable(files).map(Arrays::asList).orElse(Collections.emptyList()))
		);
	}

	@PreAuthorize("hasAuthority('" + STORY_WRITE + "')")
	@ApiOperation("Обновление группы историй по ID")
	@PutMapping(path = STORY_GROUP_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public StoryGroupView updateStoryGroup(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId,
										   @Valid @RequestBody UpdateStoryGroupRequest request) {
		log.info("Request to update StoryGroup with id '{}' with request: {}", storyGroupId, request);
		return storyGroupMapper.toView(storyGroupService.updateStoryGroup(request, storyGroupId));
	}

	@PreAuthorize("hasAuthority('" + STORY_WRITE + "')")
	@ApiOperation("Удаление группы историй по ID")
	@DeleteMapping(STORY_GROUP_ID_URI_PARAM)
	public SuccessResponse deleteStoryGroup(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID id) {
		log.info("Request for deleting story group by id: {}", id);
		storyGroupService.deleteStoryGroup(id);
		return new SuccessResponse();
	}

//--STORIES-----------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + STORY_READ + "')")
	@ApiOperation("Получение списка историй из группы с сортировкой и фильтрацией")
	@GetMapping(STORY_GROUP_ID_URI_PARAM + STORIES_URI)
	public ListView<StoryView> findAllStories(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId,
											  @RequestParam(required = false, defaultValue = "0") int page,
											  @RequestParam(required = false, defaultValue = "10") int size,
											  @RequestParam(required = false, defaultValue = "priority,asc") String sort,
											  @RequestParam(required = false) String search) {
		log.info("Request for Story list of StoryGroup with id '{}'", storyGroupId);
		var criteria = getCriteria(search);
		var stories = storyService.getAllStoriesFromGroup(page, size, sort, criteria, storyGroupId);
		return new ListView<>(stories.getTotalElements(),
							  stories.getContent()
									 .stream()
									 .map(storyMapper::toView)
									 .collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + STORY_READ + "')")
	@ApiOperation("Получение истории по ID")
	@GetMapping(STORY_GROUP_ID_URI_PARAM + STORIES_URI + STORY_ID_URI_PARAM)
	public StoryView findOneStory(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId,
								  @PathVariable(STORY_ID_URI_PARAM_NAME) UUID storyId) {
		log.info("Request for Story with id '{}' from StoryGroup with id '{}'", storyId, storyGroupId);
		return storyMapper.toView(storyService.getStory(storyGroupId, storyId));
	}

	@PreAuthorize("hasAuthority('" + STORY_WRITE + "')")
	@ApiOperation("Создание истории в группе")
	@ApiImplicitParam(name = "request", value = "Use it in a form with type application/json", required = true,
					  dataType = "ru.loylabs.portal.presentation.request.CreateStoryRequest", paramType = "body")
	@PostMapping(path = STORY_GROUP_ID_URI_PARAM + STORIES_URI,
				 consumes = { MediaType.APPLICATION_JSON_VALUE, MediaType.MULTIPART_FORM_DATA_VALUE })
	public StoryView createStory(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId,
								 @Valid @RequestPart CreateStoryRequest request,
								 @RequestPart MultipartFile file) {
		log.info("Request to create new Story for StoryGroup with id '{}' with request: {}", storyGroupId, request);
		return storyMapper.toView(storyService.saveStory(request, storyGroupId, getCredential().getPerson(), file));
	}

	@PreAuthorize("hasAuthority('" + STORY_WRITE + "')")
	@ApiOperation("Обновление истории в группе по ID")
	@PutMapping(path = STORY_GROUP_ID_URI_PARAM + STORIES_URI + STORY_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public StoryView updateStory(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId,
								 @PathVariable(STORY_ID_URI_PARAM_NAME) UUID storyId,
								 @Valid @RequestBody UpdateStoryRequest request) {
		log.info("Request to update Story with id '{}' from StoryGroup with id '{}' with request: {}",
				 storyId, storyGroupId, request);
		return storyMapper.toView(storyService.updateStory(request, storyGroupId, storyId));
	}

	@PreAuthorize("hasAuthority('" + STORY_WRITE + "')")
	@ApiOperation("Удаление истории из группы по ID")
	@DeleteMapping(STORY_GROUP_ID_URI_PARAM + STORIES_URI + STORY_ID_URI_PARAM)
	public SuccessResponse deleteStory(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId,
									   @PathVariable(STORY_ID_URI_PARAM_NAME) UUID storyId) {
		log.info("Request to delete Story with id '{}' from StoryGroup with id '{}'", storyId, storyGroupId);
		storyService.deleteStory(storyGroupId, storyId);
		return new SuccessResponse();
	}

//--STORY-ATTACHMENTS-------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + STORY_WRITE + "') && @sec.isPrincipalRecordAuthor(#storyId)")
	@ApiOperation("Замена вложения истории по ID")
	@PutMapping(path = STORY_GROUP_ID_URI_PARAM + STORIES_URI + STORY_ID_URI_PARAM + ATTACHMENTS_URI,
				consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
	public StoryView updateStoryAttachment(@PathVariable(STORY_GROUP_ID_URI_PARAM_NAME) UUID storyGroupId,
										   @PathVariable(STORY_ID_URI_PARAM_NAME) UUID storyId,
										   @RequestPart(name = "file") MultipartFile file) {
		log.info("Request to update attachment for Story with id '{}' from StoryGroup with id '{}'",
				 storyId, storyGroupId);
		return storyMapper.toView(storyService.replaceStoryAttachment(storyGroupId, storyId, file));
	}

}
