package ru.loylabs.portal.presentation.controller;

import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.MediaType;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import ru.loylabs.portal.mapper.TitleMapper;
import ru.loylabs.portal.presentation.request.CreateTitleRequest;
import ru.loylabs.portal.presentation.request.UpdateTitleRequest;
import ru.loylabs.portal.presentation.view.TitleView;
import ru.loylabs.portal.presentation.view.common.ListView;
import ru.loylabs.portal.presentation.view.common.SuccessResponse;
import ru.loylabs.portal.service.interfaces.TitleService;

import javax.validation.Valid;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.config.URIConstants.TITLES_URI;
import static ru.loylabs.portal.config.URIConstants.TITLE_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.TITLE_ID_URI_PARAM_NAME;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.TITLE_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.TITLE_WRITE;
import static ru.loylabs.portal.presentation.WebUtils.getCriteria;

@Slf4j
@RequiredArgsConstructor
@RestController
@RequestMapping(path = VERSION_1_URI + TITLES_URI,
				produces = MediaType.APPLICATION_JSON_VALUE)
public class TitleController {

	private final TitleService titleService;
	private final TitleMapper titleMapper;

//--TITLES------------------------------------------------------------------------------------------------------------//

	@PreAuthorize("hasAuthority('" + TITLE_READ + "')")
	@ApiOperation("Получение всех должностей с сортировкой и фильтрацией")
	@GetMapping
	public ListView<TitleView> findAllTitles(@RequestParam(required = false, defaultValue = "0") int page,
											 @RequestParam(required = false, defaultValue = "10") int size,
											 @RequestParam(required = false, defaultValue = "name,asc") String sort,
											 @RequestParam(required = false) String search) {
		log.info("Request for Title list");
		var criteria = getCriteria(search);
		var titles = titleService.getAllTitle(page, size, sort, criteria);
		return new ListView<>(titles.getTotalElements(),
							  titles.getContent()
									.stream()
									.map(titleMapper::toView)
									.collect(Collectors.toList()));
	}

	@PreAuthorize("hasAuthority('" + TITLE_READ + "')")
	@ApiOperation("Получение должности по ID")
	@GetMapping(TITLE_ID_URI_PARAM)
	public TitleView findOneTitle(@PathVariable(TITLE_ID_URI_PARAM_NAME) UUID titleId) {
		log.info("Request for Title with id '{}'", titleId);
		return titleMapper.toView(titleService.getTitleById(titleId));
	}

	@PreAuthorize("hasAuthority('" + TITLE_WRITE + "')")
	@ApiOperation("Создание должности")
	@PostMapping(consumes = MediaType.APPLICATION_JSON_VALUE)
	public TitleView createTitle(@Valid @RequestBody CreateTitleRequest request) {
		log.info("Request to create new Title with request: {}", request);
		return titleMapper.toView(titleService.createTitle(request));
	}

	@PreAuthorize("hasAuthority('" + TITLE_WRITE + "')")
	@ApiOperation("Редактирование должности")
	@PutMapping(path = TITLE_ID_URI_PARAM,
				consumes = MediaType.APPLICATION_JSON_VALUE)
	public TitleView updateTitle(@PathVariable(TITLE_ID_URI_PARAM_NAME) UUID titleId,
								 @Valid @RequestBody UpdateTitleRequest request) {
		log.info("Request to update Title with id '{}' with request: {}", titleId, request);
		return titleMapper.toView(titleService.updateTitle(titleId, request));
	}

	@PreAuthorize("hasAuthority('" + TITLE_WRITE + "')")
	@ApiOperation("Удаление должности")
	@DeleteMapping(TITLE_ID_URI_PARAM)
	public SuccessResponse deleteTitle(@PathVariable(TITLE_ID_URI_PARAM_NAME) UUID titleId) {
		log.info("Request to delete Title with id: {}", titleId);
		titleService.deleteTitle(titleId);
		return new SuccessResponse();
	}

}
