package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.Achievement;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.TaskStatus;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.AchievementEvents;
import ru.loylabs.portal.mapper.AchievementMapper;
import ru.loylabs.portal.presentation.request.CreateAchievementRequest;
import ru.loylabs.portal.presentation.request.UpdateAchievementRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.AchievementRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.TaskRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.AchievementService;
import ru.loylabs.portal.service.interfaces.FilterUtils;

import javax.transaction.Transactional;
import java.time.Instant;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.enums.AchievementStatus.ARCHIVED;
import static ru.loylabs.portal.enums.AchievementStatus.DONE;
import static ru.loylabs.portal.enums.AchievementStatus.NEW;
import static ru.loylabs.portal.exception.BaseException.Exception.ACHIEVEMENT_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.ACHIEVEMENT_NOT_UPDATABLE;
import static ru.loylabs.portal.exception.BaseException.Exception.ACHIEVEMENT_NO_ARCHIVED;
import static ru.loylabs.portal.exception.BaseException.Exception.ACHIEVEMENT_WRONG_STATUS;
import static ru.loylabs.portal.exception.BaseException.Exception.ACHIEVEMENT_WRONG_SUPERVISOR;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;

@Service
@RequiredArgsConstructor
public class AchievementServiceImpl extends AbstractFilteringAndSortingService<Achievement> implements AchievementService {

	private final FilterUtils filterUtils;
	private final TaskRepository taskRepository;
	private final PersonRepository personRepository;
	private final AchievementMapper achievementMapper;
	private final AchievementRepository achievementRepository;
	private final ApplicationEventPublisher publisher;

	@Override
	public Page<Achievement> getAllAchievements(int page, int size, String sort,
												SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Achievement.class);
		return achievementRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Page<Achievement> getAllPersonalAchievements(int page, int size, String sort,
														SearchReversePolishNotation criteria, UUID ownerId) {
		criteria.add(SearchCriteria.builder().key("owner.id").operation(":").value(ownerId).build());
		return getAllAchievements(page, size, sort, criteria);
	}

	@Override
	public Achievement getAchievement(UUID achievementId) {
		return achievementRepository.findById(achievementId)
				.orElseThrow(() -> new ApplicationException(ACHIEVEMENT_NOT_FOUND, achievementId));
	}

	@Override
	public Achievement createAchievement(CreateAchievementRequest request, Person author) {
		var owner = getPerson(request.getOwnerId());
		if (request.getObserverIds().contains(request.getOwnerId()))
			throw new ApplicationException(ACHIEVEMENT_WRONG_SUPERVISOR);
		var observers = getObserversFromRequest(request);
		Achievement achievement = achievementMapper.toEntity(request, author, owner, observers, Instant.now());
		achievement = achievementRepository.save(achievement);
		publisher.publishEvent(new AchievementEvents.AchievementCreateEvent(achievement.getId()));
		return achievement;
	}

	@Override
	public Achievement updateAchievement(UUID achievementId, UpdateAchievementRequest request) {
		Achievement achievement = achievementMapper.toEntity(getAchievement(achievementId), request);
		if (achievementHaveTaskWithStatus(achievement, TaskStatus.DONE))
			throw new ApplicationException(ACHIEVEMENT_NOT_UPDATABLE, achievementId);
		return achievementRepository.save(achievement);
	}


	@Override
	public Achievement confirmAchievement(UUID achievementId, Person confirming) {
		var achievement = getAchievement(achievementId);
		if (achievement.getStatus() == DONE)
			throw new ApplicationException(ACHIEVEMENT_WRONG_STATUS, achievementId, DONE);
		achievement.setStatus(DONE);
		achievement = achievementRepository.save(achievement);
		publisher.publishEvent(AchievementEvents.AchievementConfirmEvent.of(achievementId,confirming.getId()));
		return achievement;
	}

	@Override
	public Achievement archiveAchievements(UUID achievementId) {
		Achievement achievement = getAchievement(achievementId);
		if (achievement.getStatus() != DONE)
			throw new ApplicationException(ACHIEVEMENT_WRONG_STATUS, achievementId, achievement.getStatus());
		achievement.setStatus(ARCHIVED);
		return achievementRepository.save(achievement);
	}

	@Override
	public Achievement setAchievementStatusNew(UUID achievementId) {
		Achievement achievement = getAchievement(achievementId);
		if (achievement.getStatus() == NEW)
			throw new ApplicationException(ACHIEVEMENT_WRONG_STATUS, achievementId, ARCHIVED);
		achievement.setStatus(NEW);
		return achievementRepository.save(achievement);
	}


	@Override
	@Transactional
	public void deleteAchievement(UUID achievementId) {
		publisher.publishEvent(AchievementEvents.AchievementDeleteEvent.of(getAchievement(achievementId)));
		if (achievementRepository.deleteAchievementById(achievementId) == 0)
			throw new ApplicationException(ACHIEVEMENT_NOT_FOUND, achievementId);
	}

	@Override
	@Transactional
	public void deleteArchivedAchievementForPerson(UUID personId) {
		achievementRepository.findAllByStatusAndOwnerId(ARCHIVED, personId)
				.forEach(achievement -> publisher.publishEvent(AchievementEvents.AchievementDeleteEvent.of(achievement)));
		if (achievementRepository.deleteByOwnerIdAndStatus(personId, ARCHIVED) == 0)
			throw new ApplicationException(ACHIEVEMENT_NO_ARCHIVED, personId);
	}

//--------------------------------------------------------------------------------------------------------------------//

	private boolean achievementHaveTaskWithStatus(Achievement achievement, TaskStatus status) {
		return taskRepository.existsByAchievementAndStatus(achievement, status);
	}

	private Person getPerson(UUID id) {
		return personRepository.findById(id)
				.orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, id));
	}

	private Set<Person> getObserversFromRequest(CreateAchievementRequest request) {
		return request.getObserverIds().stream()
				.map(this::getPerson)
				.collect(Collectors.toSet());
	}

}