package ru.loylabs.portal.service;

import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import net.bytebuddy.utility.RandomString;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.config.jwt.JwtProvider;
import ru.loylabs.portal.config.jwt.JwtRequest;
import ru.loylabs.portal.config.jwt.JwtResponse;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.exception.ForbiddenException;
import ru.loylabs.portal.listener.events.PersonEvents;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.redis.JwtRefreshTokenRepository;
import ru.loylabs.portal.repository.redis.LoginTokenRepository;
import ru.loylabs.portal.repository.redis.PasswordEntryAttemptRepository;
import ru.loylabs.portal.repository.redis.PasswordResetTokenRepository;
import ru.loylabs.portal.service.interfaces.AuthService;
import ru.loylabs.portal.service.interfaces.EmailNotificationFactory;
import ru.loylabs.portal.service.interfaces.NotificationSender;
import ru.loylabs.portal.service.notification.email.EmailNotification;

import java.util.List;

import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_BANNED;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.WRONG_CREDENTIAL;
import static ru.loylabs.portal.exception.BaseException.Exception.WRONG_PASSWORD;
import static ru.loylabs.portal.exception.BaseException.Exception.WRONG_REFRESH_TOKEN;

@Service
@RequiredArgsConstructor
public class AuthServiceImpl implements AuthService {

	private final JwtProvider jwtProvider;
	private final PersonRepository personRepository;
	private final PasswordEncoder passwordEncoder;
	private final JwtRefreshTokenRepository refreshRepository;
	private final LoginTokenRepository loginTokenRepository;
	private final PasswordEntryAttemptRepository passwordEntryAttemptRepository;
	private final ApplicationEventPublisher publisher;
	private final PasswordResetTokenRepository passwordRepository;
	private final EmailNotificationFactory emailNotificationFactory;
	@Qualifier("emailNotificationSenderImpl")
	private final NotificationSender<EmailNotification> emailNotificationSender;

	@Value("${password.max-attempts}")
	private Integer maxCountAttempt;

	@SneakyThrows
	@Override
	public JwtResponse login(@NonNull JwtRequest authRequest) {
		final Person person = getByUsername(authRequest.getLogin());

		if (person.isBlocked()) throw new ForbiddenException(PERSON_BANNED, person.getId());

		if (passwordEncoder.matches(authRequest.getPassword(), person.getPassword())) {
			final String accessToken = jwtProvider.generateAccessToken(person);
			final String refreshToken = jwtProvider.generateRefreshToken(person);
			refreshRepository.save(person.getId(), refreshToken); // TODO comment for auth if redis not installed locally
			passwordEntryAttemptRepository.delete(person.getId().toString());
			return new JwtResponse(person.getId(),
								   person.getUsername(),
								   person.getRole().getRole(),
								   accessToken,
								   refreshToken
			);
		} else {
			var countAttempts = checkPasswordEntryAttempts(person);
			throw new ApplicationException(WRONG_PASSWORD, countAttempts);
		}
	}

	@Override
	public JwtResponse loginViaToken(String loginToken) {
		Person person = loginTokenRepository.get(loginToken)
											.flatMap(personRepository::findById)
											.orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND));

		if (person.isBlocked()) throw new ForbiddenException(PERSON_BANNED, person.getId());

		String accessToken = jwtProvider.generateAccessToken(person);
		String refreshToken = jwtProvider.generateRefreshToken(person);
		refreshRepository.save(person.getId(), refreshToken);
		loginTokenRepository.delete(loginToken);

		return new JwtResponse(person.getId(),
							   person.getUsername(),
							   person.getRole().getRole(),
							   accessToken,
							   refreshToken
		);
	}

	@Override
	public JwtResponse refresh(@NonNull String refreshToken) {
		if (jwtProvider.validateRefreshToken(refreshToken)) {
			final Person person = getByUsername(jwtProvider.getRefreshClaims(refreshToken).getSubject());
			final String saveRefreshToken = refreshRepository.get(person.getId());

			if (saveRefreshToken != null && saveRefreshToken.equals(refreshToken)) {
				final String accessToken = jwtProvider.generateAccessToken(person);
				return new JwtResponse(null, null, null, accessToken, null);
			}
		}
		throw new ApplicationException(WRONG_REFRESH_TOKEN);
	}

	@SneakyThrows
	@Override
	public JwtResponse updateRefreshToken(@NonNull String refreshToken) {
		if (jwtProvider.validateRefreshToken(refreshToken)) {
			final Person person = getByUsername(jwtProvider.getRefreshClaims(refreshToken).getSubject());
			final String saveRefreshToken = refreshRepository.get(person.getId());

			if (saveRefreshToken != null && saveRefreshToken.equals(refreshToken)) {
				final String accessToken = jwtProvider.generateAccessToken(person);
				final String newRefreshToken = jwtProvider.generateRefreshToken(person);

				refreshRepository.save(person.getId(), newRefreshToken);
				return new JwtResponse(null, null, null, accessToken, newRefreshToken);
			}
		}
		throw new ApplicationException(WRONG_REFRESH_TOKEN);
	}

	private Person getByUsername(String username) {
		return personRepository.findByUsername(username).orElseThrow(() -> new ApplicationException(WRONG_CREDENTIAL));
	}

	private Integer checkPasswordEntryAttempts (Person person) {
		var idPerson = person.getId();
		var countAttempts = passwordEntryAttemptRepository.get(idPerson.toString()).orElse(0);
		passwordEntryAttemptRepository.update(idPerson.toString(), ++countAttempts);
		if (maxCountAttempt <= countAttempts) {
			publisher.publishEvent(PersonEvents.PersonBlockedEvent.of(idPerson));
			sendMessageBasedOnUserRole(person);
		}
		return maxCountAttempt-countAttempts;
	}

	private void sendMessageBasedOnUserRole(Person person) {
		if (person.getRole().getRole().equals("ADMIN")) {
			var token = RandomString.make(45);
			emailNotificationSender.send(emailNotificationFactory.createBlockedAdmin(person, token));
			passwordRepository.save(token, person.getId());
		} else {
			var allAdmin = personRepository.findAllByRoleRoleIn(List.of("ADMIN"));
			for(Person admin: allAdmin) {
				emailNotificationSender.send(emailNotificationFactory.createBlockedUser(admin ,person));
			}
		}
	}
}
