package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.BugReport;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.enums.BugState;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.exception.BaseException;
import ru.loylabs.portal.mapper.BugReportMapper;
import ru.loylabs.portal.presentation.request.CreateRecordRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.BugReportRepository;
import ru.loylabs.portal.repository.RecordRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.BugService;
import ru.loylabs.portal.service.interfaces.EmailNotificationFactory;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.NotificationSender;
import ru.loylabs.portal.service.interfaces.RecordAdapter;
import ru.loylabs.portal.service.notification.email.EmailNotification;

import java.util.Collection;
import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.BUG_REPORT_NOT_FOUND;

@Service
@Transactional(readOnly = true)
@RequiredArgsConstructor
public class BugServiceImpl extends AbstractFilteringAndSortingService<Record> implements BugService {

	private final RecordAdapter recordAdapter;
	private final RecordRepository recordRepository;
	private final FilterUtils filterUtils;
	private final BugReportRepository bugReportRepository;
	private final BugReportMapper bugReportMapper;
	private final EmailNotificationFactory emailNotificationFactory;
	@Qualifier("emailNotificationSenderImpl")
	private final NotificationSender<EmailNotification> emailNotificationSender;

	@Override
	public BugReport getBugReport(UUID id) {
		return bugReportRepository.findById(id)
								  .orElseThrow(() -> new ApplicationException(BaseException.Exception.BUG_REPORT_NOT_FOUND, id));
	}

	@Override
	public Page<BugReport> getAllBugReportRecord(Integer page, Integer size, String sort
			, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, BugReport.class);
		return bugReportRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}


	@Override
	@Transactional
	public BugReport saveBugReportRecord(CreateRecordRequest request, Person author, Collection<MultipartFile> files) {
		return bugReportRepository.save(bugReportMapper.toEntity(
				recordAdapter.createRecord(request, author, null, RecordType.BUG_REPORT, files)
				, BugState.NEW));
	}

	@Override
	public void sendBugReport(CreateRecordRequest request, Person author, Collection<MultipartFile> files) {
		var bug = bugReportMapper.toEntity(
				recordAdapter.createRecord(request, author, null, RecordType.BUG_REPORT, files)
				, BugState.NEW);

		emailNotificationSender.send(emailNotificationFactory.createBugReportMessage(bug, files));
	}

	@Override
	@Transactional
	public BugReport addAttachments(UUID id, Collection<MultipartFile> files) {
		return recordAdapter.addAttachments(getBugReport(id), files);
	}

	@Override
	@Transactional
	public BugReport deleteAttachments(UUID id, DeleteAttachmentsRequest request) {
		return recordAdapter.deleteAttachments(getBugReport(id), request);
	}

	@Override
	@Transactional
	public void deleteBugReportRecord(UUID id) {
		bugReportRepository.findById(id).orElseThrow(() -> new ApplicationException(BUG_REPORT_NOT_FOUND, id));
		bugReportRepository.deleteById(id);
	}

	@Override
	@Transactional
	public BugReport changeStatus(UUID id, BugState bugState) {
		var bugReport = getBugReport(id);
		bugReport.setBugState(bugState);
		return bugReportRepository.save(bugReport);
	}

	@Override
	public long countComments(UUID parentRecordId) {
		return recordRepository.countByParentRecordId(parentRecordId);
	}

}
