package ru.loylabs.portal.service;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import net.minidev.json.JSONObject;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.client.RestTemplate;
import ru.loylabs.portal.entity.CorporateCalendarEvent;
import ru.loylabs.portal.entity.HolidayData;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.CalendarEventType;
import ru.loylabs.portal.repository.CorporateCalendarEventRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.interfaces.CalendarSchedulingService;

import java.time.LocalDate;
import java.time.Year;
import java.time.ZoneId;
import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.stream.Collectors;

import static java.time.LocalDate.now;
import static java.time.ZoneOffset.UTC;

@Component
@Slf4j
@RequiredArgsConstructor
public class CalendarSchedulingServiceImpl implements CalendarSchedulingService {

    private static final String JSON_ATTRIBUTE_LIST_OF_DAYS = "days";
    private static final int MONTH_OF_CALENDAR_APPROVAL = 10;
    private final PersonRepository personRepository;
    private final CorporateCalendarEventRepository calendarRepository;
    private final ObjectMapper mapper;
    @Value("${uri.production-calendar}")
    private String productionCalendarUri;

    @Override
    @Transactional
    public int addBirthdayEvent() {
        List<Person> persons = personRepository.findAll();
        var events = persons.stream()
                .filter(p -> !p.isBlocked())
                .filter(p -> p.getDob() != null)
                .filter(p -> {
                    var time = p.getDob().with(Year.now()).isBefore(now()) ?
                            p.getDob().withYear(now().getYear() + 1) : p.getDob().with(Year.now());
                    return !calendarRepository.existByTargetAndDate(p, time.getYear(), time.getMonthValue(), time.getDayOfMonth());
                })
                .map(p -> createBirthdayEvent(p, Year.now()))
                .collect(Collectors.toList());

        calendarRepository.saveAll(events);
        return events.size();
    }

    @Override
    @Transactional
    public int addHolidayEvent() {
        var year = LocalDate.now().getMonthValue() < MONTH_OF_CALENDAR_APPROVAL
                ? LocalDate.now().getYear()
                : LocalDate.now().getYear() + 1;

        return getHolidayData(year)
                .map(holidayData -> {
                    int deletedHoliday = calendarRepository.deleteGeneratedEventAfterStartDateTime(
                            CalendarEventType.HOLIDAY, LocalDate.of(year, 1, 1).atStartOfDay(UTC));

                    var events = holidayData.stream()
                            .filter(HolidayData::isHoliday)
                            .map(this::createCalendarEvent)
                            .collect(Collectors.toSet());
                    calendarRepository.saveAll(events);

                    return events.size() - deletedHoliday;
                })
                .orElse(0);
    }

    private CorporateCalendarEvent createCalendarEvent(HolidayData holidayData) {
        return new CorporateCalendarEvent(
                holidayData.getNote(),
                holidayData.getNote(),
                CalendarEventType.HOLIDAY,
                holidayData.getDate(),
                true
        );
    }

    private Optional<List<HolidayData>> getHolidayData(int year) {
        var restTemplate = new RestTemplate();
        ResponseEntity<JSONObject> response = null;
        try {
            response = restTemplate.getForEntity(productionCalendarUri, JSONObject.class, year);
        } catch (Exception e) {
            log.error("Error while receiving holiday data: {}", e.getMessage());
        }
        return Objects.nonNull(response) && Objects.nonNull(response.getBody())
                ? mapper.convertValue(response.getBody().get(JSON_ATTRIBUTE_LIST_OF_DAYS), new TypeReference<>() {})
                : Optional.empty();
    }

    @Override
    @Transactional
    public int cleanOutdatedBirthdayEvent() {
        LocalDate yearAgo = now().minusYears(1);
        return calendarRepository.deleteByEventTypeAndStartDateTimeBefore(
                CalendarEventType.BIRTHDAY,
                yearAgo.atStartOfDay(UTC)
        );
    }

    private CorporateCalendarEvent createBirthdayEvent(Person p, Year year) {
        final LocalDate dob = p.getDob();
        final LocalDate withCurrentYear = dob.with(year).isBefore(now()) ? dob.with(year).plusYears(1) : dob.with(year);

        final String message = "%s %s празднует свой День Рождения".formatted(
                p.getName(),
                p.getSurname());
        return new CorporateCalendarEvent(
                message,
                message,
                CalendarEventType.BIRTHDAY,
                withCurrentYear.atStartOfDay(ZoneId.systemDefault()),
                withCurrentYear.atTime(23, 59, 59).atZone(ZoneId.systemDefault()),
                p);
    }
}
