package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.CommentEvents;
import ru.loylabs.portal.presentation.request.CreateRecordRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.request.UpdateRecordRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.RecordRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.CommentService;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.RecordAdapter;

import java.util.Collection;
import java.util.UUID;

import static ru.loylabs.portal.enums.RecordType.COMMENT;
import static ru.loylabs.portal.exception.BaseException.Exception.RECORD_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.RECORD_TYPE_NOT_FOUND;

@Service
@RequiredArgsConstructor
public class CommentServiceImpl extends AbstractFilteringAndSortingService<Record> implements CommentService {
	private final RecordAdapter recordAdapter;
	private final RecordRepository recordRepository;
	private final FilterUtils filterUtils;
	private final ApplicationEventPublisher publisher;

	@Override
	public Page<Record> getAllComments(Integer page, Integer size, String sort, SearchReversePolishNotation criteria, UUID parentId, RecordType parentType) {
		checkExistsParentRecord(parentId, parentType);
		criteria.add(SearchCriteria.builder().key("parentRecord.id").operation(":").value(parentId).build());
		criteria.add(SearchCriteria.builder().key("recordType").operation(":").value(COMMENT.name()).build());
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Record.class);
		return recordRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Record saveComment(CreateRecordRequest request, Person author, UUID parentId, RecordType parentType, Collection<MultipartFile> files) {
		var comment = recordRepository.save(
				recordAdapter.createRecord(
						request,
						author,
						recordRepository.findByIdAndRecordType(parentId, parentType)
								.orElseThrow(() -> new ApplicationException(RECORD_TYPE_NOT_FOUND, parentId, parentType)),
						COMMENT,
						files));
		publisher.publishEvent(CommentEvents.CommentCreateEvent.of(comment.getId()));
		return comment;
	}

	@Override
	public Record updateComment(UpdateRecordRequest request, UUID recordId, UUID parentId) {
		var record = recordRepository.findByIdAndRecordTypeAndParentRecordId(recordId, COMMENT, parentId)
				.orElseThrow(() -> new ApplicationException(RECORD_NOT_FOUND, recordId));
		return recordRepository.save(recordAdapter.updateRecord(request, record));
	}

	@Override
	public Record addAttachments(UUID id, UUID parentId, RecordType type, Collection<MultipartFile> files) {
		return recordAdapter.addAttachments(
				recordRepository.findByIdAndParentRecordIdAndParentRecordRecordType(id, parentId, type)
						.orElseThrow(() -> new ApplicationException(RECORD_TYPE_NOT_FOUND, id, type))
				, files);
	}

	@Override
	@Transactional
	public Record deleteAttachments(UUID id, UUID parentId, RecordType type, DeleteAttachmentsRequest request) {
		return recordAdapter.deleteAttachments(
				recordRepository.findByIdAndParentRecordIdAndParentRecordRecordType(id, parentId, type)
						.orElseThrow(() -> new ApplicationException(RECORD_TYPE_NOT_FOUND, id, type))
				, request);
	}

	@Override
	@Transactional
	public void deleteComment(UUID recordId, UUID parentId) {
		if (recordRepository.deleteByIdAndParentRecordId(recordId, parentId) == 0) {
			throw new ApplicationException(RECORD_NOT_FOUND, recordId);
		}
		publisher.publishEvent(CommentEvents.CommentDeleteEvent.of(recordId));
	}

	private void checkExistsParentRecord(UUID parentId, RecordType parentType) {
		if (!recordRepository.existsByIdAndRecordType(parentId, parentType))
			throw new ApplicationException(RECORD_TYPE_NOT_FOUND, parentId, parentType);
	}
}
