package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.CommunityEvents;
import ru.loylabs.portal.presentation.request.CreateRecordRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.request.UpdateRecordRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.CommunityRepository;
import ru.loylabs.portal.repository.RecordRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.CommunityRecordService;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.RecordAdapter;

import java.util.Collection;
import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.COMMUNITY_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.COMMUNITY_RECORD_NOT_FOUND;

@Service
@RequiredArgsConstructor
public class CommunityRecordServiceImpl extends AbstractFilteringAndSortingService<Record> implements CommunityRecordService {
	private final RecordRepository recordRepository;
	private final CommunityRepository communityRepository;
	private final RecordAdapter recordAdapter;
	private final FilterUtils filterUtils;
	private final ApplicationEventPublisher publisher;


	@Override
	public Record getCommunityRecord(UUID id, UUID communityId) {
		return recordRepository.findByIdAndCommunityId(id, communityId)
				.orElseThrow(() -> new ApplicationException(COMMUNITY_RECORD_NOT_FOUND, id, communityId));
	}

	@Override
	public Page<Record> getAllCommunityRecords(UUID communityId, Integer page, Integer size, String sort, SearchReversePolishNotation criteria) {
		criteria.add(SearchCriteria.builder().key("community.id").operation(":").value(communityId).build());
		criteria.add(SearchCriteria.builder().key("recordType").operation(":").value(RecordType.COMMUNITY.name()).build());
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Record.class);
		return recordRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Record sendMessage(UUID id, CreateRecordRequest request, Person author, Collection<MultipartFile> files) {
		Record record = recordAdapter.createRecord(request, author, null, RecordType.COMMUNITY, files);
		record.setCommunity(communityRepository.findById(id)
				.orElseThrow(() -> new ApplicationException(COMMUNITY_NOT_FOUND, id)));
		record = recordRepository.save(record);
		publisher.publishEvent(CommunityEvents.CommunityRecordCreateEvent.of(id,record.getId()));
		return record;
	}

	@Override
	public Record updateMessage(UUID recordId, UUID communityId, UpdateRecordRequest request) {
		var record = getCommunityRecord(recordId,communityId);
		return recordRepository.save(recordAdapter.updateRecord(request,record));
	}

	@Override
	public Record addAttachments(UUID id, UUID communityId, Collection<MultipartFile> files) {
		return recordAdapter.addAttachments(getCommunityRecord(id, communityId), files);
	}

	@Override
	public Record deleteAttachments(UUID id, UUID communityId, DeleteAttachmentsRequest request) {
		return recordAdapter.deleteAttachments(getCommunityRecord(id, communityId), request);
	}

	@Override
	@Transactional
	public void deleteMessage(UUID id, UUID communityId) {
		publisher.publishEvent(CommunityEvents.CommunityRecordDeleteEvent.of(communityId,id));
		if (recordRepository.deleteByIdAndCommunityId(id, communityId) == 0)
			throw new ApplicationException(COMMUNITY_RECORD_NOT_FOUND, id, communityId);
	}
}
