package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Community;
import ru.loylabs.portal.entity.CommunityPersonRole;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.AttachmentType;
import ru.loylabs.portal.enums.CommunityRole;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.CommunityEvents;
import ru.loylabs.portal.mapper.CommunityMapper;
import ru.loylabs.portal.mapper.CommunityPersonRoleMapper;
import ru.loylabs.portal.presentation.request.AddMembersRequest;
import ru.loylabs.portal.presentation.request.CreateCommunityRequest;
import ru.loylabs.portal.presentation.request.UpdateCommunityRequest;
import ru.loylabs.portal.presentation.request.UpdateRoleInCommunityRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.CommunityPersonRoleRepository;
import ru.loylabs.portal.repository.CommunityRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.AttachmentAdapter;
import ru.loylabs.portal.service.interfaces.CommunityService;
import ru.loylabs.portal.service.interfaces.FilterUtils;

import javax.transaction.Transactional;
import java.time.Instant;
import java.util.*;
import java.util.stream.Collectors;

import static ru.loylabs.portal.enums.CommunityRole.*;
import static ru.loylabs.portal.exception.BaseException.Exception.*;
import static ru.loylabs.portal.utility.FileAttachmentTypeFactory.getCorrespondingType;

@Service
@RequiredArgsConstructor
public class CommunityServiceImpl extends AbstractFilteringAndSortingService<Community> implements CommunityService {
	private final AttachmentAdapter attachmentAdapter;
	private final CommunityRepository communityRepository;
	private final PersonRepository personRepository;
	private final CommunityPersonRoleRepository communityPersonRoleRepository;
	private final CommunityMapper communityMapper;
	private final CommunityPersonRoleMapper communityPersonRoleMapper;
	private final FilterUtils filterUtils;
	private final ApplicationEventPublisher publisher;

	@Override
	public Community getCommunity(UUID id) {
		return communityRepository.findById(id).orElseThrow(() -> new ApplicationException(COMMUNITY_NOT_FOUND, id));
	}

	@Override
	public Page<Community> getAllCommunities(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Community.class);
		return communityRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Page<Community> getAllPublicCommunities(int page, int size, String sort, SearchReversePolishNotation criteria) {
		criteria.add(SearchCriteria.builder().key("privateCommunity").operation(":").value(false).build());
		return getAllCommunities(page, size, sort, criteria);
	}

	@Override
	public Community saveCommunity(CreateCommunityRequest request, Person owner) {
		Community community = communityRepository.save(communityMapper.toEntity(request, Instant.now()));
		communityPersonRoleRepository.save(communityPersonRoleMapper.toEntity(community, owner, OWNER));
		return community;
	}

	@Override
	public Community saveCommunity(CreateCommunityRequest request, MultipartFile file, Person owner) {
		if (!AttachmentType.IMAGE.equals(getCorrespondingType(file.getContentType())))
			throw new ApplicationException(AVATAR_WRONG_FILE_TYPE);
		Community community = communityMapper.toEntity(request, Instant.now());
		community.setAvatar(attachmentAdapter.createAttachment(file));
		communityRepository.save(community);
		communityPersonRoleRepository.save(communityPersonRoleMapper.toEntity(community, owner, OWNER));
		return community;
	}

	@Override
	public Community updateCommunity(UUID id, UpdateCommunityRequest request) {
		return communityRepository.save(communityMapper.toEntity(request, getCommunity(id)));
	}

	@Override
	@Transactional
	public void deleteCommunity(UUID id) {
		publisher.publishEvent(CommunityEvents.CommunityDeleteEvent.of(getCommunity(id)));
		if (communityRepository.deleteCommunityById(id) == 0) throw new ApplicationException(COMMUNITY_NOT_FOUND, id);
	}

	@Override
	public Community addMembers(UUID id, AddMembersRequest[] requests, Person author) {
		Community community = getCommunity(id);
		Set<CommunityPersonRole> connections = new HashSet<>();
		for (AddMembersRequest member : requests) {
			if (member.getRole() == OWNER) throw new ApplicationException(COMMUNITY_CHANGE_OF_OWNER, id);
			if (member.getRole() == null) member.setRole(MEMBER);
			var personId = member.getPersonId();
			checkUnique(community.getId(), personId);
			connections.add(communityPersonRoleMapper.toEntity(community,
					personRepository.findById(personId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, personId)),
					member.getRole()));
		}
		communityPersonRoleRepository.saveAll(connections);
		publisher.publishEvent(CommunityEvents.CommunityAddMemberEvent.of(
				id,
				author.getId(),
				Arrays.stream(requests)
						.map(AddMembersRequest::getPersonId)
						.collect(Collectors.toSet())));
		return community;
	}

	@Override
	@Transactional
	public Community changeOwner(UUID communityId, UUID personId) {
		var currentRole = getCurrentRole(communityId, personId);
		if (currentRole.getCommunityRole() != OWNER) {
			var currentOwner = communityPersonRoleRepository.findByCommunityIdAndCommunityRole(communityId, OWNER);
			currentOwner.ifPresent(owner -> {
				owner.setCommunityRole(ADMIN);
				communityPersonRoleRepository.saveAndFlush(owner);
			});
			currentRole.setCommunityRole(OWNER);
			communityPersonRoleRepository.save(currentRole);
		}
		return currentRole.getCommunity();
	}

	@Override
	public Community updatePersonRoleInCommunity(UUID communityId, UpdateRoleInCommunityRequest request) {
		var currentRole = getCurrentRole(communityId, request.getPersonId());
		if (currentRole.getCommunityRole() == OWNER || request.getRole() == OWNER)
			throw new ApplicationException(COMMUNITY_CHANGE_OF_OWNER, communityId);
		if (currentRole.getCommunityRole() != request.getRole()) {
			currentRole.setCommunityRole(request.getRole());
			communityPersonRoleRepository.save(currentRole);
		}
		return currentRole.getCommunity();
	}


	@Override
	public Community joinCommunity(UUID communityId, UUID personId) {
		var community = getCommunity(communityId);
		checkUnique(communityId, personId);
		communityPersonRoleRepository.save(communityPersonRoleMapper.toEntity(community,
				personRepository.findById(personId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, personId)),
				CommunityRole.MEMBER));
		return community;
	}

	@Override
	@Transactional
	public void leaveCommunity(UUID communityId, UUID personId) {
		if (communityPersonRoleRepository.deleteByCommunityIdAndPersonId(communityId, personId) == 0)
			throw new ApplicationException(PERSON_NOT_IN_COMMUNITY, personId, communityId);
	}

	@Override
	@Transactional
	public void deleteMembers(UUID id, UUID[] personIds) {
		var ownerId = communityPersonRoleRepository.findByCommunityIdAndCommunityRole(id, OWNER)
				.map(e -> e.getPerson().getId())
				.orElse(null);
		for (UUID personId : personIds) {
			if (personId.equals(ownerId)) throw new ApplicationException(COMMUNITY_CHANGE_OF_OWNER, id);
			if (communityPersonRoleRepository.deleteByCommunityIdAndPersonId(id, personId) == 0)
				throw new ApplicationException(PERSON_NOT_IN_COMMUNITY, personId, id);
		}
		publisher.publishEvent(CommunityEvents.CommunityDeletingMember.of(id,Set.of(personIds)));
	}

	@Override
	public Community saveAvatar(UUID id, MultipartFile file) {
		if (!AttachmentType.IMAGE.equals(getCorrespondingType(file.getContentType())))
			throw new ApplicationException(AVATAR_WRONG_FILE_TYPE);
		Community community = getCommunity(id);
		community.setAvatar(attachmentAdapter.createAttachment(file));

		return communityRepository.save(community);
	}

	private void checkUnique(UUID communityId, UUID personId) {
		if (communityPersonRoleRepository.existsCommunityPersonRoleByCommunityIdAndPersonId(communityId, personId))
			throw new ApplicationException(PERSON_ALREADY_IN_COMMUNITY, personId, communityId);
	}

	private CommunityPersonRole getCurrentRole(UUID communityId, UUID personId) {
		return communityPersonRoleRepository.findByCommunityIdAndPersonId(communityId, personId)
				.orElseThrow(() -> new ApplicationException(PERSON_NOT_IN_COMMUNITY, communityId, personId));
	}
}
