package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.CorporateCalendarEvent;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.CalendarEventType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.CalendarEvents;
import ru.loylabs.portal.presentation.request.Recurrence;
import ru.loylabs.portal.repository.CorporateCalendarEventRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.interfaces.CorporateCalendarEventService;

import java.time.LocalDate;
import java.time.Period;
import java.time.Year;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.List;
import java.util.Random;
import java.util.UUID;
import java.util.stream.Collectors;
import java.util.stream.IntStream;

import static java.time.ZoneOffset.UTC;
import static java.util.Objects.nonNull;
import static ru.loylabs.portal.enums.CalendarEventType.BIRTHDAY;
import static ru.loylabs.portal.exception.BaseException.Exception.CORPORATE_CALENDAR_EVENT_DATE_ERROR;
import static ru.loylabs.portal.exception.BaseException.Exception.CORPORATE_CALENDAR_EVENT_NOT_FOUND;

@Slf4j
@Service
@RequiredArgsConstructor
public class CorporateCalendarEventServiceImpl implements CorporateCalendarEventService {

	private final CorporateCalendarEventRepository cceRepository;
	private final PersonRepository personRepository;
	private final ApplicationEventPublisher publisher;


	private final List<String> phrases = Arrays.asList("Ты крутой!",
			"Мы тебя обожаем!",
			"Спасибо, что ты с нами!",
			"Пусть всё получится!",
			"Спасибо, что живой!",
			"Что, опять? Супер!");

	@Override
	public List<CorporateCalendarEvent> getAllEvents() {
		return cceRepository.findAll();
	}

	@Override
	public List<CorporateCalendarEvent> getAllRepeatingEvents(UUID repeatId) {
		return cceRepository.findAllByRepeatId(repeatId);
	}

	@Override
	public List<CorporateCalendarEvent> getEventsAndBirthdaysForNextMonths(int months) {
		final ZonedDateTime now = ZonedDateTime.now(UTC);
		final ZonedDateTime threshold = now.plusMonths(months);
		final List<CorporateCalendarEvent> allEventsForNextMonths = cceRepository.findAllByStartDateTimeBetween(now, threshold);

		final Year currentYear = Year.now();
		final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("MM-dd");

		if (threshold.getYear() != currentYear.getValue()) {
			personRepository.findAllBirthdaysTillDateWithYearOverlap(formatter.format(LocalDate.of(currentYear.getValue(), 12, 31)),
							formatter.format(threshold))
					.stream()
					.map(p -> personDobToNearestDobEvent(p, currentYear))
					.forEachOrdered(allEventsForNextMonths::add);
			return allEventsForNextMonths;
		}

		personRepository.findAllBirthdaysTillDate(formatter.format(threshold))
				.stream()
				.map(p -> personDobToNearestDobEvent(p, currentYear))
				.forEachOrdered(allEventsForNextMonths::add);

		return allEventsForNextMonths;
	}

	@Override
	public CorporateCalendarEvent getEvent(UUID eventId) {
		return cceRepository.findById(eventId)
				.orElseThrow(() -> new ApplicationException(CORPORATE_CALENDAR_EVENT_NOT_FOUND));
	}

	@Override
	public CorporateCalendarEvent addEvent(String eventName,
										   String description,
										   CalendarEventType eventType,
										   ZonedDateTime dateStart,
										   ZonedDateTime dateEnd,
										   Recurrence recurrence,
										   Person author) {
		var repeatUUID = recurrence != null ? UUID.randomUUID() : null;
		if (dateStart.isAfter(dateEnd)) throw new ApplicationException(CORPORATE_CALENDAR_EVENT_DATE_ERROR);
		var event = cceRepository.save(new CorporateCalendarEvent(eventName, description, eventType, dateStart, dateEnd, repeatUUID));

		if (nonNull(recurrence)) {
			int amountOfRepetitions;
			switch (recurrence.getUnit()) {
				case DAYS -> amountOfRepetitions = 366;
				case WEEKS -> amountOfRepetitions = 53;
				case MONTHS -> amountOfRepetitions = 13;
				case YEARS -> amountOfRepetitions = 11;
				default -> amountOfRepetitions = 1;
			}
			final List<CorporateCalendarEvent> recurrentEvents =
					IntStream.range(1, amountOfRepetitions)
							.mapToObj(i -> new CorporateCalendarEvent(eventName,
									description,
									eventType,
									dateStart.plus(i, recurrence.getUnit()),
									dateEnd.plus(i, recurrence.getUnit()),
									repeatUUID))
							.collect(Collectors.toList());
			cceRepository.saveAll(recurrentEvents);
		}
		publisher.publishEvent(CalendarEvents.CalendarCreateEvent.of(event.getId(),author.getId()));
		return event;
	}

	@Override
	@Transactional
	public void deleteEvent(UUID eventId, boolean repeatable) {
		if (repeatable) {
			var repeatId = getEvent(eventId).getRepeatId();
			if (repeatId != null)
				publisher.publishEvent(CalendarEvents.CalendarRepeatableDeleteEvent
						.of(cceRepository.findByRepeatId(repeatId)));
			cceRepository.deleteAllByRepeatId(repeatId);
		} else {
			publisher.publishEvent(CalendarEvents.CalendarDeleteEvent.of(eventId));
			if (cceRepository.deleteCorporateCalendarEventById(eventId) == 0) {
				throw new ApplicationException(CORPORATE_CALENDAR_EVENT_NOT_FOUND);
			}
		}
	}

	private CorporateCalendarEvent personDobToNearestDobEvent(Person p, Year year) {
		final LocalDate dob = p.getDob();
		final LocalDate withCurrentYear = dob.with(year).isBefore(LocalDate.now()) ? dob.with(year).plusYears(1) : dob.with(year);

		final String message = String.format("%s %s's %d birthday",
				p.getName(),
				p.getSurname(),
				Period.between(dob, withCurrentYear).getYears());
		return new CorporateCalendarEvent(message,
				withCurrentYear.isEqual(LocalDate.now()) ? String.format(
						"Today is %s. %s Congratulations!",
						message,
						phrases.get(new Random().nextInt(phrases.size()))) : message,
				BIRTHDAY,
				withCurrentYear);

	}
}
