package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.Department;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.DepartmentMapper;
import ru.loylabs.portal.presentation.request.CreateDepartmentRequest;
import ru.loylabs.portal.presentation.request.UpdateDepartmentRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.DepartmentRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.DepartmentService;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.JobAdapter;

import java.util.Objects;
import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.*;


@Service
@RequiredArgsConstructor
public class DepartmentServiceImpl extends AbstractFilteringAndSortingService<Department> implements DepartmentService {

	private final DepartmentRepository departmentRepository;
	private final PersonRepository personRepository;
	private final DepartmentMapper departmentMapper;
	private final FilterUtils filterUtils;

	private final JobAdapter jobAdapter;

	@Override
	public Page<Department> getAllDepartment(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Department.class);
		return departmentRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Department getDepartmentById(UUID id) {
		return departmentRepository.findById(id)
								   .orElseThrow(() -> new ApplicationException(DEPARTMENT_NOT_FOUND));
	}

	@Override
	public Department createDepartment(CreateDepartmentRequest request) {
		checkUniqueName(request.getName());
		var leader = Objects.nonNull(request.getLeaderId()) ? getPerson(request.getLeaderId()) : null;
		return departmentRepository.save(departmentMapper.toEntity(request, leader));
	}

	@Override
	public Department updateDepartment(UUID id, UpdateDepartmentRequest request) {
		var department = getDepartmentById(id);
		checkUniqueName(request.getName(), department.getName());
		var leader = Objects.nonNull(request.getLeaderId()) ? getPerson(request.getLeaderId()) : null;
		return departmentRepository.save(departmentMapper.toEntity(department, request, leader));
	}

	@Override
	public void deleteDepartment(UUID id) {
		if (departmentRepository.getDefaultDepartment().getId().equals(id))
			throw new ApplicationException(DEPARTMENT_BAN_ON_DELETION, id);

		var department = getDepartmentById(id);
		personRepository.findAllByJobDepartment(department)
				.forEach(person -> jobAdapter.setDefaultDepartment(person.getJob()));

		departmentRepository.delete(department);
	}

	private Person getPerson(UUID personId) {
		return personRepository.findById(personId)
				.orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, personId));
	}

	private void checkUniqueName(String name) {
		if (name != null && departmentRepository.existsByNameIgnoreCase(name))
			throw new ApplicationException(DEPARTMENT_ALREADY_EXISTS, name);
	}

	private void checkUniqueName(String newName, String oldName) {
		if (newName != null && !oldName.equals(newName) && departmentRepository.existsByNameIgnoreCase(newName))
			throw new ApplicationException(DEPARTMENT_ALREADY_EXISTS, newName);
	}
}
