package ru.loylabs.portal.service;

import lombok.AllArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.Emoji;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Reaction;
import ru.loylabs.portal.entity.Smile;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.EmojiMapper;
import ru.loylabs.portal.mapper.EmojiPersonMapper;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.EmojiPersonRepository;
import ru.loylabs.portal.repository.EmojiRepository;
import ru.loylabs.portal.repository.ReactionRepository;
import ru.loylabs.portal.repository.SmileRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.EmojiService;
import ru.loylabs.portal.service.interfaces.FilterUtils;

import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.*;

@Service
@AllArgsConstructor
public class EmojiServiceImpl extends AbstractFilteringAndSortingService<Emoji> implements EmojiService {
	private final ReactionRepository reactionRepository;
	private final EmojiPersonRepository emojiPersonRepository;
	private final SmileRepository smileRepository;
	private final EmojiRepository emojiRepository;
	private final EmojiMapper emojiMapper;
	private final EmojiPersonMapper emojiPersonMapper;
	private final FilterUtils filterUtils;

	@Override
	public Page<Emoji> getAllEmoji(Integer page, Integer size, String sort, SearchReversePolishNotation criteria, UUID reactionId, Person person) {
		criteria.add(SearchCriteria.builder().key("reaction.id").operation(":").value(reactionId).build());
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Emoji.class);
		return emojiRepository.findAll(getSpecification(listFilter), getPageable(listFilter)).map(e -> {
			if (emojiPersonRepository.existsByEmojiIdAndPersonId(e.getId(), person.getId())) e.setPressed(true);
			e.setClickCount(emojiPersonRepository.countAllByEmojiId(e.getId()));
			return e;
		});
	}

	@Override
	@Transactional
	public Emoji increaseEmoji(UUID reactionId, UUID smileId, Person person) {
		var emoji = createIfNotExist(reactionId, smileId);

		if (emojiPersonRepository.existsByEmojiIdAndPersonId(emoji.getId(), person.getId()))
			throw new ApplicationException(PERSON_SMILE_ALREADY_EXIST, person.getId(), smileId);

		emojiPersonRepository.save(emojiPersonMapper.toEntity(person, emoji));

		emoji.setPressed(true);
		emoji.setClickCount(emojiPersonRepository.countAllByEmojiId(emoji.getId()));

		return emoji;
	}

	@Override
	@Transactional
	public Emoji decreaseEmoji(UUID reactionId, UUID smileId, Person person) {
		var emoji = emojiRepository.findByReactionIdAndSmileId(reactionId, smileId)
				.orElseThrow(() -> new ApplicationException(REACTION_SMILE_NOT_FOUND, reactionId, smileId));

		if (emojiPersonRepository.deleteByEmojiIdAndPersonId(emoji.getId(), person.getId()) == 0)
			throw new ApplicationException(PERSON_SMILE_NOT_FOUND, person.getId(), smileId);

		emoji.setPressed(false);
		emoji.setClickCount(emojiPersonRepository.countAllByEmojiId(emoji.getId()));

		return deleteIfEmpty(emoji);
	}

	private Emoji createIfNotExist(UUID reactionId, UUID smileId) {
		return emojiRepository.findByReactionIdAndSmileId(reactionId, smileId)
				.orElseGet(() -> emojiRepository.save(emojiMapper.toEntity(getReaction(reactionId), getSmile(smileId))));
	}

	private Emoji deleteIfEmpty(Emoji emoji) {
		if (emoji.getClickCount() == 0) {
			emojiRepository.deleteById(emoji.getId());
		}
		return emoji;
	}

	private Reaction getReaction(UUID reactionId) {
		return reactionRepository.findById(reactionId).orElseThrow(() -> new ApplicationException(REACTION_NOT_FOUND, reactionId));
	}

	private Smile getSmile(UUID smileId) {
		return smileRepository.findById(smileId).orElseThrow(() -> new ApplicationException(SMILE_NOT_FOUND, smileId));
	}
}