package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Feed;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.FeedEvents;
import ru.loylabs.portal.mapper.FeedMapper;
import ru.loylabs.portal.presentation.request.CreateFeedRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.request.UpdateRecordRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.FeedRepository;
import ru.loylabs.portal.repository.RecordRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FeedService;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.RecordAdapter;

import java.util.Collection;
import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.FEED_NOT_FOUND;

@Service
@RequiredArgsConstructor
public class FeedServiceImpl extends AbstractFilteringAndSortingService<Record> implements FeedService {
	private final RecordAdapter recordAdapter;
	private final RecordRepository recordRepository;
	private final FilterUtils filterUtils;
	private final FeedRepository feedRepository;
	private final FeedMapper feedMapper;
	private final ApplicationEventPublisher publisher;

	@Override
	public Feed getFeed(UUID id) {
		return feedRepository.findById(id)
				.orElseThrow(() -> new ApplicationException(FEED_NOT_FOUND, id));
	}

	@Override
	public Page<Feed> getAllFeedRecord(Integer page, Integer size, String sort, SearchReversePolishNotation criteria, boolean important) {
		criteria.add(SearchCriteria.builder().key("important").operation(":").value(important).build());
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Feed.class);
		return feedRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Feed saveFeedRecord(CreateFeedRequest request, Person author, Collection<MultipartFile> files) {
		var feed = recordRepository.save(feedMapper.toEntity(
				recordAdapter.createRecord(request, author, null, RecordType.FEED, files), request.isImportant()));
		publisher.publishEvent(FeedEvents.FeedCreateEvent.of(feed.getId()));
		return feed;
	}

	@Override
	public Feed updateFeed(UUID feedId, UpdateRecordRequest request) {
		var feed = getFeed(feedId);
		return feedRepository.save(feedMapper.toEntity(recordAdapter.updateRecord(request, feed), feed));
	}

	@Override
	@Transactional
	public void deleteFeedRecord(UUID id) {
		var feed = getFeed(id);
		if (feedRepository.deleteIdeaById(feed.getId()) == 0) {
			throw new ApplicationException(FEED_NOT_FOUND, id);
		}
		publisher.publishEvent(FeedEvents.FeedDeleteEvent.of(feed));
	}

	@Override
	public Feed addAttachments(UUID id, Collection<MultipartFile> files) {
		return recordAdapter.addAttachments(getFeed(id), files);
	}

	@Override
	public Feed deleteAttachments(UUID id, DeleteAttachmentsRequest request) {
		return recordAdapter.deleteAttachments(getFeed(id), request);
	}

	@Override
	public long countComments(UUID parentRecordId) {
		return recordRepository.countByParentRecordId(parentRecordId);
	}
}
