package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.City;
import ru.loylabs.portal.entity.Grade;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.GradeEvents;
import ru.loylabs.portal.mapper.GradeMapper;
import ru.loylabs.portal.presentation.request.CreateGradeRequest;
import ru.loylabs.portal.presentation.request.UpdateGradeRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.GradeRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.GradeService;

import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.*;

@Service
@RequiredArgsConstructor
public class GradeServiceImpl extends AbstractFilteringAndSortingService<Grade> implements GradeService {

    private final GradeRepository gradeRepository;
    private final GradeMapper gradeMapper;
    private final ListFilterUtils filterUtils;

    private final ApplicationEventPublisher publisher;

    @Override
    public Grade getGrade(UUID id) {
        return gradeRepository.findById(id).orElseThrow(() -> new ApplicationException(GRADE_NOT_FOUND, id));
    }

    @Override
    public Page<Grade> getAllGrade(int page, int size, String sort, SearchReversePolishNotation criteria) {
        ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Grade.class);
        return gradeRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
    }

    @Override
    public Grade createGrade(CreateGradeRequest request) {
        checkUniqGradeName(request.getName());
        var grade = gradeMapper.toEntity(request);
        return gradeRepository.save(grade);
    }

    @Override
    public Grade updateGrade(UUID id, UpdateGradeRequest request) {
        var grade = getGrade(id);
        checkUniqGradeName(request.getName(), grade.getName());
        grade = gradeMapper.toEntity(getGrade(id), request);
        return gradeRepository.save(grade);
    }

    @Override
    public void deleteGrade(UUID idGrade) {
        var grade = getGrade(idGrade);
        if (grade.isDefault()) {
            throw new ApplicationException(GRADE_BAN_ON_DELETION, idGrade);
        }
        publisher.publishEvent(new GradeEvents.GradeDeletedEvent(idGrade));
        gradeRepository.delete(grade);
    }

    private void checkUniqGradeName(String name) {
        if (name != null && gradeRepository.existsByNameIgnoreCase(name))
            throw new ApplicationException(GRADE_ALREADY_EXISTS, name);
    }

    private void checkUniqGradeName(String newName, String oldName) {
        if (newName != null && !oldName.equals(newName) && gradeRepository.existsByNameIgnoreCase(newName))
            throw new ApplicationException(GRADE_ALREADY_EXISTS, newName);
    }
}