package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Idea;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.IdeaStatus;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.IdeaEvents;
import ru.loylabs.portal.mapper.IdeaMapper;
import ru.loylabs.portal.presentation.request.CreateIdeaRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.request.ProvideFeedbackRequest;
import ru.loylabs.portal.presentation.request.UpdateIdeaRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.IdeaRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.IdeaService;
import ru.loylabs.portal.service.interfaces.RecordAdapter;

import java.util.Collection;
import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.*;

@Service
@RequiredArgsConstructor
public class IdeaServiceImpl extends AbstractFilteringAndSortingService<Idea> implements IdeaService {
	private final RecordAdapter recordAdapter;
	private final IdeaRepository ideaRepository;
	private final IdeaMapper ideaMapper;
	private final FilterUtils filterUtils;
	private final ApplicationEventPublisher publisher;


	@Override
	public Idea getIdea(UUID id) {
		return ideaRepository.findById(id).orElseThrow(() -> new ApplicationException(IDEA_NOT_FOUND, id));
	}

	@Override
	public Page<Idea> getAllIdeas(Integer page, Integer size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Idea.class);
		return ideaRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Page<Idea> getFavoriteIdeas(int page, int size, String sort, SearchReversePolishNotation searchCriteria, Person person) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, searchCriteria, Idea.class);
		return ideaRepository.findAllByLikedId(person.getId(),getPageable(listFilter));
	}


	@Override
	public Idea saveIdea(CreateIdeaRequest request, Person author, Collection<MultipartFile> files) {
		checkUnique(request.getHeader());
		var idea = ideaRepository.save(
				ideaMapper.toEntity(
						recordAdapter.createRecord(request,
								author, null, RecordType.IDEA, files),
						request.getHeader(),
						IdeaStatus.NEW));
		publisher.publishEvent(IdeaEvents.IdeaCreateEvent.of(idea.getId()));
		return  idea;
	}

	@Override
	public void markAsFavorites(UUID id, Person person) {
		if(ideaRepository.existsByIdAndLikedId(id,person.getId()))
			throw new ApplicationException(IDEA_ALREADY_IN_FAVORITES,id);
		var idea = getIdea(id);
		idea.getLiked().add(person);
		ideaRepository.save(idea);
	}

	@Override
	@Transactional
	public void unmarkAsFavorites(UUID id, Person person) {
		if(!ideaRepository.existsByIdAndLikedId(id,person.getId()))
			throw new ApplicationException(IDEA_NOT_IN_FAVORITES,id);
		ideaRepository.deleteFromFavoritesIdea(id,person.getId());
	}

	@Override
    public Idea updateIdea(UUID id, UpdateIdeaRequest request) {
		var idea = getIdea(id);
		checkUnique(request.getHeader(), idea.getHeader());
        if (!idea.getStatus().equals(IdeaStatus.NEW))
            throw new ApplicationException(IDEA_CANNOT_BE_CHANGE, id, idea.getStatus());
        return ideaRepository.save(ideaMapper.toEntity(request, getIdea(id)));
    }

    @Override
	public void acceptIdea(ProvideFeedbackRequest request, UUID id, Person supervisor) {
		Idea idea = getIdea(id);
		if (!idea.getStatus().equals(IdeaStatus.NEW))
			throw new ApplicationException(IDEA_CAN_NOT_BE_APPROVED, id, idea.getStatus());
		idea.setSupervisor(supervisor);
		idea.setStatus(IdeaStatus.APPROVED);
		idea.setResponse(request.getResponse());

		ideaRepository.save(idea);
	}

	@Override
	public void rejectIdea(ProvideFeedbackRequest request, UUID id, Person supervisor) {
		Idea idea = getIdea(id);
		if (!idea.getStatus().equals(IdeaStatus.NEW))
			throw new ApplicationException(IDEA_CAN_NOT_BE_REJECTED, id, idea.getStatus());
		idea.setSupervisor(supervisor);
		idea.setStatus(IdeaStatus.DISMISSED);
		idea.setResponse(request.getResponse());

		ideaRepository.save(idea);
	}

	@Override
	public boolean isFavoriteForPerson(UUID ideaId, Person person) {
		return ideaRepository.existsByIdAndLikedId(ideaId,person.getId());
	}

	@Override
	public Idea addAttachments(UUID id, Collection<MultipartFile> files) {
		return recordAdapter.addAttachments(getIdea(id), files);
	}

	@Override
	public Idea deleteAttachments(UUID id, DeleteAttachmentsRequest request) {
		return recordAdapter.deleteAttachments(getIdea(id), request);
	}

	@Override
	@Transactional
	public void deleteIdea(UUID id) {
		publisher.publishEvent(IdeaEvents.IdeaDeleteEvent.of(getIdea(id)));
		if (ideaRepository.deleteIdeaById(id) == 0)
			throw new ApplicationException(IDEA_NOT_FOUND, id);
	}

	private void checkUnique(String header) {
		if (header != null && ideaRepository.existsByHeaderIgnoreCase(header))
			throw new ApplicationException(IDEA_ALREADY_EXISTS, header);
	}

	private void checkUnique(String newHeader, String oldHeader) {
		if (newHeader != null && !oldHeader.equals(newHeader) && ideaRepository.existsByHeaderIgnoreCase(newHeader))
			throw new ApplicationException(IDEA_ALREADY_EXISTS, newHeader);
	}
}
