package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.IntegratedCalendarLink;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.CalendarLinkMapper;
import ru.loylabs.portal.mapper.parse.DateTimeParse;
import ru.loylabs.portal.presentation.request.CreateIntegrateCalendarLink;
import ru.loylabs.portal.presentation.request.IntegratedCalendarEvent;
import ru.loylabs.portal.presentation.request.UpdateIntegrateCalendarLink;
import ru.loylabs.portal.repository.IntegratedCalendarLinkRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.interfaces.IntegratedCalendarService;
import ru.loylabs.portal.service.interfaces.RemoteCalendarClient;

import java.util.List;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.exception.BaseException.Exception.INTEGRATED_CALENDAR_LINK_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.INTEGRATION_CALENDAR_LINK_EXISTS;
import static ru.loylabs.portal.exception.BaseException.Exception.INTEGRATION_CALENDAR_NAME_AND_PERSON_EXISTS;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredentialPersonId;

@Slf4j
@Service
@RequiredArgsConstructor
public class IntegratedCalendarServiceImpl implements IntegratedCalendarService {
	private final IntegratedCalendarLinkRepository calendarLinkRepository;
	private final PersonRepository personRepository;
	private final CalendarLinkMapper calendarLinkMapper;
	private final RemoteCalendarClient remoteCalendarClient;
	private final DateTimeParse dateTimeParse;

	@Override
	public IntegratedCalendarLink addCalendarLink(CreateIntegrateCalendarLink linkForPerson) {
		var author = personRepository.findById(getCredentialPersonId())
				.orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, getCredentialPersonId()));
		checkUnique(linkForPerson);
		return calendarLinkRepository.save(calendarLinkMapper.toEntity(linkForPerson, author));
	}

	@Override
	public List<IntegratedCalendarEvent> getAllCalendarEvents() {
		return getAllEvents();
	}

	@Override
	public List<IntegratedCalendarLink> getAllLinkCalendar() {
		return calendarLinkRepository.findAllByPersonId(getCredentialPersonId());
	}

	@Override
	public IntegratedCalendarLink getLinkById(UUID linkId) {
		return calendarLinkRepository.findAllByIdAndAndPersonId(linkId, getCredentialPersonId()).orElseThrow(
				() -> new ApplicationException(INTEGRATED_CALENDAR_LINK_NOT_FOUND, linkId));
	}

	@Override
	public List<IntegratedCalendarEvent> getAllCalendarEventsById(String date) {
		return getAllEvents().stream()
				.filter(e -> e.getStartEvent().toLocalDate()
						.equals(dateTimeParse.parseStringToLocalDate(date)))
				.collect(Collectors.toList());
	}

	@Override
	public IntegratedCalendarLink updateLink(UUID linkId, UpdateIntegrateCalendarLink updateIntegrateCalendarLink) {
		return calendarLinkRepository.save(calendarLinkMapper.toEntity(getLinkById(linkId), updateIntegrateCalendarLink));
	}

	@Override
	public void deleteLink(UUID linkId) {
		calendarLinkRepository.delete(getLinkById(linkId));
	}

	private void checkUnique(CreateIntegrateCalendarLink linkForPerson) {
		if (calendarLinkRepository.existsByLinkCalendarIgnoreCase(linkForPerson.getLinkCalendar()))
			throw new ApplicationException(INTEGRATION_CALENDAR_LINK_EXISTS);
		if (calendarLinkRepository.existsByPersonIdAndLinkNameIgnoreCase(
				getCredentialPersonId(), linkForPerson.getLinkName()))
			throw new ApplicationException(INTEGRATION_CALENDAR_NAME_AND_PERSON_EXISTS, getCredentialPersonId(), linkForPerson.getLinkName());
	}

	private List<IntegratedCalendarEvent> getAllEvents() {
		return getAllLinkCalendar().stream()
				.flatMap(link -> remoteCalendarClient.getCalendarEvent(link.getLinkCalendar()).stream())
				.collect(Collectors.toList());
	}
}
