package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.KbArticle;
import ru.loylabs.portal.entity.KbSection;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.KbArticleMapper;
import ru.loylabs.portal.presentation.request.ChangeKbArticleSectionRequest;
import ru.loylabs.portal.presentation.request.CreateKbArticleRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.request.UpdateKbArticleRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.KbArticleRepository;
import ru.loylabs.portal.repository.KbSectionRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.AttachmentAdapter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.KbArticleService;

import java.util.Collection;
import java.util.List;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.exception.BaseException.Exception.ARTICLE_IN_SECTION_ALREADY_EXIST;
import static ru.loylabs.portal.exception.BaseException.Exception.ARTICLE_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.SECTION_NOT_FOUND;

@Transactional
@Service
@RequiredArgsConstructor
public class KbArticleServiceImpl extends AbstractFilteringAndSortingService<KbArticle> implements KbArticleService {
	private final AttachmentAdapter attachmentAdapter;
	private final KbSectionRepository sectionRepository;
	private final KbArticleRepository articleRepository;
	private final KbArticleMapper mapper;
	private final FilterUtils filterUtils;

	@Override
	@Transactional(readOnly = true)
	public KbArticle getArticle(UUID sectionId, UUID id) {
		return articleRepository.findByIdAndSectionId(id, sectionId)
				.orElseThrow(() -> new ApplicationException(ARTICLE_NOT_FOUND, id));
	}

	@Override
	public Page<KbArticle> getAllArticles(int page, int size, String sort, SearchReversePolishNotation criteria, UUID sectionId) {
		criteria.add(SearchCriteria.builder().key("section.id").operation(":").value(sectionId).build());
		return getAllArticles(page, size, sort, criteria);
	}

	@Override
	public Page<KbArticle> getAllArticles(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, KbArticle.class);
		return articleRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}


	@Override
	public KbArticle saveArticle(UUID sectionId, CreateKbArticleRequest request, Collection<MultipartFile> files) {
		var section = searchSection(sectionId);
		checkRepeatArticleAndSection(section, request.getName());
		var article = mapper.toEntity(request, section);
		attachmentAdapter.addAttachments(article.getAttachments(), files);
		return articleRepository.save(article);
	}

	@Override
	public KbArticle updateArticle(UUID sectionId, UUID id, UpdateKbArticleRequest request) {
		var article = mapper.toEntity(getArticle(sectionId, id), request);
		return articleRepository.save(article);
	}

	@Override
	public KbArticle moveArticle(UUID sectionId, UUID id, ChangeKbArticleSectionRequest request) {
		checkRepeatArticleAndSection(searchSection(request.getKbSectionId()), getArticle(sectionId, id).getName());
		var article = mapper.toEntity(getArticle(sectionId, id), searchSection(request.getKbSectionId()));
		return articleRepository.save(article);
	}

	@Override
	public KbArticle addAttachments(UUID sectionId, UUID id, Collection<MultipartFile> files) {
		var article = getArticle(sectionId, id);
		attachmentAdapter.addAttachments(getArticle(sectionId, id).getAttachments(), files);
		return articleRepository.save(article);
	}

	@Override
	public KbArticle deleteAttachments(UUID sectionId, UUID id, DeleteAttachmentsRequest request) {
		var article = getArticle(sectionId, id);
		attachmentAdapter.deleteAttachments(article.getAttachments(), request.getAttachmentsIds());
		return article;
	}

	@Override
	@Transactional
	public void deleteArticle(UUID sectionId, UUID id) {
		if (articleRepository.deleteByIdAndSectionId(id, sectionId) == 0)
			throw new ApplicationException(ARTICLE_NOT_FOUND, id);
	}

	private void checkRepeatArticleAndSection(KbSection section, String nameArticle){
		if(articleRepository.existsByNameAndSectionId(nameArticle, section.getId()))
			throw new ApplicationException(ARTICLE_IN_SECTION_ALREADY_EXIST, nameArticle, section.getName());
	}

	private KbSection searchSection (UUID sectionId) {
		return sectionRepository.findById(sectionId)
				.orElseThrow(() -> new ApplicationException(SECTION_NOT_FOUND, sectionId));
	}
}