package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.mapstruct.util.Experimental;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.KbSection;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.KbSectionMapper;
import ru.loylabs.portal.presentation.request.CreateKbSectionRequest;
import ru.loylabs.portal.presentation.request.UpdateKbSectionRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.KbArticleRepository;
import ru.loylabs.portal.repository.KbSectionRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.KbSectionService;

import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.SECTION_NAME_ALREADY_EXIST;
import static ru.loylabs.portal.exception.BaseException.Exception.SECTION_NOT_FOUND;

@Service
@RequiredArgsConstructor
public class KbSectionServiceImpl extends AbstractFilteringAndSortingService<KbSection> implements KbSectionService {
	private final KbSectionRepository sectionRepository;
	private final KbSectionMapper mapper;
	private final FilterUtils filterUtils;

	private final KbArticleRepository articleRepository;

	@Override
	public KbSection getSection(UUID id) {
		return sectionRepository.findById(id).orElseThrow(() -> new ApplicationException(SECTION_NOT_FOUND, id));
	}

	@Override
	public Page<KbSection> getAllSections(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, KbSection.class);
		return sectionRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public KbSection createSection(CreateKbSectionRequest request) {
		KbSection section = mapper.toEntity(request);
		checkUnique(section.getName());
		return sectionRepository.save(section);
	}

	@Override
	public KbSection updateSection(UUID id, UpdateKbSectionRequest request) {
		KbSection section = getSection(id);
		checkUnique(request.getName(), section.getName());
		return sectionRepository.save(mapper.toEntity(section, request));
	}

	@Override
	@Transactional
	public void deleteSection(UUID id) {
		if (sectionRepository.deleteSectionById(id) == 0)
			throw new ApplicationException(SECTION_NOT_FOUND, id);
	}

	@Override
	public void checkUnique(String name) {
		if (name != null && sectionRepository.existsByName(name))
			throw new ApplicationException(SECTION_NAME_ALREADY_EXIST, name);
	}

	@Override
	public void checkUnique(String newName, String oldName) {
		if (newName != null && !newName.equals(oldName) && sectionRepository.existsByName(newName))
			throw new ApplicationException(SECTION_NAME_ALREADY_EXIST, newName);
	}

	@Override
	public long countArticles(UUID sectionId) {
		return articleRepository.countBySectionId(sectionId);
	}

	@Experimental
	@Transactional(readOnly = true)
	public Page<KbSection> searchSectionWithRelevantContent(int page, int size, String sort, SearchReversePolishNotation criteria, String searchPhrase) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, KbSection.class);
		var articlesId = articleRepository.getArticlesUUIDWithContent(searchPhrase);
		return sectionRepository.findKbSectionByArticleIdWithContent(articlesId, getPageable(listFilter));
	}
}

