package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.Note;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.NoteMapper;
import ru.loylabs.portal.presentation.request.CreateNoteRequest;
import ru.loylabs.portal.presentation.request.UpdateNoteRequest;
import ru.loylabs.portal.repository.NoteRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.interfaces.NoteService;

import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.*;

@Service
@RequiredArgsConstructor
public class NoteServiceImpl implements NoteService {
	private final NoteRepository noteRepository;
	private final NoteMapper noteMapper;
	private final PersonRepository personRepository;

	@Override
	public Note getNote(UUID authorId, UUID targetId) {
		return noteRepository.findByTargetIdAndAuthorId(targetId, authorId)
				.orElseThrow(() -> new ApplicationException(NOTE_NOT_FOUND, targetId, authorId));
	}

	@Override
	public Note createNote(CreateNoteRequest request, UUID authorId, UUID targetId) {
		checkUnique(targetId, authorId);
		return noteRepository.save(
				noteMapper.toEntity(request,
						personRepository.findById(authorId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, authorId)),
						personRepository.findById(targetId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, targetId))));
	}

	@Override
	public Note updateNote(UpdateNoteRequest request, UUID authorId, UUID targetId) {
		return noteRepository.save(noteMapper.toEntity(getNote(authorId, targetId), request));
	}

	@Override
	@Transactional
	public void deleteNote(UUID authorId, UUID targetId) {
		if (noteRepository.deleteByAuthorIdAndTargetId(authorId, targetId) == 0)
			throw new ApplicationException(NOTE_NOT_FOUND, targetId, authorId);
	}

	public void checkUnique(UUID userId, UUID authorId) {
		if (noteRepository.existsByUserIdAndAuthorId(userId, authorId))
			throw new ApplicationException(NOTE_USER_ALREADY_EXIST, userId, authorId);
	}
}
