package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import net.bytebuddy.utility.RandomString;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.CommunityRole;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.PersonEvents;
import ru.loylabs.portal.presentation.request.GenerateResetTokenRequest;
import ru.loylabs.portal.presentation.request.UpdatePersonPasswordViaTokenRequest;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.redis.PasswordResetTokenRepository;
import ru.loylabs.portal.service.interfaces.EmailNotificationFactory;
import ru.loylabs.portal.service.interfaces.NotificationSender;
import ru.loylabs.portal.service.interfaces.PasswordService;
import ru.loylabs.portal.service.interfaces.PasswordValidator;
import ru.loylabs.portal.service.notification.email.EmailNotification;
import ru.loylabs.portal.utility.StringUtils;

import java.util.UUID;

import static ru.loylabs.portal.enums.CommunityRole.ADMIN;
import static ru.loylabs.portal.exception.BaseException.Exception.EMAIL_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_ADMIN;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.TOKEN_NOT_VALID;
import static ru.loylabs.portal.exception.BaseException.Exception.WEAK_PASSWORD;

@Service
@RequiredArgsConstructor
public class PasswordServiceImpl implements PasswordService {

	private final PersonRepository personRepository;
	private final PasswordResetTokenRepository passwordRepository;
	private final EmailNotificationFactory emailNotificationFactory;
	@Qualifier("emailNotificationSenderImpl")
	private final NotificationSender<EmailNotification> emailNotificationSender;
	private final PasswordEncoder encoder;
	private final PasswordValidator validator;
	private final ApplicationEventPublisher publisher;

	@Override
	public void updatePasswordResetToken(GenerateResetTokenRequest request) {
		Person person = personRepository.findByUsername(request.getEmail())
										.orElseThrow(() -> new ApplicationException(EMAIL_NOT_FOUND, request.getEmail()));
		String token = RandomString.make(45);

		emailNotificationSender.send(
				emailNotificationFactory.createResetPasswordMessage(person, token)
		);

		passwordRepository.save(token, person.getId());
	}

	@Override
	public void updatePasswordViaToken(UpdatePersonPasswordViaTokenRequest request, String token) {
		if (StringUtils.isValid(token)) {
			UUID personId = passwordRepository.get(token)
											  .orElseThrow(() -> new ApplicationException(TOKEN_NOT_VALID, token));
			if (!validator.validate(request.getPassword()).isSuccess())
				throw new ApplicationException(WEAK_PASSWORD);
			Person person = personRepository.findById(personId)
											.orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, personId));
			person.setPassword(encoder.encode(request.getPassword()));

			personRepository.save(person);
			passwordRepository.delete(token);
		} else {
			throw new ApplicationException(TOKEN_NOT_VALID, token);
		}
	}

	@Override
	public void passwordUpdateWithUnblock(UpdatePersonPasswordViaTokenRequest password, String token) {
		UUID personId = passwordRepository.get(token)
				.orElseThrow(() -> new ApplicationException(TOKEN_NOT_VALID, token));
		Person person = personRepository.findById(personId)
				.orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, personId));
		if (!person.getRole().getRole().equals("ADMIN")) throw new ApplicationException(PERSON_NOT_ADMIN);
		updatePasswordViaToken(password, token);
		publisher.publishEvent(PersonEvents.PersonUnblockEvent.of(personId));
	}
}
