package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import net.bytebuddy.utility.RandomString;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.City;
import ru.loylabs.portal.entity.Contact;
import ru.loylabs.portal.entity.Grade;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Skill;
import ru.loylabs.portal.entity.Status;
import ru.loylabs.portal.enums.AttachmentType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.PersonEvents;
import ru.loylabs.portal.mapper.ContactMapper;
import ru.loylabs.portal.mapper.PersonMapper;
import ru.loylabs.portal.mapper.PersonSkillMapper;
import ru.loylabs.portal.mapper.Yandex360UserMapper;
import ru.loylabs.portal.presentation.request.*;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.CityRepository;
import ru.loylabs.portal.repository.GradeRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.SecurityRoleRepository;
import ru.loylabs.portal.repository.SkillRepository;
import ru.loylabs.portal.repository.redis.LoginTokenRepository;
import ru.loylabs.portal.repository.redis.PasswordEntryAttemptRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.*;
import ru.loylabs.portal.service.notification.email.EmailNotification;
import ru.loylabs.portal.service.yandex.dto.Yandex360UserResponse;

import java.time.LocalDate;
import java.time.ZoneId;
import java.util.Objects;
import java.util.Optional;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.exception.BaseException.Exception.*;
import static ru.loylabs.portal.utility.FileAttachmentTypeFactory.getCorrespondingType;

@Service
@RequiredArgsConstructor
public class PersonServiceImpl extends AbstractFilteringAndSortingService<Person> implements PersonService {

	private static final String DEFAULT_ROLE = "USER";


	private final AttachmentAdapter attachmentAdapter;
	private final ContactService contactService;
	private final PersonRepository personRepository;
	private final SecurityRoleRepository securityRoleRepository;
	private final CityRepository cityRepository;
	private final PersonMapper personMapper;
	private final ContactMapper contactMapper;
	private final PasswordEncoder passwordEncoder;
	private final PasswordGenerator passwordGenerator;
	private final PasswordValidator passwordValidator;
	private final FilterUtils filterUtils;
	private final JobAdapter jobAdapter;
	private final EmailNotificationFactory emailNotificationFactory;
	@Qualifier("emailNotificationSenderImpl")
	private final NotificationSender<EmailNotification> emailNotificationSender;
	private final Yandex360UserMapper yandex360UserMapper;
	private final Yandex360UserService yandex360UserService;
	private final LoginTokenRepository loginTokenRepository;
	private final SkillRepository skillRepository;
	private final GradeRepository gradeRepository;
	private final PersonSkillMapper personSkillMapper;
	private final StatusService statusService;
	private final PasswordEntryAttemptRepository passwordEntryAttemptRepository;

	private final ApplicationEventPublisher publisher;

	@Override
	public Person getPerson(UUID id) {
		return personRepository.findById(id).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, id));
	}

	@Override
	public Person getByUsername(String username) {
		return personRepository.findByUsername(username)
							   .orElseThrow(() -> new ApplicationException(USERNAME_NOT_FOUND, username));
	}

	@Override
	public Page<Person> getAllPersons(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Person.class);
		return personRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Person createPerson(CreatePersonRequest request) {
		checkUnique(request.getUsername());

		String password;
		if (request.getPassword() != null) {
			var validationResult = passwordValidator.validate(request.getPassword());
			if (!validationResult.isSuccess()) throw new ApplicationException(WEAK_PASSWORD);
			password = request.getPassword();
		} else {
			password = passwordGenerator.generatePassword();
		}

		var person = personRepository.save(
				personMapper.toEntity(
						request,
						passwordEncoder.encode(password),
						jobAdapter.createJob(request.getJob()),
						new Contact(),
						LocalDate.now(),
						securityRoleRepository.findByRole(request.getRole())
											  .orElseThrow(() -> new ApplicationException(ROLE_NOT_FOUND,
																						  request.getRole())),
						cityRepository.findById(request.getCityId())
									  .orElseThrow(() -> new ApplicationException(CITY_NOT_FOUND, request.getCityId()))));

		emailNotificationSender.send(
				emailNotificationFactory.createGeneratedPasswordMessage(person, password)
		);

		yandex360UserService.findByEmail(person.getUsername()).ifPresent(response -> person.setExternalUserId(response.getId()));

		return personRepository.save(person);
	}

	@Override
	public Person createPersonFromYandex(String email) {
		checkUnique(email);

		Optional<Yandex360UserResponse> existingYandex360User = yandex360UserService.findByEmail(email);

		if (existingYandex360User.isEmpty()) throw new ApplicationException(YANDEX_360_USER_NOT_FOUND);

		String password = passwordGenerator.generatePassword();
		var contact = contactMapper.toEntity(existingYandex360User.get().getContacts());
		contactService.checkUnique(contact);

		Person person = personRepository.save(
				yandex360UserMapper.toEntity(
						existingYandex360User.get(),
						passwordEncoder.encode(password),
						jobAdapter.createJob(existingYandex360User.get()),
						contact,
						LocalDate.ofInstant(existingYandex360User.get().getCreatedAt(), ZoneId.systemDefault()),
						securityRoleRepository.findByRole(DEFAULT_ROLE)
											  .orElseThrow(() -> new ApplicationException(ROLE_NOT_FOUND,
																						  DEFAULT_ROLE))
				)
		);

		String token = createLoginTokenForPerson(person.getId());
		emailNotificationSender.send(
				emailNotificationFactory.createGeneratedPasswordMessageWithLoginLink(person, password, token)
		);
		publisher.publishEvent(PersonEvents.PersonCreateEvent.of(person.getId()));
		return person;
	}

	@Override
	public Person updatePerson(UpdatePersonRequest request, UUID id) {
		Person person = getPerson(id);
		contactService.checkUnique(request.getContact(), person.getContact());
		City city = null;
		if (request.getCityId() != null)
			city = cityRepository.findById(request.getCityId())
								 .orElseThrow(() -> new ApplicationException(CITY_NOT_FOUND, request.getCityId()));
		personMapper.toEntity(person, request, city);

		person = personRepository.save(person);

		yandex360UserService.updateYandexUser(person);

		if(request.getDob() != null) publisher.publishEvent(PersonEvents.PersonDobUpdateEvent.of(person.getId()));

		return person;
	}

	@Override
	public Person updatePersonInfo(UpdatePersonInfoRequest request, UUID id) {
		Person person = getPerson(id);
		contactService.checkUnique(request.getContact(), person.getContact());
		personMapper.toEntity(person, request);

		person = personRepository.save(person);

		yandex360UserService.updateYandexUser(person);

		if(request.getDob() != null) publisher.publishEvent(PersonEvents.PersonDobUpdateEvent.of(person.getId()));

		return person;
	}

	@Override
	public Person updateJob(UpdateJobRequest request, UUID id) {
		Person person = getPerson(id);
		jobAdapter.updateJob(request, person.getJob());

		return person;
	}

	@Override
	public Person updateRole(UpdatePersonRoleRequest request, UUID id) {
		var person = getPerson(id);
		if (person.getRole().getRole().equalsIgnoreCase(request.getRole())) return person;
		personMapper.toEntity(
				person,
				request.getRole() == null
				? null
				: securityRoleRepository.findByRole(request.getRole())
										.orElseThrow(() -> new ApplicationException(ROLE_NOT_FOUND,
																					request.getRole()))
		);
		person = personRepository.save(person);

		yandex360UserService.updateYandexUser(person);

		return person;
	}

	@Override
	public Person updateSkills(UpdatePersonSkillsRequest request, UUID id) {
		var person = getPerson(id);
		person.setSkills(
				request.getSkills().entrySet().stream()
						.map(e -> personSkillMapper.toEntity(person, getSkill(e.getKey()), getGrade(e.getValue())))
						.collect(Collectors.toList())
		);
		return personRepository.save(person);
	}

	private Skill getSkill(UUID id) {
		return skillRepository.findById(id).orElseThrow(() -> new ApplicationException(SKILL_NOT_FOUND, id));
	}

	private Grade getGrade(UUID id) {
		return Objects.isNull(id)
				? gradeRepository.getDefaultGrade()
				: gradeRepository.findById(id).orElseThrow(() -> new ApplicationException(GRADE_NOT_FOUND, id));
	}

	@Override
	public void updatePassword(String password, UUID id) {
		Person person = getPerson(id);

		var validationResult = passwordValidator.validate(password);
		if (!validationResult.isSuccess()) throw new ApplicationException(WEAK_PASSWORD);

		person.setPassword(passwordEncoder.encode(password));

		personRepository.save(person);
	}

	@Override
	public void blockPerson(UUID id) {
		Person person = getPerson(id);
		person.setBlocked(true);

		personRepository.save(person);
	}

	@Override
	public void unblockPerson(UUID id) {
		Person person = getPerson(id);
		person.setBlocked(false);
		passwordEntryAttemptRepository.delete(id.toString());
		personRepository.save(person);
	}

	@Override
	public Person setAvatar(UUID id, MultipartFile file) {
		if (!AttachmentType.IMAGE.equals(getCorrespondingType(file.getContentType())))
			throw new ApplicationException(AVATAR_WRONG_FILE_TYPE);
		Person person = getPerson(id);
		person.setAvatar(attachmentAdapter.createAttachment(file));

		return personRepository.save(person);
	}

	@Override
	public Person setBackground(UUID id, MultipartFile file) {
		if (Objects.nonNull(file) && (!AttachmentType.IMAGE.equals(getCorrespondingType(file.getContentType()))))
			throw new ApplicationException(AVATAR_WRONG_FILE_TYPE);
		Person person = getPerson(id);
		person.setBackground(attachmentAdapter.createAttachment(file));

		return personRepository.save(person);
	}

	@Override
	public Page<Person> getAllWithUpcomingBirthdays(int size, int page) {
		LocalDate now = LocalDate.now();
		return personRepository.findByDob(now.getMonthValue(), now.getDayOfMonth(), PageRequest.of(page, size));
	}

	@Override
	public Person updateStatus(UUID personId, UUID statusId) {
		var status = statusService.getStatus(statusId);
		return setStatus(personId, status);
	}

	@Override
	public Person deleteStatus(UUID personId) {
		return setStatus(personId, null);
	}

	@Override
	public void checkUnique(String username) {
		if (username != null && personRepository.existsByUsername(username))
			throw new ApplicationException(PERSON_USERNAME_ALREADY_EXIST, username);
	}

	private String createLoginTokenForPerson(UUID personId) {
		String token = RandomString.make(45);
		loginTokenRepository.save(token, personId);
		return token;
	}

	private Person setStatus(UUID personId, Status status){
		var person = getPerson(personId);
		person.setStatus(status);
		return personRepository.save(person);
	}
}
