package ru.loylabs.portal.service;

import static ru.loylabs.portal.exception.BaseException.Exception.ROLE_IMMUTABLE;
import static ru.loylabs.portal.exception.BaseException.Exception.ROLE_NAME_ALREADY_EXISTS;
import static ru.loylabs.portal.exception.BaseException.Exception.ROLE_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.ROLE_ROLE_ALREADY_EXISTS;
import static ru.loylabs.portal.utility.StringUtils.isValid;

import java.util.UUID;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.SecurityRole;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.SecurityRoleMapper;
import ru.loylabs.portal.presentation.request.CreateSecurityRoleRequest;
import ru.loylabs.portal.presentation.request.UpdateSecurityRoleRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.SecurityPermissionRepository;
import ru.loylabs.portal.repository.SecurityRoleRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.SecurityRoleService;
import ru.loylabs.portal.utility.CollectionUtils;

@Service
@RequiredArgsConstructor
public class SecurityRoleServiceImpl extends AbstractFilteringAndSortingService<SecurityRole> implements SecurityRoleService {

	private final SecurityRoleRepository securityRoleRepository;

	private final SecurityPermissionRepository securityPermissionRepository;

	private final SecurityRoleMapper securityRoleMapper;

	private final FilterUtils filterUtils;


	@Override
	public SecurityRole getRoleById(UUID id) {
		return securityRoleRepository.findById(id).orElseThrow(() -> new ApplicationException(ROLE_NOT_FOUND, id));
	}

	@Override
	public SecurityRole getRoleByName(String name) {
		return securityRoleRepository.findByName(name)
									 .orElseThrow(() -> new ApplicationException(ROLE_NOT_FOUND, name));
	}

	@Override
	public Page<SecurityRole> getAllRoles(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, SecurityRole.class);
		return securityRoleRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Transactional
	@Override
	public SecurityRole createRole(CreateSecurityRoleRequest request) {
		checkUnique(request.getRole(), request.getName());

		return securityRoleRepository.save(
				securityRoleMapper.toEntity(request, securityPermissionRepository.findByIdIn(request.getPermissions()))
		);
	}

	@Transactional
	@Override
	public SecurityRole updateRole(UpdateSecurityRoleRequest request, UUID id) {
		SecurityRole role = getRoleById(id);
		System.out.println("BLAH");

		if (role.getIsMutable())
			return securityRoleRepository.save(
					securityRoleMapper.toEntity(
							role,
							request,
							CollectionUtils.isValid(request.getPermissions())
								? securityPermissionRepository.findByIdIn(request.getPermissions())
								: null
					)
			);
		else
			throw new ApplicationException(ROLE_IMMUTABLE, role.getRole());
	}

	@Transactional
	@Override
	public void deleteRole(UUID id) {
		securityRoleRepository.deleteById(id);
	}

	protected void checkUnique(String role, String name) {
		if (isValid(name) && securityRoleRepository.existsByName(name))
			throw new ApplicationException(ROLE_NAME_ALREADY_EXISTS, name);
		if (isValid(role) && securityRoleRepository.existsByRole(role))
			throw new ApplicationException(ROLE_ROLE_ALREADY_EXISTS, role);
	}

}
