package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.City;
import ru.loylabs.portal.entity.Skill;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.SkillMapper;
import ru.loylabs.portal.presentation.request.CreateSkillRequest;
import ru.loylabs.portal.presentation.request.UpdateSkillRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.SkillRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.SkillService;

import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.SKILL_ALREADY_EXISTS;
import static ru.loylabs.portal.exception.BaseException.Exception.SKILL_NOT_FOUND;

@Component
@RequiredArgsConstructor
public class SkillServiceImpl extends AbstractFilteringAndSortingService<Skill> implements SkillService {

	private final SkillRepository repository;
	private final ListFilterUtils filterUtils;
	private final SkillMapper skillMapper;

	@Override
	public Page<Skill> getAllSkill(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, City.class);
		return repository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Skill getSkillById(UUID id) {
		return repository.findById(id)
				.orElseThrow(() -> new ApplicationException(SKILL_NOT_FOUND, id));
	}

	@Override
	public Skill createSkill(CreateSkillRequest request) {
		checkUnique(request.getName());
		return repository.save(
				skillMapper.toEntity(request)
		);
	}

	@Override
	public Skill updateSkill(UUID id, UpdateSkillRequest request) {
		checkUnique(request.getName());
		return repository.save(
				skillMapper.toEntity(getSkillById(id), request)
		);
	}

	@Override
	@Transactional
	public void deleteSkill(UUID id) {
		if (repository.deleteSkillById(id) == 0) {
			throw new ApplicationException(SKILL_NOT_FOUND, id);
		}
	}

	private void checkUnique(String skillName) {
		var skillOptional = repository.findByNameIgnoreCase(skillName);

		if (skillOptional.isPresent()) {
			throw new ApplicationException(SKILL_ALREADY_EXISTS, skillName);
		}
	}
}
