package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.Status;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.StatusMapper;
import ru.loylabs.portal.presentation.request.UpdateStatusRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.presentation.view.StatusView;
import ru.loylabs.portal.repository.StatusRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.StatusService;

import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.*;

@Service
@RequiredArgsConstructor
public class StatusServiceImpl extends AbstractFilteringAndSortingService<Status> implements StatusService {

    private final StatusRepository repository;
    private final StatusMapper statusMapper;
    private final ListFilterUtils listFilterUtils;

    @Override
    public Status getStatus(UUID id) {
        return repository.findById(id).orElseThrow(() -> new ApplicationException(STATUS_NOT_FOUND_BY_ID, id));
    }

    @Override
    public Status getStatus(String name) {
        return repository.findByNameIgnoreCase(name).orElseThrow(() -> new ApplicationException(STATUS_NOT_FOUND_BY_NAME, name));
    }

    @Override
    public Page<Status> getAllStatuses(int page, int size, String sort, SearchReversePolishNotation criteria) {
        ListFilter listFilter = listFilterUtils.getListFilter(page, size, sort, criteria, Status.class);
        return repository.findAll(getSpecification(listFilter), getPageable(listFilter));
    }

    @Override
    public Status createStatus(Status status) {
        checkExistingStatusByName(status.getName());
        return repository.save(status);
    }

    @Override
    public void deleteStatus(UUID id) {
        repository.deleteById(id);
    }

    @Override
    public Status updateStatus(UUID id, UpdateStatusRequest request) {
        checkExistingStatusByName(request.getName());
        var status = getStatus(id);
        statusMapper.updateEntity(request, status);
        return repository.save(status);
    }

    private void checkExistingStatusByName(String name) {
        if (repository.existsByNameIgnoreCase(name)) {
            throw new ApplicationException(STATUS_NAME_ALREADY_EXISTS, name);
        }
    }
}
