package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.StoryEvents;
import ru.loylabs.portal.mapper.StoryGroupMapper;
import ru.loylabs.portal.presentation.request.CreateStoryGroupRequest;
import ru.loylabs.portal.presentation.request.CreateStoryRequest;
import ru.loylabs.portal.presentation.request.UpdateStoryGroupRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.StoryAdapter;
import ru.loylabs.portal.service.interfaces.StoryGroupService;

import javax.transaction.Transactional;
import java.time.Instant;
import java.util.List;
import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.STORY_GROUP_NOT_FOUND;

@Service
@RequiredArgsConstructor
public class StoryGroupServiceImpl extends AbstractFilteringAndSortingService<StoryGroup> implements StoryGroupService {
	private final StoryAdapter storyAdapter;
	private final StoryGroupRepository storyGroupRepository;
	private final StoryGroupMapper storyGroupMapper;
	private final FilterUtils filterUtils;
	private final ApplicationEventPublisher publisher;


	@Override
	public StoryGroup getStoryGroup(UUID id) {
		return findGroup(id);
	}

	@Override
	public Page<StoryGroup> getAllStoryGroups(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, StoryGroup.class);
		return storyGroupRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public StoryGroup saveStoryGroup(CreateStoryGroupRequest request, Person author, List<CreateStoryRequest> requests, List<MultipartFile> files) {
		if (requests.size() != files.size())
			throw new ApplicationException(ApplicationException.Exception.STORY_WRONG_FILE_AMOUNT);
		var group = storyGroupMapper.toEntity(request, author, Instant.now());
		for (int i = 0; i < requests.size(); i++)
			group.getStories().add(storyAdapter.createStory(requests.get(i), group, author, files.get(i)));
		group = storyGroupRepository.save(group);
		publisher.publishEvent(StoryEvents.StoryGroupCreateEvent.of(group.getId()));
		return group;
	}

	@Override
	public StoryGroup updateStoryGroup(UpdateStoryGroupRequest request, UUID id) {
		return storyGroupRepository.save(storyGroupMapper.toEntity(
				findGroup(id),
				request));
	}

	@Transactional
	@Override
	public void deleteStoryGroup(UUID id) {
		publisher.publishEvent(StoryEvents.StoryGroupDeleteEvent.of(id));
		if (storyGroupRepository.deleteStoryGroupById(id) == 0)
			throw new ApplicationException(STORY_GROUP_NOT_FOUND, id);
	}

	private StoryGroup findGroup(UUID id) {
		return storyGroupRepository.findById(id).orElseThrow(() -> new ApplicationException(STORY_GROUP_NOT_FOUND, id));
	}
}
