package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.annotation.UpdateStoryGroupTime;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.enums.AttachmentType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.presentation.request.CreateStoryRequest;
import ru.loylabs.portal.presentation.request.UpdateStoryRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.repository.StoryRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.RecordAdapter;
import ru.loylabs.portal.service.interfaces.StoryAdapter;
import ru.loylabs.portal.service.interfaces.StoryService;

import java.time.Instant;
import java.time.LocalDateTime;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.exception.BaseException.Exception.*;
import static ru.loylabs.portal.utility.FileAttachmentTypeFactory.getCorrespondingType;

@Service
@RequiredArgsConstructor
public class StoryServiceImpl extends AbstractFilteringAndSortingService<Story> implements StoryService {
	private final StoryAdapter storyAdapter;
	private final StoryRepository storyRepository;
	private final StoryGroupRepository storyGroupRepository;
	private final FilterUtils filterUtils;
	private final RecordAdapter recordAdapter;

	@Override
	public Story getStory(UUID groupId, UUID id) {
		return storyRepository.findByIdAndGroupId(id, groupId).orElseThrow(() -> new ApplicationException(STORY_NOT_FOUND, id));
	}

	@Override
	public Page<Story> getAllStoriesFromGroup(int page, int size, String sort, SearchReversePolishNotation criteria, UUID groupId) {
		criteria.add(SearchCriteria.builder().key("group.id").operation(":").value(groupId).build());
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Story.class);
		return storyRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	@UpdateStoryGroupTime
	public Story saveStory(CreateStoryRequest request, UUID groupId, Person author, MultipartFile file) {
		return storyRepository.save(storyAdapter.createStory(request,
				storyGroupRepository.findById(groupId).orElseThrow(() -> new ApplicationException(STORY_GROUP_NOT_FOUND, groupId)),
				author,
				file));
	}

	@Override
	public Story updateStory(UpdateStoryRequest request, UUID groupId, UUID id) {
		return storyRepository.save(storyAdapter.updateStory(request, getStory(groupId, id)));
	}

	@Override
	@Transactional
	@UpdateStoryGroupTime
	public StoryGroup deleteStory(UUID groupId, UUID id) {
		var group = storyGroupRepository.findById(groupId)
				.orElseThrow(() -> new ApplicationException(STORY_GROUP_NOT_FOUND, groupId));
		var story = getStory(groupId, id);

		var stories = group.getStories();
		stories.stream().filter(s -> s.getId().equals(story.getId())).collect(Collectors.toList()).forEach(group::detachStory);

		if (stories.size() == 0) throw new ApplicationException(STORY_GROUP_EMPTY, groupId);

		return storyGroupRepository.save(group);
	}

	@Override
	public Story replaceStoryAttachment(UUID groupId, UUID id, MultipartFile file) {
		if (!AttachmentType.IMAGE.equals(getCorrespondingType(file.getContentType())))
			throw new ApplicationException(STORY_WRONG_FILE_TYPE);

		return recordAdapter.replaceAttachments(getStory(groupId, id), file);
	}
}
