package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Achievement;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.entity.Task;
import ru.loylabs.portal.entity.TaskTag;
import ru.loylabs.portal.enums.TaskStatus;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.TaskEvents;
import ru.loylabs.portal.mapper.TaskMapper;
import ru.loylabs.portal.presentation.request.*;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.*;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.RecordAdapter;
import ru.loylabs.portal.service.interfaces.TaskService;

import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.stream.Collectors;

import static java.util.Comparator.comparingInt;
import static java.util.Objects.nonNull;
import static ru.loylabs.portal.enums.AchievementStatus.ARCHIVED;
import static ru.loylabs.portal.enums.RecordType.TASK;
import static ru.loylabs.portal.exception.BaseException.Exception.*;
import static ru.loylabs.portal.utility.CollectionUtils.isValid;


@Service
@RequiredArgsConstructor
public class TaskServiceImpl extends AbstractFilteringAndSortingService<Task> implements TaskService {

	private final TaskMapper taskMapper;
	private final FilterUtils filterUtils;
	private final RecordAdapter recordAdapter;
	private final TaskRepository taskRepository;
	private final AchievementRepository achievementRepository;
	private final PersonRepository personRepository;
	private final TaskTagRepository tagRepository;
	private final ApplicationEventPublisher publisher;


	@Override
	public Page<Task> getAllTasks(UUID id, int page, int size, String sort,
								  SearchReversePolishNotation searchCriteria) {
		searchCriteria.add(SearchCriteria.builder().key("achievement.id").operation(":").value(id).build());
		var listFilter = filterUtils.getListFilter(page, size, sort, searchCriteria, Task.class);
		return taskRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Task getTask(UUID achievementId, UUID taskId) {
		return taskRepository.findByIdAndAchievementId(taskId, achievementId)
				.orElseThrow(() -> new ApplicationException(TASK_NOT_FOUND, taskId, achievementId));
	}

	@Override
	public Task createTask(CreateTaskRequest request, UUID achievementId, Collection<MultipartFile> files, Person author) {
		Achievement achievement = getAchievement(achievementId);
		Set<Person> observers;

		if (isValid(request.getObserverIds()) && !request.getObserverIds().contains(achievement.getOwner().getId()))
			observers = convertIdsToPerson(request.getObserverIds());
		else observers = new HashSet<>(achievement.getObservers());

		var tags = request.getTagIds()
				.stream()
				.map(this::getTag)
				.collect(Collectors.toSet());

		var task = taskRepository.save(taskMapper.toEntity(
				recordAdapter.createRecord(
						new CreateRecordRequest(request.getContent()),
						author,
						null,
						TASK,
						files),
				request,
				achievement,
				observers,
				tags,
				taskRepository.getMaxSequenceInStatus(achievementId, request.getStatus()) + 1
		));
		publisher.publishEvent(TaskEvents.TaskCreateEvent.of(achievementId, task.getId()));
		return task;
	}

	@Override
	@Transactional
	public Task updateTask(UpdateTaskRequest request, UUID achievementId, UUID taskId, Person updateBy) {
		var task = getTask(achievementId, taskId);
		checkTaskUpdatable(task);
		var oldObserversIds = task.getObservers().stream().map(Person::getId).collect(Collectors.toSet());
		if (nonNull(request.getObserverIds()) && !request.getObserverIds().contains(task.getAchievement().getOwner().getId())) {

			var observers = request.getObserverIds().isEmpty() ?
					Set.of(task.getAuthor())
					:
					convertIdsToPerson(request.getObserverIds());

			task.getObservers().clear();
			task.getObservers().addAll(observers);
		}
		task.setModified(true);
		task = taskRepository.save(taskMapper.toEntity(request, task));

		publisher.publishEvent(TaskEvents.TaskUpdateObserversEvent.of(achievementId,
				taskId,
				updateBy.getId(),
				oldObserversIds));

		return task;
	}

	@Override
	@Transactional
	public Task updateTaskStatusAndSequence(UpdateTaskSequenceStatusRequest request, UUID achievementId, UUID taskId,Person updateBy) {
		var task = getTask(achievementId, taskId);
		checkTaskUpdatable(task);

		if (task.getSequence() == request.getSequence() && taskInStatus(task, request.getStatus()))
			return task;

		if (!taskInStatus(task, request.getStatus()))
			decreaseSequenceInStatus(task.getSequence(), task.getStatus(), task);

		var status = request.getStatus() == null ? task.getStatus() : request.getStatus();
		increaseSequenceInStatus(request.getSequence(), status, task);
		task = taskRepository.save(taskMapper.toEntity(request, task));
		publisher.publishEvent(TaskEvents.TaskStatusUpdateEvent.of(achievementId,taskId,updateBy.getId()));
		return task;
	}

	@Override
	@Transactional
	public Task updateTaskStatus(UUID achievementId, UUID id, TaskStatus status,Person updateBy) {
		var task = getTask(achievementId, id);
		checkTaskUpdatable(task);
		if (taskInStatus(task, status)) throw new ApplicationException(TASK_WRONG_STATUS, status);
		decreaseSequenceInStatus(task.getSequence(), task.getStatus(), task);
		task.setStatus(status);
		task.setSequence(taskRepository.getMaxSequenceInStatus(achievementId, status) + 1);
		task = taskRepository.save(task);
		publisher.publishEvent(TaskEvents.TaskStatusUpdateEvent.of(achievementId,id,updateBy.getId()));
		return task;
	}

	@Override
	@Transactional
	public void deleteTask(UUID achievementId, UUID taskId) {
		var task = getTask(achievementId, taskId);
		var achievement = task.getAchievement();
		var commentIds = task.getComments()
				.stream()
				.map(Record::getId)
				.collect(Collectors.toSet());

		checkTaskUpdatable(task);
		decreaseSequenceInStatus(task.getSequence(), task.getStatus(), task);
		achievement.getTasks().remove(task);
		publisher.publishEvent(TaskEvents.TaskDeleteEvent.of(achievementId,taskId,commentIds));
		achievementRepository.save(achievement);
	}

	@Override
	public Task addAttachments(UUID achievementId, UUID taskId, Collection<MultipartFile> files) {
		return recordAdapter.addAttachments(getTask(achievementId, taskId), files);
	}

	@Override
	public Task deleteAttachments(UUID achievementId, UUID taskId, DeleteAttachmentsRequest request) {
		return recordAdapter.deleteAttachments(getTask(achievementId, taskId), request);
	}

	//-------------------------------------------------------------------------------------------------------------------//

	private Achievement getAchievement(UUID achievementId) {
		return achievementRepository.findById(achievementId)
				.orElseThrow(() -> new ApplicationException(ACHIEVEMENT_NOT_FOUND, achievementId));
	}

	private Person getPerson(UUID personId) {
		return personRepository.findById(personId)
				.orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, personId));
	}

	private Set<Person> convertIdsToPerson(Collection<UUID> uuids) {
		return uuids.stream().map(this::getPerson).collect(Collectors.toSet());
	}

	private TaskTag getTag(UUID tagId) {
		return tagRepository.findById(tagId)
				.orElseThrow(() -> new ApplicationException(TASK_TAG_NOT_FOUND, tagId));
	}

	private void checkTaskUpdatable(Task task) {
		if (task.getAchievement().getStatus() == ARCHIVED)
			throw new ApplicationException(ACHIEVEMENT_NOT_UPDATABLE, task.getAchievement().getId());
	}

	private boolean taskInStatus(Task task, TaskStatus status) {
		return task.getStatus() == status;
	}

	private void increaseSequenceInStatus(int startSequence, TaskStatus status, Task task) {
		var count = new AtomicInteger(startSequence);
		taskRepository.saveAll(task.getAchievement().getTasks().stream()
				.filter(t -> t.getStatus() == status
						&& !t.getId().equals(task.getId())
						&& t.getSequence() >= startSequence)
				.sorted(comparingInt(Task::getSequence))
				.peek(t -> t.setSequence(count.incrementAndGet()))
				.collect(Collectors.toList()));
	}

	private void decreaseSequenceInStatus(int startSequence, TaskStatus status, Task task) {
		var count = new AtomicInteger(startSequence);
		taskRepository.saveAll(task.getAchievement().getTasks().stream()
				.filter(t -> t.getStatus() == status
						&& !t.getId().equals(task.getId())
						&& t.getSequence() >= startSequence)
				.sorted(comparingInt(Task::getSequence))
				.peek(t -> t.setSequence(count.getAndIncrement()))
				.collect(Collectors.toList()));
	}

}
