package ru.loylabs.portal.service;

import static ru.loylabs.portal.exception.BaseException.Exception.ACHIEVEMENT_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.TASK_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.TASK_TAG_EXIST_ON_TASK;
import static ru.loylabs.portal.exception.BaseException.Exception.TASK_TAG_NOT_FOUND;

import java.util.Collection;
import java.util.UUID;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.Achievement;
import ru.loylabs.portal.entity.Task;
import ru.loylabs.portal.entity.TaskTag;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.TaskTagMapper;
import ru.loylabs.portal.presentation.request.CreateTaskTagRequest;
import ru.loylabs.portal.presentation.request.UpdateTaskTagRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.AchievementRepository;
import ru.loylabs.portal.repository.TaskRepository;
import ru.loylabs.portal.repository.TaskTagRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.TaskTagService;

@Service
@RequiredArgsConstructor
public class TaskTagServiceImpl extends AbstractFilteringAndSortingService<TaskTag> implements TaskTagService {

	private final FilterUtils filterUtils;
	private final TaskTagRepository tagRepository;
	private final TaskTagMapper tagMapper;
	private final TaskRepository taskRepository;
	private final AchievementRepository achievementRepository;


	@Override
	public Page<TaskTag> getTagsInAchievement(int page, int size, String sort, SearchReversePolishNotation criteria, UUID achievementId) {
		checkingAchievementExists(achievementId);
		criteria.add(SearchCriteria.builder().key("achievement.id").operation(":").value(achievementId).build());
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, TaskTag.class);
		return tagRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public TaskTag getTaskTag(UUID tagId) {
		return tagRepository.findById(tagId).orElseThrow(() -> new ApplicationException(TASK_TAG_NOT_FOUND, tagId));
	}

	@Override
	public TaskTag createTaskTag(UUID achievementId, CreateTaskTagRequest request) {
		var achievement = getAchievement(achievementId);
		return tagRepository.save(tagMapper.toEntity(request, achievement));
	}

	@Override
	public TaskTag updateTaskTag(UUID achievementId, UUID tagId, UpdateTaskTagRequest request) {
		var tag = tagRepository.findByIdAndAchievementId(tagId, achievementId)
				.orElseThrow(() -> new ApplicationException(TASK_TAG_NOT_FOUND, tagId));
		return tagRepository.save(tagMapper.toEntity(request, tag));
	}

	@Override
	@Transactional
	public void deleteTaskTag(UUID achievementId, UUID tagId) {
		if (tagRepository.deleteByIdAndAchievementId(tagId, achievementId) != 1)
			throw new ApplicationException(TASK_TAG_NOT_FOUND, tagId);
	}

	@Override
	public void addTagsOnTask(UUID achievementId, UUID taskId, Collection<UUID> tagIds) {
		var task = getTask(achievementId, taskId);
		var tagsOnTask = task.getTags();
		var tags = tagIds.stream().map(this::getTaskTag).collect(Collectors.toSet());
		tags.forEach(tag -> {
			if (tagsOnTask.contains(tag))
				throw new ApplicationException(TASK_TAG_EXIST_ON_TASK, tag.getId(), taskId, achievementId);
		});
		tagsOnTask.addAll(tags);
		taskRepository.save(task);
	}

	@Override
	@Transactional
	public void deleteTagFromTask(UUID achievementId, UUID taskId, UUID tagId) {
		var task = getTask(achievementId, taskId);
		if (!task.getTags().remove(getTaskTag(tagId)))
			throw new ApplicationException(TASK_TAG_EXIST_ON_TASK, tagId, taskId, achievementId);
		taskRepository.save(task);
	}


	private Task getTask(UUID achievementId, UUID taskId) {
		return taskRepository.findByIdAndAchievementId(taskId, achievementId)
				.orElseThrow(() -> new ApplicationException(TASK_NOT_FOUND, taskId, achievementId));
	}

	private Achievement getAchievement(UUID achievementId) {
		return achievementRepository.findById(achievementId)
				.orElseThrow(() -> new ApplicationException(ACHIEVEMENT_NOT_FOUND, achievementId));
	}

	private void checkingAchievementExists(UUID achievementId){
		if(!achievementRepository.existsById(achievementId))
			throw new ApplicationException(ACHIEVEMENT_NOT_FOUND, achievementId);
	}
}
