package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.Title;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.TitleMapper;
import ru.loylabs.portal.presentation.request.CreateTitleRequest;
import ru.loylabs.portal.presentation.request.UpdateTitleRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.TitleRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.JobAdapter;
import ru.loylabs.portal.service.interfaces.TitleService;

import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.*;

@Service
@RequiredArgsConstructor
public class TitleServiceImpl extends AbstractFilteringAndSortingService<Title> implements TitleService {

	private final TitleRepository titleRepository;
	private final PersonRepository personRepository;
	private final TitleMapper titleMapper;
	private final FilterUtils filterUtils;
	private final JobAdapter jobAdapter;

	@Override
	public Page<Title> getAllTitle(int page, int size, String sort, SearchReversePolishNotation criteria) {
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, Title.class);
		return titleRepository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public Title getTitleById(UUID id) {
		return titleRepository.findById(id).orElseThrow(()
				-> new ApplicationException(TITLE_NOT_FOUND));
	}

	public Title createTitle(CreateTitleRequest request) {
		checkUniqGradeName(request.getName());
		return titleRepository.save(titleMapper.toEntity(request));
	}

	public Title updateTitle(UUID id, UpdateTitleRequest request) {
		var title = getTitleById(id);
		checkUniqGradeName(request.getName(), title.getName());
		return titleRepository.save((titleMapper.toEntity(title, request)));
	}

	public void deleteTitle(UUID id) {
		if (titleRepository.getDefaultTitle().getId().equals(id))
			throw new ApplicationException(TITLE_BAN_ON_DELETION, id);

		var title = getTitleById(id);
		personRepository.findAllByJobTitle(title)
				.forEach(person -> jobAdapter.setDefaultTitle(person.getJob()));
		titleRepository.delete(title);
	}

	private void checkUniqGradeName(String name) {
		if (name != null && titleRepository.existsByNameIgnoreCase(name))
			throw new ApplicationException(TITLE_ALREADY_EXISTS, name);
	}

	private void checkUniqGradeName(String newName, String oldName) {
		if (newName != null && !oldName.equals(newName) && titleRepository.existsByNameIgnoreCase(newName))
			throw new ApplicationException(TITLE_ALREADY_EXISTS, newName);
	}
}
