package ru.loylabs.portal.service;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.RandomStringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.CalendarYandexData;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.YandexCalendarEvent;
import ru.loylabs.portal.entity.caldav.generated.dav.Multistatus;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.repository.CalendarYandexDataRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.interfaces.YandexCalendarService;
import ru.loylabs.portal.utility.CryptoUtil;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBException;
import java.io.IOException;
import java.io.StringReader;
import java.net.Authenticator;
import java.net.PasswordAuthentication;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import java.util.stream.Collectors;

import static ru.loylabs.portal.exception.BaseException.Exception.CONTACT_EMAIL_ALREADY_EXIST;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.YANDEX_CALENDAR_DATA_ERROR;

@Slf4j
@Service
@RequiredArgsConstructor
public class YandexCalendarServiceImpl implements YandexCalendarService {
	@Value("${uri.yandex.caldav}")
	private String yandexCaldavUri;

	private final static String CALDAV_REQUEST_BODY =
			"<d:propfind xmlns:d=\"DAV:\" xmlns:cs=\"https://caldav.yandex.ru/\" xmlns:c=\"urn:ietf:params:xml:ns:caldav\">\n" +
					"    <d:prop>\n" + "        <c:calendar-data />\n" + "    </d:prop>\n" + "    <c:filter>\n" +
					"        <c:comp-filter name=\"VCALENDAR\">\n" + "            <c:comp-filter name=\"VEVENT\" />\n" +
					"        </c:comp-filter>\n" + "    </c:filter>\n" + "</d:propfind>";

	public static final String SEND_EVENT_BODY_TEMPLATE =
			"BEGIN:VCALENDAR\n" + "BEGIN:VTIMEZONE\n" + "TZID:Europe/Moscow\n" +
					"TZURL:http://tzurl.org/zoneinfo/Europe/Moscow\n" + "X-LIC-LOCATION:Europe/Moscow\n" + "END:VTIMEZONE\n" +
					"BEGIN:VEVENT\n" + "UID:%s\n" + "SUMMARY:%s\n" + "DTSTART:%s\n" + "DTEND:%s\n" + "END:VEVENT\n" +
					"END:VCALENDAR";
	private final CalendarYandexDataRepository calYaDataRepo;
	private final PersonRepository personRepo;

	public List<YandexCalendarEvent> getEvents(UUID personId) {
		final CalendarYandexData calYaData =
				calYaDataRepo.findByPersonId(personId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND));
		try {
			return get(calYaData.getEmail(), CryptoUtil.decrypt(calYaData.getPassword()));
		} catch (URISyntaxException | JAXBException | IOException | InterruptedException e) {
			log.error("Error occurred during calendar fetching: {}", e.getMessage());
			return Collections.emptyList();
		}
	}

	public void addEvent(UUID personId,
						 String eventDirectoryName,
						 String eventName,
						 String dateStartString,
						 String dateEndString) {
		final CalendarYandexData calYaData =
				calYaDataRepo.findByPersonId(personId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND));
		try {
			add(calYaData.getEmail(),
					CryptoUtil.decrypt(calYaData.getPassword()),
					eventDirectoryName,
					eventName,
					dateStartString,
					dateEndString);
		} catch (URISyntaxException | IOException | InterruptedException e) {
			log.error("Error occurred during calendar fetching: {}", e.getMessage());
		}
	}

	public void deleteEvent(UUID personId, String eventDirectoryName, String eventUid) {
		final CalendarYandexData calYaData =
				calYaDataRepo.findByPersonId(personId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND));
		try {
			delete(calYaData.getEmail(), CryptoUtil.decrypt(calYaData.getPassword()), eventDirectoryName, eventUid);
		} catch (URISyntaxException | IOException | InterruptedException e) {
			log.error("Error occurred during calendar fetching: {}", e.getMessage());
		}
	}

	public void saveYandexCredentials(UUID personId, String email, String password) {
		final String encrypt = CryptoUtil.encrypt(password);

		final Person person =
				personRepo.findById(personId).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND));
		final CalendarYandexData cyd =
				CalendarYandexData.builder().person(person).email(email).password(encrypt).build();
		if (calYaDataRepo.findByEmail(email).isPresent()) throw new ApplicationException(CONTACT_EMAIL_ALREADY_EXIST, email); // TODO уникальность нужна ли тут?
		calYaDataRepo.save(cyd);
	}

	public void deleteYandexCredentials(UUID personId) {
		calYaDataRepo.deleteByPersonId(personId);
	}

	private List<YandexCalendarEvent> get(String email, String password)
			throws URISyntaxException, JAXBException, IOException, InterruptedException {
		String url = yandexCaldavUri + email;

		final HttpClient client = getHttpClient(email, password);
		final HttpRequest request = HttpRequest.newBuilder(new URI(url))
				.method("PROPFIND",
						HttpRequest.BodyPublishers.ofString(CALDAV_REQUEST_BODY))
				.build();

		final HttpResponse<String> send = client.send(request, HttpResponse.BodyHandlers.ofString());
		final String stringResponse = send.body();

		final JAXBContext jaxbContext = JAXBContext.newInstance("ru.loylabs.portal.entity.caldav.generated.dav");
		final Multistatus unmarshal =
				(Multistatus) jaxbContext.createUnmarshaller().unmarshal(new StringReader(stringResponse));

		final String eventDirectoryName = getEventDirectoryName(unmarshal);

		return unmarshal.getResponses()
				.stream()
				.filter(r -> r.getHref().contains(".ics"))
				.map(r -> r.getPropstat().getProp().getCalendarData())
				.map(s -> handleEventResponse(s, eventDirectoryName))
				.collect(Collectors.toList());
	}

	private void add(String email,
					 String password,
					 String eventDirectoryName,
					 String eventName,
					 String dateStartString,
					 String dateEndString) throws URISyntaxException, IOException, InterruptedException {
		final HttpClient client = getHttpClient(email, password);
		final String random = RandomStringUtils.random(10, true, true);
		final HttpRequest request =
				HttpRequest.newBuilder(new URI(yandexCaldavUri + email + eventDirectoryName + "/" + random + ".ics"))
						.setHeader("Content-Type", "text/calendar; charset=utf-8")
						.method("PUT",
								HttpRequest.BodyPublishers.ofString(String.format(SEND_EVENT_BODY_TEMPLATE,
										random,
										eventName,
										dateStartString,
										dateEndString)))
						.build();

		client.send(request, HttpResponse.BodyHandlers.ofString());
	}

	private void delete(String email, String password, String eventDirectoryName, String uid)
			throws URISyntaxException, IOException, InterruptedException {
		final HttpClient client = getHttpClient(email, password);
		final HttpRequest request =
				HttpRequest.newBuilder(new URI(yandexCaldavUri + email + eventDirectoryName + "/" + uid + ".ics"))
						.method("DELETE", HttpRequest.BodyPublishers.noBody())
						.build();

		client.send(request, HttpResponse.BodyHandlers.ofString());
	}

	private YandexCalendarEvent handleEventResponse(String stringResponse, String eventDirectoryName) {
		final Map<String, String> collect = Arrays.stream(stringResponse.split("\n")).map(r -> {
			if (r.startsWith("DTSTART;TZID") || r.startsWith("DTEND;TZID")) {
				return r.split(";")[0] + "FACT:" + r.substring(r.length() - 15).trim();
			} else {
				return r;
			}
		}).collect(Collectors.toMap(k -> k.split(":")[0], v -> v.split(":")[1].trim(), (a, b) -> a));

		final String startTimestamp = collect.get("DTSTARTFACT");
		final String endTimestamp = collect.get("DTENDFACT");
		final String timezoneId = collect.get("TZID");
		final String eventName = collect.get("SUMMARY");
		final String uid = collect.get("UID");
		final DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyyMMdd'T'HHmmss");
		final ZonedDateTime startDateTime =
				LocalDateTime.parse(startTimestamp, formatter).atZone(ZoneId.of(timezoneId));
		final ZonedDateTime endDateTime = LocalDateTime.parse(endTimestamp, formatter).atZone(ZoneId.of(timezoneId));
		return new YandexCalendarEvent(eventName, eventDirectoryName, uid, startDateTime, endDateTime);
	}

	private HttpClient getHttpClient(String username, String pwd) {
		return HttpClient.newBuilder().authenticator(new Authenticator() {
			@Override
			protected PasswordAuthentication getPasswordAuthentication() {
				return new PasswordAuthentication(username, pwd.toCharArray());
			}
		}).build();
	}

	private String getEventDirectoryName(Multistatus unmarshal) {
		return unmarshal.getResponses().stream().filter(r -> r.getHref().contains("events-")).limit(1).map(r -> {
			final String href = r.getHref();
			Pattern pattern = Pattern.compile("events-[a-zA-Z0-9]+");
			Matcher matcher = pattern.matcher(href);
			if (matcher.find()) {
				return matcher.group();
			} else throw new ApplicationException(YANDEX_CALENDAR_DATA_ERROR);
		}).findFirst().orElseThrow(() -> new ApplicationException(YANDEX_CALENDAR_DATA_ERROR));
	}
}
