package ru.loylabs.portal.service.adapter;

import static ru.loylabs.portal.utility.FileAttachmentTypeFactory.getCorrespondingType;

import java.time.Instant;
import java.util.Collection;
import java.util.HashSet;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import org.apache.commons.io.FilenameUtils;
import ru.loylabs.portal.entity.Attachment;
import ru.loylabs.portal.mapper.AttachmentMapper;
import ru.loylabs.portal.repository.AttachmentRepository;
import ru.loylabs.portal.service.interfaces.AttachmentAdapter;
import ru.loylabs.portal.service.interfaces.FileStorage;

@Component
@RequiredArgsConstructor
public class AttachmentAdapterImpl implements AttachmentAdapter {

	private final FileStorage storage;

	private final AttachmentMapper attachmentMapper;

	private final AttachmentRepository attachmentRepository;

	@Override
	@Transactional
	public void addAttachments(Set<Attachment> attachments, Collection<MultipartFile> files) {
		if (files == null) return;
		for (MultipartFile file : files) {
			if (file != null && !file.isEmpty()) attachments.add(createAttachment(file));
		}
	}

	@Override
	@Transactional
	public void deleteAttachments(Set<Attachment> attachments, Set<UUID> attachmentsIds) {
		attachments.removeAll(new HashSet<>(attachmentRepository.findAllById(attachmentsIds)));
		attachmentRepository.flush();
		attachmentRepository.deleteAllByIdInBatch(attachmentsIds);
	}

	@Override
	@Transactional
	public void replaceAttachment(Set<Attachment> attachments, MultipartFile file) {
		if (file != null && !file.isEmpty()) {
			var attachmentsIds = attachments.stream().map(Attachment::getId).collect(Collectors.toSet());
			attachments.clear();
			attachmentRepository.flush();
			attachmentRepository.deleteAllByIdInBatch(attachmentsIds);
			attachments.add(createAttachment(file));
		}
	}

	@Override
	@SneakyThrows
	public Attachment createAttachment(MultipartFile file) {
		if (file == null) return null;
		String filename = generateNewName();
		String originalFilename = file.getOriginalFilename();
		return attachmentMapper.toEntity(
				getCorrespondingType(file.getContentType()),
				FilenameUtils.getBaseName(filename),
				"." + FilenameUtils.getExtension(originalFilename),
				file.getSize(),
				storage.save(file.getBytes(), filename + "." + FilenameUtils.getExtension(originalFilename)));
	}

	private String generateNewName() {
		return Instant.now().getEpochSecond() + "_" + Instant.now().getNano();
	}

}
