package ru.loylabs.portal.service.adapter;

import lombok.RequiredArgsConstructor;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Component;
import ru.loylabs.portal.entity.Job;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.PersonEvents;
import ru.loylabs.portal.mapper.JobMapper;
import ru.loylabs.portal.presentation.request.CreateJobRequest;
import ru.loylabs.portal.presentation.request.UpdateJobRequest;
import ru.loylabs.portal.repository.DepartmentRepository;
import ru.loylabs.portal.repository.GradeRepository;
import ru.loylabs.portal.repository.JobRepository;
import ru.loylabs.portal.repository.TitleRepository;
import ru.loylabs.portal.service.interfaces.JobAdapter;
import ru.loylabs.portal.service.yandex.dto.Yandex360UserResponse;

import static ru.loylabs.portal.exception.BaseException.Exception.DEPARTMENT_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.GRADE_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.TITLE_NOT_FOUND;

@Component
@RequiredArgsConstructor
public class JobAdapterImpl implements JobAdapter {

	private final JobMapper jobMapper;
	private final JobRepository jobRepository;
	private final TitleRepository titleRepository;
	private final DepartmentRepository departmentRepository;
	private final GradeRepository gradeRepository;

	private final ApplicationEventPublisher publisher;

	@Override
	public Job createJob(CreateJobRequest request) {
		return jobRepository.save(jobMapper.toEntity(
				request,
				titleRepository.findByName(request.getTitle())
							   .orElseThrow(() -> new ApplicationException(TITLE_NOT_FOUND)),
				departmentRepository.findByName(request.getDepartment())
									.orElseThrow(() -> new ApplicationException(DEPARTMENT_NOT_FOUND)),
				gradeRepository.findByName(request.getGrade()).orElseThrow(() -> new ApplicationException(GRADE_NOT_FOUND))));
	}

	@Override
	public Job createJob(Yandex360UserResponse response) {
		return jobRepository.save(jobMapper.toEntity(
				response,
				titleRepository.findByNameIgnoreCase(response.getPosition().trim())
							   /* Fallback value in a very likely case Yandex 'position' doesn't match any of portal's
							      'titles'. */
							   .orElseGet(titleRepository::getDefaultTitle),
				departmentRepository.findByYandexDepartmentId(response.getDepartmentId())
									.orElseGet(departmentRepository::getDefaultDepartment)));
	}

	@Override
	public Job updateJob(UpdateJobRequest request, Job job) {
		publisher.publishEvent(PersonEvents.PersonChangedJobEvent.of(job.getPerson().getId()));

		return jobRepository.save(jobMapper.toEntity(
				job,
				request.getGrade() == null ? null : gradeRepository.findByName(request.getGrade())
                                                                   .orElseThrow(() -> new ApplicationException(GRADE_NOT_FOUND)),
				request.getTitle() == null ? null : titleRepository.findByName(request.getTitle())
																   .orElseThrow(() -> new ApplicationException(TITLE_NOT_FOUND)),
				request.getDepartment() == null ? null : departmentRepository.findByName(request.getDepartment())
																			 .orElseThrow(() -> new ApplicationException(DEPARTMENT_NOT_FOUND))));
	}

	@Override
	public Job setDefaultDepartment(Job job) {
		publisher.publishEvent(PersonEvents.PersonChangedJobEvent.of(job.getPerson().getId()));

		job.setDepartment(departmentRepository.getDefaultDepartment());
		return jobRepository.save(job);
	}

	@Override
	public Job setDefaultTitle(Job job) {
		publisher.publishEvent(PersonEvents.PersonChangedJobEvent.of(job.getPerson().getId()));

		job.setTitle(titleRepository.getDefaultTitle());
		return jobRepository.save(job);
	}

	@Override
	public Job setDefaultGrade(Job job) {
		publisher.publishEvent(PersonEvents.PersonChangedJobEvent.of(job.getPerson().getId()));

		job.setGrade(gradeRepository.getDefaultGrade());
		return jobRepository.save(job);
	}
}
