package ru.loylabs.portal.service.filter;

import org.hibernate.query.criteria.internal.BasicPathUsageException;
import org.hibernate.query.criteria.internal.path.SingularAttributeJoin;
import org.springframework.data.jpa.domain.Specification;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchOperation;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;

import javax.persistence.criteria.*;
import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import static ru.loylabs.portal.exception.BaseException.Exception.SEARCH_FIELD_NOT_SEARCHABLE;

public class SpecificationRequest<T> implements Specification<T> {

	private final SearchCriteria criteria;

	private final Class<?> baseClassType;

	public SpecificationRequest(SearchCriteria criteria, Class<?> type) {
		super();
		this.criteria = criteria;
		this.baseClassType = type;
	}

	public static <T> Specification<T> of(SearchReversePolishNotation order, Class<?> type) {
		var stack = new LinkedList<Specification<T>>();

		for (var criteria : order.get()) {
			if (criteria instanceof SearchCriteria)
				stack.push(new SpecificationRequest<>((SearchCriteria) criteria, type));
			if (criteria instanceof SearchOperation) {
				var secondSpecification = stack.pop();
				var firstSpecification = stack.peek();
				if (firstSpecification == null) stack.push(secondSpecification);
				else {
					stack.pop();
					if (criteria == SearchOperation.OR) stack.push(firstSpecification.or(secondSpecification));
					else stack.push(firstSpecification.and(secondSpecification));
				}
			}
		}

		return stack.peek();
	}

	@Override
	public Predicate toPredicate(Root<T> root, CriteriaQuery<?> query, CriteriaBuilder criteriaBuilder) {
		if(criteria.getKey().equals("tasks.observers.id")) query.distinct(true); // TODO: 31.10.2023 Solve this problem more elegantly
		switch (criteria.getOperation()) {
			case ">":
				return criteriaBuilder.greaterThan(getPath(root), getValue());
			case ">:":
				return criteriaBuilder.greaterThanOrEqualTo(getPath(root), getValue());
			case "<":
				return criteriaBuilder.lessThan(getPath(root), getValue());
			case "<:":
				return criteriaBuilder.lessThanOrEqualTo(getPath(root), getValue());
			case "!:":
				if (getPath(root).getJavaType() == String.class) {
					return criteriaBuilder.notLike(criteriaBuilder.lower(getPath(root)), "%" + getValue().toString().toLowerCase() + "%");
				}
				if (getValue().toString().equals("null")) {
					return getPath(root).getJavaType() == List.class || getPath(root).getJavaType() == Set.class ?
							criteriaBuilder.isNotEmpty(getPath(root)) :
							criteriaBuilder.isNotNull(getPath(root));
				}
				return criteriaBuilder.notEqual(getPath(root), getValue());
			case ":":
				if (getPath(root).getJavaType() == String.class) {
					return criteriaBuilder.like(criteriaBuilder.lower(getPath(root)), "%" + getValue().toString().toLowerCase() + "%");
				}
				if (getValue().toString().equals("null")) {
					return getPath(root).getJavaType() == List.class || getPath(root).getJavaType() == Set.class ?
							criteriaBuilder.isEmpty(getPath(root)) :
							criteriaBuilder.isNull(getPath(root));
				}
				return criteriaBuilder.equal(getPath(root), getValue());
			default:
				return null;
		}
	}

	private Expression getPath(Root<T> root) {
		var field = criteria.getKey();
		if (field.contains(".")) {
			var split = field.split("\\.");
			var join = root.join(split[0]);
			for (int i = 1; i < split.length - 1; i++) {
				try {
					join = join.join(split[i]);
				} catch (BasicPathUsageException e) {
					throw new ApplicationException(SEARCH_FIELD_NOT_SEARCHABLE,
							split[i],
							((SingularAttributeJoin) join).getJavaType().toString().replace("class ", ""));
				}
			}
			return join.get(split[split.length - 1]);
		}
		return root.get(field);
	}

	private Comparable getValue() {
		var value = (Comparable) criteria.getValue();
		try {
			var fieldType = searchField(criteria.getKey()).getType();
			if (fieldType.isEnum()) {
				var enumValue = Enum.valueOf((Class<Enum>) fieldType, String.valueOf(value).toUpperCase());
				value = (Comparable) enumValue;
			}
		} catch (Exception ignored) {
		}
		return value;
	}

	private Field searchField(String name) {
		var fields = name.split("\\.");
		Field field = null;
		Class<?> fieldForSearch = baseClassType;
		for (String s : fields) {
			field = searchField(s, fieldForSearch);
			fieldForSearch = field.getType();
		}
		return field;
	}

	private Field searchField(String name, Class<?> type) {
		var allFields = getAllFields(type);
		for (Field field : allFields) {
			if (field.getName().equals(name)) {
				return field;
			}
		}
		return null;
	}

	List<Field> getAllFields(Class<?> type) {
		if (type == null) {
			return new ArrayList<>();
		}

		List<Field> result = getAllFields(type.getSuperclass());
		result.addAll(List.of(type.getDeclaredFields()));
		return result;
	}
}
