package ru.loylabs.portal.service.notification.bell;

import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.BellNotification;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.BellNotificationType;
import ru.loylabs.portal.enums.CalendarEventType;
import ru.loylabs.portal.listener.events.PersonEvents;
import ru.loylabs.portal.repository.BellNotificationRepository;
import ru.loylabs.portal.repository.CorporateCalendarEventRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.TaskRepository;
import ru.loylabs.portal.repository.redis.TaskDoneRepository;
import ru.loylabs.portal.service.interfaces.BellNotificationFactory;
import ru.loylabs.portal.service.interfaces.BellNotificationPersister;
import ru.loylabs.portal.service.interfaces.BellNotificationSchedulingService;
import ru.loylabs.portal.service.notification.bell.data.CalendarUpcomingEventNotificationData;
import ru.loylabs.portal.service.notification.bell.data.TaskDoneNotificationData;

import java.time.Instant;
import java.time.LocalDate;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Objects;
import java.util.Set;
import java.util.UUID;

import static java.util.stream.Collectors.toSet;
import static ru.loylabs.portal.enums.BellNotificationType.CALENDAR_UPCOMING_EVENT;
import static ru.loylabs.portal.enums.BellNotificationType.PERSON_BIRTHDAY;
import static ru.loylabs.portal.enums.BellNotificationType.PERSON_UPCOMING_BIRTHDAY;
import static ru.loylabs.portal.enums.TaskStatus.DONE;

@Component
@RequiredArgsConstructor
public class BellNotificationSchedulingServiceImpl implements BellNotificationSchedulingService {

	private final ApplicationEventPublisher publisher;
	private final BellNotificationRepository repository;
	private final CorporateCalendarEventRepository calendarRepository;
	private final BellNotificationFactory notificationFactory;
	private final PersonRepository personRepository;
	private final TaskRepository taskRepository;
	private final TaskDoneRepository taskDoneRepository;
	private final BellNotificationPersister persister;

	@Value("${notification.bell.ttl.hours}")
	private long ttlHours;

	@Override
	public int upcomingEventNotification() {
		ZonedDateTime tomorrowStart = ZonedDateTime.now().toLocalDate().plusDays(1).atStartOfDay(ZoneId.systemDefault());
		ZonedDateTime tomorrowEnd = tomorrowStart.withHour(23).withMinute(59).withSecond(59);
		var tomorrowEvents = calendarRepository.findAllByStartDateTimeBetween(tomorrowStart, tomorrowEnd);
		if (tomorrowEvents.size() == 0) return 0;
		Set<BellNotification> notifications = tomorrowEvents.stream()
				.filter(event -> event.getEventType() == CalendarEventType.CORPORATE_EVENT)
				.filter(event -> notificationNotExist(event.getId(), CALENDAR_UPCOMING_EVENT))
				.map(event -> notificationFactory.create(new CalendarUpcomingEventNotificationData(event)))
				.peek(persister::persistForAll)
				.collect(toSet());

		return notifications.size();
	}

	@Override
	public int taskDoneDelayedNotificationSend() {
		var notification = taskDoneRepository.findAllCreatedTimeAgo(15, ChronoUnit.MINUTES)
				.stream()
				.map(t -> {
					taskDoneRepository.delete(t);
					var task = taskRepository.findById(t);
					return task.filter(value -> value.getStatus() == DONE).orElseGet( null);
				})
				.filter(Objects::nonNull)
				.map(t -> notificationFactory.create(new TaskDoneNotificationData( t.getAchievement(),t)))
				.peek(n -> persister.persistForPersonWithRole(n, "ADMIN", "EDITOR"))
				.collect(toSet());

		return notification.size();

	}

	@Override
	public int upcomingPersonsBirthdayNotification() {
		var tomorrow = LocalDate.now().plusDays(1);
		var birthdayPeople = personRepository.findAllByDob(tomorrow.getMonthValue(), tomorrow.getDayOfMonth());
		if (birthdayPeople.size() == 0) return 0;
		var notifications = birthdayPeople.stream()
				.filter(p -> notificationNotExist(p.getId(), PERSON_UPCOMING_BIRTHDAY))
				.peek(p -> publisher.publishEvent(PersonEvents.PersonUpcomingDobEvent.of(p.getId())))
				.collect(toSet());

		return notifications.size();
	}

	@Override
	@Transactional
	public int birthdaysNotification() {
		var date = LocalDate.now();
		var birthdayPeople = personRepository.findAllByDob(date.getMonthValue(), date.getDayOfMonth());
		if (birthdayPeople.size() == 0) return 0;
		var notifications = birthdayPeople.stream()
				.filter(p -> personNotBlocked(p) && notificationNotExist(p.getId(), PERSON_BIRTHDAY))
				.peek(p -> {
					if (!notificationNotExist(p.getId(), PERSON_UPCOMING_BIRTHDAY)) {
						repository.deleteAllBySubjectIdAndNotificationType(p.getId(), PERSON_UPCOMING_BIRTHDAY);
					}
					publisher.publishEvent(PersonEvents.PersonDobEvent.of(p.getId()));
				})
				.collect(toSet());
		return notifications.size();
	}

	@Override
	@Transactional
	public int cleanOutdatedBellNotification() {
		Instant cutoffDate = Instant.now().minus(ttlHours, ChronoUnit.HOURS);
		return repository.deleteAllByDateIsBefore(cutoffDate);
	}

	private boolean personNotBlocked(Person person) {
		return !person.isBlocked();
	}

	private boolean notificationNotExist(UUID eventId, BellNotificationType type) {
		return !repository.existsBySubjectIdAndNotificationType(eventId, type);
	}

}
