package ru.loylabs.portal.service.notification.bell;

import lombok.RequiredArgsConstructor;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.PersonalBellNotification;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.PersonalBellNotificationRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.PersonalBellNotificationService;

import java.util.Collection;
import java.util.UUID;

import static ru.loylabs.portal.exception.BaseException.Exception.BELL_NOTIFICATION_NOT_FOUND;

@Component
@RequiredArgsConstructor
public class PersonalBellNotificationServiceImpl extends AbstractFilteringAndSortingService<PersonalBellNotification> implements PersonalBellNotificationService {

	private final PersonalBellNotificationRepository repository;

	private final FilterUtils filterUtils;


	@Override
	public Page<PersonalBellNotification> getAllForPerson(Integer page,
														  Integer size,
														  String sort,
														  SearchReversePolishNotation criteria,
														  UUID personId,
														  boolean seen) {

		criteria.add(SearchCriteria.builder().key("person.id").operation(":").value(personId).build());
		criteria.add(SearchCriteria.builder().key("seen").operation(":").value(seen).build());
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, PersonalBellNotification.class);
		return repository.findAll(getSpecification(listFilter), getPageable(listFilter));
	}

	@Override
	public void markAsSeen(UUID personId, UUID notificationId) {
		var notification = repository.findByBellNotificationIdAndPersonId(notificationId, personId)
				.orElseThrow(() -> new ApplicationException(BELL_NOTIFICATION_NOT_FOUND, notificationId));
		notification.setSeen(true);
		repository.save(notification);
	}

	@Override
	public void markAsSeen(UUID personId, Collection<UUID> notificationIds) {
	 repository.findByPersonIdAndBellNotificationIdIn(personId, notificationIds).stream()
			 .peek(notification -> notification.setSeen(true))
			 .forEach(repository::save);
	}

	@Override
	public void markAllAsSeen(UUID personId) {
		repository.findAllByPersonId(personId).stream()
				.peek(notification -> notification.setSeen(true))
				.forEach(repository::save);
	}

	@Override
	public long countUnseenForPerson(UUID personId) {
		return repository.countByPersonIdAndSeenIsFalse(personId);
	}

	@Override
	@Transactional
	public void deleteNotificationForPerson(UUID notificationId, UUID personId) {
		if (repository.deleteByBellNotificationIdAndPersonId(notificationId, personId) < 1)
			throw new ApplicationException(BELL_NOTIFICATION_NOT_FOUND, notificationId);
	}

	@Override
	@Transactional
	public void deleteAllArchivedNotification(UUID personId) {
		if(repository.deleteAllByPersonIdAndSeenIsTrue(personId) < 1)
			throw new ApplicationException(BELL_NOTIFICATION_NOT_FOUND);
	}
}
