package ru.loylabs.portal.service.notification.email;

import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.util.UriComponentsBuilder;
import ru.loylabs.portal.config.FrontEndConfigProperties;
import ru.loylabs.portal.entity.BugReport;
import ru.loylabs.portal.entity.Idea;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.ReviewPersonSkill;
import ru.loylabs.portal.entity.Task;
import ru.loylabs.portal.enums.EmailNotificationType;
import ru.loylabs.portal.enums.SettingsTagType;
import ru.loylabs.portal.service.interfaces.EmailNotificationFactory;
import ru.loylabs.portal.service.interfaces.EmailNotificationManager;
import ru.loylabs.portal.utility.EmailEmbeddableFile;
import ru.loylabs.portal.utility.EmailImageUtils;
import ru.loylabs.portal.utility.settings.SettingsArgsBuilder;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Map;
import java.util.Set;

import static ru.loylabs.portal.enums.EmailNotificationType.FORBIDDEN_USER;
import static ru.loylabs.portal.enums.EmailNotificationType.FORBIDDEN_ADMIN;
import static ru.loylabs.portal.enums.EmailNotificationType.BUG_REPORT;
import static ru.loylabs.portal.enums.EmailNotificationType.GENERATED_PASSWORD;
import static ru.loylabs.portal.enums.EmailNotificationType.GENERATED_PASSWORD_FOR_ADMIN;
import static ru.loylabs.portal.enums.EmailNotificationType.GENERATED_PASSWORD_WITH_LINK;
import static ru.loylabs.portal.enums.EmailNotificationType.NEW_IDEA;
import static ru.loylabs.portal.enums.EmailNotificationType.RESET_PASSWORD;
import static ru.loylabs.portal.enums.EmailNotificationType.REVIEW_USER;
import static ru.loylabs.portal.enums.EmailNotificationType.TASK_APPROVAL;
import static ru.loylabs.portal.enums.SettingsTagType.PORTAL_LINK;
import static ru.loylabs.portal.utility.CollectionUtils.isValid;

@Component
@RequiredArgsConstructor
public class EmailNotificationFactoryImpl implements EmailNotificationFactory {

	private final EmailNotificationManager notificationManager;

	private final FrontEndConfigProperties frontEndProps;

	private final EmailImageUtils emailImageUtils;

	@Value("${bug-report.report-receivers}")
	private final List<String> reportReceivers;

	@Override
	public EmailNotification createResetPasswordMessage(Person target, String passwordCode) {
		return create(
				new SettingsArgsBuilder().target(target)
										 .passwordCode(passwordCode)
										 .emailLoginImg()
										 .emailLogoImg()
										 // Manually setting portal link because injecting values from config files in
										 // SettingsArgsBuilder is inconvenient
										 .put(PORTAL_LINK,
											  UriComponentsBuilder.newInstance()
																  .scheme(frontEndProps.getScheme())
																  .host(frontEndProps.getHost())
																  .port(frontEndProps.getPort())
																  .pathSegment(frontEndProps.getResetPasswordPath())
																  .pathSegment(passwordCode)
																  .build()
																  .toUriString())
										 .build(),
				RESET_PASSWORD,
				Collections.singleton(target),
				null
		);
	}

	@Override
	public EmailNotification createBlockedAdmin(Person target, String passwordCode) {
		return create(
				new SettingsArgsBuilder().target(target)
						.passwordCode(passwordCode)
						.emailLogoImg()
						.put(PORTAL_LINK,
								UriComponentsBuilder.newInstance()
										.scheme(frontEndProps.getScheme())
										.host(frontEndProps.getHost())
										.port(frontEndProps.getPort())
										.pathSegment(frontEndProps.getResetPasswordPath()) // TODO Изменить на ссылку со сбросом пароля и разблокировкой
										.pathSegment(passwordCode)
										.build()
										.toUriString())
						.build(),
				FORBIDDEN_ADMIN,
				Collections.singleton(target),
				null
		);
	}

	@Override
	public EmailNotification createBlockedUser(Person target, Person blockedUser) {
		return create(
				new SettingsArgsBuilder().target(target).author(blockedUser)
						.emailLogoImg()
						.author(blockedUser)
						.put(PORTAL_LINK,
								UriComponentsBuilder.newInstance()
										.scheme(frontEndProps.getScheme())
										.host(frontEndProps.getHost())
										.port(frontEndProps.getPort())
										.pathSegment("/company") // TODO Тестовый вариант, не известно заработает или нет
										.build()
										.toUriString())
						.build(),
				FORBIDDEN_USER,
				Collections.singleton(target),
				null
		);
	}

	@Override
	public EmailNotification createReview(Person reviewer, ReviewPersonSkill review) {
		return create(
				new SettingsArgsBuilder()
						.target(reviewer)
						.emailLogoImg()
						.put(PORTAL_LINK,
								UriComponentsBuilder.newInstance()
										.scheme(frontEndProps.getScheme())
										.host(frontEndProps.getHost())
										.port(frontEndProps.getPort())
										.pathSegment(String.format("profile/%s/review/%s", review.getOwner().getId().toString(), review.getId().toString()))
										.build()
										.toUriString())
						.author(review.getOwner())
						.reviewSkill(review.getSkill())
						.reviewEndDate(review.getEndReview())
						.build(),
				REVIEW_USER,
				Collections.singleton(reviewer),
				null
		);
	}

	@Override
	public EmailNotification createTaskApprovalMessage(Task task, Person taskAuthor, Person taskReviewer,
													   boolean isApproved) {
		return create(
				new SettingsArgsBuilder().target(task.getAuthor())
										 .author(task.getAchievement().getAuthor())
										 .task(task)
										 .taskApprovalResult(isApproved)
										 .build(),
				TASK_APPROVAL,
				Collections.singleton(taskAuthor),
				null
		);
	}

	@Override
	public EmailNotification createBugReportMessage(BugReport report, Collection<MultipartFile> files) {
		EmailNotification emailNotification = create(
				new SettingsArgsBuilder().emailLogoImg()
						.author(report.getAuthor())
						.bugReport(report.getContent())
						.put(PORTAL_LINK,
								UriComponentsBuilder.newInstance()
										.scheme(frontEndProps.getScheme())
										.host(frontEndProps.getHost())
										.port(frontEndProps.getPort())
										.build()
										.toUriString())
						.build(),
				BUG_REPORT,
				null,
				files
		);
		// Set manually, cause emails are not Persons =)
		emailNotification.setReceiverEmails(reportReceivers);

		return emailNotification;
	}

	@Override
	public EmailNotification createNewIdeaMessage(Person target, Person ideaAuthor, Idea idea) {
		return create(
				new SettingsArgsBuilder()
						.target(target)
						.emailLogoImg()
						.put(PORTAL_LINK,
								UriComponentsBuilder.newInstance()
										.scheme(frontEndProps.getScheme())
										.host(frontEndProps.getHost())
										.port(frontEndProps.getPort())
										.pathSegment(frontEndProps.getIdeasLinkPath())
										.pathSegment(idea.getId().toString())
										.build()
										.toUriString())
						.author(ideaAuthor)
						.idea(idea)
						.build(),
				NEW_IDEA,
				Collections.singleton(target),
				null
		);
	}

	@Override
	public EmailNotification createGeneratedPasswordMessage(Person target, String password) {
		return create(
				new SettingsArgsBuilder().target(target)
										 .passwordRaw(password)
										 .build(),
				GENERATED_PASSWORD,
				Collections.singleton(target),
				null
		);
	}

	@Override
	public EmailNotification createGeneratedPasswordMessageWithLoginLink(Person target, String password, String link) {
		return create(
				new SettingsArgsBuilder().target(target)
										 .passwordRaw(password)
										 .emailLoginImg()
										 .emailLogoImg()
										 // Manually setting portal link because injecting values from config files in
										 // SettingsArgsBuilder is inconvenient
										 .put(PORTAL_LINK,
											  UriComponentsBuilder.newInstance()
																  .scheme(frontEndProps.getScheme())
																  .host(frontEndProps.getHost())
																  .port(frontEndProps.getPort())
																  .pathSegment(frontEndProps.getLoginLinkPath())
																  .pathSegment(link)
																  .build()
																  .toUriString())
										 .build(),
				GENERATED_PASSWORD_WITH_LINK,
				Collections.singleton(target),
				null
		);
	}

	@Override
	public EmailNotification createGeneratedPasswordForAdmin(Person target, Person createdPerson, String password) {
		return create(
				new SettingsArgsBuilder().target(target)
										 .author(createdPerson)
										 .passwordRaw(password)
										 .build(),
				GENERATED_PASSWORD_FOR_ADMIN,
				Collections.singleton(target),
				null
		);
	}

	private EmailNotification create(Map<SettingsTagType, String> args, EmailNotificationType notificationType,
									 Collection<Person> receivers, Collection<MultipartFile> files) {
		var settingsPack = notificationManager.getNotificationSettingsPack(notificationType);

		EmailNotification emailNotification = new EmailNotification();

		emailNotification.setNotificationType(notificationType);
		emailNotification.setReceivers(receivers);
		emailNotification.setHeader(settingsPack.getMessageTitle());
		emailNotification.setContent(settingsPack.getMessage(args));

		if (isValid(files)) emailNotification.setFiles(files);

		Collection<EmailEmbeddableFile> embeddableFiles = getEmbeddableFiles(args.keySet());
		if (isValid(embeddableFiles)) emailNotification.setEmbeddableFiles(embeddableFiles);

		return emailNotification;
	}

	private Collection<EmailEmbeddableFile> getEmbeddableFiles(Set<SettingsTagType> tags) {
		List<EmailEmbeddableFile> embeddableFiles = null;
		Set<SettingsTagType> imgTagTypes = EmailImageUtils.getImgTagTypes();
		for (SettingsTagType imgTagType : imgTagTypes) {
			if (tags.contains(imgTagType)) {
				if (embeddableFiles == null) embeddableFiles = new ArrayList<>(imgTagTypes.size());
				embeddableFiles.add(emailImageUtils.getEmbeddableFile(imgTagType));
			}
		}

		return embeddableFiles;
	}

}
