package ru.loylabs.portal.service.output.email;

import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.core.io.ByteArrayResource;
import org.springframework.core.task.TaskExecutor;
import org.springframework.mail.MailSendException;
import org.springframework.mail.SimpleMailMessage;
import org.springframework.mail.javamail.JavaMailSender;
import org.springframework.mail.javamail.MimeMessageHelper;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.service.interfaces.EmailAdapter;
import ru.loylabs.portal.utility.EmailEmbeddableFile;

import javax.mail.internet.MimeMessage;
import java.util.Collection;

import static ru.loylabs.portal.utility.CollectionUtils.isValid;

@Slf4j
@Component
public class EmailAdapterImpl implements EmailAdapter {

	private static final String EMAIL_ENCODING = "utf-8";
	private final TaskExecutor taskExecutor;

	private final JavaMailSender emailSender;
	private final String noReplyAddress;

	public EmailAdapterImpl(@Autowired JavaMailSender emailSender,
							@Value("${spring.mail.smtp.no_reply_address}") String noReplyAddress,
							@Autowired @Qualifier("emailSenderTaskExecutor") TaskExecutor taskExecutor) {
		this.emailSender = emailSender;
		this.noReplyAddress = noReplyAddress;
		this.taskExecutor = taskExecutor;
	}

	@Override
	public void sendSimpleMessage(String to, String subject, String text) {

		SimpleMailMessage message = new SimpleMailMessage();
		message.setFrom(noReplyAddress);
		message.setTo(to);
		message.setSubject(subject);
		message.setText(text);

		sendAsync(message);
	}

	@Override
	@SneakyThrows
	public void sendMimeMessage(String to, String subject, String text, Collection<MultipartFile> files,
								Collection<EmailEmbeddableFile> embeddableFiles) {
		MimeMessage message = emailSender.createMimeMessage();
		MimeMessageHelper messageHelper = new MimeMessageHelper(message, true, "utf-8");
		addEmailInfo(messageHelper, to, subject, text);
		addEmbeddableFiles(messageHelper, embeddableFiles);
		addAttachmentFiles(messageHelper, files);

		sendAsync(message);
	}

	private boolean isMultipart(Collection<MultipartFile> files, Collection<EmailEmbeddableFile> embeddableFiles) {
		return isValid(files) || isValid(embeddableFiles);
	}

	@SneakyThrows
	private void addEmailInfo(MimeMessageHelper messageHelper, String to, String subject, String text) {
		messageHelper.setFrom(noReplyAddress);
		messageHelper.setTo(to);
		messageHelper.setSubject(subject);
		messageHelper.setText(text, true);
	}

	@SneakyThrows
	private void addEmbeddableFiles(MimeMessageHelper messageHelper, Collection<EmailEmbeddableFile> embeddableFiles) {
		if (isValid(embeddableFiles))
			for (EmailEmbeddableFile embeddableFile : embeddableFiles)
				messageHelper.addInline(embeddableFile.getContentId(), embeddableFile.getResource());
	}

	@SneakyThrows
	private void addAttachmentFiles(MimeMessageHelper messageHelper, Collection<MultipartFile> files) {
		if (isValid(files))
			for (MultipartFile file : files) {
				int count = 0;
				ByteArrayResource fileResource = null;
				fileResource = new ByteArrayResource(file.getBytes());
				messageHelper.addAttachment(
						file.getOriginalFilename() == null ? "file" + ++count : file.getOriginalFilename(),
						fileResource
				);
			}
	}

	private void sendAsync(MimeMessage message) {
		taskExecutor.execute(() -> {
		try {
			emailSender.send(message);
		} catch (MailSendException e) {
			log.error("Couldn't send Email message", e);
		}});
	}

	private void sendAsync(SimpleMailMessage message) {
		taskExecutor.execute(() -> {
		try {
			emailSender.send(message);
		} catch (MailSendException e) {
			log.error("Couldn't send Email message", e);
		}});
	}

}
