package ru.loylabs.portal.service.review;

import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.Idea;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.ReviewPersonSkill;
import ru.loylabs.portal.entity.ReviewPersonSkillRating;
import ru.loylabs.portal.entity.Reviewers;
import ru.loylabs.portal.entity.Skill;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.listener.events.ReviewEvents;
import ru.loylabs.portal.mapper.ReviewPersonSkillMapper;
import ru.loylabs.portal.mapper.ReviewPersonSkillRatingMapper;
import ru.loylabs.portal.mapper.ReviewerSkillMapper;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkill;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkillRating;
import ru.loylabs.portal.presentation.request.UpdateReviewPersonSkill;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.ReviewPersonSkillRatingRepository;
import ru.loylabs.portal.repository.ReviewPersonSkillRepository;
import ru.loylabs.portal.repository.ReviewersRepository;
import ru.loylabs.portal.repository.SkillRepository;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilter;
import ru.loylabs.portal.service.interfaces.EmailNotificationFactory;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.NotificationSender;
import ru.loylabs.portal.service.interfaces.ReviewPersonService;
import ru.loylabs.portal.service.notification.email.EmailNotification;

import java.time.LocalDate;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.UUID;
import java.util.stream.Collectors;

import static ru.loylabs.portal.exception.BaseException.Exception.EDIT_REVIEW_YOURSELF;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_BANNED;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEWER_ALREADY_VOTED;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEWER_NOT_FOUNT;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEW_DATE_ERROR;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEW_NOT_FOUNT;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEW_OWNER_IS_REVIEWER;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEW_OWNER_IS_YOURSELF;
import static ru.loylabs.portal.exception.BaseException.Exception.SKILL_NOT_FOUND;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredentialPersonId;

@Service
@RequiredArgsConstructor
public class ReviewPersonServiceImpl extends AbstractFilteringAndSortingService<Idea> implements ReviewPersonService {
	private final ReviewPersonSkillRatingMapper reviewPersonSkillRatingMapper;
	private final ReviewPersonSkillMapper reviewPersonSkillMapper;
	private final ReviewerSkillMapper reviewerSkillMapper;
	private final ReviewPersonSkillRatingRepository reviewPersonSkillRatingRepository;
	private final ReviewPersonSkillRepository reviewPersonSkillRepository;
	private final ReviewersRepository reviewersRepository;
	private final PersonRepository personRepository;
	private final SkillRepository skillRepository;
	private final FilterUtils filterUtils;
	private final ApplicationEventPublisher publisher;
	private final EmailNotificationFactory emailNotificationFactory;
	@Qualifier("emailNotificationSenderImpl")
	private final NotificationSender<EmailNotification> emailNotificationSender;

	@Override
	public Page<ReviewPersonSkill> getAllReviewPersonSkillByOwner(Integer page, Integer size, String sort,
																  SearchReversePolishNotation criteria, UUID ownerId) {
		var owner = getPersonById(ownerId);
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, ReviewPersonSkill.class);
		return reviewPersonSkillRepository.findAllByOwnerId(owner.getId(), getPageable(listFilter));
	}

	@Override
	public Page<ReviewPersonSkill> getAllReviewPersonSkillByIdReviewer(Integer page, Integer size, String sort,
																	   SearchReversePolishNotation criteria,
																	   UUID reviewerId) {
		var reviewer = getPersonById(reviewerId);
		ListFilter listFilter = filterUtils.getListFilter(page, size, sort, criteria, ReviewPersonSkill.class);
		return reviewPersonSkillRepository.findAllByReviewersReviewerId(reviewer.getId(), getPageable(listFilter));
	}

	@Override
	public ReviewPersonSkill getReviewPersonSkillById(UUID id) {
		return reviewPersonSkillRepository.findById(id).orElseThrow(
				() -> new ApplicationException(REVIEW_NOT_FOUNT, id));
	}

	@Override
	public List<ReviewPersonSkillRating> getRatingByReviewId(UUID id) {
		var review = getReviewPersonSkillById(id);
		return reviewPersonSkillRatingRepository.findAllByReview(review);
	}

	@Override
	public ReviewPersonSkill createReview(CreateReviewPersonSkill reviewPersonSkill) {
		var owner = getPersonById(reviewPersonSkill.getOwnerId());
		var skill = getSkillById(reviewPersonSkill.getSkillId());
		var review = reviewPersonSkillMapper.toEntity(owner, skill, LocalDate.now(), reviewPersonSkill.getEndReview());

		review.setReviewers(getReviewers(reviewPersonSkill.getReviewersId(), review));

		verificationReviewUser(review);
		checkStartEndDate(review);

		review = reviewPersonSkillRepository.save(review);

		sendNotification(review.getReviewers(), review);
		publisher.publishEvent(ReviewEvents.ReviewCreateEventOwner.of(owner.getId(), review.getId()));

		return review;
	}

	@Override
	public ReviewPersonSkillRating createRating(UUID reviewId, CreateReviewPersonSkillRating createReviewPersonSkillRating) {
		var review = getReviewPersonSkillById(reviewId);
		rateSkill(review);

		return reviewPersonSkillRatingRepository.save(
				reviewPersonSkillRatingMapper.toEntity(review, createReviewPersonSkillRating.getRating()));
	}

	@Override
	@Transactional
	public ReviewPersonSkill updateReviewPersonSkill(UUID reviewId, UpdateReviewPersonSkill update) {
		var review = getReviewPersonSkillById(reviewId);
		List<Reviewers> newReviewers = new ArrayList<>();

		if (review.getOwner().getId().equals(getCredentialPersonId()))
			throw new ApplicationException(EDIT_REVIEW_YOURSELF, reviewId);

		if (update.getReviewersId() != null) {
			newReviewers = updateReviewers(review, update.getReviewersId());
		}

		if (update.getEndReview() != null) {
			review.setEndReview(update.getEndReview());
		}

		review = reviewPersonSkillRepository.save(review);

		sendNotification(newReviewers, review);

		return review;
	}

	@Override
	@Transactional
	public void removeReview(UUID reviewId) {
		var review = getReviewPersonSkillById(reviewId);
		reviewPersonSkillRatingRepository.removeAllByReviewId(reviewId);
		reviewPersonSkillRepository.deleteById(reviewId);
		publisher.publishEvent(ReviewEvents.ReviewDeleteEvent.of(review.getId(),
				review.getReviewers().stream().map(r -> r.getReviewer().getId()).collect(Collectors.toList()),
				review.getOwner().getId()));
	}

	private void checkStartEndDate(ReviewPersonSkill review) {
		if (review.getEndReview() != null && review.getStartReview().isAfter(review.getEndReview()))
			throw new ApplicationException(REVIEW_DATE_ERROR);
	}

	private void checkCreateReviewYourself(ReviewPersonSkill review) {
		if (review.getOwner().getId().equals(getCredentialPersonId()))
			throw new ApplicationException(REVIEW_OWNER_IS_YOURSELF, getCredentialPersonId());
	}

	private void verificationReviewUser(ReviewPersonSkill review) {
		isPersonBlocked(review.getOwner());
		for (Reviewers r : review.getReviewers()) {
			isPersonBlocked(r.getReviewer());
			checkOwnerIsReviewer(review.getOwner(), r.getReviewer());
		}
		checkCreateReviewYourself(review);
	}

	private void isPersonBlocked(Person person) {
		if (person.isBlocked()) throw new ApplicationException(PERSON_BANNED, person.getId());
	}

	private void checkOwnerIsReviewer(Person owner, Person reviewer) {
		if (reviewer.equals(owner)) throw new ApplicationException(REVIEW_OWNER_IS_REVIEWER, owner.getId());
	}

	private List<Reviewers> updateReviewers(ReviewPersonSkill review, Set<UUID> newReviewerId) {
		var newReviewers = getReviewers(newReviewerId, review);
		review.getReviewers().removeIf(r -> {
			var voted = r.isVoted();

			if (!voted && !newReviewers.contains(r)) {
				publisher.publishEvent(ReviewEvents.ReviewSingleDeleteEvent.of(r.getReview().getId(),
						r.getReviewer().getId(),
						r.getReview().getOwner().getId()));
				return true;
			}

			return !(voted || newReviewers.contains(r));
		});

		newReviewers.removeIf(r -> review.getReviewers().contains(r));
		review.getReviewers().addAll(newReviewers);
		verificationReviewUser(review);
		return newReviewers;
	}

	private void sendNotification(List<Reviewers> reviewers, ReviewPersonSkill review) {
		for (Reviewers rev : reviewers) {
			emailNotificationSender.send(emailNotificationFactory.createReview(rev.getReviewer(), review));
			publisher.publishEvent(ReviewEvents.ReviewCreateEvent.of(rev.getReview().getId(), rev.getReviewer().getId(), rev.getReview().getOwner().getId()));
		}
	}

	private void rateSkill(ReviewPersonSkill review) {
		var idUser = getCredentialPersonId();
		var reviewer = reviewersRepository.findByReviewerIdAndReviewId(idUser, review.getId()).orElseThrow(
				() -> new ApplicationException(REVIEWER_NOT_FOUNT, idUser, review.getId()));

		if (reviewer.isVoted()) {
			throw new ApplicationException(REVIEWER_ALREADY_VOTED, idUser, review.getId());
		}

		reviewer.setVoted(true);
		reviewersRepository.save(reviewer);
	}

	private List<Reviewers> getReviewers(Set<UUID> reviewersId, ReviewPersonSkill review) {
		if (reviewersId != null) {
			return reviewersId.stream().map(r -> {
						var reviewer = getPersonById(r);
						return reviewerSkillMapper.toEntity(reviewer, review);
					})
					.collect(Collectors.toList());
		}
		return new ArrayList<>();
	}

	private Skill getSkillById(UUID id) {
		return skillRepository.findById(id)
				.orElseThrow(() -> new ApplicationException(SKILL_NOT_FOUND, id));
	}

	private Person getPersonById(UUID id) {
		return personRepository.findById(id).orElseThrow(
				() -> new ApplicationException(PERSON_NOT_FOUND, id));
	}
}