package ru.loylabs.portal.service.security;

import lombok.RequiredArgsConstructor;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.stereotype.Service;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.enums.CommunityRole;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.repository.*;

import java.util.Objects;
import java.util.UUID;

import static ru.loylabs.portal.enums.CredentialPermission.PERSON_WRITE_PROTECTION;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.RECORD_NOT_FOUND;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredential;
import static ru.loylabs.portal.presentation.SecurityUtil.getCredentialPersonId;

@Service("sec")
@RequiredArgsConstructor
public class SecurityAccess {

	private final AchievementRepository achievementRepository;
	private final PersonRepository personRepository;
	private final CommunityPersonRoleRepository communityPersonRoleRepository;
	private final RecordRepository recordRepository;
	private final TaskRepository taskRepository;
	private final CommunityRepository communityRepository;
	private final PasswordEncoder encoder;

	/**
	 * @param id Record id
	 *
	 * @return {@literal true} if current Principal is the author of specified Record, {@literal false} otherwise
	 */
	public boolean isPrincipalRecordAuthor(UUID id) {
		return hasSameId(getRecordById(id).getAuthor(), getCredential().getPerson());
	}

	/**
	 * @param id Person id
	 *
	 * @return {@literal true} if current Principal is the same Person as one being edited, {@literal false} otherwise
	 */
	public boolean isPrincipalAccountOwner(UUID id) {
		return hasSameId(getPersonById(id), getCredential().getPerson());
	}

	/**
	 * @param username Person username (email)
	 *
	 * @return {@literal true} if current Principal is the same Person as one being edited, {@literal false} otherwise
	 */
	public boolean isPrincipalAccountOwnerByUsername(String username) {
		return hasSameId(getPersonByUsername(username), getCredential().getPerson());
	}

	/**
	 * @param id Person id
	 *
	 * @return {@literal true} if Person with id provided is write-protected (special permission), {@literal false}
	 * otherwise
	 */
	public boolean isTargetWriteProtected(UUID id) {
		return getPersonById(id).getRole().hasCredentialPermission(PERSON_WRITE_PROTECTION);
	}

	/**
	 * @param id Person id
	 *
	 * @return {@literal false} if Person with id provided is write-protected (special permission), {@literal true}
	 * otherwise
	 */
	public boolean isTargetNotWriteProtected(UUID id) {
		return !getPersonById(id).getRole().hasCredentialPermission(PERSON_WRITE_PROTECTION);
	}

	/**
	 * @param id          Person id
	 * @param oldPassword old password
	 *
	 * @return {@literal true} if provided password String matches with the specified Person's current password,
	 * {@literal false} otherwise
	 */
	public boolean isPasswordCorrect(UUID id, String oldPassword) {
		return encoder.matches(oldPassword, getPersonById(id).getPassword());
	}

	/**
	 * @param id Community id
	 *
	 * @return {@literal true} if Community with id provided is public ({@code privateCommunity == false}),
	 * {@literal false} otherwise
	 */
	public boolean isCommunityPublic(UUID id) {
		return communityRepository.existsByIdAndPrivateCommunity(id, false);
	}

	/**
	 * @param id Community id
	 *
	 * @return {@literal true} if current Principal is a member of specified Community, {@literal false} otherwise
	 */
	public boolean isPrincipalCommunityMember(UUID id) {
		return communityPersonRoleRepository.existsCommunityPersonRoleByCommunityIdAndPersonId(
				id, getCredentialPersonId()
		);
	}

	/**
	 * @param id Community id
	 *
	 * @return {@literal true} if current Principal is an {@link CommunityRole#OWNER} of specified Community,
	 * {@literal false} otherwise
	 */
	public boolean isPrincipalCommunityOwner(UUID id) {
		return communityPersonRoleRepository.existsByCommunityIdAndPersonIdAndCommunityRole(
				id, getCredentialPersonId(), CommunityRole.OWNER
		);
	}

	/**
	 * @param id Community id
	 *
	 * @return {@literal true} if current Principal is an {@link CommunityRole#ADMIN} of specified Community,
	 * {@literal false} otherwise
	 */
	public boolean isPrincipalCommunityAdmin(UUID id) {
		return communityPersonRoleRepository.existsByCommunityIdAndPersonIdAndCommunityRole(
				id, getCredentialPersonId(), CommunityRole.ADMIN
		);
	}

	/**
	 * @param id Community id
	 *
	 * @return {@literal true} if current Principal is an {@link CommunityRole#OWNER} or an {@link CommunityRole#ADMIN} of
	 * specified Community, {@literal false} otherwise
	 */
	public boolean isPrincipalCommunityOwnerOrAdmin(UUID id) {
		return isPrincipalCommunityOwner(id) || isPrincipalCommunityAdmin(id);
	}

	/**
	 * @param achievementId Achievement id
	 *
	 * @return {@literal true} if current Principal is an owner of provided Achievement, {@literal false} otherwise
	 */
	public boolean isPrincipalAchievementOwner(UUID achievementId) {
		return achievementRepository.existsByIdAndOwnerId(achievementId, getCredentialPersonId());
	}

	/**
	 * @param achievementId Achievement id
	 *
	 * @return {@literal true} if current Principal is a participant (i.e. is author of, is owner of, or has assigned
	 * Tasks
	 * in) of provided Achievement, {@literal false} otherwise
	 */
	public boolean isPrincipalAchievementParticipant(UUID achievementId) {
		UUID personId = getCredentialPersonId();
		return achievementRepository.existsByIdAndAuthorId(achievementId, personId) ||
			   achievementRepository.existsByIdAndOwnerId(achievementId, personId) ||
			   taskRepository.existsByAchievementIdAndObserversId(achievementId, personId);
	}

	/**
	 * @param achievementId Achievement id
	 * @param taskId        Task id
	 *
	 * @return {@literal true} if current Principal is a participant (i.e. is author of parent Achievement, is owner of
	 * parent Achievement, or is observer of Task) of provided Task, {@literal false} otherwise
	 */
	public boolean isPrincipalTaskParticipant(UUID achievementId, UUID taskId) {
		UUID personId = getCredentialPersonId();
		return achievementRepository.existsByIdAndAuthorId(achievementId, personId) ||
			   achievementRepository.existsByIdAndOwnerId(achievementId, personId) ||
			   taskRepository.existsByIdAndObserversId(taskId, personId);
	}

	public boolean doesPrincipalHaveRole(UUID roleId) {
		return Objects.equals(getCredential().getPerson().getRole().getId(), roleId);
	}

	private boolean hasSameId(Person personFirst, Person personSecond) {
		return Objects.equals(personFirst.getId(), personSecond.getId());
	}

	private Record getRecordById(UUID id) {
		return recordRepository.findById(id).orElseThrow(() -> new ApplicationException(RECORD_NOT_FOUND, id)); // TODO был FORBIDDEN
	}

	private Person getPersonById(UUID id) {
		return personRepository.findById(id).orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, id));
	}

	private Person getPersonByUsername(String username) {
		return personRepository.findByUsername(username)
							   .orElseThrow(() -> new ApplicationException(PERSON_NOT_FOUND, username));
	}

}