package ru.loylabs.portal.service.story.updater;

import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Reaction;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.enums.StoryGroupType;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.repository.StoryRepository;
import ru.loylabs.portal.service.interfaces.StoryManager;

import javax.annotation.PostConstruct;
import java.time.Instant;
import java.time.LocalDate;
import java.time.Year;
import java.time.format.DateTimeFormatter;
import java.util.Collections;
import java.util.List;
import java.util.stream.Collectors;


@Component
@Slf4j
@Getter
public class SystemBirthdaysStoryUpdater extends SystemStoryUpdater<Person> {

    private final PersonRepository personRepository;
    @Value("${story.ttl.days.before-dob}")
    private int daysBeforeDob;
    @Value("${story.ttl.days.after-dob}")
    private int daysAfterDob;


    public SystemBirthdaysStoryUpdater(StoryGroupRepository storyGroupRepository,
                                       StoryRepository storyRepository,
                                       StoryManager storyManager,
                                       ApplicationEventPublisher publisher,
                                       PersonRepository personRepository) {
        super(storyGroupRepository,
                storyRepository,
                StoryGroupType.BIRTHDAY,
                storyManager,
                storyManager.getStorySettingsPack(StoryGroupType.BIRTHDAY),
                publisher);
        this.personRepository = personRepository;
    }

    @PostConstruct
    private void postConstruct() {
        this.priorityFunction = (person) -> {
            int result = person.getDob().getDayOfYear();
            var now = LocalDate.now();
            if (now.getMonthValue() >= 11 || now.getMonthValue() <= 2) {
                result = result < (now.lengthOfYear() + 1) / 2 ? result + now.lengthOfYear() : result;
            }
            return result;
        };
        this.expireTimeFunction = (person) -> person.getDob().withYear(Year.now().getValue()).atStartOfDay().plusDays(daysAfterDob + 1);
    }

    @Override
    @Transactional
    public int refreshSystemGroup() {
        log.info("{} started refreshing system stories group {}.", this.getClass().getName(), storyGroupType.toString());
        var formatter = DateTimeFormatter.ofPattern("MM-dd");
        var now = LocalDate.now();
        var before = now.minusDays(daysBeforeDob);
        var after = now.plusDays(daysAfterDob);
        List<Person> persons = null;
        if(before.getYear() == after.getYear()) {
            persons = personRepository.findByDobBetween(formatter.format(before), formatter.format(after));
        }else{
            persons = personRepository.findByDobBetween(formatter.format(before),
                    formatter.format(LocalDate.of(before.getYear(), 12, 31)));
            persons.addAll(
                    personRepository.findByDobBetween(formatter.format(LocalDate.of(after.getYear(), 1, 1)),
                            formatter.format(after))
            );
        }
        return super.refreshSystemGroup(persons.stream()
                .filter(person -> {
                    if (person.getFired() == null) {
                        return true;
                    }
                    return person.getFired().isAfter(now);
                }).collect(Collectors.toList()));
    }

    @Override
    @Transactional
    public int clearOutdatedStories() {
        log.info("{} started clearing outdated stories group {}.", this.getClass().getName(), storyGroupType.toString());
        return super.clearOutdatedStories();
    }

    @Override
    protected Story createStory(Person person, StoryGroup storyGroup) {
        return Story.builder()
                .date(Instant.now())
                .reaction(new Reaction())
                .author(person)
                .content(settings.getStoryContent(person))
                .header(settings.getStoryHeader(person))
                .expireTime(expireTimeFunction.apply(person))
                .priority(priorityFunction.apply(person))
                .group(storyGroup)
                .attachments(Collections.singletonList(person.getAvatar()))
                .build();
    }

    @Override
    protected boolean isStoryExist(Person person, StoryGroup group) {
        return storyRepository.existsByAuthorIdAndGroupGroupType(person.getId(), group.getGroupType());
    }
}
