package ru.loylabs.portal.service.story.updater;

import lombok.Getter;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Reaction;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.enums.StoryGroupType;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.repository.StoryRepository;
import ru.loylabs.portal.service.interfaces.StoryManager;

import javax.annotation.PostConstruct;
import java.time.Instant;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.Collections;

@Component
@Slf4j
@Getter
public class SystemNewEmployeeStoryUpdater extends SystemStoryUpdater<Person> {
    private final PersonRepository personRepository;

    @Value("${story.ttl.hours.welcome}")
    private long welcomeTtlHours;

    public SystemNewEmployeeStoryUpdater(StoryGroupRepository storyGroupRepository,
                                         StoryRepository storyRepository,
                                         StoryManager storyManager,
                                         ApplicationEventPublisher publisher,
                                         PersonRepository personRepository) {
        super(storyGroupRepository,
                storyRepository,
                StoryGroupType.WELCOME,
                storyManager,
                storyManager.getStorySettingsPack(StoryGroupType.WELCOME),
                publisher);
        this.personRepository = personRepository;
    }

    @PostConstruct
    private void postConstruct() {
        this.priorityFunction = (person) -> person.getName().compareToIgnoreCase("");
        this.expireTimeFunction = (person) -> LocalDate.now().atTime(LocalTime.MAX).plusHours(welcomeTtlHours);
    }

    @Override
    @Transactional
    public int refreshSystemGroup() {
        log.info("{} started refreshing system stories group {}.", this.getClass().getName(), storyGroupType.toString());
        var now = LocalDate.now();
        var persons = personRepository.findByHiredBetweenOrderByHired(now.minusDays(welcomeTtlHours / 24), LocalDate.now());
        return super.refreshSystemGroup(persons);
    }

    @Override
    @Transactional
    public int clearOutdatedStories() {
        log.info("{} started clearing outdated stories group {}.", this.getClass().getName(), storyGroupType.toString());
        return super.clearOutdatedStories();
    }

    @Override
    protected Story createStory(Person person, StoryGroup storyGroup) {
        return Story.builder()
                .date(Instant.now())
                .reaction(new Reaction())
                .author(person)
                .content(settings.getStoryContent(person))
                .header(settings.getStoryHeader(person))
                .expireTime(expireTimeFunction.apply(person))
                .priority(priorityFunction.apply(person))
                .group(storyGroup)
                .attachments(Collections.singletonList(person.getAvatar()))
                .build();
    }

    @Override
    protected boolean isStoryExist(Person person, StoryGroup group) {
        return storyRepository.existsByAuthorIdAndGroupGroupType(person.getId(), group.getGroupType());
    }
}
