package ru.loylabs.portal.service.story.updater;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationEventPublisher;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.enums.StoryGroupType;
import ru.loylabs.portal.listener.events.StoryEvents;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.repository.StoryRepository;
import ru.loylabs.portal.service.interfaces.StoryManager;

import java.time.Instant;
import java.time.LocalDateTime;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Slf4j
@RequiredArgsConstructor
public abstract class SystemStoryUpdater<T> {

    protected final StoryGroupRepository storyGroupRepository;
    protected final StoryRepository storyRepository;
    protected final StoryGroupType storyGroupType;
    protected Function<T, LocalDateTime> expireTimeFunction = t -> LocalDateTime.now().plusDays(1);
    protected Function<T, Integer> priorityFunction = t -> 0;
    protected final StoryManager storyManager;
    protected final StoryManager.StorySettingsPack settings;
    protected final ApplicationEventPublisher publisher;

    abstract public int refreshSystemGroup();

    abstract protected Story createStory(T t, StoryGroup storyGroup);

    abstract protected boolean isStoryExist(T t, StoryGroup group);

    protected int refreshSystemGroup(List<T> list) {
        var storyGroup = createIfNotExist(storyGroupType);

        storyGroup.detachAllStories();

        list.forEach(t -> storyGroup.addStory(createStory(t, storyGroup)));

        if (!deleteGroupIfEmpty(storyGroup)) storyGroupRepository.save(storyGroup);
        return list.size();
    }


    public int clearOutdatedStories() {
        StoryGroup storyGroup;
        var optionalStoryGroup = storyGroupRepository.findByGroupType(storyGroupType);

        if (optionalStoryGroup.isEmpty()) {
            return 0;
        } else {
            storyGroup = optionalStoryGroup.get();
        }

        var stories = storyGroup.getStories();

        var outdatedStories = stories.stream()
                .filter(story -> story.getExpireTime().isBefore(LocalDateTime.now()))
                .collect(Collectors.toList());

        int storyCounter = outdatedStories.size();
        outdatedStories.forEach(storyGroup::detachStory);

        if (!deleteGroupIfEmpty(storyGroup)) storyGroupRepository.save(storyGroup);
        return storyCounter;
    }

    protected StoryGroup createIfNotExist(StoryGroupType groupType) {
        var optionalStoryGroup = storyGroupRepository.findByGroupType(groupType);
        return optionalStoryGroup.orElseGet(() -> {
            log.info("StoryGroup {} not found, creating new group.", groupType);
            return StoryGroup.builder()
                .pinned(true)
                .groupType(groupType)
                .updateTime(Instant.now())
                .build();});
    }

    protected boolean deleteGroupIfEmpty(StoryGroup storyGroup) {
        var stories = storyGroup.getStories();
        if (stories.isEmpty()) {
            log.info("StoryGroup {} is empty, deleting StoryGroup {}.", storyGroup.getGroupType(), storyGroup.getGroupType());
            publisher.publishEvent(StoryEvents.StoryGroupDeleteEvent.of(storyGroup.getId()));
            storyGroupRepository.deleteStoryGroupById(storyGroup.getId());
            return true;
        }
        return false;
    }
}
