package ru.loylabs.portal.service.telegram;

import org.telegram.abilitybots.api.db.DBContext;
import org.telegram.abilitybots.api.sender.SilentSender;
import org.telegram.telegrambots.meta.api.methods.send.SendMessage;
import org.telegram.telegrambots.meta.api.methods.updatingmessages.EditMessageText;
import org.telegram.telegrambots.meta.api.objects.CallbackQuery;
import org.telegram.telegrambots.meta.api.objects.Message;
import org.telegram.telegrambots.meta.api.objects.Update;
import org.telegram.telegrambots.meta.api.objects.replykeyboard.InlineKeyboardMarkup;
import org.telegram.telegrambots.meta.api.objects.replykeyboard.ReplyKeyboardRemove;
import ru.loylabs.portal.entity.KbArticle;
import ru.loylabs.portal.entity.KbSection;
import ru.loylabs.portal.repository.KbArticleRepository;
import ru.loylabs.portal.repository.KbSectionRepository;
import ru.loylabs.portal.service.telegram.reader.ArticleContentReader;

import javax.ws.rs.NotFoundException;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import static ru.loylabs.portal.service.telegram.Constants.*;
import static ru.loylabs.portal.service.telegram.UserState.*;
import static org.telegram.abilitybots.api.util.AbilityUtils.*;

public class ResponseHandler {

    private final SilentSender sender;
    private final Map<Long, UserState> chatStates;
    private final KeyboardFactory keyboardFactory;
    private final KbSectionRepository sectionRepository;
    private final KbArticleRepository articleRepository;

    public ResponseHandler(SilentSender sender, DBContext db, KbSectionRepository sectionRepository, KbArticleRepository articleRepository) {
        this.sender = sender;
        chatStates = db.getMap(CHAT_STATES);
        this.sectionRepository = sectionRepository;
        this.articleRepository = articleRepository;
        chatStates.clear();
        this.keyboardFactory = new KeyboardFactory();
    }


    public void replyToStart(long chatId) {
        SendMessage message = new SendMessage();
        message.setChatId(chatId);
        message.setText(HELLO_MESSAGE);
        message.setReplyMarkup(keyboardFactory.getFunctionsKeyboard());
        sender.execute(message);
        chatStates.put(chatId, UNDEFINED);
    }

    public void replyToButtons(Update update) {
        long chatId = getChatId(update);
        Message message = update.getMessage();
        if (message.getText().equalsIgnoreCase("/stop")) {
            stopChat(chatId);
        } else if(message.getText().equalsIgnoreCase("/start")){
            replyToStart(chatId);
            return;
        }

        switch (chatStates.get(chatId)) {
            case FAQ_SECTIONS_AWAITING -> displaySections(chatId, message);
            default -> replyToFunctionSelection(chatId, message);
        }
    }

    private void replyToFunctionSelection(long chatId, Message message) {
        switch (message.getText()) {
            case FAQ -> displaySections(chatId, message);
            default -> unexpectedMessage(chatId);
        }
    }

    public void replyToMessageButtons(Update update) {
        long chatId = getChatId(update);
        CallbackQuery callbackQuery = update.getCallbackQuery();

        switch (chatStates.get(chatId)) {
            case FAQ_ARTICLES_AWAITING, FAQ_ARTICLE_READING -> replyToArticlesQuery(chatId, callbackQuery);
            case FAQ_ARTICLE_AWAITING -> replyToArticleQuery(chatId, callbackQuery);
            default -> unexpectedMessage(chatId);
        }
    }

    private void replyToArticlesQuery(long chatId, CallbackQuery callbackQuery) {
        int messageId = callbackQuery.getMessage().getMessageId();
        List<KbArticle> articles = articleRepository.findAllBySectionId(UUID.fromString(callbackQuery.getData()));
        InlineKeyboardMarkup keyboardMarkup = keyboardFactory.getArticlesKeyboard(articles);
        editMessage(chatId, messageId, ARTICLES_REQUEST, keyboardMarkup);

        chatStates.put(chatId, FAQ_ARTICLE_AWAITING);
    }

    private void replyToArticleQuery(long chatId, CallbackQuery callbackQuery) {
        int messageId = callbackQuery.getMessage().getMessageId();
        KbArticle article = articleRepository.findById(UUID.fromString(callbackQuery.getData()))
                .orElseThrow(NotFoundException::new);
        String text = ArticleContentReader.getArticleText(article.getContent());
        InlineKeyboardMarkup keyboardMarkup = keyboardFactory.getArticleKeyboard(article);
        editMessage(chatId, messageId, text, keyboardMarkup);

        chatStates.put(chatId, FAQ_ARTICLE_READING);
    }

    private void displaySections(long chatId, Message message) {
        List<KbSection> sections = sectionRepository.findAll();
        SendMessage sendMessage = new SendMessage();
        sendMessage.setChatId(chatId);
        sendMessage.setReplyMarkup(keyboardFactory.getSectionKeyboard(sections));
        sendMessage.setText("Какая секция вас интересует?");
        sender.execute(sendMessage);

        chatStates.put(chatId, FAQ_ARTICLES_AWAITING);
    }

    public void stopChat(long chatId) {
        SendMessage sendMessage = new SendMessage();
        sendMessage.setChatId(chatId);
        sendMessage.setText(STOP_MESSAGE);
        chatStates.remove(chatId);
        sendMessage.setReplyMarkup(new ReplyKeyboardRemove(true));
        sender.execute(sendMessage);
    }

    public boolean userIsActive(Long chatId) {
        return chatStates.containsKey(chatId);
    }

    private void editMessage(long chatId, int messageId, String text, InlineKeyboardMarkup markup) {
        EditMessageText message = new EditMessageText();
        message.setChatId(chatId);
        message.setMessageId(messageId);
        message.setText(text);
        message.setReplyMarkup(markup);
        message.enableMarkdown(true);
        message.disableWebPagePreview();
        sender.execute(message);
    }

    private void unexpectedMessage(long chatId) {
        SendMessage sendMessage = new SendMessage();
        sendMessage.setChatId(chatId);
        sendMessage.setText(UNEXPECTED_MESSAGE);
        sender.execute(sendMessage);
    }
}