package ru.loylabs.portal.service.websocket;

import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.event.EventListener;
import org.springframework.messaging.simp.SimpMessageHeaderAccessor;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;
import org.springframework.web.socket.messaging.AbstractSubProtocolEvent;
import org.springframework.web.socket.messaging.SessionConnectEvent;
import org.springframework.web.socket.messaging.SessionDisconnectEvent;
import ru.loylabs.portal.config.jwt.JwtProvider;
import ru.loylabs.portal.config.jwt.JwtUtils;
import ru.loylabs.portal.repository.redis.OnlineStatusRepository;
import ru.loylabs.portal.service.interfaces.OnlineStatusService;

import java.util.ArrayList;
import java.util.Map;
import java.util.Set;
import java.util.UUID;

@Service
@Slf4j
@RequiredArgsConstructor
public class OnlineStatusServiceImpl implements OnlineStatusService {

	private final static String REQUEST_HEADER = "nativeHeaders";
	private final static String SESSION_HEADER = "simpSessionId";

	private final OnlineStatusRepository onlineStatusRepository;
	private final JwtUtils jwtUtils;
	private final JwtProvider jwtProvider;
	@Value("${jwt.header}")
	private String authHeader;

	@Override
	public Set<UUID> getAll() {
		return onlineStatusRepository.getAll();
	}

	@EventListener
	@SneakyThrows
	public void handleSessionConnected(SessionConnectEvent event) {
		SimpMessageHeaderAccessor headers = getSocketHeaders(event);
		String token = getTokenFromWebsocket(headers);
		var webSocketSessionId = getWebsocketSession(headers);

		onlineStatusRepository.save(webSocketSessionId, jwtUtils.generate(jwtProvider.getAccessClaims(token)).getPerson().getId());

		log.info("Session {} connected to ws", headers.getSessionId());
	}

	@EventListener
	public void handleSessionDisconnect(SessionDisconnectEvent event) {
		var headers = getSocketHeaders(event);
		onlineStatusRepository.delete(getWebsocketSession(headers));

		log.info("Session {} disconnected from ws", headers.getSessionId());
	}

	@SuppressWarnings("unchecked")
	private String getTokenFromWebsocket(SimpMessageHeaderAccessor headers) {
		var temp = ((Map<String, ArrayList<String>>) headers.getHeader(REQUEST_HEADER)).get(authHeader);
		final String bearer = temp == null ? null : temp.stream().findFirst().orElse(null);

		if (StringUtils.hasText(bearer) && bearer.startsWith("Bearer ")) {
			return bearer.substring(7);
		} else {
			return bearer;
		}
	}

	private SimpMessageHeaderAccessor getSocketHeaders(AbstractSubProtocolEvent event) {
		return SimpMessageHeaderAccessor.wrap(event.getMessage());
	}

	private String getWebsocketSession(SimpMessageHeaderAccessor headers) {
		return String.valueOf(headers.getHeader(SESSION_HEADER));
	}
}
