package ru.loylabs.portal.service.yandex;

import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.web.client.RestTemplateBuilder;
import org.springframework.context.annotation.Primary;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.http.HttpHeaders;
import org.springframework.http.client.HttpComponentsClientHttpRequestFactory;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import ru.loylabs.portal.config.resttemplate.Yandex360ResponseErrorHandler;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.Yandex360UserMapper;
import ru.loylabs.portal.presentation.SecurityUtil;
import ru.loylabs.portal.repository.redis.YandexRequestCounterRepository;
import ru.loylabs.portal.service.interfaces.EmailNotificationFactory;
import ru.loylabs.portal.service.interfaces.NotificationSender;
import ru.loylabs.portal.service.interfaces.PasswordGenerator;
import ru.loylabs.portal.service.interfaces.Yandex360UserService;
import ru.loylabs.portal.service.notification.email.EmailNotification;
import ru.loylabs.portal.service.yandex.dto.Yandex360CreateUserRequest;
import ru.loylabs.portal.service.yandex.dto.Yandex360UserPageResponse;
import ru.loylabs.portal.service.yandex.dto.Yandex360UserResponse;

import java.util.Optional;

import static ru.loylabs.portal.exception.BaseException.Exception.YANDEX_360_NUMBER_OF_REQUESTS_EXCEEDED;
import static ru.loylabs.portal.exception.BaseException.Exception.YANDEX_360_USER_SERVICE_UNAVAILABLE;

@Primary
@ConditionalOnProperty(value = "yandex.service.enabled", havingValue = "true")
@Service
@Slf4j
public class Yandex360UserServiceImpl implements Yandex360UserService {

	private static final String OAUTH_TOKEN_PREFIX = "OAuth ";

	private final String getUserUri;
	private final String getAllUsersUri;
	private final String createUserUri;
	private final String updateUserUri;
	private final RestTemplate restTemplate;
	private final Yandex360UserMapper yandex360UserMapper;
	private final PasswordGenerator passwordGenerator;
	private final EmailNotificationFactory emailNotificationFactory;
	private final NotificationSender<EmailNotification> emailNotificationSender;
	private final YandexRequestCounterRepository yandexRequestCounterRepository;

	private final boolean mapToYandex;

	@Value("${yandex.service.max_requests_in_period}")
	private int maxRequestsInPeriod;

	@Autowired
	public Yandex360UserServiceImpl(RestTemplateBuilder restTemplateBuilder,
									Yandex360UserMapper yandex360UserMapper,
									Yandex360ResponseErrorHandler responseErrorHandler,
									PasswordGenerator passwordGenerator,
									EmailNotificationFactory emailNotificationFactory,
									@Qualifier("emailNotificationSenderImpl") NotificationSender<EmailNotification> emailNotificationSender,
									@Value("${yandex.service.org_id}") String orgId,
									// TODO: Update token without restarting application
									@Value("${yandex.service.token}") String token,
									@Value("${uri.yandex.service}") String yandex360RootUrl,
									YandexRequestCounterRepository yandexRequestCounterRepository, @Value("${yandex.service.map_to_yandex.enabled:false}") Boolean mapToYandex) {
		this.yandexRequestCounterRepository = yandexRequestCounterRepository;
		// Using Apache HttpComponents, because default Spring implementation doesn't support PATCH HTTP method
		this.restTemplate = restTemplateBuilder.requestFactory(HttpComponentsClientHttpRequestFactory::new)
											   .errorHandler(responseErrorHandler)
											   .defaultHeader(HttpHeaders.AUTHORIZATION, OAUTH_TOKEN_PREFIX + token)
											   .build();
		this.yandex360UserMapper = yandex360UserMapper;
		this.passwordGenerator = passwordGenerator;

		this.emailNotificationFactory = emailNotificationFactory;
		this.emailNotificationSender = emailNotificationSender;

		this.getUserUri = yandex360RootUrl + "/v1/org/" + orgId + "/users/{userId}";
		this.getAllUsersUri = yandex360RootUrl + "/v1/org/" + orgId + "/users?page={page}&perPage={size}";
		this.createUserUri = yandex360RootUrl + "/v1/org/" + orgId + "/users";
		this.updateUserUri = yandex360RootUrl + "/v1/org/" + orgId + "/users/{userId}";

		// TODO 2023-10-10: Find a better way than a boolean check, or remove if this check won't be required.
		this.mapToYandex = mapToYandex;
	}

	@Override
	public Yandex360UserResponse getYandexUser(String userId) {
		try {
			return restTemplate.getForObject(
					getUserUri, Yandex360UserResponse.class, userId
			);
		} catch (RestClientException rce) {
			log.error("Error exchanging with Yandex 360 service", rce);
			throw new ApplicationException(YANDEX_360_USER_SERVICE_UNAVAILABLE);
		}
	}

	@Override
	public Yandex360UserPageResponse getAllYandexUsers(Pageable pageable) {
		try {
			return restTemplate.getForObject(
					getAllUsersUri,
					Yandex360UserPageResponse.class,
					pageable.getPageNumber() + 1, /* +1 because Yandex API pages are one-based, while
													                 Spring pageable pages are zero-based */
					pageable.getPageSize()
			);
		} catch (RestClientException rce) {
			log.error("Error exchanging with Yandex 360 service", rce);
			throw new ApplicationException(YANDEX_360_USER_SERVICE_UNAVAILABLE);
		}
	}

	@Override
	public Yandex360UserResponse createYandexUser(Person person) {
		if (!mapToYandex) return Yandex360UserResponse.EMPTY;

		Yandex360CreateUserRequest request = yandex360UserMapper.toCreateUserRequest(person);
		String password = passwordGenerator.generatePassword();
		request.setPassword(password);
		try {
			Yandex360UserResponse response = restTemplate.postForObject(
					createUserUri,
					request,
					Yandex360UserResponse.class
			);

			Person author = SecurityUtil.getCredential().getPerson();

			emailNotificationSender.send(
					emailNotificationFactory.createGeneratedPasswordForAdmin(author, person, password)
			);

			return response;
		} catch (RestClientException rce) {
			log.error("Error exchanging with Yandex 360 service", rce);
			throw new ApplicationException(YANDEX_360_USER_SERVICE_UNAVAILABLE);
		}
	}

	@Override
	public Yandex360UserResponse updateYandexUser(Person person) {
		if (!mapToYandex) return Yandex360UserResponse.EMPTY;

		try {
			return restTemplate.patchForObject(
					updateUserUri,
					yandex360UserMapper.toUpdateUserRequest(person),
					Yandex360UserResponse.class,
					person.getExternalUserId()
			);
		} catch (RestClientException rce) {
			log.error("Error exchanging with Yandex 360 service", rce);
			throw new ApplicationException(YANDEX_360_USER_SERVICE_UNAVAILABLE);
		}
	}

	@Override
	public Optional<Yandex360UserResponse> findByEmail(String email) {
		increaseRequestsCounterToYandex();
		Pageable pageable = PageRequest.of(0, 10);
		Yandex360UserPageResponse userResponse;
		Optional<Yandex360UserResponse> result;
		do {
			userResponse = this.getAllYandexUsers(pageable);
			pageable = userResponse.nextPageable();
			result = userResponse.stream()
								 .filter(response -> response.getEmail().equals(email))
								 .findFirst();
		} while (result.isEmpty() && userResponse.hasNext());

		return result;
	}

	private void increaseRequestsCounterToYandex() {

		long count = yandexRequestCounterRepository.increment();

		if (count >= maxRequestsInPeriod) {
			throw new ApplicationException(YANDEX_360_NUMBER_OF_REQUESTS_EXCEEDED, yandexRequestCounterRepository.getRemainingRequestPeriod());
		}
	}
}
