package ru.loylabs.portal.utility.settings;

import ru.loylabs.portal.entity.Achievement;
import ru.loylabs.portal.entity.Community;
import ru.loylabs.portal.entity.Feed;
import ru.loylabs.portal.entity.Idea;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.entity.Skill;
import ru.loylabs.portal.entity.Task;
import ru.loylabs.portal.enums.SettingsTagType;
import ru.loylabs.portal.utility.EmailImageUtils;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import static ru.loylabs.portal.enums.SettingsTagType.ACHIEVEMENT_NAME;
import static ru.loylabs.portal.enums.SettingsTagType.AUTHOR_DAY_OF_BIRTH;
import static ru.loylabs.portal.enums.SettingsTagType.AUTHOR_EMAIL;
import static ru.loylabs.portal.enums.SettingsTagType.AUTHOR_NAME;
import static ru.loylabs.portal.enums.SettingsTagType.AUTHOR_SKILL;
import static ru.loylabs.portal.enums.SettingsTagType.AUTHOR_SURNAME;
import static ru.loylabs.portal.enums.SettingsTagType.AUTHOR_TITLE;
import static ru.loylabs.portal.enums.SettingsTagType.BUG_REPORT;
import static ru.loylabs.portal.enums.SettingsTagType.CALENDAR_EVENT_DATE;
import static ru.loylabs.portal.enums.SettingsTagType.CALENDAR_EVENT_NAME;
import static ru.loylabs.portal.enums.SettingsTagType.COMMUNITY_NAME;
import static ru.loylabs.portal.enums.SettingsTagType.FEED_CONTENT;
import static ru.loylabs.portal.enums.SettingsTagType.IDEA_HEADER;
import static ru.loylabs.portal.enums.SettingsTagType.IMG_EMAIL_LOGIN;
import static ru.loylabs.portal.enums.SettingsTagType.IMG_EMAIL_LOGO;
import static ru.loylabs.portal.enums.SettingsTagType.PASSWORD_CODE;
import static ru.loylabs.portal.enums.SettingsTagType.PASSWORD_RAW;
import static ru.loylabs.portal.enums.SettingsTagType.RECORD_CONTENT;
import static ru.loylabs.portal.enums.SettingsTagType.REVIEW_END_DATE;
import static ru.loylabs.portal.enums.SettingsTagType.TARGET_NAME;
import static ru.loylabs.portal.enums.SettingsTagType.TARGET_PATRONYMIC;
import static ru.loylabs.portal.enums.SettingsTagType.TARGET_SURNAME;
import static ru.loylabs.portal.enums.SettingsTagType.TASK_APPROVAL_RESULT;
import static ru.loylabs.portal.enums.SettingsTagType.TASK_NAME;
import static ru.loylabs.portal.enums.SettingsTagType.TASK_STATUS;
import static ru.loylabs.portal.utility.EmailImageUtils.CID_PREFIX;
import static ru.loylabs.portal.utility.StringUtils.isValid;
import static ru.loylabs.portal.utility.settings.SettingsContentResolver.getApprovalMessage;

public class SettingsArgsBuilder {

	private final Map<SettingsTagType, String> argsMap = new HashMap<>(SettingsTagType.values().length / 2);

	public Map<SettingsTagType, String> build() {
		return this.argsMap;
	}

	public SettingsArgsBuilder put(SettingsTagType tag, String value) {
		if (tag != null) putIfValid(tag, value);

		return this;
	}

	public SettingsArgsBuilder author(Person author) {
		if (author != null) {
			putIfValid(AUTHOR_NAME, author.getName());
			putIfValid(AUTHOR_SURNAME, author.getSurname());
			putIfValid(AUTHOR_EMAIL, author.getUsername());
			if (author.getDob() != null)
				putIfValid(
						AUTHOR_DAY_OF_BIRTH,
						author.getDob()
							  .format(DateTimeFormatter.ofPattern("dd MMMM yyyy", new Locale("ru")))
				);
			if (author.getJob() != null && author.getJob().getTitle() != null)
				putIfValid(
						AUTHOR_TITLE,
						author.getJob().getTitle().toString()
				);
		}

		return this;
	}

	public SettingsArgsBuilder reviewSkill(Skill skill) {
		putIfValid(AUTHOR_SKILL, skill.getName());

		return this;
	}

	public SettingsArgsBuilder passwordCode(String passwordCode) {
		putIfValid(PASSWORD_CODE, passwordCode);

		return this;
	}

	public SettingsArgsBuilder target(Person target) {
		if (target != null) {
			putIfValid(TARGET_NAME, target.getName());
			putIfValid(TARGET_SURNAME, target.getSurname());
			putIfValid(TARGET_PATRONYMIC, target.getPatronymic());
		}

		return this;
	}

	public SettingsArgsBuilder achievement(Achievement achievement) {
		if (achievement != null) {
			putIfValid(ACHIEVEMENT_NAME, achievement.getName());
		}

		return this;
	}

	public SettingsArgsBuilder feed(Record record) {
		if (record instanceof Feed) {
			var feed = (Feed) record;
			putIfValid(FEED_CONTENT, feed.getContent());
		}

		return this;
	}

	public SettingsArgsBuilder task(Record record) {
		if (record instanceof Task) {
			var task = (Task) record;
			putIfValid(TASK_NAME, task.getHeader());
			putIfValid(TASK_STATUS, task.getStatus().getDescription());
		}

		return this;
	}

	public SettingsArgsBuilder taskApprovalResult(boolean taskApprovalResult) {
		putIfValid(TASK_APPROVAL_RESULT, getApprovalMessage(taskApprovalResult));

		return this;
	}

	public SettingsArgsBuilder taskApprovalResult(String taskApprovalResult) {
		putIfValid(TASK_APPROVAL_RESULT, taskApprovalResult);

		return this;
	}

	public SettingsArgsBuilder passwordRaw(String passwordRaw) {
		putIfValid(PASSWORD_RAW, passwordRaw);

		return this;
	}

	public SettingsArgsBuilder bugReport(String bugReport) {
		putIfValid(BUG_REPORT, bugReport);

		return this;
	}

	public SettingsArgsBuilder community(Community community) {
		if (community != null) {
			putIfValid(COMMUNITY_NAME, community.getName());
		}

		return this;
	}

	public SettingsArgsBuilder record(Record record) {
		if (record != null) {
			putIfValid(RECORD_CONTENT, record.getContent());
		}

		return this;
	}

	public SettingsArgsBuilder idea(Record record) {
		if (record instanceof Idea) {
			var idea = (Idea) record;
			putIfValid(IDEA_HEADER, idea.getHeader());
		}

		return this;
	}

	public SettingsArgsBuilder calendarEventName(String calendarEventName) {
		putIfValid(CALENDAR_EVENT_NAME, calendarEventName);

		return this;
	}

	public SettingsArgsBuilder calendarEventDate(ZonedDateTime calendarEventDate) {
		putIfValid(CALENDAR_EVENT_DATE, calendarEventDate.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));

		return this;
	}

	public SettingsArgsBuilder reviewEndDate(LocalDate reviewEndDate) {
		putIfValid(REVIEW_END_DATE, reviewEndDate.format(DateTimeFormatter.ofPattern("yyyy-MM-dd")));
		return this;
	}

	public SettingsArgsBuilder emailLogoImg() {
		put(IMG_EMAIL_LOGO, CID_PREFIX + EmailImageUtils.getContentId(IMG_EMAIL_LOGO));

		return this;
	}

	public SettingsArgsBuilder emailLoginImg() {
		put(IMG_EMAIL_LOGIN, CID_PREFIX + EmailImageUtils.getContentId(IMG_EMAIL_LOGIN));

		return this;
	}

	private String putIfValid(SettingsTagType tag, String value) {
		return isValid(value) ? argsMap.put(tag, value) : null;
	}

}