package ru.loylabs.portal.integration.controller;

import lombok.SneakyThrows;
import org.instancio.Instancio;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.test.web.servlet.MvcResult;
import ru.loylabs.portal.integration.IntegrationTestBaseMVC;
import ru.loylabs.portal.integration.util.JsonResponse;
import ru.loylabs.portal.integration.util.TestUtils;
import ru.loylabs.portal.integration.util.data.MultipartFileGeneratorUtils;
import ru.loylabs.portal.integration.util.data.PersonTestData;
import ru.loylabs.portal.presentation.request.*;
import ru.loylabs.portal.repository.CityRepository;

import java.time.LocalDate;
import java.util.UUID;
import java.util.stream.Stream;

import static org.instancio.Select.field;
import static org.junit.jupiter.api.Assertions.*;
import static org.junit.jupiter.params.provider.Arguments.arguments;
import static org.springframework.http.HttpHeaders.AUTHORIZATION;
import static org.springframework.http.HttpMethod.PUT;
import static org.springframework.http.HttpStatus.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.integration.util.MvcResultAssert.assertThat;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.TestUtils.randomFromBD;

@DisplayName("Person controller IT")
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
public class PersonControllerIT extends IntegrationTestBaseMVC {

	@Autowired
	private CityRepository cityRepository;

	private final static String PERSON_BASE_URI = VERSION_1_URI + PERSONS_URI;

	@ParameterizedTest(name = "Find all person." + ARGUMENT_INDEX)
	@MethodSource("findAllPersonArguments")
	void findAllPersonTest(UUID requestFrom, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(get(PERSON_BASE_URI)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();
		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	private Stream<Arguments> findAllPersonArguments() {
		var size = personRepository.findAll().size();
		return Stream.of(
				arguments(admin.getId(), OK, JsonResponse.expectedFieldsBuilder().totalItems(size).build()),
				arguments(editor.getId(), OK, JsonResponse.expectedFieldsBuilder().totalItems(size).build()),
				arguments(user.getId(), OK, JsonResponse.expectedFieldsBuilder().totalItems(size).build())
		);
	}


	@ParameterizedTest(name = "Find person by id." + ARGUMENT_INDEX)
	@MethodSource("findOnePersonArguments")
	void findOnePersonTest(UUID requestFrom, UUID personId, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(get(PERSON_BASE_URI + PERSON_ID_URI_PARAM, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	private Stream<Arguments> findOnePersonArguments() {
		return Stream.of(
				arguments(user.getId(), user.getId(), OK, JsonResponse.expectedFieldsBuilder().id(user.getId()).build()),
				arguments(user.getId(), admin.getId(), OK, JsonResponse.expectedFieldsBuilder().id(admin.getId()).build()),
				arguments(editor.getId(), nonExistUUID, BAD_REQUEST, JsonResponse.expectedFieldsBuilder().errorMessage(nonExistUUID.toString()).build())
		);
	}

	@ParameterizedTest(name = "Find self info." + ARGUMENT_INDEX)
	@MethodSource("getSelfArguments")
	void findSelfTest(UUID requestFrom, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(get(PERSON_BASE_URI + SELF_URI)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> getSelfArguments() {
		return Stream.of(
				arguments(user.getId(), OK, JsonResponse.expectedFieldsBuilder().id(user.getId()).build()),
				arguments(editor.getId(), OK, JsonResponse.expectedFieldsBuilder().id(editor.getId()).build()),
				arguments(admin.getId(), OK, JsonResponse.expectedFieldsBuilder().id(admin.getId()).build())
		);
	}


	@ParameterizedTest(name = "Find user login by id." + ARGUMENT_INDEX)
	@MethodSource("findOnePersonLoginArguments")
	void findOnePersonLoginTest(UUID requestFrom, UUID personId, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(get(PERSON_BASE_URI + PERSON_ID_URI_PARAM + USERNAME_URI, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> findOnePersonLoginArguments() {
		return Stream.of(
				arguments(user.getId(), user.getId(), OK, JsonResponse.expectedFieldsBuilder().expected("username", user.getUsername()).build()),
				arguments(admin.getId(), user.getId(), OK, JsonResponse.expectedFieldsBuilder().expected("username", user.getUsername()).build()),
				arguments(user.getId(), admin.getId(), FORBIDDEN, null),
				arguments(admin.getId(), nonExistUUID, HttpStatus.BAD_REQUEST, JsonResponse.expectedFieldsBuilder().errorMessage(nonExistUUID.toString()).build())
		);
	}


	@ParameterizedTest(name = "Create person." + ARGUMENT_INDEX)
	@MethodSource("createPersonArguments")
	void createPersonTest(UUID requestFrom, HttpStatus status, CreatePersonRequest request, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(post(PERSON_BASE_URI)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(TestUtils.convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> createPersonArguments() {
		var request = PersonTestData.generateCreatePersonRequest(departmentRepository, titleRepository, cityRepository);

		var requestWrongDepartment = PersonTestData.generateCreatePersonRequest(departmentRepository, titleRepository, cityRepository);
		requestWrongDepartment.getJob().setDepartment("Non-exist department");

		var requestWrongTitle = PersonTestData.generateCreatePersonRequest(departmentRepository, titleRepository, cityRepository);
		requestWrongTitle.getJob().setTitle("Non-exist title");

		var requestWrongCity = PersonTestData.generateCreatePersonRequest(departmentRepository, titleRepository, cityRepository);
		requestWrongCity.setCityId(nonExistUUID);

		return Stream.of(
				arguments(user.getId(), FORBIDDEN, request, null),
				arguments(editor.getId(), OK, request, JsonResponse.expectedFieldsBuilder().name(request.getName()).build()),
				arguments(admin.getId(), OK, request, JsonResponse.expectedFieldsBuilder().name(request.getName()).build()),
				arguments(admin.getId(), BAD_REQUEST, requestWrongDepartment, JsonResponse.expectedFieldsBuilder().errorMessage("department not found").build()),
				arguments(admin.getId(), BAD_REQUEST, requestWrongTitle, JsonResponse.expectedFieldsBuilder().errorMessage("Title not found").build()),
				arguments(admin.getId(), BAD_REQUEST, requestWrongCity, JsonResponse.expectedFieldsBuilder().errorMessage(nonExistUUID.toString()).build())
		);
	}


	@ParameterizedTest(name = "Update person by id." + ARGUMENT_INDEX)
	@MethodSource("updatePersonArguments")
	void updatePersonTest(UUID requestFrom, UUID personId, HttpStatus status, UpdatePersonRequest request, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(TestUtils.convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> updatePersonArguments() {
		var request = Instancio.of(UpdatePersonRequest.class)
				.set(field(UpdatePersonRequest::getDob), LocalDate.now().minusYears(15))
				.ignore(field(UpdatePersonRequest::getGender))
				.ignore(field(UpdatePersonRequest::getCityId)).create();

		return Stream.of(
				arguments(user.getId(), user.getId(), FORBIDDEN, request, null),
				arguments(user.getId(), admin.getId(), FORBIDDEN, request, null),
				arguments(editor.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().name(request.getName()).build()),
				arguments(admin.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().name(request.getName()).build()),
				arguments(admin.getId(), nonExistUUID, BAD_REQUEST, request, JsonResponse.expectedFieldsBuilder().errorMessage(nonExistUUID.toString()).build())
		);
	}


	@ParameterizedTest(name = "Update personal info person by id." + ARGUMENT_INDEX)
	@MethodSource("updatePersonPersonalInfoArguments")
	void updatePersonPersonalInfoTest(UUID requestFrom, UUID personId, HttpStatus status, UpdatePersonInfoRequest request, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM + PERSONAL_URI, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(TestUtils.convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> updatePersonPersonalInfoArguments() {
		var request = Instancio.of(UpdatePersonInfoRequest.class).create();

		return Stream.of(
				arguments(user.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().expected("description", request.getDescription()).build()),
				arguments(user.getId(), admin.getId(), FORBIDDEN, request, null),
				arguments(editor.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().expected("description", request.getDescription()).build()),
				arguments(admin.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().expected("description", request.getDescription()).build()),
				arguments(admin.getId(), nonExistUUID, BAD_REQUEST, request, JsonResponse.expectedFieldsBuilder().errorMessage(nonExistUUID.toString()).build())
		);
	}


	@ParameterizedTest(name = "Update person job by id." + ARGUMENT_INDEX)
	@MethodSource("updateJobArguments")
	void updateJobTest(UUID requestFrom, UUID personId, HttpStatus status, UpdateJobRequest request, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM + JOB_URI, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(TestUtils.convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> updateJobArguments() {
		var request = UpdateJobRequest.builder().department(randomFromBD(departmentRepository).getName()).title(randomFromBD(titleRepository).getName()).grade("MIDDLE").build();
		var requestWrongDepartment = UpdateJobRequest.builder().department("non-exist department").title(randomFromBD(titleRepository).getName()).grade("MIDDLE").build();
		var requestWrongTitle = UpdateJobRequest.builder().department(randomFromBD(departmentRepository).getName()).title("non-exist title").grade("MIDDLE").build();

		return Stream.of(
				arguments(user.getId(), user.getId(), FORBIDDEN, request, null),
				arguments(editor.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().id(user.getId()).build()),
				arguments(admin.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().id(user.getId()).build()),
				arguments(admin.getId(), nonExistUUID, HttpStatus.BAD_REQUEST, request, JsonResponse.expectedFieldsBuilder().errorMessage(nonExistUUID.toString()).build()),
				arguments(admin.getId(), editor.getId(), HttpStatus.BAD_REQUEST, requestWrongDepartment, JsonResponse.expectedFieldsBuilder().errorMessage("department not found").build()),
				arguments(admin.getId(), editor.getId(), HttpStatus.BAD_REQUEST, requestWrongTitle, JsonResponse.expectedFieldsBuilder().errorMessage("title not found").build())
		);
	}


	@ParameterizedTest(name = "Update person role by id." + ARGUMENT_INDEX)
	@MethodSource("updateRoleArguments")
	void updateRoleTest(UUID requestFrom, UUID personId, HttpStatus status, UpdatePersonRoleRequest request, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(put("/v1/persons/{id}/role", personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> updateRoleArguments() {
		var request = new UpdatePersonRoleRequest("EDITOR");

		return Stream.of(
				arguments(user.getId(), admin.getId(), FORBIDDEN, request, null),
				arguments(user.getId(), editor.getId(), FORBIDDEN, request, null),
				arguments(editor.getId(), user.getId(), FORBIDDEN, request, null),
				arguments(admin.getId(), user.getId(), OK, request, JsonResponse.expectedFieldsBuilder().id(user.getId()).build())
		);
	}


	@ParameterizedTest(name = "Update password for person by id." + ARGUMENT_INDEX)
	@MethodSource("updatePasswordArguments")
	void updatePasswordTest(UUID requestFrom, UUID personId, HttpStatus status, UpdatePersonPasswordRequest request,
							JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM + PASSWORD_URI, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	public Stream<Arguments> updatePasswordArguments() {
		var userRequest = UpdatePersonPasswordRequest.builder().oldPassword("HPCibozn-1").newPassword("123testPassword-").build();
		var userWrongOldPassword = UpdatePersonPasswordRequest.builder().oldPassword("wrong password").newPassword("123testPassword-").build();
		var editorRequest = UpdatePersonPasswordRequest.builder().oldPassword("HPCibozn-2").newPassword("123testPassword-").build();

		return Stream.of(
				arguments(user.getId(), user.getId(), OK, userRequest, null),
				arguments(user.getId(), user.getId(), FORBIDDEN, userWrongOldPassword, null),
				arguments(admin.getId(), nonExistUUID, BAD_REQUEST, userRequest, JsonResponse.expectedFieldsBuilder().errorMessage(nonExistUUID.toString()).build()),
				arguments(user.getId(), editor.getId(), FORBIDDEN, editorRequest, null),
				arguments(editor.getId(), user.getId(), FORBIDDEN, userRequest, null),
				arguments(admin.getId(), editor.getId(), OK, editorRequest, null)
		);
	}


	@ParameterizedTest(name = "Block person by id." + ARGUMENT_INDEX)
	@MethodSource("blockPersonArguments")
	void blockPersonTest(UUID requestFrom, UUID personId, HttpStatus status) throws Exception {
		MvcResult result = mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM + BLOCK_URI, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();
		assertThat(result).hasStatus(status);
		if (status == OK) assertTrue(personRepository.getReferenceById(personId).isBlocked());
	}

	public Stream<Arguments> blockPersonArguments() {
		return Stream.of(
				arguments(user.getId(), editor.getId(), FORBIDDEN),
				arguments(editor.getId(), user.getId(), FORBIDDEN),
				arguments(admin.getId(), user.getId(), OK),
				arguments(admin.getId(), editor.getId(), OK),
				arguments(admin.getId(), nonExistUUID, BAD_REQUEST),
				arguments(admin.getId(), admin.getId(), FORBIDDEN)
		);
	}

	@ParameterizedTest(name = "Unblock person by id." + ARGUMENT_INDEX)
	@MethodSource("unblockPersonArguments")
	void unblockPersonTest(UUID requestFrom, UUID personId, HttpStatus status) throws Exception {
		personRepository.findById(personId).ifPresent(p -> {
			p.setBlocked(true);
			personRepository.save(p);
		});
		MvcResult result = mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM + UNBLOCK_URI, personId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();
		assertThat(result).hasStatus(status);
		if (status == OK) assertFalse(personRepository.getReferenceById(personId).isBlocked());
	}

	public Stream<Arguments> unblockPersonArguments() {
		return Stream.of(
				arguments(user.getId(), editor.getId(), FORBIDDEN),
				arguments(editor.getId(), user.getId(), FORBIDDEN),
				arguments(admin.getId(), user.getId(), OK),
				arguments(admin.getId(), editor.getId(), OK),
				arguments(admin.getId(), nonExistUUID, BAD_REQUEST),
				arguments(admin.getId(), admin.getId(), FORBIDDEN)
		);
	}


	@ParameterizedTest(name = "Update avatar for  person by id." + ARGUMENT_INDEX)
	@MethodSource("updatePersonAvatarArguments")
	void updatePersonAvatarTest(UUID requestFrom, UUID personId, MockMultipartFile file, HttpStatus status) throws Exception {
		MvcResult result = mockMvc.perform(multipart(PUT, PERSON_BASE_URI + PERSON_ID_URI_PARAM + AVATAR_URI, personId)
						.file(file)
						.contentType(MediaType.MULTIPART_FORM_DATA)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();

		assertThat(result).hasStatus(status);
		if (status == OK) assertNotNull(personRepository.getReferenceById(personId).getAvatar().getId());
	}

	@SneakyThrows
	public Stream<Arguments> updatePersonAvatarArguments() {
		var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
		var textFile = MultipartFileGeneratorUtils.generateFile(MediaType.TEXT_HTML);
		return Stream.of(
				arguments(user.getId(), user.getId(), file, OK),
				arguments(user.getId(),user.getId(),textFile,BAD_REQUEST),
				arguments(editor.getId(), user.getId(), file, FORBIDDEN),
				arguments(admin.getId(), user.getId(), file, OK),
				arguments(admin.getId(), editor.getId(), file, OK),
				arguments(admin.getId(), nonExistUUID, file, BAD_REQUEST)
		);
	}


	@ParameterizedTest(name = "Update background for person by id." + ARGUMENT_INDEX)
	@MethodSource("updatePersonBackgroundArguments")
	void updateBackgroundTest(UUID requestFrom, UUID personId, MockMultipartFile file, HttpStatus status) throws Exception {
		MvcResult result = mockMvc.perform(multipart(PUT, PERSON_BASE_URI + PERSON_ID_URI_PARAM + BACKGROUND_URI, personId)
						.file(file)
						.contentType(MediaType.MULTIPART_FORM_DATA)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();

		assertThat(result).hasStatus(status);
		if (status == OK) assertNotNull(personRepository.getReferenceById(personId).getBackground().getId());
	}

	@SneakyThrows
	public Stream<Arguments> updatePersonBackgroundArguments() {
		var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
		var textFile = MultipartFileGeneratorUtils.generateFile(MediaType.TEXT_HTML);
		return Stream.of(
				arguments(user.getId(), user.getId(), file, OK),
				arguments(user.getId(),user.getId(),textFile,BAD_REQUEST),
				arguments(editor.getId(), user.getId(), file, FORBIDDEN),
				arguments(admin.getId(), user.getId(), file, OK),
				arguments(admin.getId(), editor.getId(), file, OK),
				arguments(admin.getId(), nonExistUUID, file, BAD_REQUEST)
		);
	}

}
