package ru.loylabs.portal.integration.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MvcResult;
import ru.loylabs.portal.integration.IntegrationTestBaseMVC;
import ru.loylabs.portal.integration.util.JsonResponse;
import ru.loylabs.portal.integration.util.TestUtils;
import ru.loylabs.portal.integration.util.data.JobTestData;
import ru.loylabs.portal.presentation.request.CreateTitleRequest;
import ru.loylabs.portal.presentation.request.UpdateTitleRequest;

import java.util.UUID;

import static org.springframework.http.HttpHeaders.AUTHORIZATION;
import static org.springframework.http.HttpStatus.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.integration.util.MvcResultAssert.assertThat;
import static ru.loylabs.portal.integration.util.data.JobTestData.generateTitle;

@DisplayName("Title controller IT")
public class TitleControllerIT extends IntegrationTestBaseMVC {

	private final static String TITLE_BASE_URI = VERSION_1_URI + TITLES_URI;


	@Test
	@DisplayName("Find all title. Request from user")
	void findAllTitleRequestFromUserTest() throws Exception {
		var size = titleRepository.findAll().size();
		var expectedBody = JsonResponse.expectedFieldsBuilder().totalItems(size).build();
		findAllPerform(user.getId(), OK, expectedBody);
	}

	@Test
	@DisplayName("Find all title. Request from editor")
	void findAllTitleRequestFromEditorTest() throws Exception {
		var size = titleRepository.findAll().size();
		var expectedBody = JsonResponse.expectedFieldsBuilder().totalItems(size).build();
		findAllPerform(editor.getId(), OK, expectedBody);
	}

	@Test
	@DisplayName("Find all title. Request from admin")
	void findAllTitleRequestFromAdminTest() throws Exception {
		var size = titleRepository.findAll().size();
		var expectedBody = JsonResponse.expectedFieldsBuilder().totalItems(size).build();
		findAllPerform(admin.getId(), OK, expectedBody);
	}

	private void findAllPerform(UUID requestFrom, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(get(TITLE_BASE_URI)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();
		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	@Test
	@DisplayName("Find one title by id. Request from user")
	void findOneTitleRequestFromUserTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		var expectedBody = JsonResponse.expectedFieldsBuilder().id(title.getId()).build();
		findOneTitlePerform(user.getId(), title.getId(), OK, expectedBody);
	}

	@Test
	@DisplayName("Find one title by id. Request from editor")
	void findOneTitleRequestFromEditorTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		var expectedBody = JsonResponse.expectedFieldsBuilder().id(title.getId()).build();
		findOneTitlePerform(editor.getId(), title.getId(), OK, expectedBody);
	}

	@Test
	@DisplayName("Find one title by id. Request from admin")
	void findOneTitleRequestFromAdminTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		var expectedBody = JsonResponse.expectedFieldsBuilder().id(title.getId()).build();
		findOneTitlePerform(editor.getId(), title.getId(), OK, expectedBody);
	}

	@Test
	@DisplayName("Find one title by id. Request from admin : Title not exist")
	void findOneTitleRequestFromAdminNotExistTest() throws Exception {
		findOneTitlePerform(editor.getId(), nonExistUUID, BAD_REQUEST, null);
	}

	private void findOneTitlePerform(UUID requestFrom, UUID titleId, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(get(TITLE_BASE_URI + TITLE_ID_URI_PARAM, titleId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	@Test
	@DisplayName("Create title. Request from user")
	void createTitleRequestFromUserTest() throws Exception {
		var request = JobTestData.generateCreateTitleRequest();
		createTitlePerform(user.getId(), request, FORBIDDEN, null);
	}

	@Test
	@DisplayName("Create title. Request from editor")
	void createTitleRequestFromEditorTest() throws Exception {
		var request = JobTestData.generateCreateTitleRequest();
		var expectedBody = JsonResponse.expectedFieldsBuilder().name(request.getName()).build();
		createTitlePerform(editor.getId(), request, OK, expectedBody);
	}

	@Test
	@DisplayName("Create title. Request from admin")
	void createTitleRequestFromAdminTest() throws Exception {
		var request = JobTestData.generateCreateTitleRequest();
		var expectedBody = JsonResponse.expectedFieldsBuilder().name(request.getName()).build();
		createTitlePerform(editor.getId(), request, OK, expectedBody);
	}

	void createTitlePerform(UUID requestFrom, CreateTitleRequest request, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(post(TITLE_BASE_URI)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(TestUtils.convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	@Test
	@DisplayName("Update title by id. Request from user")
	void updateTitleRequestFromUserTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		var request = JobTestData.generateUpdateTitleRequest();
		updateTitlePerform(user.getId(), title.getId(), request, FORBIDDEN, null);
	}

	@Test
	@DisplayName("Update title by id. Request from editor")
	void updateTitleRequestFromEditorTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		var request = JobTestData.generateUpdateTitleRequest();
		var expectedBody = JsonResponse.expectedFieldsBuilder().name(request.getName()).build();
		updateTitlePerform(editor.getId(), title.getId(), request, OK, expectedBody);
	}

	@Test
	@DisplayName("Update title by id. Request from admin")
	void updateTitleRequestFromAdminTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		var request = JobTestData.generateUpdateTitleRequest();
		var expectedBody = JsonResponse.expectedFieldsBuilder().name(request.getName()).build();
		updateTitlePerform(admin.getId(), title.getId(), request, OK, expectedBody);
	}

	@Test
	@DisplayName("Update title by id. Request from admin : Title not exist")
	void updateTitleRequestFromAdminNotExistTest() throws Exception {
		var request = JobTestData.generateUpdateTitleRequest();
		updateTitlePerform(admin.getId(), nonExistUUID, request, BAD_REQUEST, null);
	}

	void updateTitlePerform(UUID requestFrom, UUID titleId, UpdateTitleRequest request, HttpStatus status, JsonResponse expectedBody) throws Exception {
		MvcResult result = mockMvc.perform(put(TITLE_BASE_URI + TITLE_ID_URI_PARAM, titleId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom))
						.content(TestUtils.convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andReturn();

		assertThat(result).hasStatus(status).hasBody(expectedBody);
	}

	@Test
	@DisplayName("Delete title by id. Request from user")
	void deleteTitleRequestFromUserTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		deleteTitlePerform(user.getId(), title.getId(), FORBIDDEN);
	}

	@Test
	@DisplayName("Delete title by id. Request from editor")
	void deleteTitleRequestFromEditorTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		deleteTitlePerform(editor.getId(), title.getId(), OK);
	}

	@Test
	@DisplayName("Delete title by id. Request from admin")
	void deleteTitleRequestFromAdminTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		deleteTitlePerform(admin.getId(), title.getId(), OK);
	}

	@Test
	@DisplayName("Delete title by id. Request from admin : Title not exist")
	void deleteTitleRequestFromAdminNotExistTest() throws Exception {
		var title = titleRepository.save(generateTitle());
		deleteTitlePerform(admin.getId(), title.getId(), OK);
	}

	void deleteTitlePerform(UUID requestFrom, UUID titleId, HttpStatus status) throws Exception {
		MvcResult result = mockMvc.perform(delete(TITLE_BASE_URI + TITLE_ID_URI_PARAM, titleId)
						.header(AUTHORIZATION, tokenProvider.getPersonalTokenById(requestFrom)))
				.andReturn();

		assertThat(result).hasStatus(status);
	}
}
