package ru.loylabs.portal.integration.util;

import com.jayway.jsonpath.JsonPath;
import com.jayway.jsonpath.PathNotFoundException;
import org.assertj.core.api.AbstractAssert;
import org.hamcrest.MatcherAssert;
import org.hamcrest.Matchers;
import org.springframework.http.HttpStatus;
import org.springframework.test.web.servlet.MvcResult;

import java.util.Map;

import static org.hamcrest.Matchers.emptyString;
import static org.hamcrest.Matchers.not;

public class MvcResultAssert extends AbstractAssert<MvcResultAssert, MvcResult> {

	public MvcResultAssert(MvcResult actual) {
		super(actual, MvcResultAssert.class);
	}


	public static MvcResultAssert assertThat(MvcResult actual) {
		return new MvcResultAssert(actual);
	}

	public MvcResultAssert hasStatus(HttpStatus expectedStatus) {
		var actualStatus = actual.getResponse().getStatus();
		if (actualStatus != expectedStatus.value()) {
			failWithMessage("Expected http status to be <%s> but was <%s>", expectedStatus.value(), actualStatus);
		}
		return this;
	}

	public MvcResultAssert hasBody(JsonResponse body) {
		if (body != null) {
			String responseBody = getResponseBody();
			Map<String, Object> expectedBody = body.getValues();
			if (responseBody == null || responseBody.isEmpty()) {
				failWithMessage("Expected response body should not have been empty");
			}
			expectedBody.forEach(this::assertFieldInResponseBody);
		}
		return this;
	}

	private String getResponseBody() {
		try {
			return actual.getResponse().getContentAsString();
		} catch (Exception e) {
			throw new AssertionError("Failed to get response body", e);
		}
	}

	private void assertFieldInResponseBody(String field, Object value) {
		try {
			if (value instanceof String) {
				MatcherAssert.assertThat(JsonPath.read(getResponseBody(), field), Matchers.containsStringIgnoringCase((String) value));
			} else if (value instanceof NotEmptyField) {
				MatcherAssert.assertThat(JsonPath.read(getResponseBody(), field), Matchers.is(not(emptyString())));
			} else {
				MatcherAssert.assertThat(JsonPath.read(getResponseBody(), field), Matchers.is(value));
			}
		} catch (PathNotFoundException ex) {
			failWithMessage("Expected field <%s> not found in response", field.replaceFirst("\\$\\.",""));
		}
	}

	public static class NotEmptyField{}
}
