package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.PageRequest;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.entity.Community;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Record;
import ru.loylabs.portal.enums.CommunityRole;
import ru.loylabs.portal.enums.RecordType;
import ru.loylabs.portal.integration.util.data.MultipartFileGeneratorUtils;
import ru.loylabs.portal.mapper.CommentMapper;
import ru.loylabs.portal.mapper.CommunityMapper;
import ru.loylabs.portal.mapper.CommunityRecordMapper;
import ru.loylabs.portal.mapper.PersonMapper;
import ru.loylabs.portal.presentation.controller.CommunityController;
import ru.loylabs.portal.presentation.request.AddMembersRequest;
import ru.loylabs.portal.presentation.request.CreateCommunityRequest;
import ru.loylabs.portal.presentation.request.CreateRecordRequest;
import ru.loylabs.portal.presentation.request.DeleteAttachmentsRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.service.CommunityServiceImpl;
import ru.loylabs.portal.service.interfaces.CommentService;
import ru.loylabs.portal.service.interfaces.CommunityPersonRoleService;
import ru.loylabs.portal.service.interfaces.CommunityRecordService;
import ru.loylabs.portal.service.security.SecurityAccess;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.UUID;

import static org.hamcrest.Matchers.is;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;
import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.multipart;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.ATTACHMENTS_URI;
import static ru.loylabs.portal.config.URIConstants.AVATAR_URI;
import static ru.loylabs.portal.config.URIConstants.COMMENTS_URI;
import static ru.loylabs.portal.config.URIConstants.COMMENT_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.COMMUNITY_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.COMMUNITY_URI;
import static ru.loylabs.portal.config.URIConstants.JOIN_URI;
import static ru.loylabs.portal.config.URIConstants.LEAVE_URI;
import static ru.loylabs.portal.config.URIConstants.MEMBERS_URI;
import static ru.loylabs.portal.config.URIConstants.MESSAGES_URI;
import static ru.loylabs.portal.config.URIConstants.MESSAGE_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.NEW_AVATAR_URI;
import static ru.loylabs.portal.config.URIConstants.OWNER_URI;
import static ru.loylabs.portal.config.URIConstants.PERSONAL_URI;
import static ru.loylabs.portal.config.URIConstants.PUBLIC_URI;
import static ru.loylabs.portal.config.URIConstants.ROLES_URI;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_INVITE_KICK;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_INVITE_KICK_OWN;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_JOIN_LEAVE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_JOIN_PRIVATE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_READ_COMMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_READ_OWN;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_READ_OWN_COMMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_READ_PUBLIC;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_WRITE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_WRITE_COMMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_WRITE_MESSAGE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_WRITE_OWN;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_WRITE_OWN_COMMENT;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.COMMUNITY_WRITE_OWN_MESSAGE;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.createAddMemberRequest;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.createCommunityRole;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.createUpdateRoleInCommunityRequest;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCommunity;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCommunityListView;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCommunityPersonRole;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCommunityRecordView;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCommunityView;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCreateCommunityRequest;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateDeleteMemberRequest;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generatePersonCommunityView;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateUpdateCommunityRequest;
import static ru.loylabs.portal.integration.util.data.RecordTestData.generateCommentView;
import static ru.loylabs.portal.integration.util.data.RecordTestData.generateCreateRecordRequest;
import static ru.loylabs.portal.integration.util.data.RecordTestData.generateDeleteAttachmentsRequest;
import static ru.loylabs.portal.integration.util.data.RecordTestData.generateRecord;
import static ru.loylabs.portal.integration.util.data.RecordTestData.generateUpdateRecordRequest;

@DisplayName("Community controller test")
@WebMvcTest(CommunityController.class)
public class CommunityControllerTest extends BaseControllerTest {

    @MockBean
    private CommunityServiceImpl communityService;
    @MockBean
    private CommunityMapper communityMapper;
    @MockBean
    private PersonMapper personMapper;
    @MockBean
    private CommunityRecordMapper communityRecordMapper;
    @MockBean
    private CommentMapper commentMapper;
    @MockBean
    private CommunityRecordService communityRecordService;
    @MockBean
    private CommunityPersonRoleService communityPersonRoleService;
    @MockBean
    private CommentService commentService;
    @MockBean(name = "sec")
    private SecurityAccess securityAccess;

    private final static String BASE_URI = VERSION_1_URI + COMMUNITY_URI;

    //--COMMUNITIES-------------------------------------------------------------------------------------------------------//
    @Test
    @DisplayName("Find all groups. Request from user with authorities COMMUNITY_READ")
    @WithMockUser(authorities = COMMUNITY_READ)
    public void findAllCommunitiesFromUserWithAuthoritiesTest() throws Exception {
        var community = generateCommunity(UUID.randomUUID());
        var communityPage = new PageImpl<>(List.of(community), PageRequest.of(0, 1), 1);
        var communityListView = generateCommunityListView(community.getId());

        when(communityMapper.toCommunityListView(community, community.getMembers().size())).thenReturn(communityListView);
        when(communityService.getAllCommunities(
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPage);

        mockMvc.perform(get(BASE_URI))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(community.getId().toString())));
    }

    @Test
    @DisplayName("Find all groups. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_READ")
    public void findAllCommunitiesFromUserWithoutAuthoritiesTest() throws Exception {
        var community = generateCommunity(UUID.randomUUID());
        var communityPage = new PageImpl<>(List.of(community), PageRequest.of(0, 1), 1);
        var communityListView = generateCommunityListView(community.getId());

        when(communityMapper.toCommunityListView(community, community.getMembers().size())).thenReturn(communityListView);
        when(communityService.getAllCommunities(
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPage);

        mockMvc.perform(get(BASE_URI))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Find all personal communities. Request from user with authorities COMMUNITY_READ_OWN")
    @WithMockUser(authorities = COMMUNITY_READ_OWN)
    public void findAllPersonalCommunitiesFromUserWithAuthoritiesTest() throws Exception {
        var communityPersonRole = generateCommunityPersonRole();
        var communityPersonRolePage = new PageImpl<>(List.of(communityPersonRole), PageRequest.of(0, 1), 1);
        var communityListView = generateCommunityListView(communityPersonRole.getId());

        when(communityMapper.toCommunityListView(any(Community.class), anyInt())).thenReturn(communityListView);
        when(communityPersonRoleService.getAllPersonalCommunities(anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class),
                any())
        ).thenReturn(communityPersonRolePage);

        mockMvc.perform(get(BASE_URI + PERSONAL_URI))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(communityPersonRole.getId().toString())));

    }

    @Test
    @DisplayName("Find all personal communities. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_READ_OWN")
    public void findAllPersonalCommunitiesFromUserWithoutAuthoritiesTest() throws Exception {
        var communityPersonRole = generateCommunityPersonRole();
        var communityPersonRolePage = new PageImpl<>(List.of(communityPersonRole), PageRequest.of(0, 1), 1);
        var communityListView = generateCommunityListView(communityPersonRole.getId());

        when(communityMapper.toCommunityListView(any(Community.class), anyInt())).thenReturn(communityListView);
        when(communityPersonRoleService.getAllPersonalCommunities(anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class),
                any())
        ).thenReturn(communityPersonRolePage);

        mockMvc.perform(get(BASE_URI + PERSONAL_URI))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Find all open communities. Request from user with authorities COMMUNITY_READ_PUBLIC")
    @WithMockUser(authorities = COMMUNITY_READ_PUBLIC)
    public void findAllOpenCommunitiesFromUserWithAuthoritiesTest() throws Exception {
        var community = generateCommunity(UUID.randomUUID());
        var communityPage = new PageImpl<>(List.of(community), PageRequest.of(0, 1), 1);
        var communityListView = generateCommunityListView(community.getId());

        when(communityMapper.toCommunityListView(any(Community.class), anyInt())).thenReturn(communityListView);
        when(communityService.getAllPublicCommunities(anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPage);

        mockMvc.perform(get(BASE_URI + PUBLIC_URI))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(community.getId().toString())));
    }

    @Test
    @DisplayName("Find all open communities. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_READ_PUBLIC")
    public void findAllOpenCommunitiesFromUserWithoutAuthoritiesTest() throws Exception {
        var community = generateCommunity(UUID.randomUUID());
        var communityPage = new PageImpl<>(List.of(community), PageRequest.of(0, 1), 1);
        var communityListView = generateCommunityListView(community.getId());

        when(communityMapper.toCommunityListView(any(Community.class), anyInt())).thenReturn(communityListView);
        when(communityService.getAllPublicCommunities(anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPage);

        mockMvc.perform(get(BASE_URI + PUBLIC_URI))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Find community by id. Request from user with authorities COMMUNITY_READ")
    @WithMockUser(authorities = COMMUNITY_READ)
    public void findCommunityByIdFromUserWithAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var community = generateCommunity(idCommunity);
        var communityView = generateCommunityView(idCommunity);

        when(communityService.getCommunity(any())).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.MEMBER);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(community.getId().toString())));
    }

    @Test
    @DisplayName("Find community by id. Request from user with authorities COMMUNITY_READ_PUBLIC")
    @WithMockUser(authorities = COMMUNITY_READ_PUBLIC)
    public void findCommunityByIdFromUserWithAuthoritiesCommunityReadPublicTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var community = generateCommunity(idCommunity);
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isCommunityPublic(idCommunity)).thenReturn(true);
        when(communityService.getCommunity(any())).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.MEMBER);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(community.getId().toString())));
    }

    @Test
    @DisplayName("Find community by id. Request from user with authorities COMMUNITY_READ_OWN")
    @WithMockUser(authorities = COMMUNITY_READ_OWN)
    public void findCommunityByIdFromUserWithAuthoritiesCommunityReadOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var community = generateCommunity(idCommunity);
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityService.getCommunity(any())).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.MEMBER);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);


        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(community.getId().toString())));

    }

    @Test
    @DisplayName("Find community by id. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_READ")
    public void findCommunityByIdFromUserWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var community = generateCommunity(idCommunity);
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityService.getCommunity(any())).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.MEMBER);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM, UUID.randomUUID()))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Create community. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void createCommunityFromUserWithAuthoritiesCommunityWriteTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = generateCreateCommunityRequest();
        var response = generateCommunity(idCommunity);
        var communityView = generateCommunityView(idCommunity);

        when(communityService.saveCommunity(any(CreateCommunityRequest.class), any(Person.class))).thenReturn(response);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(BASE_URI)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));

    }

    @Test
    @DisplayName("Create community. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void createCommunityFromUserWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = generateCreateCommunityRequest();
        var response = generateCommunity(idCommunity);
        var communityView = generateCommunityView(idCommunity);

        when(communityService.saveCommunity(any(CreateCommunityRequest.class), any(Person.class))).thenReturn(response);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(BASE_URI)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));

    }

    @Test
    @DisplayName("Create community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN")
    public void createCommunityFromUserWithoutAuthoritiesTest() throws Exception {
        var request = generateCreateCommunityRequest();

        when(communityService.saveCommunity(any(CreateCommunityRequest.class), any(Person.class))).thenReturn(new Community());
        when(communityPersonRoleService.getPersonalRoleInCommunity(UUID.randomUUID(), UUID.randomUUID())).thenReturn(createCommunityRole());

        mockMvc.perform(multipart(BASE_URI)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Create community with avatar. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void createCommunityWithAvatarFromUserWithAuthoritiesCommunityWriteTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = generateCreateCommunityRequest();
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var communityView = generateCommunityView(idCommunity);
        var response = generateCommunity(idCommunity);

        when(communityService.saveCommunity(any(CreateCommunityRequest.class), any(MultipartFile.class), any(Person.class))).thenReturn(response);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + NEW_AVATAR_URI)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Create community with avatar. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void createCommunityWithAvatarFromUserWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = generateCreateCommunityRequest();
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var communityView = generateCommunityView(idCommunity);
        var response = generateCommunity(idCommunity);


        when(communityService.saveCommunity(any(CreateCommunityRequest.class), any(MultipartFile.class), any(Person.class))).thenReturn(response);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + NEW_AVATAR_URI)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Create community with avatar. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN")
    public void createCommunityWithAvatarFromUserWithoutAuthoritiesTest() throws Exception {
        var request = generateCreateCommunityRequest();
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);

        when(communityService.saveCommunity(any(CreateCommunityRequest.class), any(MultipartFile.class), any(Person.class))).thenReturn(new Community());
        when(communityPersonRoleService.getPersonalRoleInCommunity(UUID.randomUUID(), UUID.randomUUID())).thenReturn(createCommunityRole());

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + NEW_AVATAR_URI)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Update community. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void updateCommunityFromUserWithAuthoritiesCommunityWriteTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = generateUpdateCommunityRequest();
        var communityView = generateCommunityView(idCommunity);
        var community = generateCommunity(idCommunity);

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        when(communityService.updateCommunity(idCommunity, request)).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));

    }

    @Test
    @DisplayName("Update community. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void updateCommunityFromUserWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = generateUpdateCommunityRequest();
        var communityView = generateCommunityView(idCommunity);
        var community = generateCommunity(idCommunity);

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        when(communityService.updateCommunity(idCommunity, request)).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Update community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN")
    public void updateCommunityFromUserWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = generateUpdateCommunityRequest();
        var communityView = generateCommunityView(idCommunity);
        var community = generateCommunity(idCommunity);

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        when(communityService.updateCommunity(idCommunity, request)).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Set community avatar by ID. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void setCommunityAvatarWithAuthoritiesCommunityWriteTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var communityView = generateCommunityView(idCommunity);
        var community = generateCommunity(idCommunity);

        when(communityService.saveAvatar(any(), any())).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + AVATAR_URI, idCommunity)
                        .file(request)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Set community avatar by ID. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void setCommunityAvatarWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var communityView = generateCommunityView(idCommunity);
        var community = generateCommunity(idCommunity);

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        when(communityService.saveAvatar(any(), any())).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + AVATAR_URI, idCommunity)
                        .file(request)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Set community avatar by ID. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN")
    public void setCommunityAvatarWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var communityView = generateCommunityView(idCommunity);
        var community = generateCommunity(idCommunity);

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        when(communityService.saveAvatar(any(), any())).thenReturn(community);
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + AVATAR_URI, idCommunity)
                        .file(request)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Delete community by ID. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void deleteCommunityWithAuthoritiesCommunityWriteTest() throws Exception {
        var idCommunity = UUID.randomUUID();

        doNothing().when(communityService).deleteCommunity(any());

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));

    }

    @Test
    @DisplayName("Delete community by ID. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void deleteCommunityWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        doNothing().when(communityService).deleteCommunity(any());

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));

    }

    @Test
    @DisplayName("Delete community by ID. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE")
    public void deleteCommunityWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();

        doNothing().when(communityService).deleteCommunity(any());

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM, idCommunity))
                .andExpect(status().isForbidden());
    }

    //--COMMUNITY-MEMBERS-------------------------------------------------------------------------------------------------//
    @Test
    @DisplayName("Find all users in a communities. Request from user with authorities COMMUNITY_READ")
    @WithMockUser(authorities = COMMUNITY_READ)
    public void findAllUsersInCommunitiesFromUserWithAuthoritiesCommunityReadTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityPersonRole = generateCommunityPersonRole();
        var communityPersonRolePage = new PageImpl<>(List.of(communityPersonRole), PageRequest.of(0, 1), 1);
        var personCommunityView = generatePersonCommunityView(idCommunity);

        when(communityPersonRoleService.getAllPersonRoleInCommunity(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPersonRolePage);
        when(personMapper.toPersonCommunityView(any(), any())).thenReturn(personCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Find all users in a communities. Request from user with authorities COMMUNITY_READ_PUBLIC")
    @WithMockUser(authorities = COMMUNITY_READ_PUBLIC)
    public void findAllUsersInCommunitiesFromUserWithAuthoritiesCommunityReadPublicTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityPersonRole = generateCommunityPersonRole();
        var communityPersonRolePage = new PageImpl<>(List.of(communityPersonRole), PageRequest.of(0, 1), 1);
        var personCommunityView = generatePersonCommunityView(idCommunity);

        when(securityAccess.isCommunityPublic(idCommunity)).thenReturn(true);
        when(communityPersonRoleService.getAllPersonRoleInCommunity(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPersonRolePage);
        when(personMapper.toPersonCommunityView(any(), any())).thenReturn(personCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Find all users in a communities. Request from user with authorities COMMUNITY_READ_OWN")
    @WithMockUser(authorities = COMMUNITY_READ_OWN)
    public void findAllUsersInCommunitiesFromUserWithAuthoritiesCommunityReadOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityPersonRole = generateCommunityPersonRole();
        var communityPersonRolePage = new PageImpl<>(List.of(communityPersonRole), PageRequest.of(0, 1), 1);
        var personCommunityView = generatePersonCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityPersonRoleService.getAllPersonRoleInCommunity(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPersonRolePage);
        when(personMapper.toPersonCommunityView(any(), any())).thenReturn(personCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Find all users in a communities. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_READ_OWN")
    public void findAllUsersInCommunitiesFromUserWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityPersonRole = generateCommunityPersonRole();
        var communityPersonRolePage = new PageImpl<>(List.of(communityPersonRole), PageRequest.of(0, 1), 1);
        var personCommunityView = generatePersonCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityPersonRoleService.getAllPersonRoleInCommunity(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(communityPersonRolePage);
        when(personMapper.toPersonCommunityView(any(), any())).thenReturn(personCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Add users to community. Request from user with authorities COMMUNITY_INVITE_KICK")
    @WithMockUser(authorities = COMMUNITY_INVITE_KICK)
    public void addUsersToCommunityWithAuthoritiesCommunityInviteKickTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);
        AddMembersRequest[] request = {createAddMemberRequest(UUID.randomUUID()), createAddMemberRequest(UUID.randomUUID())};

        when(communityService.addMembers(any(), any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Add users to community. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void addUsersToCommunityWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);
        AddMembersRequest[] request = {createAddMemberRequest(UUID.randomUUID()), createAddMemberRequest(UUID.randomUUID())};

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        when(communityService.addMembers(any(), any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Add users to community. Request from user with authorities COMMUNITY_INVITE_KICK_OWN")
    @WithMockUser(authorities = COMMUNITY_INVITE_KICK_OWN)
    public void addUsersToCommunityWithAuthoritiesCommunityInviteKickOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);
        AddMembersRequest[] request = {createAddMemberRequest(UUID.randomUUID()), createAddMemberRequest(UUID.randomUUID())};

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        when(communityService.addMembers(any(), any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Add users to community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_INVITE_KICK_OWN")
    public void addUsersToCommunityWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);
        AddMembersRequest[] request = {createAddMemberRequest(UUID.randomUUID()), createAddMemberRequest(UUID.randomUUID())};

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        when(communityService.addMembers(any(), any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(CommunityRole.ADMIN);
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);


        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Join to community. Request from user with authorities COMMUNITY_JOIN_PRIVATE")
    @WithMockUser(authorities = COMMUNITY_JOIN_PRIVATE)
    public void joinToCommunityWithAuthoritiesCommunityJoinPrivateTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);

        when(communityService.joinCommunity(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + JOIN_URI)
                        .param("communityId", idCommunity.toString())
                        .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Join to community. Request from user with authorities COMMUNITY_JOIN_LEAVE")
    @WithMockUser(authorities = COMMUNITY_JOIN_LEAVE)
    public void joinToCommunityWithAuthoritiesCommunityJoinLeaveTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isCommunityPublic(idCommunity)).thenReturn(true);
        when(communityService.joinCommunity(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + JOIN_URI)
                        .param("communityId", idCommunity.toString())
                        .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Join to community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_JOIN_LEAVE")
    public void joinToCommunityWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isCommunityPublic(idCommunity)).thenReturn(true);
        when(communityService.joinCommunity(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + JOIN_URI)
                        .param("communityId", idCommunity.toString())
                        .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Leave community. Request from user with authorities COMMUNITY_JOIN_LEAVE")
    @WithMockUser(authorities = COMMUNITY_JOIN_LEAVE)
    public void leaveCommunityWithAuthoritiesCommunityJoinLeaveTest() throws Exception {
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        doNothing().when(communityService).leaveCommunity(any(), any());

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + LEAVE_URI)
                        .param("communityId", idCommunity.toString())
                        .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Leave community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_JOIN_LEAVE")
    public void leaveCommunityWithoutAuthoritiesTest() throws Exception {
        var id = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityMember(id)).thenReturn(true);
        doNothing().when(communityService).leaveCommunity(any(), any());

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + LEAVE_URI)
                        .param("communityId", id.toString())
                        .contentType(MediaType.APPLICATION_JSON)
                )
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Changing a community members role. Request from user with authorities COMMUNITY_INVITE_KICK")
    @WithMockUser(authorities = COMMUNITY_INVITE_KICK)
    public void changingCommunityMembersRoleWithAuthoritiesCommunityInviteKickTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = createUpdateRoleInCommunityRequest();
        var communityView = generateCommunityView(idCommunity);

        when(communityService.updatePersonRoleInCommunity(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Changing a community members role. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void changingCommunityMembersRoleWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = createUpdateRoleInCommunityRequest();
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        when(communityService.updatePersonRoleInCommunity(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Changing a community members role. Request from user with authorities COMMUNITY_INVITE_KICK_OWN")
    @WithMockUser(authorities = COMMUNITY_INVITE_KICK_OWN)
    public void changingCommunityMembersRoleWithAuthoritiesCommunityInviteKickOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = createUpdateRoleInCommunityRequest();
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        when(communityService.updatePersonRoleInCommunity(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Changing a community members role. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_INVITE_KICK_OWN")
    public void changingCommunityMembersRoleWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var request = createUpdateRoleInCommunityRequest();
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        when(communityService.updatePersonRoleInCommunity(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Assign a new community owner. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void assignNewCommunityOwnerWithAuthoritiesCommunityWriteTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idPerson = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);

        when(communityService.changeOwner(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI + OWNER_URI, idCommunity)
                        .param("personId", idPerson.toString())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Assign a new community owner. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void assignNewCommunityOwnerWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idPerson = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        when(communityService.changeOwner(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI + OWNER_URI, idCommunity)
                        .param("personId", idPerson.toString())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Assign a new community owner. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN")
    public void assignNewCommunityOwnerWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idPerson = UUID.randomUUID();
        var communityView = generateCommunityView(idCommunity);

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        when(communityService.changeOwner(any(), any())).thenReturn(generateCommunity(idCommunity));
        when(communityPersonRoleService.getPersonalRoleInCommunity(any(), any())).thenReturn(createCommunityRole());
        when(communityMapper.toView(any(Community.class), any(CommunityRole.class))).thenReturn(communityView);


        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI + ROLES_URI + OWNER_URI, idCommunity)
                        .param("personId", idPerson.toString())
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Delete users from a community. Request from user with authorities COMMUNITY_INVITE_KICK")
    @WithMockUser(authorities = COMMUNITY_INVITE_KICK)
    public void deleteUsersFromCommunityWithAuthoritiesCommunityInviteKickTest() throws Exception {
        var request = generateDeleteMemberRequest();
        var idCommunity = UUID.randomUUID();

        doNothing().when(communityService).deleteMembers(any(), any());

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete users from a community. Request from user with authorities COMMUNITY_WRITE_OWN")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN)
    public void deleteUsersFromCommunityWithAuthoritiesCommunityWriteOwnTest() throws Exception {
        var request = generateDeleteMemberRequest();
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityOwner(idCommunity)).thenReturn(true);
        doNothing().when(communityService).deleteMembers(any(), any());

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete users from a community. Request from user with authorities COMMUNITY_INVITE_KICK_OWN")
    @WithMockUser(authorities = COMMUNITY_INVITE_KICK_OWN)
    public void deleteUsersFromCommunityWithAuthoritiesCommunityInviteKickOwnTest() throws Exception {
        var request = generateDeleteMemberRequest();
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        doNothing().when(communityService).deleteMembers(any(), any());

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete users from a community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_INVITE_KICK_OWN")
    public void deleteUsersFromCommunityWithoutAuthoritiesTest() throws Exception {
        var request = generateDeleteMemberRequest();
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityOwnerOrAdmin(idCommunity)).thenReturn(true);
        doNothing().when(communityService).deleteMembers(any(), any());

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MEMBERS_URI, idCommunity)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    //--COMMUNITY-MESSAGES------------------------------------------------------------------------------------------------//
    @Test
    @DisplayName("Find all messages in a communities. Request from user with authorities COMMUNITY_READ")
    @WithMockUser(authorities = COMMUNITY_READ)
    public void findAllMessagesInCommunitiesFromUserWithAuthoritiesCommunityReadTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var record = generateRecord();
        var community = generateCommunity(idCommunity);
        record.setCommunity(community);
        var recordPage = new PageImpl<>(List.of(record), PageRequest.of(0, 1), 1);
        var recordCommunityView = generateCommunityRecordView(record.getId());

        when(communityRecordService.getAllCommunityRecords(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(recordPage);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(record.getId().toString())));
    }

    @Test
    @DisplayName("Find all messages in a communities. Request from user with authorities COMMUNITY_READ_PUBLIC")
    @WithMockUser(authorities = COMMUNITY_READ_PUBLIC)
    public void findAllMessagesInCommunitiesFromUserWithAuthoritiesCommunityReadPublicTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var record = generateRecord();
        var community = generateCommunity(idCommunity);
        record.setCommunity(community);
        var recordPage = new PageImpl<>(List.of(record), PageRequest.of(0, 1), 1);
        var recordCommunityView = generateCommunityRecordView(record.getId());

        when(securityAccess.isCommunityPublic(idCommunity)).thenReturn(true);
        when(communityRecordService.getAllCommunityRecords(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(recordPage);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(record.getId().toString())));
    }

    @Test
    @DisplayName("Find all messages in a communities. Request from user with authorities COMMUNITY_READ_OWN")
    @WithMockUser(authorities = COMMUNITY_READ_OWN)
    public void findAllMessagesInCommunitiesFromUserWithAuthoritiesCommunityReadOwnTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var record = generateRecord();
        var community = generateCommunity(idCommunity);
        record.setCommunity(community);
        var recordPage = new PageImpl<>(List.of(record), PageRequest.of(0, 1), 1);
        var recordCommunityView = generateCommunityRecordView(record.getId());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.getAllCommunityRecords(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(recordPage);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI, idCommunity))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(record.getId().toString())));
    }

    @Test
    @DisplayName("Find all messages in a communities. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_READ_OWN")
    public void findAllMessagesInCommunitiesFromUserWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var record = generateRecord();
        var community = generateCommunity(idCommunity);
        record.setCommunity(community);
        var recordPage = new PageImpl<>(List.of(record), PageRequest.of(0, 1), 1);
        var recordCommunityView = generateCommunityRecordView(record.getId());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.getAllCommunityRecords(
                any(),
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(recordPage);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI, idCommunity))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Find message by ID. Request from user with authorities COMMUNITY_READ")
    @WithMockUser(authorities = COMMUNITY_READ)
    public void findMessageByIdFromUserWithAuthoritiesCommunityReadTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);

        when(communityRecordService.getCommunityRecord(any(), any())).thenReturn(generateRecord());
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Find message by ID. Request from user with authorities COMMUNITY_READ_PUBLIC")
    @WithMockUser(authorities = COMMUNITY_READ_PUBLIC)
    public void findMessageByIdFromUserWithAuthoritiesCommunityReadPublicTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);

        when(securityAccess.isCommunityPublic(idCommunity)).thenReturn(true);
        when(communityRecordService.getCommunityRecord(any(), any())).thenReturn(generateRecord());
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Find message by ID. Request from user with authorities COMMUNITY_READ_OWN")
    @WithMockUser(authorities = COMMUNITY_READ_OWN)
    public void findMessageByIdFromUserWithAuthoritiesCommunityReadOwnTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.getCommunityRecord(any(), any())).thenReturn(generateRecord());
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Find message by ID. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_READ_OWN")
    public void findMessageByIdFromUserWithoutAuthoritiesTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.getCommunityRecord(any(), any())).thenReturn(generateRecord());
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Write a message to the community. Request from user with authorities COMMUNITY_WRITE_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_MESSAGE)
    public void writeMessageToCommunityWithAuthoritiesCommunityWriteMessageTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idRecord = UUID.randomUUID();
        var request = generateCreateRecordRequest();
        var record = generateRecord(idRecord);
        var recordCommunityView = generateCommunityRecordView(idCommunity);
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);

        when(communityRecordService.sendMessage(
                any(),
                any(CreateRecordRequest.class),
                any(Person.class),
                any()
        )).thenReturn(record);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI, idCommunity)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Write a message to the community. Request from user with authorities COMMUNITY_WRITE_OWN_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_MESSAGE)
    public void writeMessageToCommunityWithAuthoritiesCommunityWriteOwnMessageTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idRecord = UUID.randomUUID();
        var request = generateCreateRecordRequest();
        var record = generateRecord(idRecord);
        var recordCommunityView = generateCommunityRecordView(idCommunity);
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.sendMessage(
                any(),
                any(CreateRecordRequest.class),
                any(Person.class),
                any()
        )).thenReturn(record);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI, idCommunity)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Write a message to the community. Request from user without authorities.")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void writeMessageToCommunityWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idRecord = UUID.randomUUID();
        var request = generateCreateRecordRequest();
        var record = generateRecord(idRecord);
        var recordCommunityView = generateCommunityRecordView(idCommunity);
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.sendMessage(
                any(),
                any(CreateRecordRequest.class),
                any(Person.class),
                any()
        )).thenReturn(record);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI, idCommunity)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Edit a message in a community by ID. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void editMessageInCommunityWithAuthoritiesCommunityWriteTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(communityRecordService.updateMessage(idMessage, idCommunity, request)).thenReturn(response);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Edit a message in a community by ID. Request from user with authorities COMMUNITY_WRITE_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_MESSAGE)
    public void editMessageInCommunityWithAuthoritiesCommunityWriteMessageTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.updateMessage(idMessage, idCommunity, request)).thenReturn(response);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Edit a message in a community by ID. Request from user with authorities COMMUNITY_WRITE_OWN_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_MESSAGE)
    public void editMessageInCommunityWithAuthoritiesCommunityWriteOwnMessageTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(communityRecordService.updateMessage(idMessage, idCommunity, request)).thenReturn(response);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idCommunity.toString())));
    }

    @Test
    @DisplayName("Edit a message in a community by ID. Request from user without authorities.")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void editMessageInCommunityWithoutAuthoritiesTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var recordCommunityView = generateCommunityRecordView(idCommunity);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(communityRecordService.updateMessage(idMessage, idCommunity, request)).thenReturn(response);
        when(communityRecordMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Delete a message from a community by ID. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void deleteMessageFromCommunityWithAuthoritiesCommunityWriteTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();

        doNothing().when(communityRecordService).deleteMessage(idMessage, idCommunity);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete a message from a community by ID. Request from user with authorities COMMUNITY_WRITE_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_MESSAGE)
    public void deleteMessageFromCommunityWithAuthoritiesCommunityWriteMessageTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        doNothing().when(communityRecordService).deleteMessage(idMessage, idCommunity);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete a message from a community by ID. Request from user with authorities COMMUNITY_WRITE_OWN_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_MESSAGE)
    public void deleteMessageFromCommunityWithAuthoritiesCommunityWriteOwnMessageTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        doNothing().when(communityRecordService).deleteMessage(idMessage, idCommunity);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete a message from a community by ID. Request from user without authorities.")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void deleteMessageFromCommunityWithoutAuthoritiesTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        doNothing().when(communityRecordService).deleteMessage(idMessage, idCommunity);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM, idCommunity, idMessage))
                .andExpect(status().isForbidden());
    }

//--COMMUNITY-MESSAGE-COMMENTS----------------------------------------------------------------------------------------//

    @Test
    @DisplayName("Find all comments on a message in a communities. Request from user with authorities COMMUNITY_READ_COMMENT")
    @WithMockUser(authorities = COMMUNITY_READ_COMMENT)
    public void findAllCommentsOnMessageInCommunitiesFromUserWithAuthoritiesCommunityReadCommentTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var comment = generateRecord(idComment);
        var commentView = generateCommentView(idComment);
        var recordPage = new PageImpl<>(List.of(comment), PageRequest.of(0, 1), 1);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(false);
        when(commentService.getAllComments(
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class),
                any(UUID.class),
                any(RecordType.class))
        ).thenReturn(recordPage);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(comment.getId().toString())));
    }

    @Test
    @DisplayName("Find all comments on a message in a communities. Request from user with authorities COMMUNITY_READ_OWN_COMMENT")
    @WithMockUser(authorities = COMMUNITY_READ_OWN_COMMENT)
    public void findAllCommentsOnMessageInCommunitiesFromUserWithAuthoritiesCommunityReadOwnCommentTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var comment = generateRecord(idComment);
        var commentView = generateCommentView(idComment);
        var recordPage = new PageImpl<>(List.of(comment), PageRequest.of(0, 1), 1);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(commentService.getAllComments(
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class),
                any(UUID.class),
                any(RecordType.class))
        ).thenReturn(recordPage);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI, idCommunity, idMessage))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.totalItems", is(1)))
                .andExpect(jsonPath("$.items[0].id", is(comment.getId().toString())));
    }

    @Test
    @DisplayName("Find all comments on a message in a communities. Request from user without authorities.")
    @WithMockUser(authorities = "NO_COMMUNITY_READ_OWN_COMMENT")
    public void findAllCommentsOnMessageInCommunitiesFromUserWithoutAuthoritiesTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var comment = generateRecord(idComment);
        var commentView = generateCommentView(idComment);
        var recordPage = new PageImpl<>(List.of(comment), PageRequest.of(0, 1), 1);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(commentService.getAllComments(
                anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class),
                any(UUID.class),
                any(RecordType.class))
        ).thenReturn(recordPage);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(get(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI, idCommunity, idMessage))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Add a comment to a message in a community. Request from user with authorities COMMUNITY_WRITE_COMMENT")
    @WithMockUser(authorities = COMMUNITY_WRITE_COMMENT)
    public void addCommentToMessageInCommunityWithAuthoritiesCommunityWriteCommentTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var request = generateCreateRecordRequest();
        var comment = generateRecord(idComment);
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var recordCommunityView = generateCommentView(idComment);

        when(commentService.saveComment(
                any(CreateRecordRequest.class),
                any(Person.class),
                any(),
                any(RecordType.class),
                any()
        )).thenReturn(comment);
        when(commentMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI, idCommunity, idMessage)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Add a comment to a message in a community. Request from user with authorities COMMUNITY_WRITE_OWN_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_MESSAGE)
    public void addCommentToMessageInCommunityWithAuthoritiesCommunityWriteOwnCommentTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var request = generateCreateRecordRequest();
        var comment = generateRecord(idComment);
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var recordCommunityView = generateCommentView(idComment);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(commentService.saveComment(
                any(CreateRecordRequest.class),
                any(Person.class),
                any(),
                any(RecordType.class),
                any()
        )).thenReturn(comment);
        when(commentMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI, idCommunity, idMessage)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Add a comment to a message in a community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void addCommentToMessageInCommunityWithoutAuthoritiesTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var request = generateCreateRecordRequest();
        var comment = generateRecord(idComment);
        var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
        var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
        var recordCommunityView = generateCommentView(idComment);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(commentService.saveComment(
                any(CreateRecordRequest.class),
                any(Person.class),
                any(),
                any(RecordType.class),
                any()
        )).thenReturn(comment);
        when(commentMapper.toView(any())).thenReturn(recordCommunityView);

        mockMvc.perform(multipart(HttpMethod.POST, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI, idCommunity, idMessage)
                        .file(file)
                        .file(part)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Edit a comment on a message in a community. Request from user with authorities COMMUNITY_WRITE_COMMENT.")
    @WithMockUser(authorities = COMMUNITY_WRITE_COMMENT)
    public void editCommentOnMessageInCommunityWithAuthoritiesCommunityWriteCommentTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var request = generateUpdateRecordRequest();
        var response = generateRecord(idComment);
        response.setContent(request.getContent());
        var commentView = generateCommentView(idComment);

        when(commentService.updateComment(request, idMessage, idComment)).thenReturn(response);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM, idCommunity, idMessage, idComment)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Edit a comment on a message in a community. Request from user with authorities COMMUNITY_WRITE_OWN_MESSAGE.")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_MESSAGE)
    public void editCommentOnMessageInCommunityWithAuthoritiesCommunityWriteOwnCommentTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var request = generateUpdateRecordRequest();
        var response = generateRecord(idComment);
        response.setContent(request.getContent());
        var commentView = generateCommentView(idComment);

        when(securityAccess.isPrincipalRecordAuthor(idComment)).thenReturn(true);
        when(commentService.updateComment(request, idMessage, idComment)).thenReturn(response);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM, idCommunity, idMessage, idComment)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Edit a comment on a message in a community. Request from user without authorities.")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void editCommentOnMessageInCommunityWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var request = generateUpdateRecordRequest();
        var response = generateRecord(idComment);
        response.setContent(request.getContent());
        var commentView = generateCommentView(idComment);

        when(securityAccess.isPrincipalRecordAuthor(idComment)).thenReturn(true);
        when(commentService.updateComment(request, idMessage, idComment)).thenReturn(response);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM, idCommunity, idMessage, idComment)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Delete a comment on a message in a community. Request from user with authorities COMMUNITY_WRITE_COMMENT")
    @WithMockUser(authorities = COMMUNITY_WRITE_COMMENT)
    public void deleteCommentOnMessageInCommunityWithAuthoritiesCommunityWriteCommentTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var idComment = UUID.randomUUID();

        doNothing().when(commentService).deleteComment(idComment, idMessage);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM, idCommunity, idMessage, idComment))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete a comment on a message in a community. Request from user with authorities COMMUNITY_WRITE_OWN_COMMENT")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_COMMENT)
    public void deleteCommentOnMessageInCommunityWithAuthoritiesCommunityWriteOwnCommentTest() throws Exception {
        UUID idMessage = UUID.randomUUID();
        UUID idCommunity = UUID.randomUUID();
        UUID idComment = UUID.randomUUID();

        when(securityAccess.isPrincipalRecordAuthor(idComment)).thenReturn(true);
        doNothing().when(commentService).deleteComment(idComment, idMessage);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM, idCommunity, idMessage, idComment))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.status", is("success")));
    }

    @Test
    @DisplayName("Delete a comment on a message in a community. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_COMMENT")
    public void deleteCommentOnMessageInCommunityWithoutAuthoritiesTest() throws Exception {
        UUID idMessage = UUID.randomUUID();
        UUID idCommunity = UUID.randomUUID();
        UUID idComment = UUID.randomUUID();

        when(securityAccess.isPrincipalRecordAuthor(idComment)).thenReturn(true);
        doNothing().when(commentService).deleteComment(idComment, idMessage);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM, idCommunity, idMessage, idComment))
                .andExpect(status().isForbidden());
    }

//--COMMUNITY-MESSAGE-ATTACHMENTS-------------------------------------------------------------------------------------//

    @Test
    @DisplayName("Add attachments to a community message by ID. Request from user with authorities COMMUNITY_WRITE.")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void addAttachmentsToCommunityMessageWithAuthoritiesCommunityWriteTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
        var communityRecordView = generateCommunityRecordView(idMessage);
        Collection<MultipartFile> allFile = new ArrayList<>();
        allFile.add(file);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(communityRecordService.addAttachments(idMessage, idCommunity, allFile)).thenReturn(response);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .file(file)
                        .contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idMessage.toString())));

    }

    @Test
    @DisplayName("Add attachments to a community message by ID. Request from user with authorities COMMUNITY_WRITE_MESSAGE.")
    @WithMockUser(authorities = COMMUNITY_WRITE_MESSAGE)
    public void addAttachmentsToCommunityMessageWithAuthoritiesCommunityWriteMessageTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
        var communityRecordView = generateCommunityRecordView(idMessage);
        Collection<MultipartFile> allFile = new ArrayList<>();
        allFile.add(file);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.addAttachments(idMessage, idCommunity, allFile)).thenReturn(response);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .file(file)
                        .contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idMessage.toString())));
    }

    @Test
    @DisplayName("Add attachments to a community message by ID. Request from user with authorities COMMUNITY_WRITE_OWN_MESSAGE.")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_MESSAGE)
    public void addAttachmentsToCommunityMessageWithAuthoritiesCommunityWriteOwnMessageTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
        var communityRecordView = generateCommunityRecordView(idMessage);
        Collection<MultipartFile> allFile = new ArrayList<>();
        allFile.add(file);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(communityRecordService.addAttachments(idMessage, idCommunity, allFile)).thenReturn(response);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .file(file)
                        .contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idMessage.toString())));
    }

    @Test
    @DisplayName("Add attachments to a community message by ID. Request from user without authorities.")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void addAttachmentsToCommunityMessageWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
        var communityRecordView = generateCommunityRecordView(idMessage);
        Collection<MultipartFile> allFile = new ArrayList<>();
        allFile.add(file);
        var request = generateUpdateRecordRequest();
        var response = generateRecord();
        response.setContent(request.getContent());

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(communityRecordService.addAttachments(idMessage, idCommunity, allFile)).thenReturn(response);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .file(file)
                        .contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Delete community message attachments by ID. Request from user with authorities COMMUNITY_WRITE")
    @WithMockUser(authorities = COMMUNITY_WRITE)
    public void deleteCommunityMessageAttachmentsWithAuthoritiesCommunityWriteTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var attachment = generateDeleteAttachmentsRequest();
        var message = generateRecord(idMessage);
        var communityRecordView = generateCommunityRecordView(idMessage);

        when(communityRecordService.deleteAttachments(any(), any(), any(DeleteAttachmentsRequest.class))).thenReturn(message);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .content(convertToJsonString(attachment))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idMessage.toString())));
    }

    @Test
    @DisplayName("Delete community message attachments by ID. Request from user with authorities COMMUNITY_WRITE_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_MESSAGE)
    public void deleteCommunityMessageAttachmentsWithAuthoritiesCommunityWriteMessageTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var attachment = generateDeleteAttachmentsRequest();
        var message = generateRecord(idMessage);
        var communityRecordView = generateCommunityRecordView(idMessage);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(communityRecordService.deleteAttachments(any(), any(), any(DeleteAttachmentsRequest.class))).thenReturn(message);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .content(convertToJsonString(attachment))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idMessage.toString())));
    }

    @Test
    @DisplayName("Delete community message attachments by ID. Request from user with authorities COMMUNITY_WRITE_OWN_MESSAGE")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_MESSAGE)
    public void deleteCommunityMessageAttachmentsWithAuthoritiesCommunityWriteOwnMessageTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var attachment = generateDeleteAttachmentsRequest();
        var message = generateRecord(idMessage);
        var communityRecordView = generateCommunityRecordView(idMessage);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(communityRecordService.deleteAttachments(any(), any(), any(DeleteAttachmentsRequest.class))).thenReturn(message);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .content(convertToJsonString(attachment))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idMessage.toString())));
    }

    @Test
    @DisplayName("Delete community message attachments by ID. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void deleteCommunityMessageAttachmentsWithoutAuthoritiesTest() throws Exception {
        var idMessage = UUID.randomUUID();
        var idCommunity = UUID.randomUUID();
        var attachment = generateDeleteAttachmentsRequest();
        var message = generateRecord(idMessage);
        var communityRecordView = generateCommunityRecordView(idMessage);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(communityRecordService.deleteAttachments(any(), any(), any(DeleteAttachmentsRequest.class))).thenReturn(message);
        when(communityRecordMapper.toView(any(Record.class))).thenReturn(communityRecordView);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage)
                        .content(convertToJsonString(attachment))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }

    //--COMMUNITY-MESSAGE-COMMENT-ATTACHMENTS-----------------------------------------------------------------------------//
    @Test
    @DisplayName("Add attachments to comment in community message by ID. Request from user with authorities COMMUNITY_WRITE_COMMENT.")
    @WithMockUser(authorities = COMMUNITY_WRITE_COMMENT)
    public void addAttachmentsToCommentInCommunityMessageWithAuthoritiesCommunityWriteCommentTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
        var commentView = generateCommentView(idComment);
        var response = generateRecord(idComment);

        when(commentService.addAttachments(any(), any(), any(), any())).thenReturn(response);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage, idComment)
                        .file(file)
                        .contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Add attachments to a community message by ID. Request from user with authorities COMMUNITY_WRITE_OWN_COMMENT.")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_COMMENT)
    public void addAttachmentsToCommentInCommunityMessageWithAuthoritiesCommunityWriteOwnCommentTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
        var commentView = generateCommentView(idComment);
        var response = generateRecord(idComment);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(commentService.addAttachments(any(), any(), any(), any())).thenReturn(response);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage, idComment)
                        .file(file)
                        .contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Add attachments to a community message by ID. Request from user without authorities.")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_MESSAGE")
    public void addAttachmentsToCommentInCommunityMessageWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var file = MultipartFileGeneratorUtils.generateFile(MediaType.IMAGE_JPEG);
        var commentView = generateCommentView(idComment);
        var response = generateRecord(idComment);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(commentService.addAttachments(any(), any(), any(), any())).thenReturn(response);
        when(commentMapper.toView(any(Record.class))).thenReturn(commentView);

        mockMvc.perform(multipart(HttpMethod.PUT, BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage, idComment)
                        .file(file)
                        .contentType(MediaType.MULTIPART_FORM_DATA_VALUE))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Delete community message attachments by ID. Request from user with authorities COMMUNITY_WRITE_COMMENT")
    @WithMockUser(authorities = COMMUNITY_WRITE_COMMENT)
    public void deleteAttachmentsToCommentInCommunityMessageWithAuthoritiesCommunityWriteCommentTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var attachment = generateDeleteAttachmentsRequest();
        var comment = generateRecord(idComment);
        var commentView = generateCommentView(idComment);

        when(commentService.deleteAttachments(any(), any(), any(), any())).thenReturn(comment);
        when(commentMapper.toView(any())).thenReturn(commentView);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage, idComment)
                        .content(convertToJsonString(attachment))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Delete community message attachments by ID. Request from user with authorities COMMUNITY_WRITE_OWN_COMMENT")
    @WithMockUser(authorities = COMMUNITY_WRITE_OWN_COMMENT)
    public void deleteAttachmentsToCommentInCommunityMessageWithAuthoritiesCommunityWriteOwnCommentTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var attachment = generateDeleteAttachmentsRequest();
        var comment = generateRecord(idComment);
        var commentView = generateCommentView(idComment);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(commentService.deleteAttachments(any(), any(), any(), any())).thenReturn(comment);
        when(commentMapper.toView(any())).thenReturn(commentView);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage, idComment)
                        .content(convertToJsonString(attachment))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(jsonPath("$.id", is(idComment.toString())));
    }

    @Test
    @DisplayName("Delete community message attachments by ID. Request from user without authorities")
    @WithMockUser(authorities = "NO_COMMUNITY_WRITE_OWN_COMMENT")
    public void deleteAttachmentsToCommentInCommunityMessageWithoutAuthoritiesTest() throws Exception {
        var idCommunity = UUID.randomUUID();
        var idMessage = UUID.randomUUID();
        var idComment = UUID.randomUUID();
        var attachment = generateDeleteAttachmentsRequest();
        var comment = generateRecord(idComment);
        var commentView = generateCommentView(idComment);

        when(securityAccess.isPrincipalCommunityMember(idCommunity)).thenReturn(true);
        when(securityAccess.isPrincipalRecordAuthor(idMessage)).thenReturn(true);
        when(commentService.deleteAttachments(any(), any(), any(), any())).thenReturn(comment);
        when(commentMapper.toView(any())).thenReturn(commentView);

        mockMvc.perform(delete(BASE_URI + COMMUNITY_ID_URI_PARAM + MESSAGES_URI + MESSAGE_ID_URI_PARAM + COMMENTS_URI + COMMENT_ID_URI_PARAM + ATTACHMENTS_URI, idCommunity, idMessage, idComment)
                        .content(convertToJsonString(attachment))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isForbidden());
    }
}