package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import ru.loylabs.portal.mapper.GradeMapper;
import ru.loylabs.portal.mapper.GradeMapperImpl;
import ru.loylabs.portal.presentation.controller.GradeController;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.service.interfaces.GradeService;

import java.util.UUID;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.GRADE_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.GRADE_WRITE;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.data.GradeTestData.*;

@DisplayName("Grade controller test")
@WebMvcTest(GradeController.class)
@Import({GradeMapperImpl.class})
public class GradeControllerTest extends BaseControllerTest {

	@MockBean
	private GradeService gradeService;
	@Autowired
	private GradeMapper gradeMapper;

	private final static String GRADE_BASE_URI = VERSION_1_URI + GRADE_URI;

	@Test
	@DisplayName("Find all grade. Request from user with authorities")
	@WithMockUser(authorities = GRADE_READ)
	void findAllGradeRequestFromUserWithAuthoritiesTest() throws Exception {
		when(gradeService.getAllGrade(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class))
		).thenReturn(Page.empty());

		mockMvc.perform(get(GRADE_BASE_URI))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find all grade. Request from user without authorities")
	@WithMockUser(authorities = "NOT_GRADE_READ")
	void findAllGradeRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(GRADE_BASE_URI))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Find one grade by id. Request from user with authorities")
	@WithMockUser(authorities = GRADE_READ)
	void findOneGradeRequestFromUserWithAuthoritiesTest() throws Exception {
		var grade = generateGrade();
		when(gradeService.getGrade(any())).thenReturn(grade);

		mockMvc.perform(get(GRADE_BASE_URI + GRADE_ID_URI_PARAM, grade.getId()))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find one grade. Request from user without authorities")
	@WithMockUser(authorities = "NOT_GRADE_READ")
	void findOneGradeRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(GRADE_BASE_URI + GRADE_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Create grade. Request from user with authorities")
	@WithMockUser(authorities = GRADE_WRITE)
	void createGradeRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateCreateGradeRequest();
		var grade = generateGrade();
		when(gradeService.createGrade(any())).thenReturn(grade);

		mockMvc.perform(post(GRADE_BASE_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Create grade. Request from user without authorities")
	@WithMockUser(authorities = "NO_GRADE_WRITE")
	void createGradeRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateCreateGradeRequest();
		var grade = generateGrade();
		when(gradeService.createGrade(any())).thenReturn(grade);

		mockMvc.perform(post(GRADE_BASE_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isForbidden()
				);
	}


	@Test
	@DisplayName("Update grade by id. Request from user with authorities")
	@WithMockUser(authorities = GRADE_WRITE)
	void updateGradeRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateUpdateGradeRequest();
		var grade = generateGrade();
		when(gradeService.updateGrade(grade.getId(), request)).thenReturn(grade);

		mockMvc.perform(put(GRADE_BASE_URI + GRADE_ID_URI_PARAM, grade.getId())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Update grade by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_GRADE_WRITE")
	void updateGradeRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateUpdateGradeRequest();
		var grade = generateGrade();
		when(gradeService.updateGrade(grade.getId(), request)).thenReturn(grade);

		mockMvc.perform(put(GRADE_BASE_URI + GRADE_ID_URI_PARAM, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Delete grade by id. Request from with authorities")
	@WithMockUser(authorities = GRADE_WRITE)
	void deleteGradeRequestFromWithAuthoritiesTest() throws Exception {
		doNothing().when(gradeService).deleteGrade(any());

		mockMvc.perform(delete(GRADE_BASE_URI + GRADE_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Delete grade by id. Request from without authorities")
	@WithMockUser(authorities = "NO_GRADE_WRITE")
	void deleteGradeRequestFromWithoutAuthoritiesTest() throws Exception {
		doNothing().when(gradeService).deleteGrade(any());

		mockMvc.perform(delete(GRADE_BASE_URI + GRADE_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isForbidden());
	}
}
