package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.security.test.context.support.WithMockUser;
import ru.loylabs.portal.mapper.*;
import ru.loylabs.portal.presentation.controller.PersonController;
import ru.loylabs.portal.service.interfaces.CityService;
import ru.loylabs.portal.service.interfaces.PersonService;

import java.util.UUID;

import static org.mockito.Mockito.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.*;
import static ru.loylabs.portal.integration.util.data.PersonTestData.*;
import static ru.loylabs.portal.integration.util.data.StatusTestData.*;
import static org.mockito.ArgumentMatchers.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;

@DisplayName("Person controller test")
@WebMvcTest(PersonController.class)
@Import({PersonMapperImpl.class,
        ContactMapperImpl.class,
        JobMapperImpl.class,
        AttachmentMapperImpl.class,
        CityMapperImpl.class,
        SecurityRoleMapperImpl.class,
        PersonSkillMapperImpl.class,
        SkillMapperImpl.class,
        StatusMapperImpl.class,
        GradeMapperImpl.class})
public class PersonControllerTest extends BaseControllerTest {

    @MockBean
    private PersonService personService;
    @MockBean
    private CityService cityService;
    @Autowired
    private PersonMapper personMapper;

    private final static String PERSON_BASE_URI = VERSION_1_URI + PERSONS_URI;

    @Test
    @DisplayName("Updating status of person. Request from user with authorities")
    @WithMockUser(authorities = PERSON_MODIFY_STATUS)
    void testUpdateStatusWithAuthorities() throws Exception {
        var person = generatePerson(UUID.randomUUID());
        var status = generateStatus();
        when(personService.updateStatus(any(), any())).thenReturn(person);

        mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM + STATUS_URI + STATUS_ID_URI_PARAM,
                        person.getId(), status.getId()))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Updating status of person. Request from user without authorities")
    @WithMockUser(authorities = "NOT_PERSON_MODIFY_STATUS")
    void testUpdateStatusWithoutAuthorities() throws Exception {
        mockMvc.perform(put(PERSON_BASE_URI + PERSON_ID_URI_PARAM + STATUS_URI + STATUS_ID_URI_PARAM,
                        UUID.randomUUID(), UUID.randomUUID()))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Deleting status of person. Request from user with authorities")
    @WithMockUser(authorities = PERSON_MODIFY_STATUS)
    void testDeleteStatusWithAuthorities() throws Exception {
        var person = generatePerson(UUID.randomUUID());
        when(personService.deleteStatus(any())).thenReturn(person);

        mockMvc.perform(delete(PERSON_BASE_URI + PERSON_ID_URI_PARAM + STATUS_URI,
                        person.getId()))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Deleting status of person. Request from user without authorities")
    @WithMockUser(authorities = "NOT_PERSON_MODIFY_STATUS")
    void testDeleteStatusWithoutAuthorities() throws Exception {
        mockMvc.perform(delete(PERSON_BASE_URI + PERSON_ID_URI_PARAM + STATUS_URI,
                        UUID.randomUUID()))
                .andExpect(status().isForbidden());
    }
}
