package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import ru.loylabs.portal.entity.ReviewPersonSkill;
import ru.loylabs.portal.mapper.AttachmentMapper;
import ru.loylabs.portal.mapper.CityMapper;
import ru.loylabs.portal.mapper.ContactMapper;
import ru.loylabs.portal.mapper.GradeMapper;
import ru.loylabs.portal.mapper.JobMapper;
import ru.loylabs.portal.mapper.PersonMapper;
import ru.loylabs.portal.mapper.PersonSkillMapper;
import ru.loylabs.portal.mapper.ReviewPersonSkillMapper;
import ru.loylabs.portal.mapper.ReviewPersonSkillRatingMapper;
import ru.loylabs.portal.mapper.ReviewerSkillMapper;
import ru.loylabs.portal.mapper.SecurityRoleMapper;
import ru.loylabs.portal.mapper.SkillMapper;
import ru.loylabs.portal.mapper.StatusMapper;
import ru.loylabs.portal.presentation.controller.ReviewController;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkill;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkillRating;
import ru.loylabs.portal.presentation.request.UpdateReviewPersonSkill;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.service.interfaces.ReviewPersonService;

import java.util.UUID;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.ArgumentMatchers.anyString;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.delete;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.multipart;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.put;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.OWNER_REVIEW_URI;
import static ru.loylabs.portal.config.URIConstants.PERSON_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.PERSON_URI;
import static ru.loylabs.portal.config.URIConstants.REVIEWER_URI;
import static ru.loylabs.portal.config.URIConstants.REVIEW_ID_URI_PARAM;
import static ru.loylabs.portal.config.URIConstants.REVIEW_RATING_URI;
import static ru.loylabs.portal.config.URIConstants.REVIEW_URI;
import static ru.loylabs.portal.config.URIConstants.VERSION_1_URI;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REVIEW_RATING_WRITE;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REVIEW_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.REVIEW_WRITE;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateCreateReviewPersonSkill;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateCreateReviewPersonSkillRating;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateReviewPersonSkill;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateReviewPersonSkillRating;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateUpdateReviewPersonSkill;

@DisplayName("Review controller test")
@WebMvcTest({ReviewController.class, ReviewPersonSkillMapper.class, ReviewPersonSkillRatingMapper.class,
		PersonMapper.class, ContactMapper.class, JobMapper.class, GradeMapper.class, AttachmentMapper.class,
		CityMapper.class, SecurityRoleMapper.class, PersonSkillMapper.class, SkillMapper.class, StatusMapper.class,
		ReviewerSkillMapper.class})
public class ReviewControllerTest extends BaseControllerTest {

	@MockBean
	private ReviewPersonService reviewPersonService;

	private final static String REVIEW_BASE_URI = VERSION_1_URI + REVIEW_URI;

	@Test
	@DisplayName("Find all person reviewers. Request from user with authorities")
	@WithMockUser(authorities = REVIEW_READ)
	void findAllPersonReviewersRequestFromUserWithAuthoritiesTest() throws Exception {
		when(reviewPersonService.getAllReviewPersonSkillByOwner(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class),
				any())
		).thenReturn(Page.empty());

		mockMvc.perform(get(REVIEW_BASE_URI + PERSON_URI + PERSON_ID_URI_PARAM + OWNER_REVIEW_URI + REVIEW_URI,
						UUID.randomUUID()))
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Find all person reviewers. Request from user without authorities")
	@WithMockUser(authorities = "NOT_REVIEW_READ")
	void findAllPersonReviewersRequestFromUserWithoutAuthoritiesTest() throws Exception {
		when(reviewPersonService.getAllReviewPersonSkillByOwner(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class),
				any())
		).thenReturn(Page.empty());

		mockMvc.perform(get(REVIEW_BASE_URI + PERSON_URI + PERSON_ID_URI_PARAM + OWNER_REVIEW_URI + REVIEW_URI,
						UUID.randomUUID()))
				.andExpect(status().isForbidden());
	}

	@Test
	@DisplayName("Find all person participating in review. Request from user with authorities")
	@WithMockUser(authorities = REVIEW_READ)
	void findAllPersonParticipatingInReviewRequestFromUserWithAuthoritiesTest() throws Exception {
		when(reviewPersonService.getAllReviewPersonSkillByIdReviewer(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class),
				any())
		).thenReturn(Page.empty());

		mockMvc.perform(get(REVIEW_BASE_URI + PERSON_URI + PERSON_ID_URI_PARAM + REVIEWER_URI + REVIEW_URI,
						UUID.randomUUID()))
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Find all person participating in review. Request from user without authorities")
	@WithMockUser(authorities = "NOT_REVIEW_READ")
	void findAllPersonParticipatingInReviewRequestFromUserWithoutAuthoritiesTest() throws Exception {
		when(reviewPersonService.getAllReviewPersonSkillByIdReviewer(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class),
				any())
		).thenReturn(Page.empty());

		mockMvc.perform(get(REVIEW_BASE_URI + PERSON_URI + PERSON_ID_URI_PARAM + REVIEWER_URI + REVIEW_URI,
						UUID.randomUUID()))
				.andExpect(status().isForbidden());
	}

	@Test
	@DisplayName("Find review by id. Request from user with authorities")
	@WithMockUser(authorities = REVIEW_READ)
	void findReviewByIdRequestFromUserWithAuthoritiesTest() throws Exception {
		when(reviewPersonService.getReviewPersonSkillById(any())).thenReturn(new ReviewPersonSkill());

		mockMvc.perform(get(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Find review by id. Request from user without authorities")
	@WithMockUser(authorities = "NOT_REVIEW_READ")
	void findReviewByIdRequestFromUserWithoutAuthoritiesTest() throws Exception {
		when(reviewPersonService.getReviewPersonSkillById(any())).thenReturn(new ReviewPersonSkill());

		mockMvc.perform(get(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isForbidden());
	}

	@Test
	@DisplayName("Create review. Request from user with authorities")
	@WithMockUser(authorities = REVIEW_WRITE)
	void createReviewRequestFromUserWithAuthoritiesTest() throws Exception {
		var id = UUID.randomUUID();
		var review = generateReviewPersonSkill();
		review.setId(id);
		var request = generateCreateReviewPersonSkill();
		when(reviewPersonService.createReview(any(CreateReviewPersonSkill.class))).thenReturn(review);

		mockMvc.perform(multipart(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(review.getId().toString()));
	}

	@Test
	@DisplayName("Create review. Request from user without authorities")
	@WithMockUser(authorities = "NOT_REVIEW_WRITE")
	void createReviewRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var id = UUID.randomUUID();
		var review = generateReviewPersonSkill();
		review.setId(id);
		var request = generateCreateReviewPersonSkill();
		when(reviewPersonService.createReview(any(CreateReviewPersonSkill.class))).thenReturn(review);

		mockMvc.perform(multipart(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isForbidden());
	}

	@Test
	@DisplayName("Create rating review. Request from user with authorities")
	@WithMockUser(authorities = REVIEW_RATING_WRITE)
	void createRatingReviewRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateCreateReviewPersonSkillRating(5);
		var ratingReview = generateReviewPersonSkillRating();
		when(reviewPersonService.createRating(any(), any(CreateReviewPersonSkillRating.class))).thenReturn(ratingReview);

		mockMvc.perform(multipart(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM + REVIEW_RATING_URI, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(ratingReview.getId().toString()));
	}

	@Test
	@DisplayName("Create rating review. Request from user without authorities")
	@WithMockUser(authorities = "NOT_REVIEW_RATING_WRITE")
	void createRatingReviewRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateCreateReviewPersonSkillRating(5);
		var ratingReview = generateReviewPersonSkillRating();
		when(reviewPersonService.createRating(any(), any(CreateReviewPersonSkillRating.class))).thenReturn(ratingReview);

		mockMvc.perform(multipart(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM + REVIEW_RATING_URI, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isForbidden());
	}

	@Test
	@DisplayName("Update review. Request from user with authorities")
	@WithMockUser(authorities = REVIEW_WRITE)
	void updateReviewRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateUpdateReviewPersonSkill();
		var review = generateReviewPersonSkill();
		when(reviewPersonService.updateReviewPersonSkill(any(), any(UpdateReviewPersonSkill.class))).thenReturn(review);

		mockMvc.perform(put(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(review.getId().toString()));
	}

	@Test
	@DisplayName("Update review. Request from user without authorities")
	@WithMockUser(authorities = "NOT_REVIEW_WRITE")
	void updateReviewRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateUpdateReviewPersonSkill();
		var review = generateReviewPersonSkill();
		when(reviewPersonService.updateReviewPersonSkill(any(), any(UpdateReviewPersonSkill.class))).thenReturn(review);

		mockMvc.perform(put(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isForbidden());
	}

	@Test
	@DisplayName("Delete review. Request from user with authorities")
	@WithMockUser(authorities = REVIEW_WRITE)
	void deleteReviewRequestFromUserWithAuthoritiesTest() throws Exception {
		doNothing().when(reviewPersonService).removeReview(any());

		mockMvc.perform(delete(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Delete review. Request from user without authorities")
	@WithMockUser(authorities = "NOT_REVIEW_WRITE")
	void deleteReviewRequestFromUserWithoutAuthoritiesTest() throws Exception {
		doNothing().when(reviewPersonService).removeReview(any());

		mockMvc.perform(delete(REVIEW_BASE_URI + PERSON_URI + REVIEW_URI + REVIEW_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isForbidden());
	}

}
