package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import ru.loylabs.portal.mapper.SkillMapper;
import ru.loylabs.portal.mapper.SkillMapperImpl;
import ru.loylabs.portal.presentation.controller.SkillController;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.service.interfaces.SkillService;

import java.util.UUID;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.SKILL_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.SKILL_WRITE;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.data.SkillTestData.*;

@DisplayName("Skill controller test")
@WebMvcTest(SkillController.class)
@Import({SkillMapperImpl.class})
public class SkillControllerTest extends BaseControllerTest {

	@MockBean
	private SkillService skillService;
	@Autowired
	private SkillMapper skillMapper;

	private final static String SKILL_BASE_URI = VERSION_1_URI + SKILL_URI;

	@Test
	@DisplayName("Find all skill. Request from user with authorities")
	@WithMockUser(authorities = SKILL_READ)
	void findAllSkillRequestFromUserWithAuthoritiesTest() throws Exception {
		when(skillService.getAllSkill(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class))
		).thenReturn(Page.empty());

		mockMvc.perform(get(SKILL_BASE_URI))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find all skill. Request from user without authorities")
	@WithMockUser(authorities = "NOT_SKILL_READ")
	void findAllSkillRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(SKILL_BASE_URI))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Find one skill by id. Request from user with authorities")
	@WithMockUser(authorities = SKILL_READ)
	void findOneSkillRequestFromUserWithAuthoritiesTest() throws Exception {
		var skill = generateSkill();
		when(skillService.getSkillById(any())).thenReturn(skill);

		mockMvc.perform(get(SKILL_BASE_URI + SKILL_ID_URI_PARAM, skill.getId()))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find one skill. Request from user without authorities")
	@WithMockUser(authorities = "NOT_SKILL_READ")
	void findOneSkillRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(SKILL_BASE_URI + SKILL_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Create skill. Request from user with authorities")
	@WithMockUser(authorities = SKILL_WRITE)
	void createSkillRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateCreateSkillRequest();
		var skill = generateSkill();
		when(skillService.createSkill(any())).thenReturn(skill);

		mockMvc.perform(post(SKILL_BASE_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Create skill. Request from user without authorities")
	@WithMockUser(authorities = "NO_SKILL_WRITE")
	void createSkillRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateCreateSkillRequest();
		var skill = generateSkill();
		when(skillService.createSkill(any())).thenReturn(skill);

		mockMvc.perform(post(SKILL_BASE_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isForbidden()
				);
	}


	@Test
	@DisplayName("Update skill by id. Request from user with authorities")
	@WithMockUser(authorities = SKILL_WRITE)
	void updateSkillRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateUpdateSkillRequest();
		var skill = generateSkill();
		when(skillService.updateSkill(skill.getId(), request)).thenReturn(skill);

		mockMvc.perform(put(SKILL_BASE_URI + SKILL_ID_URI_PARAM, skill.getId())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Update skill by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_SKILL_WRITE")
	void updateSkillRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateUpdateSkillRequest();
		var skill = generateSkill();
		when(skillService.updateSkill(skill.getId(), request)).thenReturn(skill);

		mockMvc.perform(put(SKILL_BASE_URI + SKILL_ID_URI_PARAM, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Delete skill by id. Request from with authorities")
	@WithMockUser(authorities = SKILL_WRITE)
	void deleteSkillRequestFromWithAuthoritiesTest() throws Exception {
		doNothing().when(skillService).deleteSkill(any());

		mockMvc.perform(delete(SKILL_BASE_URI + SKILL_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Delete skill by id. Request from without authorities")
	@WithMockUser(authorities = "NO_SKILL_WRITE")
	void deleteSkillRequestFromWithoutAuthoritiesTest() throws Exception {
		doNothing().when(skillService).deleteSkill(any());

		mockMvc.perform(delete(SKILL_BASE_URI + SKILL_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isForbidden());
	}
}
