package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.annotation.Import;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.testcontainers.shaded.org.apache.commons.lang3.RandomStringUtils;
import ru.loylabs.portal.mapper.StatusMapper;
import ru.loylabs.portal.mapper.StatusMapperImpl;
import ru.loylabs.portal.presentation.controller.StatusController;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.service.interfaces.StatusService;

import java.util.UUID;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.*;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.data.StatusTestData.*;

@DisplayName("Status controller test")
@WebMvcTest(StatusController.class)
@Import({StatusMapperImpl.class})
public class StatusControllerTest extends BaseControllerTest {

    @MockBean
    private StatusService statusService;
    @Autowired
    private StatusMapper statusMapper;

    private final static String STATUS_BASE_URI = VERSION_1_URI + STATUS_URI;

    @Test
    @DisplayName("Find all statuses. Request from user with authorities")
    @WithMockUser(authorities = STATUS_READ)
    void testGetAllStatusesWithAuthorities() throws Exception {
        when(statusService.getAllStatuses(anyInt(),
                anyInt(),
                anyString(),
                any(SearchReversePolishNotation.class))
        ).thenReturn(Page.empty());

        mockMvc.perform(get(STATUS_BASE_URI))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Find all statuses. Request from user without authorities")
    @WithMockUser(authorities = "NOT_STATUS_READ")
    void testGetAllStatusesWithoutAuthoritiesTest() throws Exception {
        mockMvc.perform(get(STATUS_BASE_URI))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Find one status by id. Request from user with authorities")
    @WithMockUser(authorities = STATUS_READ)
    void testGetStatusByIdWithAuthoritiesTest() throws Exception {
        var status = generateStatus();
        when(statusService.getStatus(any(UUID.class))).thenReturn(status);

        mockMvc.perform(get(STATUS_BASE_URI + STATUS_ID_URI_PARAM, status.getId()))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Find one status by id. Request from user without authorities")
    @WithMockUser(authorities = "NOT_STATUS_READ")
    void testGetStatusByNameWithoutAuthoritiesTest() throws Exception {
        mockMvc.perform(get(STATUS_BASE_URI + STATUS_ID_URI_PARAM, UUID.randomUUID()))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Find one status by name. Request from user with authorities")
    @WithMockUser(authorities = STATUS_READ)
    void testGetStatusByNameWithAuthoritiesTest() throws Exception {
        var status = generateStatus();
        when(statusService.getStatus(anyString())).thenReturn(status);

        mockMvc.perform(get(STATUS_BASE_URI + STATUS_NAME_URI_PARAM, status.getName()))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Find one status by name. Request from user without authorities")
    @WithMockUser(authorities = "NOT_STATUS_READ")
    void testGetStatusByIdWithoutAuthoritiesTest() throws Exception {
        mockMvc.perform(get(STATUS_BASE_URI + STATUS_NAME_URI_PARAM, RandomStringUtils.randomAlphabetic(29)))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Create status. Request from user with authorities")
    @WithMockUser(authorities = STATUS_WRITE)
    void createGradeRequestFromUserWithAuthoritiesTest() throws Exception {
        var request = generateCreateStatusRequest();
        var status = generateStatus();
        when(statusService.createStatus(any())).thenReturn(status);

        mockMvc.perform(post(STATUS_BASE_URI)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Create status. Request from user without authorities")
    @WithMockUser(authorities = "NO_STATUS_WRITE")
    void createGradeRequestFromUserWithoutAuthoritiesTest() throws Exception {
        var request = generateCreateStatusRequest();
        var status = generateStatus();
        when(statusService.createStatus(any())).thenReturn(status);

        mockMvc.perform(post(STATUS_BASE_URI)
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isForbidden());
    }


    @Test
    @DisplayName("Update status by id. Request from user with authorities")
    @WithMockUser(authorities = STATUS_WRITE)
    void updateGradeRequestFromUserWithAuthoritiesTest() throws Exception {
        var request = generateUpdateStatusRequest();
        var status = generateStatus();
        when(statusService.updateStatus(status.getId(), request)).thenReturn(status);

        mockMvc.perform(put(STATUS_BASE_URI + STATUS_ID_URI_PARAM, status.getId())
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Update status by id. Request from user without authorities")
    @WithMockUser(authorities = "NO_STATUS_WRITE")
    void updateGradeRequestFromUserWithoutAuthoritiesTest() throws Exception {
        var request = generateUpdateStatusRequest();
        var status = generateStatus();
        when(statusService.updateStatus(status.getId(), request)).thenReturn(status);

        mockMvc.perform(put(STATUS_BASE_URI + STATUS_ID_URI_PARAM, UUID.randomUUID())
                        .content(convertToJsonString(request))
                        .contentType(MediaType.APPLICATION_JSON_VALUE))
                .andExpect(status().isForbidden());
    }

    @Test
    @DisplayName("Delete status by id. Request from with authorities")
    @WithMockUser(authorities = STATUS_WRITE)
    void deleteGradeRequestFromWithAuthoritiesTest() throws Exception {

        mockMvc.perform(delete(STATUS_BASE_URI + STATUS_ID_URI_PARAM, UUID.randomUUID()))
                .andExpect(status().isOk());
    }

    @Test
    @DisplayName("Delete grade by id. Request from without authorities")
    @WithMockUser(authorities = "NO_STATUS_WRITE")
    void deleteGradeRequestFromWithoutAuthoritiesTest() throws Exception {

        mockMvc.perform(delete(STATUS_BASE_URI + STATUS_ID_URI_PARAM, UUID.randomUUID()))
                .andExpect(status().isForbidden());
    }
}
