package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.test.context.support.WithMockUser;
import ru.loylabs.portal.integration.util.data.MultipartFileGeneratorUtils;
import ru.loylabs.portal.mapper.StoryGroupMapper;
import ru.loylabs.portal.mapper.StoryMapper;
import ru.loylabs.portal.presentation.SecurityUtil;
import ru.loylabs.portal.presentation.controller.StoryController;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.service.interfaces.StoryGroupService;
import ru.loylabs.portal.service.interfaces.StoryService;
import ru.loylabs.portal.service.security.SecurityAccess;

import java.util.UUID;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;
import static org.springframework.http.MediaType.APPLICATION_JSON_VALUE;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.jsonPath;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.STORY_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.STORY_WRITE;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.data.StoryTestData.*;

@DisplayName("Story controller test")
@WebMvcTest({StoryController.class, StoryGroupMapper.class, StoryMapper.class})
public class StoryControllerTest extends BaseControllerTest {

	@MockBean
	private StoryGroupService storyGroupService;
	@MockBean
	private StoryService storyService;
	@MockBean
	private SecurityUtil securityUtil;
	@MockBean(name = "sec")
	private SecurityAccess securityAccess;

	public static final String STORY_GROUP_BASE_URI = VERSION_1_URI + STORY_GROUPS_URI;

	public static final String STORY_BASE_URI = STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM + STORIES_URI;



	//--STORY-GROUPS------------------------------------------------------------------------------------------------------//
	@Test
	@DisplayName("Find all story group. Request from user with authorities")
	@WithMockUser(authorities = STORY_READ)
	void findAllStoryGroupRequestFromUserWithAuthoritiesTest() throws Exception {
		when(storyGroupService.getAllStoryGroups(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class))
		).thenReturn(Page.empty());

		mockMvc.perform(get(STORY_GROUP_BASE_URI))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find all story group. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_READ")
	void findAllStoryGroupRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(STORY_GROUP_BASE_URI))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Find one story group by id. Request from user with authorities")
	@WithMockUser(authorities = STORY_READ)
	void findOneStoryGroupRequestFromUserWithAuthoritiesTest() throws Exception {
		var storyGroupId = UUID.randomUUID();
		var storyGroup = generateEmptyStoryGroup();
		storyGroup.setId(storyGroupId);
		when(storyGroupService.getStoryGroup(any()))
				.thenReturn(storyGroup);

		mockMvc.perform(get(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, storyGroupId))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(storyGroupId.toString()));

	}

	@Test
	@DisplayName("Find one story group by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_READ")
	void findOneStoryGroupRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Create story group. Request from user with authorities")
	@WithMockUser(authorities = STORY_WRITE)
	void createStoryGroupRequestFromUserWithAuthoritiesTest() throws Exception {
		var storyGroupId = UUID.randomUUID();
		var storyGroup = generateEmptyStoryGroup();
		storyGroup.setId(storyGroupId);
		var mockFile = MultipartFileGeneratorUtils.generateFile("files", MediaType.IMAGE_JPEG);
		var request = generateCreateStoryGroupRequest();
		var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
		when(storyGroupService.saveStoryGroup(any(), any(), any(), any()))
				.thenReturn(storyGroup);


		mockMvc.perform(multipart(STORY_GROUP_BASE_URI)
						.file(mockFile)
						.file(part)
						.contentType(MediaType.MULTIPART_FORM_DATA))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(storyGroupId.toString()));

	}

	@Test
	@DisplayName("Create story group. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_WRITE")
	void createStoryGroupRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var mockFile = MultipartFileGeneratorUtils.generateFile("files", MediaType.IMAGE_JPEG);
		var request = generateCreateStoryGroupRequest();
		var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());

		mockMvc.perform(multipart(STORY_GROUP_BASE_URI)
						.file(mockFile)
						.file(part)
						.contentType(MediaType.MULTIPART_FORM_DATA))
				.andExpect(status().isForbidden());

	}

	@Test
	@DisplayName("update story group. Request from user with authorities")
	@WithMockUser(authorities = STORY_WRITE)
	void updateStoryGroupRequestFromUserWithAuthoritiesTest() throws Exception {
		var storyGroupId = UUID.randomUUID();
		var storyGroup = generateEmptyStoryGroup();
		storyGroup.setId(storyGroupId);
		var request = generateUpdateStoryGroupRequest();
		when(storyGroupService.updateStoryGroup(any(), any()))
				.thenReturn(storyGroup);


		mockMvc.perform(put(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, storyGroupId)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(storyGroupId.toString()));

	}

	@Test
	@DisplayName("update story group. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_WRITE")
	void updateStoryGroupRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateUpdateStoryGroupRequest();

		mockMvc.perform(put(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isForbidden());

	}

	@Test
	@DisplayName("delete story group by id. Request from user with authorities")
	@WithMockUser(authorities = STORY_WRITE)
	void deleteStoryGroupByUdRequestFromUserWithAuthoritiesTest() throws Exception {
		doNothing().when(storyGroupService).deleteStoryGroup(any());

		mockMvc.perform(delete(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isOk());

	}

	@Test
	@DisplayName("delete story group by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_WRITE")
	void deleteStoryGroupByUdRequestFromUserWithoutAuthoritiesTest() throws Exception {
		doNothing().when(storyGroupService).deleteStoryGroup(any());

		mockMvc.perform(delete(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isForbidden());

	}

////--STORIES-----------------------------------------------------------------------------------------------------------//

	@Test
	@DisplayName("Find all story in group. Request from user with authorities")
	@WithMockUser(authorities = STORY_READ)
	void findAllStoryRequestFromUserWithAuthoritiesTest() throws Exception {
		when(storyService.getAllStoriesFromGroup(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class),
				any())
		).thenReturn(Page.empty());

		mockMvc.perform(get(STORY_BASE_URI, UUID.randomUUID()))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find all story in group. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_READ")
	void findAllStoryRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(STORY_BASE_URI, UUID.randomUUID()))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Find one story in group by id. Request from user with authorities")
	@WithMockUser(authorities = STORY_READ)
	void findOneStoryRequestFromUserWithAuthoritiesTest() throws Exception {
		var storyId = UUID.randomUUID();
		var story = generateStory();
		story.setId(storyId);
		when(storyService.getStory(any(), any()))
				.thenReturn(story);

		mockMvc.perform(get(STORY_BASE_URI + STORY_ID_URI_PARAM, UUID.randomUUID(), storyId))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(storyId.toString()));
	}

	@Test
	@DisplayName("Find one story in group by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_READ")
	void findOneStoryRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(STORY_BASE_URI + STORY_ID_URI_PARAM, UUID.randomUUID(), UUID.randomUUID()))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Create story. Request from user with authorities")
	@WithMockUser(authorities = STORY_WRITE)
	void createStoryRequestFromUserWithAuthoritiesTest() throws Exception {
		var storyId = UUID.randomUUID();
		var story = generateStory();
		story.setId(storyId);
		var mockFile = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
		var request = generateCreateStoryRequest();
		var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());
		when(storyService.saveStory(any(), any(), any(), any()))
				.thenReturn(story);

		mockMvc.perform(multipart(STORY_BASE_URI, UUID.randomUUID())
						.file(mockFile)
						.file(part)
						.contentType(MediaType.MULTIPART_FORM_DATA))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(storyId.toString()));
	}

	@Test
	@DisplayName("Create story. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_WRITE")
	void createStoryRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var mockFile = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
		var request = generateCreateStoryRequest();
		var part = new MockMultipartFile("request", "", APPLICATION_JSON_VALUE, convertToJsonString(request).getBytes());

		mockMvc.perform(multipart(STORY_BASE_URI, UUID.randomUUID())
						.file(mockFile)
						.file(part)
						.contentType(MediaType.MULTIPART_FORM_DATA))
				.andExpect(status().isForbidden());

	}

	@Test
	@DisplayName("update story. Request from user with authorities")
	@WithMockUser(authorities = STORY_WRITE)
	void updateStoryRequestFromUserWithAuthoritiesTest() throws Exception {
		var storyId = UUID.randomUUID();
		var story = generateStory();
		story.setId(storyId);
		var request = generateUpdateStoryRequest();
		when(storyService.updateStory(any(), any(), any()))
				.thenReturn(story);

		mockMvc.perform(put(STORY_BASE_URI + STORY_ID_URI_PARAM, UUID.randomUUID(), UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(storyId.toString()));
	}

	@Test
	@DisplayName("update story. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_WRITE")
	void updateStoryRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateUpdateStoryRequest();

		mockMvc.perform(put(STORY_BASE_URI + STORY_ID_URI_PARAM, UUID.randomUUID(), UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(status().isForbidden());

	}

	@Test
	@DisplayName("delete story by id. Request from user with authorities")
	@WithMockUser(authorities = STORY_WRITE)
	void deleteStoryByUdRequestFromUserWithAuthoritiesTest() throws Exception {
		doNothing().when(storyGroupService).deleteStoryGroup(any());

		mockMvc.perform(delete(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isOk());

	}

	@Test
	@DisplayName("delete story by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_WRITE")
	void deleteStoryByUdRequestFromUserWithoutAuthoritiesTest() throws Exception {
		doNothing().when(storyGroupService).deleteStoryGroup(any());

		mockMvc.perform(delete(STORY_GROUP_BASE_URI + STORY_GROUP_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isForbidden());

	}

//	//--STORY-ATTACHMENTS-------------------------------------------------------------------------------------------------/

	@Test
	@DisplayName("Update story attachment by id. Request from user with authorities")
	@WithMockUser(authorities = STORY_WRITE)
	void updateStoryAttachmentRequestFromUserWithAuthoritiesTest() throws Exception {
		var storyId = UUID.randomUUID();
		var story = generateStory();
		story.setId(storyId);
		var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
		when(storyService.replaceStoryAttachment(any(), any(), any()))
				.thenReturn(story);
		when(securityAccess.isPrincipalRecordAuthor(any())).thenReturn(true);

		mockMvc.perform(multipart(HttpMethod.PUT, STORY_BASE_URI + STORY_ID_URI_PARAM + ATTACHMENTS_URI, UUID.randomUUID(), UUID.randomUUID())
						.file(file)
						.contentType(MediaType.MULTIPART_FORM_DATA))
				.andExpect(status().isOk())
				.andExpect(jsonPath("$.id").value(storyId.toString()));
	}

	@Test
	@DisplayName("Update story attachment by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_STORY_WRITE")
	void updateStoryAttachmentRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var file = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);
		when(securityAccess.isPrincipalRecordAuthor(any())).thenReturn(true);

		mockMvc.perform(multipart(HttpMethod.PUT, STORY_BASE_URI + STORY_ID_URI_PARAM + ATTACHMENTS_URI, UUID.randomUUID(), UUID.randomUUID())
						.file(file)
						.contentType(MediaType.MULTIPART_FORM_DATA))
				.andExpect(status().isForbidden());

	}
}
