package ru.loylabs.portal.unit.controller;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.autoconfigure.web.servlet.WebMvcTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import ru.loylabs.portal.entity.Title;
import ru.loylabs.portal.mapper.TitleMapper;
import ru.loylabs.portal.presentation.controller.TitleController;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.service.interfaces.TitleService;

import java.util.UUID;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.when;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;
import static ru.loylabs.portal.config.URIConstants.*;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.TITLE_READ;
import static ru.loylabs.portal.enums.CredentialPermission.Fields.TITLE_WRITE;
import static ru.loylabs.portal.integration.util.TestUtils.convertToJsonString;
import static ru.loylabs.portal.integration.util.data.JobTestData.generateCreateTitleRequest;
import static ru.loylabs.portal.integration.util.data.JobTestData.generateUpdateTitleRequest;

@DisplayName("Title controller test")
@WebMvcTest(TitleController.class)
public class TitleControllerTest extends BaseControllerTest {

	@MockBean
	private TitleService titleService;
	@MockBean
	private TitleMapper titleMapper;

	private final static String TITLE_BASE_URI = VERSION_1_URI + TITLES_URI;

	@Test
	@DisplayName("Find all title. Request from user with authorities")
	@WithMockUser(authorities = TITLE_READ)
	void findAllTitleRequestFromUserWithAuthoritiesTest() throws Exception {
		when(titleService.getAllTitle(anyInt(),
				anyInt(),
				anyString(),
				any(SearchReversePolishNotation.class))
		).thenReturn(Page.empty());

		mockMvc.perform(get(TITLE_BASE_URI))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find all title. Request from user without authorities")
	@WithMockUser(authorities = "NOT_TITLE_READ")
	void findAllTitleRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(TITLE_BASE_URI))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Find one title by id. Request from user with authorities")
	@WithMockUser(authorities = TITLE_READ)
	void findOneTitleRequestFromUserWithAuthoritiesTest() throws Exception {
		var titleId = UUID.randomUUID();
		when(titleService.getTitleById(any())).thenReturn(new Title());

		mockMvc.perform(get(TITLE_BASE_URI + TITLE_ID_URI_PARAM, titleId))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Find one title. Request from user without authorities")
	@WithMockUser(authorities = "NOT_TITLE_READ")
	void findOneTitleRequestFromUserWithoutAuthoritiesTest() throws Exception {
		mockMvc.perform(get(TITLE_BASE_URI + TITLE_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Create title. Request from user with authorities")
	@WithMockUser(authorities = TITLE_WRITE)
	void createTitleRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateCreateTitleRequest();
		when(titleService.createTitle(any())).thenReturn(new Title());

		mockMvc.perform(post(TITLE_BASE_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Create title. Request from user without authorities")
	@WithMockUser(authorities = "NO_TITLE_WRITE")
	void createTitleRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateCreateTitleRequest();
		when(titleService.createTitle(any())).thenReturn(new Title());

		mockMvc.perform(post(TITLE_BASE_URI)
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isForbidden()
				);
	}


	@Test
	@DisplayName("Update title by id. Request from user with authorities")
	@WithMockUser(authorities = TITLE_WRITE)
	void updateTitleRequestFromUserWithAuthoritiesTest() throws Exception {
		var request = generateUpdateTitleRequest();
		when(titleService.createTitle(any())).thenReturn(new Title());

		mockMvc.perform(put(TITLE_BASE_URI + TITLE_ID_URI_PARAM, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isOk()
				);
	}

	@Test
	@DisplayName("Update title by id. Request from user without authorities")
	@WithMockUser(authorities = "NO_TITLE_WRITE")
	void updateTitleRequestFromUserWithoutAuthoritiesTest() throws Exception {
		var request = generateUpdateTitleRequest();
		when(titleService.createTitle(any())).thenReturn(new Title());

		mockMvc.perform(put(TITLE_BASE_URI + TITLE_ID_URI_PARAM, UUID.randomUUID())
						.content(convertToJsonString(request))
						.contentType(MediaType.APPLICATION_JSON))
				.andExpect(
						status().isForbidden()
				);
	}

	@Test
	@DisplayName("Delete title by id. Request from user with authorities")
	@WithMockUser(authorities = TITLE_WRITE)
	void deleteTitleRequestFromWithAuthoritiesTest() throws Exception {
		doNothing().when(titleService).deleteTitle(any());

		mockMvc.perform(delete(TITLE_BASE_URI + TITLE_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isOk());
	}

	@Test
	@DisplayName("Delete title by id. Request from without authorities")
	@WithMockUser(authorities = "NO_TITLE_WRITE")
	void deleteTitleRequestFromWithoutAuthoritiesTest() throws Exception {
		doNothing().when(titleService).deleteTitle(any());

		mockMvc.perform(delete(TITLE_BASE_URI + TITLE_ID_URI_PARAM, UUID.randomUUID()))
				.andExpect(status().isForbidden());
	}
}
