package ru.loylabs.portal.unit.mapper;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import ru.loylabs.portal.entity.Status;
import ru.loylabs.portal.mapper.StatusMapper;
import ru.loylabs.portal.mapper.StatusMapperImpl;
import ru.loylabs.portal.presentation.request.CreateStatusRequest;
import ru.loylabs.portal.presentation.request.UpdateStatusRequest;
import ru.loylabs.portal.presentation.view.StatusView;

import java.util.*;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class StatusMapperTest {

    @Spy
    private StatusMapper statusMapper = new StatusMapperImpl();

    private UUID id = UUID.randomUUID();
    private String name;
    private Status status;
    private StatusView statusView;
    private CreateStatusRequest createStatusRequest;
    private UpdateStatusRequest updateStatusRequest;

    @BeforeEach
    void setUp() {
        name = "name";
        status = Status.builder()
                .id(id)
                .name(name)
                .build();
        statusView = new StatusView(id.toString(), name);
        createStatusRequest = new CreateStatusRequest(name);
    }

    @Test
    void testViewToEntity() {
        var correctStatus = status;
        var actual = statusMapper.toEntity(statusView);
        assertThat(actual).isEqualTo(correctStatus);
    }

    @Test
    void testCreateRequestToEntity() {
        var correctStatus = Status.builder()
                .name(name)
                .build();
        var actual = statusMapper.toEntity(createStatusRequest);
        assertThat(actual).isEqualTo(correctStatus);
    }

    @Test
    void testToView() {
        var correctView = statusView;
        var actual = statusMapper.toView(status);
        assertThat(actual).isEqualTo(correctView);
    }

    @Test
    void testToListView() {
        UUID id1 = UUID.randomUUID();
        UUID id2 = UUID.randomUUID();
        UUID id3 = UUID.randomUUID();

        String name1 = "name1";
        String name2 = "name2";
        String name3 = "name3";

        List<Status> statuses = new ArrayList<>();
        statuses.add(Status.builder().id(id1).name(name1).build());
        statuses.add(Status.builder().id(id2).name(name2).build());
        statuses.add(Status.builder().id(id3).name(name3).build());

        List<StatusView> statusViews = new ArrayList<>();
        statusViews.add(new StatusView(id1.toString(), name1));
        statusViews.add(new StatusView(id2.toString(), name2));
        statusViews.add(new StatusView(id3.toString(), name3));

        var actual = statusMapper.toListView(statuses);

        assertThat(actual).hasSameElementsAs(statusViews);
    }

    @Test
    void testToSetView() {
        UUID id1 = UUID.randomUUID();
        UUID id2 = UUID.randomUUID();
        UUID id3 = UUID.randomUUID();

        String name1 = "name1";
        String name2 = "name2";
        String name3 = "name3";

        Set<Status> statuses = new HashSet<>();
        statuses.add(Status.builder().id(id1).name(name1).build());
        statuses.add(Status.builder().id(id2).name(name2).build());
        statuses.add(Status.builder().id(id3).name(name3).build());

        Set<StatusView> statusViews = new HashSet<>();
        statusViews.add(new StatusView(id1.toString(), name1));
        statusViews.add(new StatusView(id2.toString(), name2));
        statusViews.add(new StatusView(id3.toString(), name3));

        var actual = statusMapper.toSetView(statuses);

        assertThat(actual).hasSameElementsAs(statusViews);
    }

    @Test
    void testUpdateEntity() {
        String newName = "newName";
        updateStatusRequest = new UpdateStatusRequest(newName);
        statusMapper.updateEntity(updateStatusRequest, status);
        assertThat(status.getName()).isEqualTo(newName);
    }

    @Test
    void testMapId() {
        var actual = statusMapper.mapId(id.toString());
        assertThat(actual).isEqualTo(id);
    }
}