package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.jpa.domain.Specification;
import ru.loylabs.portal.entity.Grade;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.GradeMapperImpl;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.presentation.request.CreateGradeRequest;
import ru.loylabs.portal.presentation.request.UpdateGradeRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.GradeRepository;
import ru.loylabs.portal.service.GradeServiceImpl;
import ru.loylabs.portal.unit.BaseTest;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doReturn;
import static org.mockito.Mockito.when;
import static ru.loylabs.portal.exception.BaseException.Exception.GRADE_ALREADY_EXISTS;
import static ru.loylabs.portal.exception.BaseException.Exception.GRADE_NOT_FOUND;
import static ru.loylabs.portal.integration.util.data.GradeTestData.generateGrade;

@DisplayName("Grade service")
public class GradeServiceTest extends BaseTest {

    private static Grade grade;
    private static Page<Grade> pageGrades;
    private static SearchReversePolishNotation searchCriteria;
    private static CreateGradeRequest createGradeRequest;
    private static CreateGradeRequest createGradeRequestExistingGradeName;
    private static UpdateGradeRequest updateGradeRequest;
    private static UpdateGradeRequest updateGradeRequestExistingGradeName;

    @Spy
    private GradeMapperImpl gradeMapper;
    @Mock
    private GradeRepository gradeRepository;
    @Mock
    private ApplicationEventPublisher publisher;
    @InjectMocks
    private GradeServiceImpl gradeService;

    @BeforeAll
    static void setup() {
        grade = new Grade();
        grade.setName("grade");
        pageGrades = new PageImpl<>(List.of(grade));
        searchCriteria = WebUtils.getCriteria("");
        createGradeRequest = new CreateGradeRequest();
        createGradeRequest.setName("newGrade");
        updateGradeRequest = new UpdateGradeRequest();
        updateGradeRequest.setName("newGrade");
        createGradeRequestExistingGradeName = new CreateGradeRequest();
        createGradeRequestExistingGradeName.setName(grade.getName());
        updateGradeRequestExistingGradeName = new UpdateGradeRequest();
        updateGradeRequestExistingGradeName.setName("newGrade");
    }

    @Test
    @DisplayName("Get all grade")
    void getAllGradeTest() {
        doReturn(pageGrades).when(gradeRepository).findAll(ArgumentMatchers.<Specification<Grade>>any(), any());

        var result = gradeService.getAllGrade(0, 10, "date,asc", searchCriteria);

        assertEquals(pageGrades.getTotalElements(), result.getTotalElements());
        assertEquals(pageGrades.getContent(), result.getContent());
    }

    @Test
    @DisplayName("Get grade by id")
    void getGradeByIdTest() {
        when(gradeRepository.findById(grade.getId())).thenReturn(Optional.of(grade));

        var result = gradeService.getGrade(grade.getId());

        assertEquals(grade.getId(), result.getId());
    }

    @Test
    @DisplayName("Get grade by id. Grade not found")
    void getGradeByIdTestGradeNotFound() {
        var exception = assertThrows(ApplicationException.class,
                () -> gradeService.getGrade(UUID.randomUUID()));

        assertEquals(exception.getErrorCode(), GRADE_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Create grade")
    void createGradeTest() {
        when(gradeRepository.existsByNameIgnoreCase(createGradeRequest.getName())).thenReturn(false);
        when(gradeRepository.save(any())).thenReturn(grade);

        var result = gradeService.createGrade(createGradeRequest);

        assertEquals(grade.getId(), result.getId());
        assertEquals(createGradeRequest.getName(), result.getName());
    }

    @Test
    @DisplayName("Create grade. Grade already exists")
    void createGradeTestGradeAlreadyExists() {
        when(gradeRepository.existsByNameIgnoreCase(createGradeRequestExistingGradeName.getName())).thenReturn(true);

        var exception = assertThrows(ApplicationException.class,
                () -> gradeService.createGrade(createGradeRequestExistingGradeName));

        assertEquals(exception.getErrorCode(), GRADE_ALREADY_EXISTS.getCode());
    }

    @Test
    @DisplayName("Update grade")
    void updateGradeTest() {
        when(gradeRepository.existsByNameIgnoreCase(updateGradeRequest.getName())).thenReturn(false);
        when(gradeRepository.findById(grade.getId())).thenReturn(Optional.of(grade));
        when(gradeRepository.save(any())).thenReturn(grade);

        var result = gradeService.updateGrade(grade.getId(), updateGradeRequest);

        assertEquals(grade.getId(), result.getId());
        assertEquals(updateGradeRequest.getName(), result.getName());
    }

    @Test
    @DisplayName("Update grade. Grade already exists")
    void updateGradeTestGradeAlreadyExists() {
        when(gradeRepository.findById(grade.getId())).thenReturn(Optional.of(grade));
        when(gradeRepository.existsByNameIgnoreCase(updateGradeRequestExistingGradeName.getName())).thenReturn(true);

        var exception = assertThrows(ApplicationException.class,
                () -> gradeService.updateGrade(grade.getId(), updateGradeRequestExistingGradeName));

        assertEquals(exception.getErrorCode(), GRADE_ALREADY_EXISTS.getCode());
    }

    @Test
    @DisplayName("Delete grade")
    void deleteGradeTest() {
        var grade = generateGrade();
        when(gradeRepository.findById(grade.getId())).thenReturn(Optional.of(grade));

        gradeService.deleteGrade(grade.getId());
    }

    @Test
    @DisplayName("Delete grade. grade not found")
    void deleteGradeTestGradeNotFound() {
        var exception = assertThrows(ApplicationException.class,
                () -> gradeService.deleteGrade(UUID.randomUUID()));

        assertEquals(exception.getErrorCode(), GRADE_NOT_FOUND.getCode());
    }
}