package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.Mock;
import org.mockito.MockedStatic;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContext;
import org.springframework.security.core.context.SecurityContextHolder;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.ReviewPersonSkill;
import ru.loylabs.portal.entity.ReviewPersonSkillRating;
import ru.loylabs.portal.entity.Reviewers;
import ru.loylabs.portal.entity.Skill;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.ReviewPersonSkillMapper;
import ru.loylabs.portal.mapper.ReviewPersonSkillRatingMapper;
import ru.loylabs.portal.mapper.ReviewerSkillMapper;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkill;
import ru.loylabs.portal.presentation.request.CreateReviewPersonSkillRating;
import ru.loylabs.portal.presentation.request.UpdateReviewPersonSkill;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.ReviewPersonSkillRatingRepository;
import ru.loylabs.portal.repository.ReviewPersonSkillRepository;
import ru.loylabs.portal.repository.ReviewersRepository;
import ru.loylabs.portal.repository.SkillRepository;
import ru.loylabs.portal.service.review.ReviewPersonServiceImpl;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.EmailNotificationFactory;
import ru.loylabs.portal.service.interfaces.NotificationSender;
import ru.loylabs.portal.service.interfaces.ReviewPersonService;
import ru.loylabs.portal.service.notification.email.EmailNotification;
import ru.loylabs.portal.service.security.Credential;
import ru.loylabs.portal.unit.BaseTest;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.anyInt;
import static org.mockito.Mockito.doNothing;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.mockStatic;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEWER_ALREADY_VOTED;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEWER_NOT_FOUNT;
import static ru.loylabs.portal.exception.BaseException.Exception.REVIEW_NOT_FOUNT;
import static ru.loylabs.portal.exception.BaseException.Exception.SKILL_NOT_FOUND;
import static ru.loylabs.portal.integration.util.data.PersonTestData.generatePerson;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateCreateReviewPersonSkill;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateCreateReviewPersonSkillRating;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateReviewPersonSkill;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateReviewPersonSkillRating;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateReviewersSkill;
import static ru.loylabs.portal.integration.util.data.ReviewTestData.generateUpdateReviewPersonSkill;
import static ru.loylabs.portal.integration.util.data.SkillTestData.generateSkill;

@DisplayName("Review person service")
@SpringBootTest(classes = {ReviewPersonServiceImpl.class, ListFilterUtils.class})
public class ReviewPersonServiceTest extends BaseTest {

	private static Reviewers reviewers;
	private static ReviewPersonSkill reviewPersonSkill;
	private static ReviewPersonSkillRating reviewPersonSkillRating;
	private static Page<Reviewers> pageReviewersSkill;
	private static Page<ReviewPersonSkill> pageReviewPersonSkill;
	private static SearchReversePolishNotation searchCriteria;
	private static CreateReviewPersonSkill createReviewPersonSkill;
	private static CreateReviewPersonSkillRating createReviewPersonSkillRating;
	private static Person owner;
	private static Person reviewer;
	private static Skill skill;


	private static Person updateOwner;
	private static Person updateReviewer;
	private static Person resultUpdateReviewer;
	private static ReviewPersonSkill updateReviewPersonSkill;
	private static ReviewPersonSkill resultUpdateReviewPersonSkill;
	private static Skill updateSkill;
	private static Skill resultUpdateSkill;
	private static Reviewers updateReviewers;
	private static Reviewers resultUpdateReviewers;
	private static UpdateReviewPersonSkill updateSkillRequest;


	private static MockedStatic<SecurityContextHolder> mockedStatic;

	@MockBean
	private ReviewerSkillMapper reviewerSkillMapper;
	@MockBean
	private ReviewPersonSkillMapper reviewPersonSkillMapper;
	@MockBean
	private ReviewPersonSkillRatingMapper reviewPersonSkillRatingMapper;
	@MockBean
	private ReviewersRepository reviewersRepository;
	@MockBean
	private ReviewPersonSkillRatingRepository reviewPersonSkillRatingRepository;
	@MockBean
	private EmailNotificationFactory emailNotificationFactory;
	@MockBean
	private NotificationSender<EmailNotification> notificationSenderMock;
	@MockBean
	private ReviewPersonSkillRepository reviewPersonSkillRepository;
	@MockBean
	private PersonRepository personRepository;
	@MockBean
	private SkillRepository skillRepository;

	@Autowired
	private ReviewPersonService reviewPersonService;
	@Mock
	private SecurityContext securityContext;

	@BeforeAll
	static void setup() {
		var idReview = UUID.randomUUID();
		var idReviewer = UUID.randomUUID();
		var idSkill = UUID.randomUUID();
		var idOwner = UUID.randomUUID();
		var estimation = 9;

		owner = generatePerson(idOwner);
		reviewer = generatePerson(idReviewer);
		skill = generateSkill(idSkill);


		reviewPersonSkill = generateReviewPersonSkill(idReview, skill, owner);
		reviewers = generateReviewersSkill(reviewPersonSkill, reviewer);
		reviewPersonSkill.setReviewers(Arrays.asList(reviewers));
		reviewPersonSkillRating = generateReviewPersonSkillRating(reviewPersonSkill, estimation);
		createReviewPersonSkill = generateCreateReviewPersonSkill(reviewPersonSkill);
		createReviewPersonSkillRating = generateCreateReviewPersonSkillRating(estimation);
		searchCriteria = WebUtils.getCriteria("");
		pageReviewersSkill = new PageImpl<>(List.of(reviewers));
		pageReviewPersonSkill = new PageImpl<>(List.of(reviewPersonSkill));
		mockedStatic = mockStatic(SecurityContextHolder.class);


		var idUpdateOwner = UUID.randomUUID();
		var idUpdateReviewer = UUID.randomUUID();
		var idResultUpdateReviewer = UUID.randomUUID();
		var idUpdateSkill = UUID.randomUUID();
		var idResultUpdateSkill = UUID.randomUUID();
		var idUpdateReviewPersonSkill = UUID.randomUUID();


		updateOwner = generatePerson(idUpdateOwner);
		updateReviewer = generatePerson(idUpdateReviewer);
		resultUpdateReviewer = generatePerson(idResultUpdateReviewer);

		updateSkill = generateSkill(idUpdateSkill);
		resultUpdateSkill = generateSkill(idResultUpdateSkill);

		updateReviewPersonSkill = generateReviewPersonSkill(idUpdateReviewPersonSkill, updateSkill, updateOwner);
		resultUpdateReviewPersonSkill = generateReviewPersonSkill(idUpdateReviewPersonSkill, resultUpdateSkill, updateOwner);

		updateReviewers = generateReviewersSkill(updateReviewPersonSkill, updateReviewer);
		resultUpdateReviewers = generateReviewersSkill(resultUpdateReviewPersonSkill, updateReviewer);

		updateReviewPersonSkill.setReviewers(Arrays.asList(updateReviewers));
		resultUpdateReviewPersonSkill.setReviewers(Arrays.asList(resultUpdateReviewers));
		updateSkillRequest = generateUpdateReviewPersonSkill(new HashSet<>(Arrays.asList(idResultUpdateReviewer)));
	}

	@Test
	@DisplayName("Get all review person skill by owner")
	public void getAllReviewPersonSkillByOwnerTest() {
		when(reviewPersonSkillRepository.findAllByOwnerId(any(), any())).thenReturn(pageReviewPersonSkill);
		when(personRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill.getOwner()));
		var result = reviewPersonService.getAllReviewPersonSkillByOwner(0, 10, "date,asc", searchCriteria, reviewPersonSkill.getOwner().getId());

		assertEquals(reviewPersonSkill.getId(), result.getContent().get(0).getId());
		assertEquals(owner, result.getContent().get(0).getOwner());
		assertEquals(skill, result.getContent().get(0).getSkill());
		assertEquals(reviewers, result.getContent().get(0).getReviewers().get(0));
	}

	@Test
	@DisplayName("Get all review person skill by owner. Owner not found")
	public void getAllReviewPersonSkillByOwnerNotFoundTest() {
		var exception = assertThrows(ApplicationException.class, () -> {
			reviewPersonService.getAllReviewPersonSkillByOwner(0, 10, "date,asc", searchCriteria, reviewPersonSkill.getOwner().getId());
		});

		assertEquals(exception.getErrorCode(), PERSON_NOT_FOUND.getCode());
	}

	@Test
	@DisplayName("Get all review person skill by reviewer")
	public void getAllReviewPersonSkillByReviewerTest() {
		when(reviewPersonSkillRepository.findAllByReviewersReviewerId(any(), any())).thenReturn(pageReviewPersonSkill);
		when(personRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill.getReviewers().get(0).getReviewer()));
		var result = reviewPersonService.getAllReviewPersonSkillByIdReviewer(0, 10, "date,asc", searchCriteria, reviewPersonSkill.getReviewers().get(0).getReviewer().getId());

		assertEquals(reviewPersonSkill.getId(), result.getContent().get(0).getId());
		assertEquals(owner, result.getContent().get(0).getOwner());
		assertEquals(skill, result.getContent().get(0).getSkill());
		assertEquals(reviewers, result.getContent().get(0).getReviewers().get(0));
	}

	@Test
	@DisplayName("Get all review person skill by reviewer. Reviewer not found")
	public void getAllReviewPersonSkillByReviewerNotFoundTest() {
		var exception = assertThrows(ApplicationException.class, () -> {
			reviewPersonService.getAllReviewPersonSkillByIdReviewer(0, 10, "date,asc", searchCriteria, reviewPersonSkill.getReviewers().get(0).getReviewer().getId());
		});

		assertEquals(exception.getErrorCode(), PERSON_NOT_FOUND.getCode());
	}

	@Test
	@DisplayName("Get all review person by id")
	public void getReviewPersonByIdTest() {
		when(reviewPersonSkillRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill));
		var result = reviewPersonService.getReviewPersonSkillById(reviewPersonSkill.getId());

		assertEquals(reviewPersonSkill.getId(), result.getId());
		assertEquals(owner, result.getOwner());
		assertEquals(skill, result.getSkill());
		assertEquals(reviewers, result.getReviewers().get(0));
	}

	@Test
	@DisplayName("Get all review person by id. Review not found")
	public void getReviewPersonByIdNotFoundTest() {
		var exception = assertThrows(ApplicationException.class, () -> {
			reviewPersonService.getReviewPersonSkillById(reviewPersonSkill.getId());
		});

		assertEquals(exception.getErrorCode(), REVIEW_NOT_FOUNT.getCode());
	}

	@Test
	@DisplayName("Create review person")
	public void createReviewPersonTest() {
		when(personRepository.findById(reviewPersonSkill.getOwner().getId())).thenReturn(Optional.ofNullable(reviewPersonSkill.getOwner()));
		when(skillRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill.getSkill()));
		when(personRepository.findById(reviewPersonSkill.getReviewers().get(0).getReviewer().getId())).thenReturn(Optional.ofNullable(reviewPersonSkill.getReviewers().get(0).getReviewer()));
		when(reviewPersonSkillRepository.save(any())).thenReturn(reviewPersonSkill);
		//when(reviewPersonSkillMapper.toEntity(any(Person.class), any(Skill.class))).thenReturn(reviewPersonSkill);
		when(reviewerSkillMapper.toEntity(any(Person.class), any(ReviewPersonSkill.class))).thenReturn(reviewers);

		var result = reviewPersonService.createReview(createReviewPersonSkill);

		assertEquals(reviewPersonSkill.getId(), result.getId());
		assertEquals(owner, result.getOwner());
		assertEquals(skill, result.getSkill());
		assertEquals(reviewers, result.getReviewers().get(0));
	}

	@Test
	@DisplayName("Create review person. Owner not found")
	public void createReviewPersonOwnerNotFoundTest() {
		var exception = assertThrows(ApplicationException.class, () -> {
			reviewPersonService.createReview(createReviewPersonSkill);
		});

		assertEquals(exception.getErrorCode(), PERSON_NOT_FOUND.getCode());
	}

	@Test
	@DisplayName("Create review person. Skill not found")
	public void createReviewPersonSkillNotFoundTest() {
		when(personRepository.findById(reviewPersonSkill.getOwner().getId())).thenReturn(Optional.ofNullable(reviewPersonSkill.getOwner()));
		var exception = assertThrows(ApplicationException.class, () -> {
			reviewPersonService.createReview(createReviewPersonSkill);
		});

		assertEquals(exception.getErrorCode(), SKILL_NOT_FOUND.getCode());
	}

	@Test
	@DisplayName("Create rating review person")
	public void createRatingReviewPersonTest() {
		mockedStatic.when(SecurityContextHolder::getContext).thenReturn(securityContext);
		Credential credential = new Credential(reviewer);
		Authentication authentication = mock(Authentication.class);
		when(authentication.getPrincipal()).thenReturn(credential);
		when(securityContext.getAuthentication()).thenReturn(authentication);
		when(reviewPersonSkillRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill));
		when(reviewersRepository.findByReviewerIdAndReviewId(any(), any())).thenReturn(Optional.ofNullable(reviewers));
		when(reviewPersonSkillRatingMapper.toEntity(any(ReviewPersonSkill.class), anyInt())).thenReturn(reviewPersonSkillRating);
		when(reviewPersonSkillRatingRepository.save(any(ReviewPersonSkillRating.class))).thenReturn(reviewPersonSkillRating);

		var result = reviewPersonService.createRating(reviewPersonSkill.getId(), createReviewPersonSkillRating);

		assertEquals(reviewPersonSkillRating.getId(), result.getId());
		assertEquals(reviewPersonSkill, result.getReview());
	}

	@Test
	@DisplayName("Create rating review person. Review not found")
	public void createRatingReviewPersonReviewNotFoundTest() {
		mockedStatic.when(SecurityContextHolder::getContext).thenReturn(securityContext);
		Credential credential = new Credential(reviewer);
		Authentication authentication = mock(Authentication.class);
		when(authentication.getPrincipal()).thenReturn(credential);
		when(securityContext.getAuthentication()).thenReturn(authentication);
		when(reviewPersonSkillRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill));

		var exception = assertThrows(ApplicationException.class, () -> {
			reviewPersonService.createRating(reviewPersonSkill.getId(), createReviewPersonSkillRating);
		});

		assertEquals(exception.getErrorCode(), REVIEWER_NOT_FOUNT.getCode());
	}

	@Test
	@DisplayName("Create rating review person. Reviewer is voted")
	public void createRatingReviewPersonReviewReviewerIsVotedTest() {
		var reviewerIsVoted = reviewers;
		reviewerIsVoted.setVoted(true);

		mockedStatic.when(SecurityContextHolder::getContext).thenReturn(securityContext);
		Credential credential = new Credential(reviewer);
		Authentication authentication = mock(Authentication.class);
		when(authentication.getPrincipal()).thenReturn(credential);
		when(securityContext.getAuthentication()).thenReturn(authentication);
		when(reviewPersonSkillRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill));
		when(reviewersRepository.findByReviewerIdAndReviewId(any(), any())).thenReturn(Optional.of(reviewerIsVoted));

		var exception = assertThrows(ApplicationException.class, () -> {
			reviewPersonService.createRating(reviewPersonSkill.getId(), createReviewPersonSkillRating);
		});

		assertEquals(exception.getErrorCode(), REVIEWER_ALREADY_VOTED.getCode());
	}

	@Test
	@DisplayName("Update review person")
	public void updateReviewPersonTest() {
		var newReviewer = new ArrayList<>(updateReviewPersonSkill.getReviewers());
		updateReviewPersonSkill.setReviewers(newReviewer);

		when(reviewPersonSkillRepository.findById(any())).thenReturn(Optional.ofNullable(updateReviewPersonSkill));
		when(skillRepository.findById(any())).thenReturn(Optional.ofNullable(updateSkill));
		when(personRepository.findById(any())).thenReturn(Optional.ofNullable(resultUpdateReviewers.getReviewer()));
		when(reviewerSkillMapper.toEntity(any(Person.class), any(ReviewPersonSkill.class))).thenReturn(resultUpdateReviewers);
		when(reviewPersonSkillRepository.save(any(ReviewPersonSkill.class))).thenReturn(resultUpdateReviewPersonSkill);

		var result = reviewPersonService.updateReviewPersonSkill(updateReviewPersonSkill.getId(), updateSkillRequest);

		assertEquals(updateReviewPersonSkill.getId(), result.getId());
		assertEquals(updateOwner, result.getOwner());
		assertEquals(resultUpdateSkill, result.getSkill());
		assertEquals(resultUpdateReviewPersonSkill.getReviewers().get(0), result.getReviewers().get(0));
	}

	@Test@DisplayName("Remove review")
	public void removeReview() {
		when(reviewPersonSkillRepository.findById(any())).thenReturn(Optional.ofNullable(reviewPersonSkill));
		doNothing().when(reviewPersonSkillRatingRepository).removeAllByReviewId(any());
		doNothing().when(reviewersRepository).removeAllByReviewId(any());
		doNothing().when(reviewPersonSkillRepository).delete(any(ReviewPersonSkill.class));

		reviewPersonService.removeReview(reviewPersonSkill.getId());

		verify(reviewPersonSkillRatingRepository).removeAllByReviewId(reviewPersonSkill.getId());
		verify(reviewersRepository).removeAllByReviewId(reviewPersonSkill.getId());
		verify(reviewPersonSkillRepository).delete(reviewPersonSkill);
	}

}