package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import ru.loylabs.portal.entity.Skill;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.SkillMapperImpl;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.presentation.request.CreateSkillRequest;
import ru.loylabs.portal.presentation.request.UpdateSkillRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.SkillRepository;
import ru.loylabs.portal.service.SkillServiceImpl;
import ru.loylabs.portal.unit.BaseTest;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;
import static ru.loylabs.portal.exception.BaseException.Exception.SKILL_ALREADY_EXISTS;
import static ru.loylabs.portal.exception.BaseException.Exception.SKILL_NOT_FOUND;

@DisplayName("Skill service")
public class SkillServiceTest extends BaseTest {

    private static Skill skill;
    private static Page<Skill> pageSkills;
    private static SearchReversePolishNotation searchCriteria;
    private static CreateSkillRequest createSkillRequest;
    private static CreateSkillRequest createSkillRequestExistingSkillName;
    private static UpdateSkillRequest updateSkillRequest;
    private static UpdateSkillRequest updateSkillRequestExistingSkillName;

    @Spy
    private SkillMapperImpl skillMapper;
    @Mock
    private SkillRepository skillRepository;
    @InjectMocks
    private SkillServiceImpl skillService;

    @BeforeAll
    static void setup() {
        skill = new Skill();
        skill.setName("skill");
        pageSkills = new PageImpl<>(List.of(skill));
        searchCriteria = WebUtils.getCriteria("");
        createSkillRequest = new CreateSkillRequest();
        createSkillRequest.setName("newSill");
        updateSkillRequest = new UpdateSkillRequest();
        updateSkillRequest.setName("newSill");
        createSkillRequestExistingSkillName = new CreateSkillRequest();
        createSkillRequestExistingSkillName.setName(skill.getName());
        updateSkillRequestExistingSkillName = new UpdateSkillRequest();
        updateSkillRequestExistingSkillName.setName(skill.getName());
    }

    @Test
    @DisplayName("Get all skill")
    void getAllSkillTest() {
        doReturn(pageSkills).when(skillRepository).findAll(ArgumentMatchers.<Specification<Skill>>any(), any(Pageable.class));

        var result = skillService.getAllSkill(0, 10, "date,asc", searchCriteria);

        assertEquals(pageSkills.getTotalElements(), result.getTotalElements());
        assertEquals(pageSkills.getContent(), result.getContent());
    }

    @Test
    @DisplayName("Get skill by id")
    void getSkillByIdTest() {
        when(skillRepository.findById(skill.getId())).thenReturn(Optional.of(skill));

        var result = skillService.getSkillById(skill.getId());

        assertEquals(skill.getId(), result.getId());
        assertEquals(skill.getName(), result.getName());
    }

    @Test
    @DisplayName("Get skill by id. Skill not found")
    void getSkillByIdTestSkillNotFound() {
        var exception = assertThrows(ApplicationException.class,
                () -> skillService.getSkillById(UUID.randomUUID()));

        assertEquals(exception.getErrorCode(), SKILL_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Create skill")
    void createSkillTest() {
        when(skillRepository.findByNameIgnoreCase(createSkillRequest.getName())).thenReturn(Optional.empty());
        when(skillRepository.save(any())).thenReturn(skill);

        var result = skillService.createSkill(createSkillRequest);

        assertEquals(skill.getId(), result.getId());
        assertEquals(skill.getName(), result.getName());
    }

    @Test
    @DisplayName("Create skill. Skill already exists")
    void createSkillTestSkillAlreadyExists() {
        when(skillRepository.findByNameIgnoreCase(createSkillRequestExistingSkillName.getName())).thenReturn(Optional.of(skill));

        var exception = assertThrows(ApplicationException.class,
                () -> skillService.createSkill(createSkillRequestExistingSkillName));

        assertEquals(exception.getErrorCode(), SKILL_ALREADY_EXISTS.getCode());
    }

    @Test
    @DisplayName("Update skill")
    void updateSkillTest() {
        when(skillRepository.findByNameIgnoreCase(updateSkillRequest.getName())).thenReturn(Optional.empty());
        when(skillRepository.findById(skill.getId())).thenReturn(Optional.of(skill));
        when(skillRepository.save(skillMapper.toEntity(skill, updateSkillRequest))).thenReturn(skill);

        var result = skillService.updateSkill(skill.getId(), updateSkillRequest);

        assertEquals(skill.getId(), result.getId());
        assertEquals(updateSkillRequest.getName(), result.getName());
    }

    @Test
    @DisplayName("Update skill. Skill already exists")
    void updateSkillTestSkillAlreadyExists() {
        when(skillRepository.findByNameIgnoreCase(updateSkillRequestExistingSkillName.getName())).thenReturn(Optional.of(skill));

        var exception = assertThrows(ApplicationException.class,
                () -> skillService.updateSkill(skill.getId(), updateSkillRequestExistingSkillName));

        assertEquals(exception.getErrorCode(), SKILL_ALREADY_EXISTS.getCode());
    }

    @Test
    @DisplayName("Delete skill")
    void deleteSkillTest() {
        when(skillRepository.deleteSkillById(skill.getId())).thenReturn(1);

        skillService.deleteSkill(skill.getId());
    }

    @Test
    @DisplayName("Delete Skill. Skill not found")
    void deleteSkillTestSkillNotFound() {
        var exception = assertThrows(ApplicationException.class,
                () -> skillService.deleteSkill(UUID.randomUUID()));

        assertEquals(exception.getErrorCode(), SKILL_NOT_FOUND.getCode());
    }
}