package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.ArgumentMatchers;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import ru.loylabs.portal.entity.Status;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.StatusMapper;
import ru.loylabs.portal.mapper.StatusMapperImpl;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.presentation.request.UpdateStatusRequest;
import ru.loylabs.portal.presentation.search.SearchReversePolishNotation;
import ru.loylabs.portal.repository.StatusRepository;
import ru.loylabs.portal.service.StatusServiceImpl;
import ru.loylabs.portal.unit.BaseTest;

import java.util.List;
import java.util.Optional;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;
import static ru.loylabs.portal.integration.util.data.StatusTestData.*;

@DisplayName("StatusServiceImpl test")
@ExtendWith(MockitoExtension.class)
class StatusServiceImplTest extends BaseTest {

    @Mock
    private StatusRepository repository;
    @Spy
    private StatusMapper statusMapper = new StatusMapperImpl();
    @InjectMocks
    private StatusServiceImpl service;

    private Status status;
    private UpdateStatusRequest updateRequest;
    private Page<Status> statusPage;
    private SearchReversePolishNotation searchCriteria;

    @BeforeEach
    void setUp() {
        status = generateStatus();
        updateRequest = generateUpdateStatusRequest();
        statusPage = new PageImpl<>(List.of(status));
        searchCriteria = WebUtils.getCriteria("");
    }

    @Test
    @DisplayName("Test get status by id")
    void testGetStatusById() {
        doReturn(Optional.of(status)).when(repository).findById(status.getId());
        var actual = service.getStatus(status.getId());
        assertThat(actual).isEqualTo(status);
    }

    @Test
    @DisplayName("Test status not found by id")
    void testNotFoundStatusById() {
        doReturn(Optional.empty()).when(repository).findById(status.getId());
        assertThatExceptionOfType(ApplicationException.class)
                .isThrownBy(() -> service.getStatus(status.getId()));
    }

    @Test
    @DisplayName("Test get status by name")
    void testGetStatusByName() {
        doReturn(Optional.of(status)).when(repository).findByNameIgnoreCase(status.getName());
        var actual = service.getStatus(status.getName());
        assertThat(actual).isEqualTo(status);
    }

    @Test
    @DisplayName("Test status not found by name")
    void testNotFoundStatusByName() {
        doReturn(Optional.empty()).when(repository).findByNameIgnoreCase(status.getName());
        assertThatExceptionOfType(ApplicationException.class)
                .isThrownBy(() -> service.getStatus(status.getName()));
    }

    @Test
    @DisplayName("Test get all statuses")
    void getAllStatuses() {
        doReturn(statusPage).when(repository).findAll(ArgumentMatchers.<Specification<Status>>any(), any(Pageable.class));

        var result = service.getAllStatuses(0, 10, "date,asc", searchCriteria);

        assertThat(result.getTotalElements()).isEqualTo(statusPage.getTotalElements());
        assertThat(result.getContent()).hasSameElementsAs(statusPage.getContent());
    }

    @Test
    @DisplayName("Test creat status with unique name")
    void testCreateUniqueStatus() {
        doReturn(false).when(repository).existsByNameIgnoreCase(status.getName());
        service.createStatus(status);
        verify(repository).save(status);
    }

    @Test
    @DisplayName("Test create status with existing name")
    void testCreateExistingStatus() {
        doReturn(true).when(repository).existsByNameIgnoreCase(status.getName());
        assertThatExceptionOfType(ApplicationException.class)
                .isThrownBy(() -> service.createStatus(status));
    }

    @Test
    @DisplayName("Test delete status by id")
    void testDeleteStatus() {
        service.deleteStatus(status.getId());
        verify(repository).deleteById(status.getId());
    }

    @Test
    @DisplayName("Test update status")
    void testUpdateStatus() {
        doReturn(false).when(repository).existsByNameIgnoreCase(updateRequest.getName());
        doReturn(Optional.of(status)).when(repository).findById(status.getId());

        service.updateStatus(status.getId(), updateRequest);

        verify(statusMapper).updateEntity(updateRequest, status);
        verify(repository).save(status);
        assertThat(status.getName()).isEqualTo(updateRequest.getName());
    }

    @Test
    @DisplayName("Test update status name to existing")
    void testUpdateStatusWithExistingName() {
        doReturn(true).when(repository).existsByNameIgnoreCase(updateRequest.getName());

        assertThatExceptionOfType(ApplicationException.class)
                .isThrownBy(() -> service.updateStatus(status.getId(), updateRequest));
    }
}