package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.config.ServiceConfig;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.enums.StoryGroupType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.StoryGroupMapper;
import ru.loylabs.portal.mapper.StoryGroupMapperImpl;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.presentation.request.CreateStoryGroupRequest;
import ru.loylabs.portal.presentation.request.CreateStoryRequest;
import ru.loylabs.portal.presentation.request.UpdateStoryGroupRequest;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.service.StoryGroupServiceImpl;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.StoryAdapter;
import ru.loylabs.portal.utility.StaticMessageSource;

import java.util.Collections;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static java.util.UUID.randomUUID;
import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;
import static ru.loylabs.portal.exception.BaseException.Exception.STORY_GROUP_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.STORY_WRONG_FILE_AMOUNT;

@ExtendWith(MockitoExtension.class)
@DisplayName("Story group service test")
class StoryGroupServiceImplTest extends AbstractFilteringAndSortingService<StoryGroup> {
    private StoryGroup storyGroup;
    private Person person;

    @Spy
    private StaticMessageSource staticMessageSource = new StaticMessageSource(new ServiceConfig().messageSource());
    @Spy
    private StoryGroupMapper storyGroupMapper = new StoryGroupMapperImpl();
    @Spy
    private FilterUtils filterUtils = new ListFilterUtils();
    @Mock
    private StoryAdapter storyAdapter;
    @Mock
    private StoryGroupRepository storyGroupRepository;
    @InjectMocks
    private StoryGroupServiceImpl storyGroupService;

    @BeforeEach
    void setup() {
        storyGroup = new StoryGroup();
        storyGroup.setId(randomUUID());
        storyGroup.setGroupType(StoryGroupType.WELCOME);
        storyGroup.setPinned(false);

        person = new Person();
        person.setId(randomUUID());

        lenient().doAnswer(answer -> answer.getArgument(0).equals(storyGroup.getId()) ?
                        Optional.of(storyGroup) : Optional.empty())
                .when(storyGroupRepository).findById(any(UUID.class));
    }

    @Test
    @DisplayName("Getting story group by id")
    void getStoryGroupTest() {

        //Getting story group by id
        var result = storyGroupService.getStoryGroup(storyGroup.getId());
        assertNotNull(result);
        assertThat(result.getId()).isEqualTo(storyGroup.getId());

        //Getting story group by non-existent id
        var exception = assertThrows(ApplicationException.class,
                () -> storyGroupService.getStoryGroup(randomUUID()));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_GROUP_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Getting all story groups")
    void getAllStoryGroupsTest() {
        var criteria = WebUtils.getCriteria("search");
        var listFilter = filterUtils.getListFilter(0, 10, "id,desc", criteria, StoryGroup.class);
        Page<StoryGroup> pageOfGroups = new PageImpl<>(List.of(storyGroup));

        doReturn(pageOfGroups).when(storyGroupRepository).findAll(getSpecification(listFilter), getPageable(listFilter));

        //Getting all story groups
        var result = storyGroupService.getAllStoryGroups(0, 10, "id,desc", criteria);
        assertThat(result.getContent()).isEqualTo(pageOfGroups.getContent());
    }

    @Test
    @DisplayName("Save story group")
    void saveStoryGroupTest() {
        var requests = List.of(new CreateStoryRequest());
        var groupRequest = new CreateStoryGroupRequest(true, StoryGroupType.WELCOME, requests);
        var file = new MockMultipartFile(
                "test", "test.jpg", String.valueOf(MediaType.IMAGE_JPEG), "1".getBytes());

        doReturn(new Story()).when(storyAdapter).createStory(
                any(CreateStoryRequest.class), any(StoryGroup.class), any(Person.class), any());
        doAnswer(answer -> answer.getArgument(0)).when(storyGroupRepository).save(any(StoryGroup.class));

        //Validating saving a story group
        var result = storyGroupService.saveStoryGroup(groupRequest, person, requests, List.of(file));
        assertThat(result).isNotNull();
        assertThat(result.getPinned()).isTrue();

        //Saving a group of stories where the number of stories and attachments are not equal
        var exception = assertThrows(ApplicationException.class,
                () -> storyGroupService.saveStoryGroup(groupRequest, person, requests, Collections.emptyList()));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_WRONG_FILE_AMOUNT.getCode());
    }

    @Test
    @DisplayName("Updating a story group")
    void updateStoryGroupTest() {
        var updateRequest = new UpdateStoryGroupRequest();
        updateRequest.setPinned(true);

        doAnswer(obj -> obj.getArgument(0)).when(storyGroupRepository).save(any(StoryGroup.class));

        //Updating an existing story group
        var res = storyGroupService.updateStoryGroup(updateRequest, storyGroup.getId());
        assertThat(res).isNotNull();
        assertThat(res.getId()).isEqualTo(storyGroup.getId());
        assertThat(res.getPinned()).isEqualTo(updateRequest.getPinned());

        //Updating non-existent story group
       var e = assertThrows(ApplicationException.class,
                () -> storyGroupService.updateStoryGroup(updateRequest, randomUUID()));
       assertThat(e.getErrorCode()).isEqualTo(STORY_GROUP_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Delete story group by id")
    void deleteStoryGroupTest() {
        doAnswer(answer -> answer.getArgument(0).equals(storyGroup.getId()) ? 1 : 0)
                .when(storyGroupRepository).deleteStoryGroupById(any(UUID.class));

        //Deleting an existing group
        assertDoesNotThrow(() -> storyGroupService.deleteStoryGroup(storyGroup.getId()));

        //Deleting an non-existent group
        var exception = assertThrows(ApplicationException.class,
                () -> storyGroupService.deleteStoryGroup(randomUUID()));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_GROUP_NOT_FOUND.getCode());
    }
}