package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.config.ServiceConfig;
import ru.loylabs.portal.entity.Attachment;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.enums.AttachmentType;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.presentation.request.CreateStoryRequest;
import ru.loylabs.portal.presentation.request.UpdateStoryRequest;
import ru.loylabs.portal.presentation.search.SearchCriteria;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.repository.StoryRepository;
import ru.loylabs.portal.service.StoryServiceImpl;
import ru.loylabs.portal.service.filter.AbstractFilteringAndSortingService;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.FilterUtils;
import ru.loylabs.portal.service.interfaces.RecordAdapter;
import ru.loylabs.portal.service.interfaces.StoryAdapter;
import ru.loylabs.portal.utility.FileAttachmentTypeFactory;
import ru.loylabs.portal.utility.StaticMessageSource;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static java.util.UUID.randomUUID;
import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.*;
import static ru.loylabs.portal.exception.BaseException.Exception.*;

@ExtendWith(MockitoExtension.class)
@DisplayName("Story service test")
class StoryServiceImplTest extends AbstractFilteringAndSortingService<Story> {
    private static Story story;
    private static final Story secondStory = new Story();
    private static Attachment storyAttachment;
    private static StoryGroup storyGroup;

    @Spy
    private StaticMessageSource staticMessageSource = new StaticMessageSource(new ServiceConfig().messageSource());
    @Spy
    private FileAttachmentTypeFactory fileAttachmentTypeFactor = new FileAttachmentTypeFactory();
    @Spy
    private FilterUtils filterUtils = new ListFilterUtils();
    @Mock
    private StoryAdapter storyAdapter;
    @Mock
    private StoryRepository storyRepository;
    @Mock
    private StoryGroupRepository storyGroupRepository;
    @Mock
    private RecordAdapter recordAdapter;
    @InjectMocks
    private StoryServiceImpl storyService;

    @BeforeEach
    void setStoryAndStoryGroup() {
        storyGroup = new StoryGroup();
        storyGroup.setId(randomUUID());

        storyAttachment = new Attachment();
        storyAttachment.setId(randomUUID());
        storyAttachment.setName("storyAttachment");
        storyAttachment.setExtension(String.valueOf(MediaType.IMAGE_JPEG));

        story = new Story();
        story.getAttachments().add(storyAttachment);
        story.setId(randomUUID());
        story.setContent("test content story");
        story.setGroup(storyGroup);
        story.getAttachments().add(storyAttachment);

        secondStory.setId(randomUUID());
        secondStory.setContent("test content secondStory");
        secondStory.setGroup(storyGroup);

        storyGroup.getStories().add(story);
        storyGroup.getStories().add(secondStory);

        lenient().doAnswer(answer ->
                        answer.getArgument(0).equals(story.getId()) && answer.getArgument(1).equals(storyGroup.getId())
                                ? Optional.of(story) : Optional.empty())
                .when(storyRepository).findByIdAndGroupId(any(UUID.class), any(UUID.class));
        lenient().doAnswer(answer ->
                        answer.getArgument(0).equals(storyGroup.getId()) ? Optional.of(storyGroup) : Optional.empty())
                .when(storyGroupRepository).findById(any(UUID.class));
    }

    @Test
    @DisplayName("Getting story by id")
    void getStoryTest() {

        //Getting story by id
        var actualRes = storyService.getStory(storyGroup.getId(), story.getId());
        assertThat(actualRes).isNotNull();
        assertThat(actualRes.getId()).isEqualTo(story.getId());
        assertThat(actualRes.getGroup().getId()).isEqualTo(storyGroup.getId());

        //Getting non-existent story
        var exception = assertThrows(ApplicationException.class,
                () -> storyService.getStory(storyGroup.getId(), randomUUID()));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Getting all stories from group")
    void getAllStoriesTest() {
        var criteria = WebUtils.getCriteria("search");
        criteria.add(SearchCriteria.builder().key("group.id").operation(":").value(storyGroup.getId()).build());
        var listFilter = filterUtils.getListFilter(0, 10, "date,asc", criteria, Story.class);
        Page<Story> pageOfStories = new PageImpl<>(List.of(story, secondStory));

        doReturn(pageOfStories).when(storyRepository).findAll(any(Specification.class), eq(getPageable(listFilter)));

        //Getting all stories from group
        var result = storyService.getAllStoriesFromGroup(0, 10, "date,asc", criteria, storyGroup.getId());
        assertThat(result).isNotNull();
        assertThat(result.getContent()).isEqualTo(pageOfStories.getContent());
    }

    @Test
    @DisplayName("Updating stories")
    void updateStoryTest() {
        var updateRequest = new UpdateStoryRequest("update header", "update content");
        var updateStory = new Story();
        updateStory.setId(story.getId());
        updateStory.setGroup(storyGroup);
        updateStory.setHeader(updateRequest.getHeader());
        updateStory.setContent(updateRequest.getContent());

        doReturn(updateStory).when(storyRepository).save(updateStory);
        doReturn(updateStory).when(storyAdapter).updateStory(updateRequest, story);

        //Updating an existing story
        var actualRes = storyService.updateStory(updateRequest, storyGroup.getId(), story.getId());
        assertThat(actualRes).isNotNull();
        assertThat(actualRes.getContent()).isEqualTo(updateStory.getContent());

        //Updating a non-existent story
        var exception = assertThrows(ApplicationException.class,
                () -> storyService.updateStory(updateRequest, storyGroup.getId(), randomUUID()));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Deleting story by id and group id")
    void deleteStoryTest() {
        doAnswer(answer -> answer.getArgument(0)).when(storyGroupRepository).save(any(StoryGroup.class));
        doReturn(Optional.of(secondStory)).when(storyRepository).findByIdAndGroupId(secondStory.getId(),storyGroup.getId());

        //Deleting story by id and group id
        var actualRes = storyService.deleteStory(storyGroup.getId(), story.getId());
        assertThat(actualRes.getStories()).doesNotContain(story);
        assertThat(actualRes.getStories()).hasSize(1);

        //Deleting the only story in a group
        var exception = assertThrows(ApplicationException.class,
                () -> storyService.deleteStory(storyGroup.getId(), secondStory.getId()));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_GROUP_EMPTY.getCode());

        //Deleting story in a non-existent group
        exception = assertThrows(ApplicationException.class,
                () -> storyService.deleteStory(randomUUID(), story.getId()));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_GROUP_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Replacing story attachment")
    void replaceStoryAttachmentTest() {
        MultipartFile file = fileJPG();
        MultipartFile wrongFile = fileHtml();
        var attachmentFromFile = new Attachment();
        attachmentFromFile.setName(file.getName());
        attachmentFromFile.setAttachmentType(AttachmentType.IMAGE);

        doAnswer(answer -> {
            var result = (Story) answer.getArgument(0);
            result.getAttachments().clear();
            result.getAttachments().add(attachmentFromFile);
            return result;
        }).when(recordAdapter).replaceAttachments(story, file);

        //Replacing story attachment
        var result = storyService.replaceStoryAttachment(storyGroup.getId(), story.getId(), file);
        assertThat(result).isNotNull();
        assertThat(result.getAttachments()).doesNotContain(storyAttachment);

        //Replacing story attachment by wrong file
        var exception = assertThrows(ApplicationException.class,
                () -> storyService.replaceStoryAttachment(storyGroup.getId(), story.getId(), wrongFile),
                STORY_WRONG_FILE_TYPE.getLabel());
        assertThat(exception.getErrorCode()).isEqualTo(STORY_WRONG_FILE_TYPE.getCode());
    }

//    @Test
//    @DisplayName("Saving new story")
//    void saveStoryTest() {
//        var request = new CreateStoryRequest("story request header", "story request content", 6000);
//        var newStory = Story.builder()
//                .header(request.getHeader())
//                .expireTime(LocalDateTime.now())
//                .group(storyGroup)
//                .priority(0)
//                .build();
//        var file = fileJPG();
//
//        doAnswer(answer -> answer.getArgument(0)).when(storyRepository).save(any(Story.class));
//        doReturn(newStory).when(storyAdapter).createStory(
//                any(CreateStoryRequest.class), any(StoryGroup.class), any(Person.class), any(MultipartFile.class));
//
//        //Saving story
//        var result = storyService.saveStory(request, storyGroup.getId(), new Person(), file);
//        assertThat(result).isNotNull();
//        assertThat(result.getHeader()).isEqualTo(request.getHeader());
//
//        //Saving story in non-existent group
//        var exception = assertThrows(ApplicationException.class,
//                () -> storyService.saveStory(request, randomUUID(), new Person(), file));
//        assertThat(exception.getErrorCode()).isEqualTo(STORY_GROUP_NOT_FOUND.getCode());
//    }

    private MockMultipartFile fileHtml() {
        return new MockMultipartFile("file", "file.html", MediaType.TEXT_HTML_VALUE, "1".getBytes());
    }

    private MockMultipartFile fileJPG() {
        return new MockMultipartFile("file", "file.jpg", MediaType.IMAGE_JPEG_VALUE, "1".getBytes());
    }
}