package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.repository.StoryRepository;
import ru.loylabs.portal.service.StoryServiceImpl;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.RecordAdapter;
import ru.loylabs.portal.service.interfaces.StoryAdapter;
import ru.loylabs.portal.service.interfaces.StoryService;
import ru.loylabs.portal.unit.BaseTest;

import java.io.IOException;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.when;
import static ru.loylabs.portal.exception.BaseException.Exception.*;
import static ru.loylabs.portal.integration.util.data.MultipartFileGeneratorUtils.generateFile;
import static ru.loylabs.portal.integration.util.data.PersonTestData.generatePerson;
import static ru.loylabs.portal.integration.util.data.StoryTestData.*;

@DisplayName("Story service test")
@SpringBootTest(classes = {StoryServiceImpl.class,
        ListFilterUtils.class})
public class StoryServiceTest extends BaseTest {

    @MockBean
    private StoryAdapter storyAdapter;
    @MockBean
    private StoryRepository storyRepository;
    @MockBean
    private StoryGroupRepository storyGroupRepository;
    @MockBean
    private RecordAdapter recordAdapter;

    @Autowired
    private StoryService storyService;

    @Test
    @DisplayName("Get story")
    void getStoryTest() {
        var story = generateStory();
        when(storyRepository.findByIdAndGroupId(story.getId(), story.getGroup().getId())).thenReturn(Optional.of(story));

        var result = storyService.getStory(story.getGroup().getId(), story.getId());

        assertEquals(story.getId(), result.getId());
    }

    @Test
    @DisplayName("Get story. Story not found")
    void getStoryTestStoryNotFound() {
        var exception = assertThrows(ApplicationException.class, () -> {
            storyService.getStory(UUID.randomUUID(), UUID.randomUUID());
        });

        assertEquals(exception.getErrorCode(), STORY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Save story")
    void saveStoryTest() {
        var createStoryRequest = generateCreateStoryRequest();
        var person = generatePerson();
        var file = fileJPG();
        var storyGroup = generateEmptyStoryGroup();
        var story = generateStory();
        when(storyGroupRepository.findById(storyGroup.getId())).thenReturn(Optional.of(storyGroup));
        when(storyAdapter.createStory(any(), any(), any(), any())).thenReturn(story);
        when(storyRepository.save(story)).thenReturn(story);

        var result = storyService.saveStory(createStoryRequest, storyGroup.getId(), person, file);

        assertEquals(story.getId(), result.getId());
    }

    @Test
    @DisplayName("Save story. Story group not found")
    void saveStoryTestStoryGroupNotFound() {
        var createStoryRequest = generateCreateStoryRequest();
        var person = generatePerson();
        var file = fileJPG();
        var storyGroup = generateEmptyStoryGroup();

        var exception = assertThrows(ApplicationException.class, () -> {
            storyService.saveStory(createStoryRequest, storyGroup.getId(), person, file);
        });

        assertEquals(exception.getErrorCode(), STORY_GROUP_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Update story")
    void updateStoryTest() {
        var story = generateStory();
        var updateStoryRequest = generateUpdateStoryRequest();
        String newContentForStory = "new content";
        var storyAfterUpdate = story;
        storyAfterUpdate.setContent(newContentForStory);
        updateStoryRequest.setContent(newContentForStory);
        when(storyRepository.findByIdAndGroupId(story.getId(), story.getGroup().getId())).thenReturn(Optional.of(story));
        when(storyAdapter.updateStory(updateStoryRequest, story)).thenReturn(story);
        when(storyRepository.save(story)).thenReturn(storyAfterUpdate);

        var result = storyService.updateStory(updateStoryRequest, story.getGroup().getId(), story.getId());

        assertEquals(story.getId(), result.getId());
        assertEquals(newContentForStory, result.getContent());
    }

    @Test
    @DisplayName("Update story. Story not found")
    void updateStoryTestStoryNotFound() {
        var story = generateStory();
        var updateStoryRequest = generateUpdateStoryRequest();
        var exception = assertThrows(ApplicationException.class, () -> {
            storyService.updateStory(updateStoryRequest, story.getGroup().getId(), story.getId());
        });

        assertEquals(exception.getErrorCode(), STORY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Delete story")
    void deleteStoryTest() {
        var firstStory = generateStory();
        var secondStory = generateStory();
        var group = firstStory.getGroup();
        group.getStories().add(firstStory);
        group.getStories().add(secondStory);
        when(storyGroupRepository.findById(firstStory.getGroup().getId())).thenReturn(Optional.of(firstStory.getGroup()));
        when(storyRepository.findByIdAndGroupId(firstStory.getId(), firstStory.getGroup().getId())).thenReturn(Optional.of(firstStory));
        when(storyGroupRepository.save(firstStory.getGroup())).thenReturn(firstStory.getGroup());

        var result = storyService.deleteStory(firstStory.getGroup().getId(), firstStory.getId());

        assertEquals(group.getId(), result.getId());
        assertFalse(result.getStories().contains(firstStory));
    }

    @Test
    @DisplayName("Delete story. Story group not found")
    void deleteStoryTestStoryGroupNotFound() {
        var exception = assertThrows(ApplicationException.class, () -> {
            storyService.deleteStory(UUID.randomUUID(), UUID.randomUUID());
        });

        assertEquals(exception.getErrorCode(), STORY_GROUP_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Delete story. Story not found")
    void deleteStoryTestStoryNotFound() {
        var storyGroup = generateEmptyStoryGroup();
        when(storyGroupRepository.findById(storyGroup.getId())).thenReturn(Optional.of(storyGroup));

        var exception = assertThrows(ApplicationException.class, () -> {
            storyService.deleteStory(storyGroup.getId(), UUID.randomUUID());
        });

        assertEquals(exception.getErrorCode(), STORY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Delete story. Story group not be empty")
    void deleteStoryTestStoryGroupNotBeEmpty() {
        var story = generateStory();
        var group = story.getGroup();
        group.getStories().add(story);
        when(storyGroupRepository.findById(story.getGroup().getId())).thenReturn(Optional.of(story.getGroup()));
        when(storyRepository.findByIdAndGroupId(story.getId(), story.getGroup().getId())).thenReturn(Optional.of(story));
        when(storyGroupRepository.save(story.getGroup())).thenReturn(story.getGroup());

        var exception = assertThrows(ApplicationException.class, () -> {
            storyService.deleteStory(story.getGroup().getId(), story.getId());
        });

        assertEquals(exception.getErrorCode(), STORY_GROUP_EMPTY.getCode());
    }

    @Test
    @DisplayName("Replace story attachment")
    void replaceStoryAttachmentTest() {
        var file = fileJPG();
        var story = generateStory();
        when(storyRepository.findByIdAndGroupId(story.getId(), story.getGroup().getId())).thenReturn(Optional.of(story));
        when(recordAdapter.replaceAttachments(story, file)).thenReturn(story);

        var result = storyService.replaceStoryAttachment(story.getGroup().getId(), story.getId(), file);

        assertEquals(story.getId(), result.getId());
    }

    @Test
    @DisplayName("Replace story attachment. Wrong file type")
    void replaceStoryAttachmentTestWrongFileType() throws IOException {
        var file = generateFile(MediaType.APPLICATION_JSON);

        var exception = assertThrows(ApplicationException.class, () -> {
            storyService.replaceStoryAttachment(UUID.randomUUID(), UUID.randomUUID(), file);
        });

        assertEquals(exception.getErrorCode(), STORY_WRONG_FILE_TYPE.getCode());
    }

    private MockMultipartFile fileJPG() {
        return new MockMultipartFile("file", "file.jpg", MediaType.IMAGE_JPEG_VALUE, "1".getBytes());
    }
}






























