package ru.loylabs.portal.unit.service;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.jpa.domain.Specification;
import ru.loylabs.portal.entity.Title;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.TitleMapper;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.TitleRepository;
import ru.loylabs.portal.service.TitleServiceImpl;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.JobAdapter;
import ru.loylabs.portal.service.interfaces.TitleService;
import ru.loylabs.portal.unit.BaseTest;

import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.ArgumentMatchers.eq;
import static org.mockito.Mockito.*;
import static ru.loylabs.portal.exception.BaseException.Exception.TITLE_BAN_ON_DELETION;
import static ru.loylabs.portal.exception.BaseException.Exception.TITLE_NOT_FOUND;
import static ru.loylabs.portal.integration.util.data.JobTestData.*;

@DisplayName("Title service")
@SpringBootTest(classes = {TitleServiceImpl.class,
        ListFilterUtils.class})
public class TitleServiceTest extends BaseTest {
    @MockBean
    private TitleRepository titleRepository;
    @MockBean
    private PersonRepository personRepository;
    @MockBean
    private TitleMapper titleMapper;
    @MockBean
    private JobAdapter jobAdapter;

    @Autowired
    private TitleService titleService;

    @Test
    @DisplayName("Get all title")
    void getAllTitleTest() {
        var pageTitles = new PageImpl<>(List.of(generateTitle()));
        var criteria = WebUtils.getCriteria("");
        doReturn(pageTitles).when(titleRepository).findAll((Specification<Title>) eq(null), any());


        var result = titleService.getAllTitle(0, 10, "date,asc", criteria);

        assertEquals(pageTitles.getTotalElements(), result.getTotalElements());
        assertEquals(pageTitles.getContent(), result.getContent());
    }

    @Test
    @DisplayName("Get title by id")
    void getTitleByIdTest() {
        var title = generateTitle();
        when(titleRepository.findById(title.getId())).thenReturn(Optional.of(title));

        var result = titleService.getTitleById(title.getId());

        assertEquals(title.getId(), result.getId());
    }

    @Test
    @DisplayName("Get title by id. Title not found")
    void getTitleByIdTestTitleNotFound() {
        var exception = assertThrows(ApplicationException.class, () -> {
            titleService.getTitleById(UUID.randomUUID());
        });

        assertEquals(exception.getErrorCode(), TITLE_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Create title")
    void createTitleTest() {
        var createTitleRequest = generateCreateTitleRequest();
        var title = generateTitle();
        when(titleRepository.save(any())).thenReturn(title);

        var result = titleService.createTitle(createTitleRequest);

        assertEquals(title.getId(), result.getId());
    }

    @Test
    @DisplayName("Update title")
    void updateTitleTest() {
        var updateTitleRequest = generateUpdateTitleRequest();
        var title = generateTitle();
        when(titleRepository.findById(title.getId())).thenReturn(Optional.of(title));
        when(titleRepository.save(any())).thenReturn(title);

        var result = titleService.updateTitle(title.getId(), updateTitleRequest);

        assertEquals(title.getId(), result.getId());
    }

    @Test
    @DisplayName("Delete title")
    void deleteTitleTest() {
        var title = generateTitle();
        var defaultTitle = generateTitle();
        when(titleRepository.getDefaultTitle()).thenReturn(defaultTitle);
        when(titleRepository.findById(title.getId())).thenReturn(Optional.of(title));

        titleService.deleteTitle(title.getId());

        verify(titleRepository).delete(title);
    }

    @Test
    @DisplayName("Delete title. Cannot delete default title")
    void deleteTitleTestCannotDeleteDefaultTitle() {
        var defaultTitle = generateTitle();
        when(titleRepository.getDefaultTitle()).thenReturn(defaultTitle);

        var exception = assertThrows(ApplicationException.class, () -> {
            titleService.deleteTitle(defaultTitle.getId());
        });

        assertEquals(exception.getErrorCode(), TITLE_BAN_ON_DELETION.getCode());
    }

    @Test
    @DisplayName("Delete title. Title not found")
    void deleteTitleTestTitleNotFound() {
        var defaultTitle = generateTitle();
        when(titleRepository.getDefaultTitle()).thenReturn(defaultTitle);

        var exception = assertThrows(ApplicationException.class, () -> {
            titleService.deleteTitle(UUID.randomUUID());
        });

        assertEquals(exception.getErrorCode(), TITLE_NOT_FOUND.getCode());
    }
}























































