package ru.loylabs.portal.unit.service.adapter;

import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.web.multipart.MultipartFile;
import ru.loylabs.portal.config.ServiceConfig;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.mapper.StoryMapper;
import ru.loylabs.portal.mapper.StoryMapperImpl;
import ru.loylabs.portal.presentation.request.CreateStoryRequest;
import ru.loylabs.portal.presentation.request.UpdateStoryRequest;
import ru.loylabs.portal.service.adapter.StoryAdapterImpl;
import ru.loylabs.portal.service.interfaces.RecordAdapter;
import ru.loylabs.portal.utility.StaticMessageSource;

import java.time.LocalDateTime;

import static org.assertj.core.api.Assertions.assertThat;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static ru.loylabs.portal.exception.BaseException.Exception.STORY_WRONG_FILE_TYPE;

@ExtendWith(MockitoExtension.class)
@DisplayName("Story adapter test")
class StoryAdapterImplTest {
    private static Person person;
    private static Story story;
    private static StoryGroup storyGroup;
    private static MultipartFile file;

    @Spy
    private static StaticMessageSource staticMessageSource = new StaticMessageSource(new ServiceConfig().messageSource());
    @Spy
    private StoryMapper storyMapper = new StoryMapperImpl();
    @Mock
    private RecordAdapter recordAdapter;
    @InjectMocks
    private StoryAdapterImpl storyAdapter;

    @BeforeAll
    static void setup() {
        file = new MockMultipartFile(
                "file", "file.jpg", String.valueOf(MediaType.IMAGE_JPEG), "test".getBytes());
        storyGroup = new StoryGroup();
        person = new Person();
        story = Story.builder()
                .header("story header")
                .expireTime(LocalDateTime.MIN)
                .group(storyGroup)
                .priority(0)
                .build();
    }

    @Test
    @DisplayName("Creating story")
    void createStoryTest() {
        var createRequest = new CreateStoryRequest("create header", "create content", 0);
        var wrongFile = new MockMultipartFile(
                "file", "file.jpg", String.valueOf(MediaType.TEXT_HTML), "test".getBytes());

        //Checking the created story
        var actualRes = storyAdapter.createStory(createRequest, storyGroup, person, file);
        assertThat(actualRes).isNotNull();
        assertThat(actualRes.getHeader()).isEqualTo(createRequest.getHeader());

        //Creating story with wrong file
        var exception = assertThrows(ApplicationException.class,
                () -> storyAdapter.createStory(createRequest, storyGroup, person, wrongFile));
        assertThat(exception.getErrorCode()).isEqualTo(STORY_WRONG_FILE_TYPE.getCode());
    }

    @Test
    @DisplayName("Updating story")
    void updateStoryTest() {
        var updateRequest = new UpdateStoryRequest("update header", "update content");

        //Checking the updated story
        var actualResult = storyAdapter.updateStory(updateRequest, story);
        assertThat(actualResult).isNotNull();
        assertThat(actualResult.getContent()).isEqualTo(updateRequest.getContent());
    }
}