package ru.loylabs.portal.unit.service.community;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;
import org.mockito.Mockito;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.MediaType;
import ru.loylabs.portal.entity.Community;
import ru.loylabs.portal.entity.CommunityPersonRole;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.enums.CommunityRole;
import ru.loylabs.portal.exception.ApplicationException;
import ru.loylabs.portal.integration.util.data.MultipartFileGeneratorUtils;
import ru.loylabs.portal.mapper.CommunityMapper;
import ru.loylabs.portal.mapper.CommunityPersonRoleMapper;
import ru.loylabs.portal.presentation.WebUtils;
import ru.loylabs.portal.presentation.request.AddMembersRequest;
import ru.loylabs.portal.presentation.request.CreateCommunityRequest;
import ru.loylabs.portal.presentation.request.UpdateCommunityRequest;
import ru.loylabs.portal.presentation.request.UpdateRoleInCommunityRequest;
import ru.loylabs.portal.repository.CommunityPersonRoleRepository;
import ru.loylabs.portal.repository.CommunityRepository;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.service.CommunityServiceImpl;
import ru.loylabs.portal.service.filter.ListFilterUtils;
import ru.loylabs.portal.service.interfaces.AttachmentAdapter;
import ru.loylabs.portal.service.interfaces.CommunityService;
import ru.loylabs.portal.unit.BaseTest;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Optional;
import java.util.Set;
import java.util.UUID;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertThrows;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.verify;
import static org.mockito.Mockito.when;
import static ru.loylabs.portal.enums.CommunityRole.OWNER;
import static ru.loylabs.portal.exception.BaseException.Exception.AVATAR_WRONG_FILE_TYPE;
import static ru.loylabs.portal.exception.BaseException.Exception.COMMUNITY_CHANGE_OF_OWNER;
import static ru.loylabs.portal.exception.BaseException.Exception.COMMUNITY_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_FOUND;
import static ru.loylabs.portal.exception.BaseException.Exception.PERSON_NOT_IN_COMMUNITY;
import static ru.loylabs.portal.exception.BaseException.Exception.TITLE_BAN_ON_DELETION;
import static ru.loylabs.portal.integration.util.data.AttachmentTestData.initPictureAttachment;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCommunity;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCommunityPersonRole;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateCreateCommunityRequest;
import static ru.loylabs.portal.integration.util.data.CommunityTestData.generateUpdateCommunityRequest;
import static ru.loylabs.portal.integration.util.data.MultipartFileGeneratorUtils.generateFile;
import static ru.loylabs.portal.integration.util.data.PersonTestData.generatePerson;

@DisplayName("Community service")
@SpringBootTest(classes = {CommunityServiceImpl.class,
        ListFilterUtils.class})
public class CommunityServiceImplTest extends BaseTest {
    @MockBean
    private CommunityRepository communityRepository;
    @MockBean
    private AttachmentAdapter attachmentAdapter;
    @MockBean
    private PersonRepository personRepository;
    @MockBean
    private CommunityPersonRoleRepository communityPersonRoleRepository;
    @MockBean
    private CommunityMapper communityMapper;
    @MockBean
    private CommunityPersonRoleMapper communityPersonRoleMapper;
    @MockBean
    private ApplicationEventPublisher publisher;

    @Autowired
    private CommunityService communityService;

    @Test
    @DisplayName("Get community by ID")
    public void getCommunityByIdTest() {
        var idCommunity = UUID.randomUUID();
        var community = generateCommunity(idCommunity);

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));

        var result = communityService.getCommunity(idCommunity);

        assertEquals(community.getName(), result.getName());
    }

    @Test
    @DisplayName("Get community by ID. Community not found")
    public void getCommunityNotFoundTest() {
        var idCommunity = UUID.randomUUID();

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.getCommunity(idCommunity);
        });

        assertEquals(exception.getErrorCode(), COMMUNITY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Get all community")
    public void getAllCommunityTest() {
        var idCommunity1 = UUID.randomUUID();
        var idCommunity2 = UUID.randomUUID();

        var community1 = generateCommunity(idCommunity1);
        var community2 = generateCommunity(idCommunity2);
        var criteria = WebUtils.getCriteria("");
        var allCommunity = new PageImpl<>(List.of(community1, community2));

        when(communityRepository.findAll((Specification<Community>) any(), any())).thenReturn(allCommunity);

        var result = communityService.getAllCommunities(0, 10, "date,asc", criteria);

        assertEquals(allCommunity.getTotalElements(), result.getTotalElements());
        assertEquals(allCommunity.getContent(), result.getContent());
    }

    @Test
    @DisplayName("Get all public communities")
    public void getAllPublicCommunitiesTest() {
        var idCommunity1 = UUID.randomUUID();
        var idCommunity2 = UUID.randomUUID();

        var community1 = generateCommunity(idCommunity1);
        var community2 = generateCommunity(idCommunity2);
        var criteria = WebUtils.getCriteria("");
        var allCommunity = new PageImpl<>(List.of(community1, community2));

        when(communityRepository.findAll((Specification<Community>) any(), any())).thenReturn(allCommunity);

        var result = communityService.getAllPublicCommunities(0, 10, "date,asc", criteria);

        assertEquals(allCommunity.getTotalElements(), result.getTotalElements());
        assertEquals(allCommunity.getContent(), result.getContent());
    }

    @Test
    @DisplayName("Save community")
    public void saveCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var request = generateCreateCommunityRequest();
        var person = generatePerson();

        when(communityRepository.save(any())).thenReturn(community);

        var result = communityService.saveCommunity(request, person);

        assertEquals(community.getName(), result.getName());
        assertEquals(community.getId(), result.getId());
    }

    @Test
    @DisplayName("Save community with an avatar")
    public void saveCommunityWithAvatarTest() throws IOException {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var request = generateCreateCommunityRequest();
        var person = generatePerson();
        var file = generateFile("file", MediaType.IMAGE_JPEG);
        var attachment = initPictureAttachment();
        community.setAvatar(attachment);

        when(communityRepository.save(any())).thenReturn(community);
        when(attachmentAdapter.createAttachment(any())).thenReturn(attachment);
        when(communityMapper.toEntity(any(CreateCommunityRequest.class), any())).thenReturn(community);

        var result = communityService.saveCommunity(request, file, person);

        assertEquals(community.getName(), result.getName());
        assertEquals(community.getId(), result.getId());
    }

    @Test
    @DisplayName("Update community")
    public void updateCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var updateCommunityRequest = generateUpdateCommunityRequest();
        var community = generateCommunity(idCommunity);

        when(communityRepository.save(any())).thenReturn(community);
        when(communityMapper.toEntity(any(UpdateCommunityRequest.class), any())).thenReturn(community);
        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));

        var result = communityService.updateCommunity(idCommunity, updateCommunityRequest);

        assertEquals(community.getName(), result.getName());
        assertEquals(community.getId(), result.getId());
    }

    @Test
    @DisplayName("Delete community by ID")
    public void deleteCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(communityRepository.deleteCommunityById(any())).thenReturn(1);

        communityService.deleteCommunity(idCommunity);

        verify(communityRepository).deleteCommunityById(idCommunity);
    }

    @Test
    @DisplayName("Delete community by ID. Community not found.")
    public void deleteCommunityNotFoundTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(communityRepository.deleteCommunityById(any())).thenReturn(0);

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.deleteCommunity(idCommunity);
        });

        assertEquals(exception.getErrorCode(), COMMUNITY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Add members to community by ID.")
    public void addMembersToCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var person = generatePerson();
        var communityPersonRole = generateCommunityPersonRole();
        List<CommunityPersonRole> connections = Collections.singletonList(communityPersonRole);
        var addMemberRequest = new AddMembersRequest();
        addMemberRequest.setPersonId(person.getId());
        AddMembersRequest[] as = {addMemberRequest};

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(personRepository.findById(any())).thenReturn(Optional.of(person));
        when(communityPersonRoleMapper.toEntity(any(Community.class), any(Person.class), any(CommunityRole.class))).thenReturn(communityPersonRole);
        when(communityRepository.deleteCommunityById(any())).thenReturn(0);
        when(communityPersonRoleRepository.saveAll(any())).thenReturn(connections);

        var result = communityService.addMembers(idCommunity, as, person);

        assertEquals(community.getId(), result.getId());
        assertEquals(community, result);
    }

    @Test
    @DisplayName("Add members to community by ID. Added member is owner of community")
    public void addMembersToCommunityErrorRoleMemberTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var person = generatePerson();
        var communityPersonRole = generateCommunityPersonRole();
        List<CommunityPersonRole> connections = Collections.singletonList(communityPersonRole);
        var addMemberRequest = new AddMembersRequest();
        addMemberRequest.setPersonId(person.getId());
        addMemberRequest.setRole(OWNER);
        AddMembersRequest[] as = {addMemberRequest};

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(personRepository.findById(any())).thenReturn(Optional.of(person));
        when(communityPersonRoleMapper.toEntity(any(Community.class), any(Person.class), any(CommunityRole.class))).thenReturn(communityPersonRole);
        when(communityRepository.deleteCommunityById(any())).thenReturn(0);
        when(communityPersonRoleRepository.saveAll(any())).thenReturn(connections);

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.addMembers(idCommunity, as, person);
        });

        assertEquals(exception.getErrorCode(), COMMUNITY_CHANGE_OF_OWNER.getCode());
    }

    @Test
    @DisplayName("Add members to community by ID. Person not found")
    public void addMembersToCommunityPersonNotFoundTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var person = generatePerson();
        var communityPersonRole = generateCommunityPersonRole();
        List<CommunityPersonRole> connections = Collections.singletonList(communityPersonRole);
        var addMemberRequest = new AddMembersRequest();
        addMemberRequest.setPersonId(person.getId());
        AddMembersRequest[] as = {addMemberRequest};

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(communityPersonRoleMapper.toEntity(any(Community.class), any(Person.class), any(CommunityRole.class))).thenReturn(communityPersonRole);
        when(communityRepository.deleteCommunityById(any())).thenReturn(0);
        when(communityPersonRoleRepository.saveAll(any())).thenReturn(connections);

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.addMembers(idCommunity, as, person);
        });

        assertEquals(exception.getErrorCode(), PERSON_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Update person role in community.")
    public void updatePersonRoleInCommunityTest() {
        var idCommunity = UUID.randomUUID();
        var idPerson = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var updateRoleInCommunityRequest = new UpdateRoleInCommunityRequest();
        updateRoleInCommunityRequest.setPersonId(idPerson);
        updateRoleInCommunityRequest.setRole(CommunityRole.MEMBER);
        var communityPersonRole = generateCommunityPersonRole(community);

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(communityPersonRoleRepository.findByCommunityIdAndPersonId(any(), any())).thenReturn(Optional.ofNullable(communityPersonRole));
        var result = communityService.updatePersonRoleInCommunity(idCommunity, updateRoleInCommunityRequest);

        assertEquals(community.getId(), result.getId());
        assertEquals(community, result);
    }

    @Test
    @DisplayName("Update person role in community. Community change of owner")
    public void updatePersonRoleInCommunityCommunityChangeOfOwnerTest() {
        var idCommunity = UUID.randomUUID();
        var idPerson = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var updateRoleInCommunityRequest = new UpdateRoleInCommunityRequest();
        updateRoleInCommunityRequest.setPersonId(idPerson);
        updateRoleInCommunityRequest.setRole(CommunityRole.OWNER);
        var communityPersonRole = generateCommunityPersonRole(community);

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(communityPersonRoleRepository.findByCommunityIdAndPersonId(any(), any())).thenReturn(Optional.ofNullable(communityPersonRole));
        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.updatePersonRoleInCommunity(idCommunity, updateRoleInCommunityRequest);
        });

        assertEquals(exception.getErrorCode(), COMMUNITY_CHANGE_OF_OWNER.getCode());
    }

    @Test
    @DisplayName("Join community.")
    public void joinCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var person = generatePerson();

        var idPerson = person.getId();

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(communityPersonRoleRepository.existsCommunityPersonRoleByCommunityIdAndPersonId(any(), any())).thenReturn(false);
        when(personRepository.findById(any())).thenReturn(Optional.ofNullable(person));

        var result = communityService.joinCommunity(idCommunity, idPerson);

        assertEquals(community.getId(), result.getId());
        assertEquals(community, result);
    }

    @Test
    @DisplayName("Join community. Community not found")
    public void joinCommunityNotFoundTest() {
        var idCommunity = UUID.randomUUID();

        var person = generatePerson();

        var idPerson = person.getId();

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(null));
        when(communityPersonRoleRepository.existsCommunityPersonRoleByCommunityIdAndPersonId(any(), any())).thenReturn(false);
        when(personRepository.findById(any())).thenReturn(Optional.ofNullable(person));

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.joinCommunity(idCommunity, idPerson);
        });

        assertEquals(exception.getErrorCode(), COMMUNITY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Join community. Personal already in community")
    public void joinCommunityPersonalAlreadyInCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var person = generatePerson();

        var idPerson = person.getId();

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(null));
        when(communityPersonRoleRepository.existsCommunityPersonRoleByCommunityIdAndPersonId(any(), any())).thenReturn(true);
        when(personRepository.findById(any())).thenReturn(Optional.ofNullable(person));

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.joinCommunity(idCommunity, idPerson);
        });

        assertEquals(exception.getErrorCode(), COMMUNITY_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Join community. Personal not found")
    public void joinCommunityPersonalNotFoundTest() {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);
        var person = generatePerson();

        var idPerson = person.getId();

        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));
        when(communityPersonRoleRepository.existsCommunityPersonRoleByCommunityIdAndPersonId(any(), any())).thenReturn(false);
        when(personRepository.findById(any())).thenReturn(Optional.ofNullable(null));

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.joinCommunity(idCommunity, idPerson);
        });

        assertEquals(exception.getErrorCode(), PERSON_NOT_FOUND.getCode());
    }

    @Test
    @DisplayName("Leave community.")
    public void leaveCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var idPerson = UUID.randomUUID();

        when(communityPersonRoleRepository.deleteByCommunityIdAndPersonId(any(), any())).thenReturn(1);

        communityService.leaveCommunity(idCommunity, idPerson);

        verify(communityPersonRoleRepository).deleteByCommunityIdAndPersonId(idCommunity, idPerson);
    }

    @Test
    @DisplayName("Leave community. Person not in community")
    public void leaveCommunityPersonNotInCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var idPerson = UUID.randomUUID();

        when(communityPersonRoleRepository.deleteByCommunityIdAndPersonId(any(), any())).thenReturn(0);

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.leaveCommunity(idCommunity, idPerson);
        });

        assertEquals(exception.getErrorCode(), PERSON_NOT_IN_COMMUNITY.getCode());
    }

    @Test
    @DisplayName("Delete members.")
    public void deleteMembersTest() {
        var idCommunity = UUID.randomUUID();

        var  communityPersonRole =  generateCommunityPersonRole();

        var id1 = UUID.randomUUID();
        var id2 =  UUID.randomUUID();

        when(communityPersonRoleRepository.deleteByCommunityIdAndPersonId(any(), any())).thenReturn(1);
        when(communityPersonRoleRepository.findByCommunityIdAndCommunityRole(any(), any())).thenReturn(Optional.ofNullable(communityPersonRole));

        communityService.deleteMembers(idCommunity, new UUID[]{id1, id2});

        verify(communityPersonRoleRepository).deleteByCommunityIdAndPersonId(idCommunity, id1);
        verify(communityPersonRoleRepository).deleteByCommunityIdAndPersonId(idCommunity, id2);
    }

    @Test
    @DisplayName("Delete members. Community change of owner")
    public void deleteMembersCommunityChangeOfOwnerTest() {
        var idCommunity = UUID.randomUUID();

        var  communityPersonRole =  generateCommunityPersonRole();

        var id1 = UUID.randomUUID();
        var id2 =  UUID.randomUUID();
        var owner = generatePerson();
        owner.setId(id1);

        communityPersonRole.setPerson(owner);

        when(communityPersonRoleRepository.deleteByCommunityIdAndPersonId(any(), any())).thenReturn(1);
        when(communityPersonRoleRepository.findByCommunityIdAndCommunityRole(any(), any())).thenReturn(Optional.ofNullable(communityPersonRole));

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.deleteMembers(idCommunity, new UUID[]{id1, id2});
        });

        assertEquals(exception.getErrorCode(), COMMUNITY_CHANGE_OF_OWNER.getCode());
    }

    @Test
    @DisplayName("Delete members. Person not in community")
    public void deleteMembersPersonNotInCommunityTest() {
        var idCommunity = UUID.randomUUID();

        var  communityPersonRole =  generateCommunityPersonRole();

        var id1 = UUID.randomUUID();
        var id2 =  UUID.randomUUID();

        when(communityPersonRoleRepository.deleteByCommunityIdAndPersonId(any(), any())).thenReturn(0);
        when(communityPersonRoleRepository.findByCommunityIdAndCommunityRole(any(), any())).thenReturn(Optional.ofNullable(communityPersonRole));

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.deleteMembers(idCommunity, new UUID[]{id1, id2});
        });

        assertEquals(exception.getErrorCode(), PERSON_NOT_IN_COMMUNITY.getCode());
    }

    @Test
    @DisplayName("Save avatar.")
    public void saveAvatarTest() throws Exception {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);

        var fileImage = MultipartFileGeneratorUtils.generateFile("file", MediaType.IMAGE_JPEG);

        when(communityRepository.save(any())).thenReturn(community);
        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));

        var result = communityService.saveAvatar(idCommunity, fileImage);

        assertEquals(result.getId(), community.getId());
        assertEquals(result, community);
    }

    @Test
    @DisplayName("Save avatar. Avatar wrong file type.")
    public void saveAvatarWrongFileTypeTest() throws Exception {
        var idCommunity = UUID.randomUUID();

        var community = generateCommunity(idCommunity);

        var fileImage = MultipartFileGeneratorUtils.generateFile("file", MediaType.APPLICATION_XML);

        when(communityRepository.save(any())).thenReturn(community);
        when(communityRepository.findById(any())).thenReturn(Optional.ofNullable(community));

        var exception = assertThrows(ApplicationException.class, () -> {
            communityService.saveAvatar(idCommunity, fileImage);
        });

        assertEquals(exception.getErrorCode(), AVATAR_WRONG_FILE_TYPE.getCode());
    }
}