package ru.loylabs.portal.unit.service.story.updater;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.mockito.junit.jupiter.MockitoExtension;
import org.springframework.context.ApplicationEventPublisher;
import ru.loylabs.portal.entity.Person;
import ru.loylabs.portal.entity.Story;
import ru.loylabs.portal.entity.StoryGroup;
import ru.loylabs.portal.enums.StoryGroupType;
import ru.loylabs.portal.listener.events.StoryEvents;
import ru.loylabs.portal.repository.PersonRepository;
import ru.loylabs.portal.repository.StoryGroupRepository;
import ru.loylabs.portal.repository.StoryRepository;
import ru.loylabs.portal.service.interfaces.StoryManager;
import ru.loylabs.portal.service.story.updater.SystemBirthdaysStoryUpdater;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Optional;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

@ExtendWith(MockitoExtension.class)
class SystemBirthdaysStoryUpdaterTest {

    @Mock
    protected StoryGroupRepository storyGroupRepository;
    @Mock
    protected StoryRepository storyRepository;
    @Mock
    protected StoryManager.StorySettingsPack settings;
    @Mock
    protected ApplicationEventPublisher publisher;
    @Mock
    private PersonRepository personRepository;
    @Spy
    protected StoryManager storyManager = new StoryManager() {
        @Override
        public StorySettingsPack getStorySettingsPack(StoryGroupType groupType) {
            return settings;
        }
    };
    @InjectMocks
    private SystemBirthdaysStoryUpdater updater;
    private StoryGroup group;
    private List<Person> persons;

    @BeforeEach
    void setUp() {
        group = new StoryGroup();
        var firedPerson = new Person(); // Должен быть проигнорирован в историях
        firedPerson.setFired(LocalDate.now().minusDays(1));
        persons = List.of(new Person(),
                new Person(),
                new Person(),
                firedPerson);
    }

    @Test
    void testRefreshSystemGroup() {
        doReturn(persons).when(personRepository).findByDobBetween(anyString(), anyString());
        doReturn(Optional.of(group)).when(storyGroupRepository).findByGroupType(any(StoryGroupType.class));
        doReturn("").when(settings).getStoryContent(any(Person.class));
        doReturn("").when(settings).getStoryHeader(any(Person.class));

        int actual = updater.refreshSystemGroup();
        verify(storyGroupRepository).save(group);

        assertThat(actual).isEqualTo(3);
        assertThat(group.getStories()).hasSize(3);
    }

    @Test
    void testClearOutdatedStories() {
        var now = LocalDateTime.now();
        group.getStories().add(Story.builder().expireTime(now.minusDays(1)).priority(0).build());
        group.getStories().add(Story.builder().expireTime(now.minusDays(1)).priority(0).build());
        group.getStories().add(Story.builder().expireTime(now.plusDays(1)).priority(0).build());

        doReturn(Optional.of(group)).when(storyGroupRepository).findByGroupType(any(StoryGroupType.class));

        int actual = updater.clearOutdatedStories();

        assertThat(actual).isEqualTo(2);
        assertThat(group.getStories()).hasSize(1);
    }

    @Test
    void testClearOutdatedStoriesStoryGroupIsNull() {
        doReturn(Optional.empty()).when(storyGroupRepository).findByGroupType(any(StoryGroupType.class));

        int actual = updater.clearOutdatedStories();

        assertThat(actual).isEqualTo(0);
    }

    @Test
    void testClearOutdatedStoriesIsEmpty() {
        doReturn(Optional.of(group)).when(storyGroupRepository).findByGroupType(any(StoryGroupType.class));

        int actual = updater.clearOutdatedStories();

        verify(publisher).publishEvent(any(StoryEvents.StoryGroupDeleteEvent.class));
        verify(storyGroupRepository).deleteStoryGroupById(any());
        assertThat(actual).isEqualTo(0);
    }
}