import React, { FC, SetStateAction, useState } from "react";
import moment from "moment";

import { useTheme } from "@mui/material/styles";
import { Typography } from "@mui/material";
import Stack from "@mui/material/Stack";

import InputWrapper from "../../../common/Inputs/InputWrapper/InputWrapper";
import DatePickerCustom from "../../../common/Inputs/DatePickerCustom/DatePickerCustom";
import InputLine from "../../../common/Inputs/InputLine/InputLine";
import { checkLanguage } from "../../../../constants/errorText";
import CustomSelect from "../../../common/Inputs/CustomSelect/CustomSelect";
import { UserInfo } from "../../../../types/UserInfo.types";
import { checkLetters } from "../../../../operation/checkLetters";
import { useJobTitiles } from "../../../../hooks/useJobTitiles";
import Roles from "../../../common/Roles/Roles";
import Gender from "../../../common/Gender/Gender";
import { useDepartments } from "../../../../hooks/useDepartments";
import CityCombobox from "../../../UserEditing/PersonalInfo/CityCombobox/CityCombobox";

interface CreateUserContentProps {
  userInfo: UserInfo;
  setUserInfo: SetStateAction<any>;
}
export const CreateUserContent: FC<CreateUserContentProps> = ({
  userInfo,
  setUserInfo,
}) => {
  const theme = useTheme();

  const [pageTitle, setPageTitle] = useState<number>(0);
  const [pageDepartment, setPageDepartment] = useState<number>(0);
  const jobTitles = useJobTitiles(pageTitle);
  const departments = useDepartments(pageDepartment);

  const handleScrollTitles = (event: any) => {
    const target = event.target;
    if (
      target.scrollTop + target.clientHeight >= target.scrollHeight &&
      jobTitles.titles.length < jobTitles.size
    ) {
      setPageTitle((prev) => prev + 1);
    }
  };

  const handleScrollDepartments = (event: any) => {
    const target = event.target;
    if (
      target.scrollTop + target.clientHeight >= target.scrollHeight &&
      departments.departments.length < departments.size
    ) {
      setPageDepartment((prev) => prev + 1);
    }
  };

  const handleChangeInputText =
    (prop: keyof UserInfo) => (event: React.ChangeEvent<HTMLInputElement>) => {
      setUserInfo((prevState: UserInfo) => ({
        ...prevState,
        [prop]: event.target.value,
      }));
    };

  const handleChangeDate = (newValue?: string | null) => {
    setUserInfo((prevState: UserInfo) => ({
      ...prevState,
      dob: moment(newValue).format("YYYY-MM-DD"),
    }));
  };

  const data = [
    {
      name: "Е-mail",
      element: (
        <InputLine
          value={userInfo?.email}
          onChange={handleChangeInputText("email")}
          placeholder="Введите е-mail сотрудника"
        />
      ),
    },
    {
      name: "Фамилия",
      element: (
        <InputLine
          value={userInfo?.surname}
          onChange={handleChangeInputText("surname")}
          error={checkLetters(userInfo?.surname)}
          helperText={checkLanguage}
          placeholder="Укажите фамилию"
        />
      ),
    },
    {
      name: "Имя",
      element: (
        <InputLine
          value={userInfo?.name}
          onChange={handleChangeInputText("name")}
          error={checkLetters(userInfo?.name)}
          helperText={checkLanguage}
          placeholder="Укажите имя"
        />
      ),
    },
    {
      name: "Отчество",
      element: (
        <InputLine
          value={userInfo?.patronymic}
          onChange={handleChangeInputText("patronymic")}
          error={checkLetters(userInfo?.patronymic)}
          helperText={checkLanguage}
          placeholder="Укажите отчество"
        />
      ),
    },
    {
      name: "Должность",
      element: (
        <CustomSelect
          items={jobTitles.titles}
          value={userInfo?.jobTitle}
          onChange={handleChangeInputText("jobTitle")}
          placeholder="Выберите должность"
          onScroll={handleScrollTitles}
        />
      ),
    },
    {
      name: "Департамент",
      element: (
        <CustomSelect
          items={departments.departments}
          value={userInfo?.department}
          onChange={handleChangeInputText("department")}
          placeholder="Выберите департамент"
          onScroll={handleScrollDepartments}
        />
      ),
    },
    {
      name: "Дата рождения",
      element: (
        <DatePickerCustom
          value={userInfo?.dob}
          styles={{ width: "100%" }}
          onChange={handleChangeDate}
        />
      ),
    },
    {
      name: "Город",
      element: (
        <CityCombobox
          personData={userInfo}
          setPersonData={setUserInfo}
          width="100%"
        />
      ),
    },
    {
      name: "Пол",
      element: <Gender personData={userInfo} setPersonData={setUserInfo} />,
    },
    {
      name: "Выберите роли",
      element: (
        <Roles
          personData={userInfo}
          setPersonData={setUserInfo}
          disabled={false}
        />
      ),
    },
  ];

  const wrapperStyle = {
    "& > .MuiTypography-root": {
      color: theme.palette.text.secondary,
    },
  };

  return (
    <>
      <Typography variant="h4" mb={3}>
        Добавить сотрудника
      </Typography>
      <Stack spacing={4}>
        {data.map((item, index) => (
          <InputWrapper
            key={index}
            title={item.name}
            element={item.element}
            flexDirection="column"
            styles={wrapperStyle}
          />
        ))}
      </Stack>
    </>
  );
};

export default CreateUserContent;
