import { initialReview } from './initialStates/initialReview'
import { reviewController } from '../services/review.controller'
import { createSlice, PayloadAction } from '@reduxjs/toolkit'
import { createAppAsyncThunk } from './utils/createAppAsyncThunk'
import { EditReviewParams, ReviewList, ReviewRating, Skill } from '../models/review.models'
import { skillController } from '../services/skill.controller'
import { AnyAction } from 'redux'

export type ReviewInfo = {
  reviewList: ReviewList[]
  skillList: Skill[]
  estimations: {
    [key: string]: {
      items: ReviewRating[]
      totalItems: number
    }
  }
  isReviewInitialized: boolean
  error: string | null
  loading: boolean
}

const initialState: ReviewInfo = initialReview

const getFetchedReviewList = createAppAsyncThunk<{ reviewList: ReviewList[] }, string>(
  'review/review_list',
  async (userId: string, thunkAPI) => {
    const { dispatch, rejectWithValue } = thunkAPI

    dispatch(reviewActions.setReviewInitialized({ isReviewInitialized: true }))

    try {
      const res = await reviewController.getAllReview(userId)
      dispatch(reviewActions.reviewList(res.data.items))
      dispatch(reviewActions.setReviewInitialized({ isReviewInitialized: false }))
      return { reviewList: res.data.items }
    } catch (e) {
      return rejectWithValue('Server Error')
    }
  },
)

const getFetchedSkills = createAppAsyncThunk<Skill[]>('review/review_skills', async (_, thunkAPI) => {
  const { dispatch, rejectWithValue } = thunkAPI
  const res = await skillController.getSkills()
  try {
    dispatch(reviewActions.skillList(res.data.items))
  } catch (e) {
    return rejectWithValue(e)
  }
  return res.data.items
})

const createReview = createAppAsyncThunk<
  { review: ReviewList },
  { ownerId: string; reviewersId: string[]; skillId: string }
>('review/create_review', async (arg, thunkAPI) => {
  const { rejectWithValue } = thunkAPI
  try {
    const res = await reviewController.createReview(arg)
    return { review: res.data }
  } catch (e) {
    return rejectWithValue('Server Error')
  }
})

const editReview = createAppAsyncThunk<void, EditReviewParams>('review/edit_review', async (arg, thunkAPI) => {
  const { dispatch, rejectWithValue } = thunkAPI
  const res = await reviewController.editReview(arg.reviewId, arg.body)
  try {
    if (res.status === 200) {
      dispatch(reviewActions.updateReviewList(res.data))
    }
  } catch (e) {
    return rejectWithValue(e)
  }
})

const deleteReview = createAppAsyncThunk<void, { reviewId: string }>('review/delete_review', async (arg, thunkAPI) => {
  const { dispatch, rejectWithValue } = thunkAPI
  const res = await reviewController.deleteReview(arg.reviewId)
  try {
    dispatch(reviewActions.deleteReview({ reviewId: arg.reviewId }))
  } catch (e) {
    return rejectWithValue(e)
  }
})

const setEstimationForReview = createAppAsyncThunk<void, { reviewId: string; estimation: number }>(
  'review/set_estimation_for_review',
  async (arg, thunkAPI) => {
    const { dispatch, rejectWithValue } = thunkAPI
    const res = await reviewController.setEstimation(arg.reviewId, {
      rating: arg.estimation,
    })
    try {
      dispatch(reviewActions.updateReviewList(res.data.review))
    } catch (e) {
      return rejectWithValue(e)
    }
  },
)

const getFetchedAllEstimations = createAppAsyncThunk<void, { reviewId: string }>(
  'review/estimations',
  async (arg, thunkAPI) => {
    const { dispatch, rejectWithValue } = thunkAPI
    const res = await reviewController.getAllEstimations(arg.reviewId)
    try {
      dispatch(reviewActions.setEstimations({ reviewId: arg.reviewId, items: res.data }))
    } catch (e) {
      return rejectWithValue(e)
    }
  },
)

const slice = createSlice({
  name: 'review',
  initialState: initialState,

  reducers: {
    reviewList: (state, action: PayloadAction<ReviewList[]>) => {
      state.reviewList = action.payload
    },
    skillList: (state, action: PayloadAction<Skill[]>) => {
      state.skillList = action.payload
    },
    updateReviewList: (state, action: PayloadAction<ReviewList>) => {
      const index = state.reviewList.findIndex((el) => el.id === action.payload.id)
      if (index !== -1) {
        state.reviewList[index] = action.payload
      } else {
        state.reviewList.push(action.payload)
      }
    },
    deleteReview: (state, action: PayloadAction<{ reviewId: string }>) => {
      state.reviewList = state.reviewList.filter((review) => review.id !== action.payload.reviewId)
    },
    setEstimations: (state, action: PayloadAction<{ reviewId: string; items: any }>) => {
      //TODO bug: сохраняются оценки в ассоциативном массиве при переходе на другого юзера
      const { reviewId, items } = action.payload
      if (!state.estimations[reviewId]) {
        state.estimations[reviewId] = { items: [], totalItems: 0 }
      }
      state.estimations[reviewId] = {
        ...state.estimations[reviewId],
        ...items,
      }
    },
    setReviewInitialized: (state, action: PayloadAction<any>) => {
      state.isReviewInitialized = action.payload.isReviewInitialized
    },
  },
  extraReducers: (builder) => {
    builder
      .addCase(createReview.pending, (state, _) => {
        state.loading = true
        state.error = null
      })
      .addCase(createReview.rejected, (state, action) => {
        state.error = action.payload as string
        state.loading = false
      })
      .addCase(createReview.fulfilled, (state, action) => {
        state.reviewList.push(action.payload.review)
        state.loading = false
        state.error = null
      })
      .addMatcher(isError, (state, action: PayloadAction<string>) => {
        state.error = action.payload
        state.loading = false
      })
  },
})

function isError(action: AnyAction) {
  return action.type.endsWith('rejected')
}

export const reviewReducer = slice.reducer
export const reviewActions = slice.actions
export const reviewThunks = {
  getFetchedReviewList,
  getFetchedAllEstimations,
  createReview,
  deleteReview,
  editReview,
  setEstimationForReview,
  getFetchedSkills,
}
